pub const config_directory: []const u8 = "/etc";
pub const prefix_directory: []const u8 = "/usr";
pub const version: []const u8 = "1.4.1";
pub const tty: u8 = 2;
pub const fallback_tty: u8 = 2;
pub const fallback_uid_min: u32 = 1000;
pub const fallback_uid_max: u32 = 60000;
pub const enable_x11_support: bool = true;
