const __root = @This();
pub const __builtin = @import("c_builtins");
pub const __helpers = @import("helpers");

pub extern fn __errno_location() [*c]c_int;
pub const __u_char = u8;
pub const __u_short = c_ushort;
pub const __u_int = c_uint;
pub const __u_long = c_ulong;
pub const __int8_t = i8;
pub const __uint8_t = u8;
pub const __int16_t = c_short;
pub const __uint16_t = c_ushort;
pub const __int32_t = c_int;
pub const __uint32_t = c_uint;
pub const __int64_t = c_long;
pub const __uint64_t = c_ulong;
pub const __int_least8_t = __int8_t;
pub const __uint_least8_t = __uint8_t;
pub const __int_least16_t = __int16_t;
pub const __uint_least16_t = __uint16_t;
pub const __int_least32_t = __int32_t;
pub const __uint_least32_t = __uint32_t;
pub const __int_least64_t = __int64_t;
pub const __uint_least64_t = __uint64_t;
pub const __quad_t = c_long;
pub const __u_quad_t = c_ulong;
pub const __intmax_t = c_long;
pub const __uintmax_t = c_ulong;
pub const __dev_t = c_ulong;
pub const __uid_t = c_uint;
pub const __gid_t = c_uint;
pub const __ino_t = c_ulong;
pub const __ino64_t = c_ulong;
pub const __mode_t = c_uint;
pub const __nlink_t = c_ulong;
pub const __off_t = c_long;
pub const __off64_t = c_long;
pub const __pid_t = c_int;
pub const __fsid_t = extern struct {
    __val: [2]c_int,
};
pub const __clock_t = c_long;
pub const __rlim_t = c_ulong;
pub const __rlim64_t = c_ulong;
pub const __id_t = c_uint;
pub const __time_t = c_long;
pub const __useconds_t = c_uint;
pub const __suseconds_t = c_long;
pub const __suseconds64_t = c_long;
pub const __daddr_t = c_int;
pub const __key_t = c_int;
pub const __clockid_t = c_int;
pub const __timer_t = ?*anyopaque;
pub const __blksize_t = c_long;
pub const __blkcnt_t = c_long;
pub const __blkcnt64_t = c_long;
pub const __fsblkcnt_t = c_ulong;
pub const __fsblkcnt64_t = c_ulong;
pub const __fsfilcnt_t = c_ulong;
pub const __fsfilcnt64_t = c_ulong;
pub const __fsword_t = c_long;
pub const __ssize_t = c_long;
pub const __syscall_slong_t = c_long;
pub const __syscall_ulong_t = c_ulong;
pub const __loff_t = __off64_t;
pub const __caddr_t = [*c]u8;
pub const __intptr_t = c_long;
pub const __socklen_t = c_uint;
pub const __sig_atomic_t = c_int;
pub const struct_flock = extern struct {
    l_type: c_short,
    l_whence: c_short,
    l_start: __off_t,
    l_len: __off_t,
    l_pid: __pid_t,
};
pub const mode_t = __mode_t;
pub const off_t = __off_t;
pub const pid_t = __pid_t;
pub const time_t = __time_t;
pub const struct_timespec = extern struct {
    tv_sec: __time_t,
    tv_nsec: __syscall_slong_t,
};
pub const struct_stat = extern struct {
    st_dev: __dev_t,
    st_ino: __ino_t,
    st_nlink: __nlink_t,
    st_mode: __mode_t,
    st_uid: __uid_t,
    st_gid: __gid_t,
    __pad0: c_int,
    st_rdev: __dev_t,
    st_size: __off_t,
    st_blksize: __blksize_t,
    st_blocks: __blkcnt_t,
    st_atim: struct_timespec,
    st_mtim: struct_timespec,
    st_ctim: struct_timespec,
    __glibc_reserved: [3]__syscall_slong_t,
};
pub extern fn fcntl(__fd: c_int, __cmd: c_int, ...) c_int;
pub extern fn open(__file: [*c]const u8, __oflag: c_int, ...) c_int;
pub extern fn openat(__fd: c_int, __file: [*c]const u8, __oflag: c_int, ...) c_int;
pub extern fn creat(__file: [*c]const u8, __mode: mode_t) c_int;
pub extern fn lockf(__fd: c_int, __cmd: c_int, __len: off_t) c_int;
pub extern fn posix_fadvise(__fd: c_int, __offset: off_t, __len: off_t, __advise: c_int) c_int;
pub extern fn posix_fallocate(__fd: c_int, __offset: off_t, __len: off_t) c_int;
pub const sig_atomic_t = __sig_atomic_t;
pub const __sigset_t = extern struct {
    __val: [16]c_ulong,
    pub const sigemptyset = __root.sigemptyset;
    pub const sigfillset = __root.sigfillset;
    pub const sigaddset = __root.sigaddset;
    pub const sigdelset = __root.sigdelset;
    pub const sigismember = __root.sigismember;
    pub const sigsuspend = __root.sigsuspend;
    pub const sigpending = __root.sigpending;
    pub const sigwait = __root.sigwait;
    pub const sigwaitinfo = __root.sigwaitinfo;
    pub const sigtimedwait = __root.sigtimedwait;
};
pub const sigset_t = __sigset_t;
pub const uid_t = __uid_t;
pub const union_sigval = extern union {
    sival_int: c_int,
    sival_ptr: ?*anyopaque,
};
pub const __sigval_t = union_sigval;
const struct_unnamed_2 = extern struct {
    si_pid: __pid_t,
    si_uid: __uid_t,
};
const struct_unnamed_3 = extern struct {
    si_tid: c_int,
    si_overrun: c_int,
    si_sigval: __sigval_t,
};
const struct_unnamed_4 = extern struct {
    si_pid: __pid_t,
    si_uid: __uid_t,
    si_sigval: __sigval_t,
};
const struct_unnamed_5 = extern struct {
    si_pid: __pid_t,
    si_uid: __uid_t,
    si_status: c_int,
    si_utime: __clock_t,
    si_stime: __clock_t,
};
const struct_unnamed_8 = extern struct {
    _lower: ?*anyopaque,
    _upper: ?*anyopaque,
};
const union_unnamed_7 = extern union {
    _addr_bnd: struct_unnamed_8,
    _pkey: __uint32_t,
};
const struct_unnamed_6 = extern struct {
    si_addr: ?*anyopaque,
    si_addr_lsb: c_short,
    _bounds: union_unnamed_7,
};
const struct_unnamed_9 = extern struct {
    si_band: c_long,
    si_fd: c_int,
};
const struct_unnamed_10 = extern struct {
    _call_addr: ?*anyopaque,
    _syscall: c_int,
    _arch: c_uint,
};
const union_unnamed_1 = extern union {
    _pad: [28]c_int,
    _kill: struct_unnamed_2,
    _timer: struct_unnamed_3,
    _rt: struct_unnamed_4,
    _sigchld: struct_unnamed_5,
    _sigfault: struct_unnamed_6,
    _sigpoll: struct_unnamed_9,
    _sigsys: struct_unnamed_10,
};
pub const siginfo_t = extern struct {
    si_signo: c_int,
    si_errno: c_int,
    si_code: c_int,
    __pad0: c_int,
    _sifields: union_unnamed_1,
    pub const psiginfo = __root.psiginfo;
};
pub const SI_ASYNCNL: c_int = -60;
pub const SI_DETHREAD: c_int = -7;
pub const SI_TKILL: c_int = -6;
pub const SI_SIGIO: c_int = -5;
pub const SI_ASYNCIO: c_int = -4;
pub const SI_MESGQ: c_int = -3;
pub const SI_TIMER: c_int = -2;
pub const SI_QUEUE: c_int = -1;
pub const SI_USER: c_int = 0;
pub const SI_KERNEL: c_int = 128;
const enum_unnamed_11 = c_int;
pub const ILL_ILLOPC: c_int = 1;
pub const ILL_ILLOPN: c_int = 2;
pub const ILL_ILLADR: c_int = 3;
pub const ILL_ILLTRP: c_int = 4;
pub const ILL_PRVOPC: c_int = 5;
pub const ILL_PRVREG: c_int = 6;
pub const ILL_COPROC: c_int = 7;
pub const ILL_BADSTK: c_int = 8;
pub const ILL_BADIADDR: c_int = 9;
const enum_unnamed_12 = c_uint;
pub const FPE_INTDIV: c_int = 1;
pub const FPE_INTOVF: c_int = 2;
pub const FPE_FLTDIV: c_int = 3;
pub const FPE_FLTOVF: c_int = 4;
pub const FPE_FLTUND: c_int = 5;
pub const FPE_FLTRES: c_int = 6;
pub const FPE_FLTINV: c_int = 7;
pub const FPE_FLTSUB: c_int = 8;
pub const FPE_FLTUNK: c_int = 14;
pub const FPE_CONDTRAP: c_int = 15;
const enum_unnamed_13 = c_uint;
pub const SEGV_MAPERR: c_int = 1;
pub const SEGV_ACCERR: c_int = 2;
pub const SEGV_BNDERR: c_int = 3;
pub const SEGV_PKUERR: c_int = 4;
pub const SEGV_ACCADI: c_int = 5;
pub const SEGV_ADIDERR: c_int = 6;
pub const SEGV_ADIPERR: c_int = 7;
pub const SEGV_MTEAERR: c_int = 8;
pub const SEGV_MTESERR: c_int = 9;
pub const SEGV_CPERR: c_int = 10;
const enum_unnamed_14 = c_uint;
pub const BUS_ADRALN: c_int = 1;
pub const BUS_ADRERR: c_int = 2;
pub const BUS_OBJERR: c_int = 3;
pub const BUS_MCEERR_AR: c_int = 4;
pub const BUS_MCEERR_AO: c_int = 5;
const enum_unnamed_15 = c_uint;
pub const TRAP_BRKPT: c_int = 1;
pub const TRAP_TRACE: c_int = 2;
pub const TRAP_BRANCH: c_int = 3;
pub const TRAP_HWBKPT: c_int = 4;
pub const TRAP_UNK: c_int = 5;
pub const TRAP_PERF: c_int = 6;
const enum_unnamed_16 = c_uint;
pub const CLD_EXITED: c_int = 1;
pub const CLD_KILLED: c_int = 2;
pub const CLD_DUMPED: c_int = 3;
pub const CLD_TRAPPED: c_int = 4;
pub const CLD_STOPPED: c_int = 5;
pub const CLD_CONTINUED: c_int = 6;
const enum_unnamed_17 = c_uint;
pub const POLL_IN: c_int = 1;
pub const POLL_OUT: c_int = 2;
pub const POLL_MSG: c_int = 3;
pub const POLL_ERR: c_int = 4;
pub const POLL_PRI: c_int = 5;
pub const POLL_HUP: c_int = 6;
const enum_unnamed_18 = c_uint;
pub const sigval_t = __sigval_t;
pub const union_pthread_attr_t = extern union {
    __size: [56]u8,
    __align: c_long,
};
pub const pthread_attr_t = union_pthread_attr_t;
const struct_unnamed_20 = extern struct {
    _function: ?*const fn (__sigval_t) callconv(.c) void,
    _attribute: [*c]pthread_attr_t,
};
const union_unnamed_19 = extern union {
    _pad: [12]c_int,
    _tid: __pid_t,
    _sigev_thread: struct_unnamed_20,
};
pub const struct_sigevent = extern struct {
    sigev_value: __sigval_t,
    sigev_signo: c_int,
    sigev_notify: c_int,
    _sigev_un: union_unnamed_19,
};
pub const sigevent_t = struct_sigevent;
pub const SIGEV_SIGNAL: c_int = 0;
pub const SIGEV_NONE: c_int = 1;
pub const SIGEV_THREAD: c_int = 2;
pub const SIGEV_THREAD_ID: c_int = 4;
const enum_unnamed_21 = c_uint;
pub const __sighandler_t = ?*const fn (c_int) callconv(.c) void;
pub extern fn __sysv_signal(__sig: c_int, __handler: __sighandler_t) __sighandler_t;
pub extern fn signal(__sig: c_int, __handler: __sighandler_t) __sighandler_t;
pub extern fn kill(__pid: __pid_t, __sig: c_int) c_int;
pub extern fn killpg(__pgrp: __pid_t, __sig: c_int) c_int;
pub extern fn raise(__sig: c_int) c_int;
pub extern fn ssignal(__sig: c_int, __handler: __sighandler_t) __sighandler_t;
pub extern fn gsignal(__sig: c_int) c_int;
pub extern fn psignal(__sig: c_int, __s: [*c]const u8) void;
pub extern fn psiginfo(__pinfo: [*c]const siginfo_t, __s: [*c]const u8) void;
pub extern fn sigpause(__sig: c_int) c_int;
pub extern fn sigblock(__mask: c_int) c_int;
pub extern fn sigsetmask(__mask: c_int) c_int;
pub extern fn siggetmask() c_int;
pub const sig_t = __sighandler_t;
pub extern fn sigemptyset(__set: [*c]sigset_t) c_int;
pub extern fn sigfillset(__set: [*c]sigset_t) c_int;
pub extern fn sigaddset(__set: [*c]sigset_t, __signo: c_int) c_int;
pub extern fn sigdelset(__set: [*c]sigset_t, __signo: c_int) c_int;
pub extern fn sigismember(__set: [*c]const sigset_t, __signo: c_int) c_int;
const union_unnamed_22 = extern union {
    sa_handler: __sighandler_t,
    sa_sigaction: ?*const fn (c_int, [*c]siginfo_t, ?*anyopaque) callconv(.c) void,
};
pub const struct_sigaction = extern struct {
    __sigaction_handler: union_unnamed_22,
    sa_mask: __sigset_t,
    sa_flags: c_int,
    sa_restorer: ?*const fn () callconv(.c) void,
};
pub extern fn sigprocmask(__how: c_int, noalias __set: [*c]const sigset_t, noalias __oset: [*c]sigset_t) c_int;
pub extern fn sigsuspend(__set: [*c]const sigset_t) c_int;
pub extern fn sigaction(__sig: c_int, noalias __act: [*c]const struct_sigaction, noalias __oact: [*c]struct_sigaction) c_int;
pub extern fn sigpending(__set: [*c]sigset_t) c_int;
pub extern fn sigwait(noalias __set: [*c]const sigset_t, noalias __sig: [*c]c_int) c_int;
pub extern fn sigwaitinfo(noalias __set: [*c]const sigset_t, noalias __info: [*c]siginfo_t) c_int;
pub extern fn sigtimedwait(noalias __set: [*c]const sigset_t, noalias __info: [*c]siginfo_t, noalias __timeout: [*c]const struct_timespec) c_int;
pub extern fn sigqueue(__pid: __pid_t, __sig: c_int, __val: union_sigval) c_int;
pub const struct__fpx_sw_bytes = extern struct {
    magic1: __uint32_t,
    extended_size: __uint32_t,
    xstate_bv: __uint64_t,
    xstate_size: __uint32_t,
    __glibc_reserved1: [7]__uint32_t,
};
pub const struct__fpreg = extern struct {
    significand: [4]c_ushort,
    exponent: c_ushort,
};
pub const struct__fpxreg = extern struct {
    significand: [4]c_ushort,
    exponent: c_ushort,
    __glibc_reserved1: [3]c_ushort,
};
pub const struct__xmmreg = extern struct {
    element: [4]__uint32_t,
};
pub const struct__fpstate = extern struct {
    cwd: __uint16_t,
    swd: __uint16_t,
    ftw: __uint16_t,
    fop: __uint16_t,
    rip: __uint64_t,
    rdp: __uint64_t,
    mxcsr: __uint32_t,
    mxcr_mask: __uint32_t,
    _st: [8]struct__fpxreg,
    _xmm: [16]struct__xmmreg,
    __glibc_reserved1: [24]__uint32_t,
};
const union_unnamed_23 = extern union {
    fpstate: [*c]struct__fpstate,
    __fpstate_word: __uint64_t,
};
pub const struct_sigcontext = extern struct {
    r8: __uint64_t,
    r9: __uint64_t,
    r10: __uint64_t,
    r11: __uint64_t,
    r12: __uint64_t,
    r13: __uint64_t,
    r14: __uint64_t,
    r15: __uint64_t,
    rdi: __uint64_t,
    rsi: __uint64_t,
    rbp: __uint64_t,
    rbx: __uint64_t,
    rdx: __uint64_t,
    rax: __uint64_t,
    rcx: __uint64_t,
    rsp: __uint64_t,
    rip: __uint64_t,
    eflags: __uint64_t,
    cs: c_ushort,
    gs: c_ushort,
    fs: c_ushort,
    __pad0: c_ushort,
    err: __uint64_t,
    trapno: __uint64_t,
    oldmask: __uint64_t,
    cr2: __uint64_t,
    unnamed_0: union_unnamed_23,
    __reserved1: [8]__uint64_t,
    pub const sigreturn = __root.sigreturn;
};
pub const struct__xsave_hdr = extern struct {
    xstate_bv: __uint64_t,
    __glibc_reserved1: [2]__uint64_t,
    __glibc_reserved2: [5]__uint64_t,
};
pub const struct__ymmh_state = extern struct {
    ymmh_space: [64]__uint32_t,
};
pub const struct__xstate = extern struct {
    fpstate: struct__fpstate,
    xstate_hdr: struct__xsave_hdr,
    ymmh: struct__ymmh_state,
};
pub extern fn sigreturn(__scp: [*c]struct_sigcontext) c_int;
pub const ptrdiff_t = c_long;
pub const wchar_t = c_int;
pub const max_align_t = extern struct {
    __aro_max_align_ll: c_longlong,
    __aro_max_align_ld: c_longdouble,
};
pub const stack_t = extern struct {
    ss_sp: ?*anyopaque,
    ss_flags: c_int,
    ss_size: usize,
    pub const sigaltstack = __root.sigaltstack;
};
pub const greg_t = c_longlong;
pub const gregset_t = [23]greg_t;
pub const struct__libc_fpxreg = extern struct {
    significand: [4]c_ushort,
    exponent: c_ushort,
    __glibc_reserved1: [3]c_ushort,
};
pub const struct__libc_xmmreg = extern struct {
    element: [4]__uint32_t,
};
pub const struct__libc_fpstate = extern struct {
    cwd: __uint16_t,
    swd: __uint16_t,
    ftw: __uint16_t,
    fop: __uint16_t,
    rip: __uint64_t,
    rdp: __uint64_t,
    mxcsr: __uint32_t,
    mxcr_mask: __uint32_t,
    _st: [8]struct__libc_fpxreg,
    _xmm: [16]struct__libc_xmmreg,
    __glibc_reserved1: [24]__uint32_t,
};
pub const fpregset_t = [*c]struct__libc_fpstate;
pub const mcontext_t = extern struct {
    gregs: gregset_t,
    fpregs: fpregset_t,
    __reserved1: [8]c_ulonglong,
};
pub const struct_ucontext_t = extern struct {
    uc_flags: c_ulong,
    uc_link: [*c]struct_ucontext_t,
    uc_stack: stack_t,
    uc_mcontext: mcontext_t,
    uc_sigmask: sigset_t,
    __fpregs_mem: struct__libc_fpstate,
    __ssp: [4]c_ulonglong,
};
pub const ucontext_t = struct_ucontext_t;
pub extern fn siginterrupt(__sig: c_int, __interrupt: c_int) c_int;
pub const SS_ONSTACK: c_int = 1;
pub const SS_DISABLE: c_int = 2;
const enum_unnamed_24 = c_uint;
pub extern fn sigaltstack(noalias __ss: [*c]const stack_t, noalias __oss: [*c]stack_t) c_int;
pub const struct_sigstack = extern struct {
    ss_sp: ?*anyopaque,
    ss_onstack: c_int,
    pub const sigstack = __root.sigstack;
};
pub extern fn sigstack(__ss: [*c]struct_sigstack, __oss: [*c]struct_sigstack) c_int;
pub extern fn sighold(__sig: c_int) c_int;
pub extern fn sigrelse(__sig: c_int) c_int;
pub extern fn sigignore(__sig: c_int) c_int;
pub extern fn sigset(__sig: c_int, __disp: __sighandler_t) __sighandler_t;
const struct_unnamed_25 = extern struct {
    __low: c_uint,
    __high: c_uint,
};
pub const __atomic_wide_counter = extern union {
    __value64: c_ulonglong,
    __value32: struct_unnamed_25,
};
pub const struct___pthread_internal_list = extern struct {
    __prev: [*c]struct___pthread_internal_list,
    __next: [*c]struct___pthread_internal_list,
};
pub const __pthread_list_t = struct___pthread_internal_list;
pub const struct___pthread_internal_slist = extern struct {
    __next: [*c]struct___pthread_internal_slist,
};
pub const __pthread_slist_t = struct___pthread_internal_slist;
pub const struct___pthread_mutex_s = extern struct {
    __lock: c_int,
    __count: c_uint,
    __owner: c_int,
    __nusers: c_uint,
    __kind: c_int,
    __spins: c_short,
    __unused: c_short,
    __list: __pthread_list_t,
};
pub const struct___pthread_rwlock_arch_t = extern struct {
    __readers: c_uint,
    __writers: c_uint,
    __wrphase_futex: c_uint,
    __writers_futex: c_uint,
    __pad3: c_uint,
    __pad4: c_uint,
    __cur_writer: c_int,
    __shared: c_int,
    __pad1: c_ulong,
    __pad2: c_ulong,
    __flags: c_uint,
};
pub const struct___pthread_cond_s = extern struct {
    __wseq: __atomic_wide_counter,
    __g1_start: __atomic_wide_counter,
    __g_size: [2]c_uint,
    __g1_orig_size: c_uint,
    __wrefs: c_uint,
    __g_signals: [2]c_uint,
    __unused_initialized_1: c_uint,
    __unused_initialized_2: c_uint,
};
pub const __tss_t = c_uint;
pub const __thrd_t = c_ulong;
pub const __once_flag = extern struct {
    __data: c_int,
};
pub const pthread_t = c_ulong;
pub const pthread_mutexattr_t = extern union {
    __size: [4]u8,
    __align: c_int,
};
pub const pthread_condattr_t = extern union {
    __size: [4]u8,
    __align: c_int,
};
pub const pthread_key_t = c_uint;
pub const pthread_once_t = c_int;
pub const pthread_mutex_t = extern union {
    __data: struct___pthread_mutex_s,
    __size: [40]u8,
    __align: c_long,
};
pub const pthread_cond_t = extern union {
    __data: struct___pthread_cond_s,
    __size: [48]u8,
    __align: c_longlong,
};
pub const pthread_rwlock_t = extern union {
    __data: struct___pthread_rwlock_arch_t,
    __size: [56]u8,
    __align: c_long,
};
pub const pthread_rwlockattr_t = extern union {
    __size: [8]u8,
    __align: c_long,
};
pub const pthread_spinlock_t = c_int;
pub const pthread_barrier_t = extern union {
    __size: [32]u8,
    __align: c_long,
};
pub const pthread_barrierattr_t = extern union {
    __size: [4]u8,
    __align: c_int,
};
pub extern fn pthread_sigmask(__how: c_int, noalias __newmask: [*c]const __sigset_t, noalias __oldmask: [*c]__sigset_t) c_int;
pub extern fn pthread_kill(__threadid: pthread_t, __signo: c_int) c_int;
pub extern fn __libc_current_sigrtmin() c_int;
pub extern fn __libc_current_sigrtmax() c_int;
pub const struct___va_list_tag_26 = extern struct {
    unnamed_0: c_uint,
    unnamed_1: c_uint,
    unnamed_2: ?*anyopaque,
    unnamed_3: ?*anyopaque,
};
pub const __builtin_va_list = [1]struct___va_list_tag_26;
pub const va_list = __builtin_va_list;
pub const __gnuc_va_list = __builtin_va_list;
pub const int_least8_t = __int_least8_t;
pub const int_least16_t = __int_least16_t;
pub const int_least32_t = __int_least32_t;
pub const int_least64_t = __int_least64_t;
pub const uint_least8_t = __uint_least8_t;
pub const uint_least16_t = __uint_least16_t;
pub const uint_least32_t = __uint_least32_t;
pub const uint_least64_t = __uint_least64_t;
pub const int_fast8_t = i8;
pub const int_fast16_t = c_long;
pub const int_fast32_t = c_long;
pub const int_fast64_t = c_long;
pub const uint_fast8_t = u8;
pub const uint_fast16_t = c_ulong;
pub const uint_fast32_t = c_ulong;
pub const uint_fast64_t = c_ulong;
pub const intmax_t = __intmax_t;
pub const uintmax_t = __uintmax_t;
const union_unnamed_27 = extern union {
    __wch: c_uint,
    __wchb: [4]u8,
};
pub const __mbstate_t = extern struct {
    __count: c_int,
    __value: union_unnamed_27,
};
pub const struct__G_fpos_t = extern struct {
    __pos: __off_t,
    __state: __mbstate_t,
};
pub const __fpos_t = struct__G_fpos_t;
pub const struct__G_fpos64_t = extern struct {
    __pos: __off64_t,
    __state: __mbstate_t,
};
pub const __fpos64_t = struct__G_fpos64_t;
pub const struct__IO_marker = opaque {}; // /usr/include/bits/types/struct_FILE.h:75:7: warning: struct demoted to opaque type - has bitfield
pub const struct__IO_FILE = opaque {
    pub const fclose = __root.fclose;
    pub const fflush = __root.fflush;
    pub const fflush_unlocked = __root.fflush_unlocked;
    pub const setbuf = __root.setbuf;
    pub const setvbuf = __root.setvbuf;
    pub const setbuffer = __root.setbuffer;
    pub const setlinebuf = __root.setlinebuf;
    pub const fprintf = __root.fprintf;
    pub const vfprintf = __root.vfprintf;
    pub const fscanf = __root.fscanf;
    pub const vfscanf = __root.vfscanf;
    pub const fgetc = __root.fgetc;
    pub const getc = __root.getc;
    pub const getc_unlocked = __root.getc_unlocked;
    pub const fgetc_unlocked = __root.fgetc_unlocked;
    pub const getw = __root.getw;
    pub const fseek = __root.fseek;
    pub const ftell = __root.ftell;
    pub const rewind = __root.rewind;
    pub const fseeko = __root.fseeko;
    pub const ftello = __root.ftello;
    pub const fgetpos = __root.fgetpos;
    pub const fsetpos = __root.fsetpos;
    pub const clearerr = __root.clearerr;
    pub const feof = __root.feof;
    pub const ferror = __root.ferror;
    pub const clearerr_unlocked = __root.clearerr_unlocked;
    pub const feof_unlocked = __root.feof_unlocked;
    pub const ferror_unlocked = __root.ferror_unlocked;
    pub const fileno = __root.fileno;
    pub const fileno_unlocked = __root.fileno_unlocked;
    pub const pclose = __root.pclose;
    pub const flockfile = __root.flockfile;
    pub const ftrylockfile = __root.ftrylockfile;
    pub const funlockfile = __root.funlockfile;
    pub const __uflow = __root.__uflow;
    pub const __overflow = __root.__overflow;
    pub const unlocked = __root.fflush_unlocked;
    pub const uflow = __root.__uflow;
    pub const overflow = __root.__overflow;
};
pub const __FILE = struct__IO_FILE;
pub const FILE = struct__IO_FILE;
pub const struct__IO_codecvt = opaque {};
pub const struct__IO_wide_data = opaque {};
pub const _IO_lock_t = anyopaque;
pub const cookie_read_function_t = fn (__cookie: ?*anyopaque, __buf: [*c]u8, __nbytes: usize) callconv(.c) __ssize_t;
pub const cookie_write_function_t = fn (__cookie: ?*anyopaque, __buf: [*c]const u8, __nbytes: usize) callconv(.c) __ssize_t;
pub const cookie_seek_function_t = fn (__cookie: ?*anyopaque, __pos: [*c]__off64_t, __w: c_int) callconv(.c) c_int;
pub const cookie_close_function_t = fn (__cookie: ?*anyopaque) callconv(.c) c_int;
pub const struct__IO_cookie_io_functions_t = extern struct {
    read: ?*const cookie_read_function_t,
    write: ?*const cookie_write_function_t,
    seek: ?*const cookie_seek_function_t,
    close: ?*const cookie_close_function_t,
};
pub const cookie_io_functions_t = struct__IO_cookie_io_functions_t;
pub const fpos_t = __fpos_t;
pub extern var stdin: ?*FILE;
pub extern var stdout: ?*FILE;
pub extern var stderr: ?*FILE;
pub extern fn remove(__filename: [*c]const u8) c_int;
pub extern fn rename(__old: [*c]const u8, __new: [*c]const u8) c_int;
pub extern fn renameat(__oldfd: c_int, __old: [*c]const u8, __newfd: c_int, __new: [*c]const u8) c_int;
pub extern fn fclose(__stream: ?*FILE) c_int;
pub extern fn tmpfile() ?*FILE;
pub extern fn tmpnam([*c]u8) [*c]u8;
pub extern fn tmpnam_r(__s: [*c]u8) [*c]u8;
pub extern fn tempnam(__dir: [*c]const u8, __pfx: [*c]const u8) [*c]u8;
pub extern fn fflush(__stream: ?*FILE) c_int;
pub extern fn fflush_unlocked(__stream: ?*FILE) c_int;
pub extern fn fopen(noalias __filename: [*c]const u8, noalias __modes: [*c]const u8) ?*FILE;
pub extern fn freopen(noalias __filename: [*c]const u8, noalias __modes: [*c]const u8, noalias __stream: ?*FILE) ?*FILE;
pub extern fn fdopen(__fd: c_int, __modes: [*c]const u8) ?*FILE;
pub extern fn fopencookie(noalias __magic_cookie: ?*anyopaque, noalias __modes: [*c]const u8, __io_funcs: cookie_io_functions_t) ?*FILE;
pub extern fn fmemopen(__s: ?*anyopaque, __len: usize, __modes: [*c]const u8) ?*FILE;
pub extern fn open_memstream(__bufloc: [*c][*c]u8, __sizeloc: [*c]usize) ?*FILE;
pub extern fn setbuf(noalias __stream: ?*FILE, noalias __buf: [*c]u8) void;
pub extern fn setvbuf(noalias __stream: ?*FILE, noalias __buf: [*c]u8, __modes: c_int, __n: usize) c_int;
pub extern fn setbuffer(noalias __stream: ?*FILE, noalias __buf: [*c]u8, __size: usize) void;
pub extern fn setlinebuf(__stream: ?*FILE) void;
pub extern fn fprintf(noalias __stream: ?*FILE, noalias __format: [*c]const u8, ...) c_int;
pub extern fn printf(noalias __format: [*c]const u8, ...) c_int;
pub extern fn sprintf(noalias __s: [*c]u8, noalias __format: [*c]const u8, ...) c_int;
pub extern fn vfprintf(noalias __s: ?*FILE, noalias __format: [*c]const u8, __arg: [*c]struct___va_list_tag_26) c_int;
pub extern fn vprintf(noalias __format: [*c]const u8, __arg: [*c]struct___va_list_tag_26) c_int;
pub extern fn vsprintf(noalias __s: [*c]u8, noalias __format: [*c]const u8, __arg: [*c]struct___va_list_tag_26) c_int;
pub extern fn snprintf(noalias __s: [*c]u8, __maxlen: usize, noalias __format: [*c]const u8, ...) c_int;
pub extern fn vsnprintf(noalias __s: [*c]u8, __maxlen: usize, noalias __format: [*c]const u8, __arg: [*c]struct___va_list_tag_26) c_int;
pub extern fn vasprintf(noalias __ptr: [*c][*c]u8, noalias __f: [*c]const u8, __arg: [*c]struct___va_list_tag_26) c_int;
pub extern fn __asprintf(noalias __ptr: [*c][*c]u8, noalias __fmt: [*c]const u8, ...) c_int;
pub extern fn asprintf(noalias __ptr: [*c][*c]u8, noalias __fmt: [*c]const u8, ...) c_int;
pub extern fn vdprintf(__fd: c_int, noalias __fmt: [*c]const u8, __arg: [*c]struct___va_list_tag_26) c_int;
pub extern fn dprintf(__fd: c_int, noalias __fmt: [*c]const u8, ...) c_int;
pub extern fn fscanf(noalias __stream: ?*FILE, noalias __format: [*c]const u8, ...) c_int;
pub extern fn scanf(noalias __format: [*c]const u8, ...) c_int;
pub extern fn sscanf(noalias __s: [*c]const u8, noalias __format: [*c]const u8, ...) c_int;
pub extern fn vfscanf(noalias __s: ?*FILE, noalias __format: [*c]const u8, __arg: [*c]struct___va_list_tag_26) c_int;
pub extern fn vscanf(noalias __format: [*c]const u8, __arg: [*c]struct___va_list_tag_26) c_int;
pub extern fn vsscanf(noalias __s: [*c]const u8, noalias __format: [*c]const u8, __arg: [*c]struct___va_list_tag_26) c_int;
pub extern fn fgetc(__stream: ?*FILE) c_int;
pub extern fn getc(__stream: ?*FILE) c_int;
pub extern fn getchar() c_int;
pub extern fn getc_unlocked(__stream: ?*FILE) c_int;
pub extern fn getchar_unlocked() c_int;
pub extern fn fgetc_unlocked(__stream: ?*FILE) c_int;
pub extern fn fputc(__c: c_int, __stream: ?*FILE) c_int;
pub extern fn putc(__c: c_int, __stream: ?*FILE) c_int;
pub extern fn putchar(__c: c_int) c_int;
pub extern fn fputc_unlocked(__c: c_int, __stream: ?*FILE) c_int;
pub extern fn putc_unlocked(__c: c_int, __stream: ?*FILE) c_int;
pub extern fn putchar_unlocked(__c: c_int) c_int;
pub extern fn getw(__stream: ?*FILE) c_int;
pub extern fn putw(__w: c_int, __stream: ?*FILE) c_int;
pub extern fn fgets(noalias __s: [*c]u8, __n: c_int, noalias __stream: ?*FILE) [*c]u8;
pub extern fn __getdelim(noalias __lineptr: [*c][*c]u8, noalias __n: [*c]usize, __delimiter: c_int, noalias __stream: ?*FILE) __ssize_t;
pub extern fn getdelim(noalias __lineptr: [*c][*c]u8, noalias __n: [*c]usize, __delimiter: c_int, noalias __stream: ?*FILE) __ssize_t;
pub extern fn getline(noalias __lineptr: [*c][*c]u8, noalias __n: [*c]usize, noalias __stream: ?*FILE) __ssize_t;
pub extern fn fputs(noalias __s: [*c]const u8, noalias __stream: ?*FILE) c_int;
pub extern fn puts(__s: [*c]const u8) c_int;
pub extern fn ungetc(__c: c_int, __stream: ?*FILE) c_int;
pub extern fn fread(noalias __ptr: ?*anyopaque, __size: usize, __n: usize, noalias __stream: ?*FILE) usize;
pub extern fn fwrite(noalias __ptr: ?*const anyopaque, __size: usize, __n: usize, noalias __s: ?*FILE) usize;
pub extern fn fread_unlocked(noalias __ptr: ?*anyopaque, __size: usize, __n: usize, noalias __stream: ?*FILE) usize;
pub extern fn fwrite_unlocked(noalias __ptr: ?*const anyopaque, __size: usize, __n: usize, noalias __stream: ?*FILE) usize;
pub extern fn fseek(__stream: ?*FILE, __off: c_long, __whence: c_int) c_int;
pub extern fn ftell(__stream: ?*FILE) c_long;
pub extern fn rewind(__stream: ?*FILE) void;
pub extern fn fseeko(__stream: ?*FILE, __off: __off_t, __whence: c_int) c_int;
pub extern fn ftello(__stream: ?*FILE) __off_t;
pub extern fn fgetpos(noalias __stream: ?*FILE, noalias __pos: [*c]fpos_t) c_int;
pub extern fn fsetpos(__stream: ?*FILE, __pos: [*c]const fpos_t) c_int;
pub extern fn clearerr(__stream: ?*FILE) void;
pub extern fn feof(__stream: ?*FILE) c_int;
pub extern fn ferror(__stream: ?*FILE) c_int;
pub extern fn clearerr_unlocked(__stream: ?*FILE) void;
pub extern fn feof_unlocked(__stream: ?*FILE) c_int;
pub extern fn ferror_unlocked(__stream: ?*FILE) c_int;
pub extern fn perror(__s: [*c]const u8) void;
pub extern fn fileno(__stream: ?*FILE) c_int;
pub extern fn fileno_unlocked(__stream: ?*FILE) c_int;
pub extern fn pclose(__stream: ?*FILE) c_int;
pub extern fn popen(__command: [*c]const u8, __modes: [*c]const u8) ?*FILE;
pub extern fn ctermid(__s: [*c]u8) [*c]u8;
pub extern fn flockfile(__stream: ?*FILE) void;
pub extern fn ftrylockfile(__stream: ?*FILE) c_int;
pub extern fn funlockfile(__stream: ?*FILE) void;
pub extern fn __uflow(?*FILE) c_int;
pub extern fn __overflow(?*FILE, c_int) c_int;
pub const div_t = extern struct {
    quot: c_int,
    rem: c_int,
};
pub const ldiv_t = extern struct {
    quot: c_long,
    rem: c_long,
};
pub const lldiv_t = extern struct {
    quot: c_longlong,
    rem: c_longlong,
};
pub extern fn __ctype_get_mb_cur_max() usize;
pub extern fn atof(__nptr: [*c]const u8) f64;
pub extern fn atoi(__nptr: [*c]const u8) c_int;
pub extern fn atol(__nptr: [*c]const u8) c_long;
pub extern fn atoll(__nptr: [*c]const u8) c_longlong;
pub extern fn strtod(noalias __nptr: [*c]const u8, noalias __endptr: [*c][*c]u8) f64;
pub extern fn strtof(noalias __nptr: [*c]const u8, noalias __endptr: [*c][*c]u8) f32;
pub extern fn strtold(noalias __nptr: [*c]const u8, noalias __endptr: [*c][*c]u8) c_longdouble;
pub extern fn strtol(noalias __nptr: [*c]const u8, noalias __endptr: [*c][*c]u8, __base: c_int) c_long;
pub extern fn strtoul(noalias __nptr: [*c]const u8, noalias __endptr: [*c][*c]u8, __base: c_int) c_ulong;
pub extern fn strtoq(noalias __nptr: [*c]const u8, noalias __endptr: [*c][*c]u8, __base: c_int) c_longlong;
pub extern fn strtouq(noalias __nptr: [*c]const u8, noalias __endptr: [*c][*c]u8, __base: c_int) c_ulonglong;
pub extern fn strtoll(noalias __nptr: [*c]const u8, noalias __endptr: [*c][*c]u8, __base: c_int) c_longlong;
pub extern fn strtoull(noalias __nptr: [*c]const u8, noalias __endptr: [*c][*c]u8, __base: c_int) c_ulonglong;
pub extern fn l64a(__n: c_long) [*c]u8;
pub extern fn a64l(__s: [*c]const u8) c_long;
pub const u_char = __u_char;
pub const u_short = __u_short;
pub const u_int = __u_int;
pub const u_long = __u_long;
pub const quad_t = __quad_t;
pub const u_quad_t = __u_quad_t;
pub const fsid_t = __fsid_t;
pub const loff_t = __loff_t;
pub const ino_t = __ino_t;
pub const dev_t = __dev_t;
pub const gid_t = __gid_t;
pub const nlink_t = __nlink_t;
pub const id_t = __id_t;
pub const daddr_t = __daddr_t;
pub const caddr_t = __caddr_t;
pub const key_t = __key_t;
pub const clock_t = __clock_t;
pub const clockid_t = __clockid_t;
pub const timer_t = __timer_t;
pub const useconds_t = __useconds_t;
pub const suseconds_t = __suseconds_t;
pub const ulong = c_ulong;
pub const ushort = c_ushort;
pub const uint = c_uint;
pub const u_int8_t = __uint8_t;
pub const u_int16_t = __uint16_t;
pub const u_int32_t = __uint32_t;
pub const u_int64_t = __uint64_t;
pub const register_t = c_int;
pub fn __bswap_16(arg___bsx: __uint16_t) callconv(.c) __uint16_t {
    var __bsx = arg___bsx;
    _ = &__bsx;
    return @byteSwap(@as(__uint16_t, __bsx));
}
pub fn __bswap_32(arg___bsx: __uint32_t) callconv(.c) __uint32_t {
    var __bsx = arg___bsx;
    _ = &__bsx;
    return @bitCast(@as(c_int, @byteSwap(@as(c_int, @bitCast(@as(c_uint, @truncate(__bsx)))))));
}
pub fn __bswap_64(arg___bsx: __uint64_t) callconv(.c) __uint64_t {
    var __bsx = arg___bsx;
    _ = &__bsx;
    return @bitCast(@as(c_long, @byteSwap(@as(c_long, @bitCast(@as(c_ulong, @truncate(__bsx)))))));
}
pub fn __uint16_identity(arg___x: __uint16_t) callconv(.c) __uint16_t {
    var __x = arg___x;
    _ = &__x;
    return __x;
}
pub fn __uint32_identity(arg___x: __uint32_t) callconv(.c) __uint32_t {
    var __x = arg___x;
    _ = &__x;
    return __x;
}
pub fn __uint64_identity(arg___x: __uint64_t) callconv(.c) __uint64_t {
    var __x = arg___x;
    _ = &__x;
    return __x;
}
pub const struct_timeval = extern struct {
    tv_sec: __time_t,
    tv_usec: __suseconds_t,
    pub const gettimeofday = __root.gettimeofday;
    pub const settimeofday = __root.settimeofday;
    pub const adjtime = __root.adjtime;
};
pub const __fd_mask = c_long;
pub const fd_set = extern struct {
    fds_bits: [16]__fd_mask,
};
pub const fd_mask = __fd_mask;
pub extern fn select(__nfds: c_int, noalias __readfds: [*c]fd_set, noalias __writefds: [*c]fd_set, noalias __exceptfds: [*c]fd_set, noalias __timeout: [*c]struct_timeval) c_int;
pub extern fn pselect(__nfds: c_int, noalias __readfds: [*c]fd_set, noalias __writefds: [*c]fd_set, noalias __exceptfds: [*c]fd_set, noalias __timeout: [*c]const struct_timespec, noalias __sigmask: [*c]const __sigset_t) c_int;
pub const blksize_t = __blksize_t;
pub const blkcnt_t = __blkcnt_t;
pub const fsblkcnt_t = __fsblkcnt_t;
pub const fsfilcnt_t = __fsfilcnt_t;
pub extern fn random() c_long;
pub extern fn srandom(__seed: c_uint) void;
pub extern fn initstate(__seed: c_uint, __statebuf: [*c]u8, __statelen: usize) [*c]u8;
pub extern fn setstate(__statebuf: [*c]u8) [*c]u8;
pub const struct_random_data = extern struct {
    fptr: [*c]i32,
    rptr: [*c]i32,
    state: [*c]i32,
    rand_type: c_int,
    rand_deg: c_int,
    rand_sep: c_int,
    end_ptr: [*c]i32,
    pub const random_r = __root.random_r;
    pub const r = __root.random_r;
};
pub extern fn random_r(noalias __buf: [*c]struct_random_data, noalias __result: [*c]i32) c_int;
pub extern fn srandom_r(__seed: c_uint, __buf: [*c]struct_random_data) c_int;
pub extern fn initstate_r(__seed: c_uint, noalias __statebuf: [*c]u8, __statelen: usize, noalias __buf: [*c]struct_random_data) c_int;
pub extern fn setstate_r(noalias __statebuf: [*c]u8, noalias __buf: [*c]struct_random_data) c_int;
pub extern fn rand() c_int;
pub extern fn srand(__seed: c_uint) void;
pub extern fn rand_r(__seed: [*c]c_uint) c_int;
pub extern fn drand48() f64;
pub extern fn erand48(__xsubi: [*c]c_ushort) f64;
pub extern fn lrand48() c_long;
pub extern fn nrand48(__xsubi: [*c]c_ushort) c_long;
pub extern fn mrand48() c_long;
pub extern fn jrand48(__xsubi: [*c]c_ushort) c_long;
pub extern fn srand48(__seedval: c_long) void;
pub extern fn seed48(__seed16v: [*c]c_ushort) [*c]c_ushort;
pub extern fn lcong48(__param: [*c]c_ushort) void;
pub const struct_drand48_data = extern struct {
    __x: [3]c_ushort,
    __old_x: [3]c_ushort,
    __c: c_ushort,
    __init: c_ushort,
    __a: c_ulonglong,
    pub const drand48_r = __root.drand48_r;
    pub const lrand48_r = __root.lrand48_r;
    pub const mrand48_r = __root.mrand48_r;
    pub const r = __root.drand48_r;
};
pub extern fn drand48_r(noalias __buffer: [*c]struct_drand48_data, noalias __result: [*c]f64) c_int;
pub extern fn erand48_r(__xsubi: [*c]c_ushort, noalias __buffer: [*c]struct_drand48_data, noalias __result: [*c]f64) c_int;
pub extern fn lrand48_r(noalias __buffer: [*c]struct_drand48_data, noalias __result: [*c]c_long) c_int;
pub extern fn nrand48_r(__xsubi: [*c]c_ushort, noalias __buffer: [*c]struct_drand48_data, noalias __result: [*c]c_long) c_int;
pub extern fn mrand48_r(noalias __buffer: [*c]struct_drand48_data, noalias __result: [*c]c_long) c_int;
pub extern fn jrand48_r(__xsubi: [*c]c_ushort, noalias __buffer: [*c]struct_drand48_data, noalias __result: [*c]c_long) c_int;
pub extern fn srand48_r(__seedval: c_long, __buffer: [*c]struct_drand48_data) c_int;
pub extern fn seed48_r(__seed16v: [*c]c_ushort, __buffer: [*c]struct_drand48_data) c_int;
pub extern fn lcong48_r(__param: [*c]c_ushort, __buffer: [*c]struct_drand48_data) c_int;
pub extern fn arc4random() __uint32_t;
pub extern fn arc4random_buf(__buf: ?*anyopaque, __size: usize) void;
pub extern fn arc4random_uniform(__upper_bound: __uint32_t) __uint32_t;
pub extern fn malloc(__size: usize) ?*anyopaque;
pub extern fn calloc(__nmemb: usize, __size: usize) ?*anyopaque;
pub extern fn realloc(__ptr: ?*anyopaque, __size: usize) ?*anyopaque;
pub extern fn free(__ptr: ?*anyopaque) void;
pub extern fn reallocarray(__ptr: ?*anyopaque, __nmemb: usize, __size: usize) ?*anyopaque;
pub extern fn alloca(__size: usize) ?*anyopaque;
pub extern fn valloc(__size: usize) ?*anyopaque;
pub extern fn posix_memalign(__memptr: [*c]?*anyopaque, __alignment: usize, __size: usize) c_int;
pub extern fn aligned_alloc(__alignment: usize, __size: usize) ?*anyopaque;
pub extern fn abort() noreturn;
pub extern fn atexit(__func: ?*const fn () callconv(.c) void) c_int;
pub extern fn at_quick_exit(__func: ?*const fn () callconv(.c) void) c_int;
pub extern fn on_exit(__func: ?*const fn (__status: c_int, __arg: ?*anyopaque) callconv(.c) void, __arg: ?*anyopaque) c_int;
pub extern fn exit(__status: c_int) noreturn;
pub extern fn quick_exit(__status: c_int) noreturn;
pub extern fn _Exit(__status: c_int) noreturn;
pub extern fn getenv(__name: [*c]const u8) [*c]u8;
pub extern fn putenv(__string: [*c]u8) c_int;
pub extern fn setenv(__name: [*c]const u8, __value: [*c]const u8, __replace: c_int) c_int;
pub extern fn unsetenv(__name: [*c]const u8) c_int;
pub extern fn clearenv() c_int;
pub extern fn mktemp(__template: [*c]u8) [*c]u8;
pub extern fn mkstemp(__template: [*c]u8) c_int;
pub extern fn mkstemps(__template: [*c]u8, __suffixlen: c_int) c_int;
pub extern fn mkdtemp(__template: [*c]u8) [*c]u8;
pub extern fn system(__command: [*c]const u8) c_int;
pub extern fn realpath(noalias __name: [*c]const u8, noalias __resolved: [*c]u8) [*c]u8;
pub const __compar_fn_t = ?*const fn (?*const anyopaque, ?*const anyopaque) callconv(.c) c_int;
pub extern fn bsearch(__key: ?*const anyopaque, __base: ?*const anyopaque, __nmemb: usize, __size: usize, __compar: __compar_fn_t) ?*anyopaque;
pub extern fn qsort(__base: ?*anyopaque, __nmemb: usize, __size: usize, __compar: __compar_fn_t) void;
pub extern fn abs(__x: c_int) c_int;
pub extern fn labs(__x: c_long) c_long;
pub extern fn llabs(__x: c_longlong) c_longlong;
pub extern fn div(__numer: c_int, __denom: c_int) div_t;
pub extern fn ldiv(__numer: c_long, __denom: c_long) ldiv_t;
pub extern fn lldiv(__numer: c_longlong, __denom: c_longlong) lldiv_t;
pub extern fn ecvt(__value: f64, __ndigit: c_int, noalias __decpt: [*c]c_int, noalias __sign: [*c]c_int) [*c]u8;
pub extern fn fcvt(__value: f64, __ndigit: c_int, noalias __decpt: [*c]c_int, noalias __sign: [*c]c_int) [*c]u8;
pub extern fn gcvt(__value: f64, __ndigit: c_int, __buf: [*c]u8) [*c]u8;
pub extern fn qecvt(__value: c_longdouble, __ndigit: c_int, noalias __decpt: [*c]c_int, noalias __sign: [*c]c_int) [*c]u8;
pub extern fn qfcvt(__value: c_longdouble, __ndigit: c_int, noalias __decpt: [*c]c_int, noalias __sign: [*c]c_int) [*c]u8;
pub extern fn qgcvt(__value: c_longdouble, __ndigit: c_int, __buf: [*c]u8) [*c]u8;
pub extern fn ecvt_r(__value: f64, __ndigit: c_int, noalias __decpt: [*c]c_int, noalias __sign: [*c]c_int, noalias __buf: [*c]u8, __len: usize) c_int;
pub extern fn fcvt_r(__value: f64, __ndigit: c_int, noalias __decpt: [*c]c_int, noalias __sign: [*c]c_int, noalias __buf: [*c]u8, __len: usize) c_int;
pub extern fn qecvt_r(__value: c_longdouble, __ndigit: c_int, noalias __decpt: [*c]c_int, noalias __sign: [*c]c_int, noalias __buf: [*c]u8, __len: usize) c_int;
pub extern fn qfcvt_r(__value: c_longdouble, __ndigit: c_int, noalias __decpt: [*c]c_int, noalias __sign: [*c]c_int, noalias __buf: [*c]u8, __len: usize) c_int;
pub extern fn mblen(__s: [*c]const u8, __n: usize) c_int;
pub extern fn mbtowc(noalias __pwc: [*c]wchar_t, noalias __s: [*c]const u8, __n: usize) c_int;
pub extern fn wctomb(__s: [*c]u8, __wchar: wchar_t) c_int;
pub extern fn mbstowcs(noalias __pwcs: [*c]wchar_t, noalias __s: [*c]const u8, __n: usize) usize;
pub extern fn wcstombs(noalias __s: [*c]u8, noalias __pwcs: [*c]const wchar_t, __n: usize) usize;
pub extern fn rpmatch(__response: [*c]const u8) c_int;
pub extern fn getsubopt(noalias __optionp: [*c][*c]u8, noalias __tokens: [*c]const [*c]u8, noalias __valuep: [*c][*c]u8) c_int;
pub extern fn posix_openpt(__oflag: c_int) c_int;
pub extern fn grantpt(__fd: c_int) c_int;
pub extern fn unlockpt(__fd: c_int) c_int;
pub extern fn ptsname(__fd: c_int) [*c]u8;
pub extern fn getloadavg(__loadavg: [*c]f64, __nelem: c_int) c_int;
pub extern fn memcpy(noalias __dest: ?*anyopaque, noalias __src: ?*const anyopaque, __n: usize) ?*anyopaque;
pub extern fn memmove(__dest: ?*anyopaque, __src: ?*const anyopaque, __n: usize) ?*anyopaque;
pub extern fn memccpy(noalias __dest: ?*anyopaque, noalias __src: ?*const anyopaque, __c: c_int, __n: usize) ?*anyopaque;
pub extern fn memset(__s: ?*anyopaque, __c: c_int, __n: usize) ?*anyopaque;
pub extern fn memset_explicit(__s: ?*anyopaque, __c: c_int, __n: usize) ?*anyopaque;
pub extern fn memcmp(__s1: ?*const anyopaque, __s2: ?*const anyopaque, __n: usize) c_int;
pub extern fn __memcmpeq(__s1: ?*const anyopaque, __s2: ?*const anyopaque, __n: usize) c_int;
pub extern fn memchr(__s: ?*const anyopaque, __c: c_int, __n: usize) ?*anyopaque;
pub extern fn strcpy(noalias __dest: [*c]u8, noalias __src: [*c]const u8) [*c]u8;
pub extern fn strncpy(noalias __dest: [*c]u8, noalias __src: [*c]const u8, __n: usize) [*c]u8;
pub extern fn strcat(noalias __dest: [*c]u8, noalias __src: [*c]const u8) [*c]u8;
pub extern fn strncat(noalias __dest: [*c]u8, noalias __src: [*c]const u8, __n: usize) [*c]u8;
pub extern fn strcmp(__s1: [*c]const u8, __s2: [*c]const u8) c_int;
pub extern fn strncmp(__s1: [*c]const u8, __s2: [*c]const u8, __n: usize) c_int;
pub extern fn strcoll(__s1: [*c]const u8, __s2: [*c]const u8) c_int;
pub extern fn strxfrm(noalias __dest: [*c]u8, noalias __src: [*c]const u8, __n: usize) usize;
pub const struct___locale_data_28 = opaque {};
pub const struct___locale_struct = extern struct {
    __locales: [13]?*struct___locale_data_28,
    __ctype_b: [*c]const c_ushort,
    __ctype_tolower: [*c]const c_int,
    __ctype_toupper: [*c]const c_int,
    __names: [13][*c]const u8,
};
pub const __locale_t = [*c]struct___locale_struct;
pub const locale_t = __locale_t;
pub extern fn strcoll_l(__s1: [*c]const u8, __s2: [*c]const u8, __l: locale_t) c_int;
pub extern fn strxfrm_l(__dest: [*c]u8, __src: [*c]const u8, __n: usize, __l: locale_t) usize;
pub extern fn strdup(__s: [*c]const u8) [*c]u8;
pub extern fn strndup(__string: [*c]const u8, __n: usize) [*c]u8;
pub extern fn strchr(__s: [*c]const u8, __c: c_int) [*c]u8;
pub extern fn strrchr(__s: [*c]const u8, __c: c_int) [*c]u8;
pub extern fn strchrnul(__s: [*c]const u8, __c: c_int) [*c]u8;
pub extern fn strcspn(__s: [*c]const u8, __reject: [*c]const u8) usize;
pub extern fn strspn(__s: [*c]const u8, __accept: [*c]const u8) usize;
pub extern fn strpbrk(__s: [*c]const u8, __accept: [*c]const u8) [*c]u8;
pub extern fn strstr(__haystack: [*c]const u8, __needle: [*c]const u8) [*c]u8;
pub extern fn strtok(noalias __s: [*c]u8, noalias __delim: [*c]const u8) [*c]u8;
pub extern fn __strtok_r(noalias __s: [*c]u8, noalias __delim: [*c]const u8, noalias __save_ptr: [*c][*c]u8) [*c]u8;
pub extern fn strtok_r(noalias __s: [*c]u8, noalias __delim: [*c]const u8, noalias __save_ptr: [*c][*c]u8) [*c]u8;
pub extern fn strcasestr(__haystack: [*c]const u8, __needle: [*c]const u8) [*c]u8;
pub extern fn memmem(__haystack: ?*const anyopaque, __haystacklen: usize, __needle: ?*const anyopaque, __needlelen: usize) ?*anyopaque;
pub extern fn __mempcpy(noalias __dest: ?*anyopaque, noalias __src: ?*const anyopaque, __n: usize) ?*anyopaque;
pub extern fn mempcpy(noalias __dest: ?*anyopaque, noalias __src: ?*const anyopaque, __n: usize) ?*anyopaque;
pub extern fn strlen(__s: [*c]const u8) usize;
pub extern fn strnlen(__string: [*c]const u8, __maxlen: usize) usize;
pub extern fn strerror(__errnum: c_int) [*c]u8;
pub extern fn strerror_r(__errnum: c_int, __buf: [*c]u8, __buflen: usize) c_int;
pub extern fn strerror_l(__errnum: c_int, __l: locale_t) [*c]u8;
pub extern fn bcmp(__s1: ?*const anyopaque, __s2: ?*const anyopaque, __n: usize) c_int;
pub extern fn bcopy(__src: ?*const anyopaque, __dest: ?*anyopaque, __n: usize) void;
pub extern fn bzero(__s: ?*anyopaque, __n: usize) void;
pub extern fn index(__s: [*c]const u8, __c: c_int) [*c]u8;
pub extern fn rindex(__s: [*c]const u8, __c: c_int) [*c]u8;
pub extern fn ffs(__i: c_int) c_int;
pub extern fn ffsl(__l: c_long) c_int;
pub extern fn ffsll(__ll: c_longlong) c_int;
pub extern fn strcasecmp(__s1: [*c]const u8, __s2: [*c]const u8) c_int;
pub extern fn strncasecmp(__s1: [*c]const u8, __s2: [*c]const u8, __n: usize) c_int;
pub extern fn strcasecmp_l(__s1: [*c]const u8, __s2: [*c]const u8, __loc: locale_t) c_int;
pub extern fn strncasecmp_l(__s1: [*c]const u8, __s2: [*c]const u8, __n: usize, __loc: locale_t) c_int;
pub extern fn explicit_bzero(__s: ?*anyopaque, __n: usize) void;
pub extern fn strsep(noalias __stringp: [*c][*c]u8, noalias __delim: [*c]const u8) [*c]u8;
pub extern fn strsignal(__sig: c_int) [*c]u8;
pub extern fn __stpcpy(noalias __dest: [*c]u8, noalias __src: [*c]const u8) [*c]u8;
pub extern fn stpcpy(noalias __dest: [*c]u8, noalias __src: [*c]const u8) [*c]u8;
pub extern fn __stpncpy(noalias __dest: [*c]u8, noalias __src: [*c]const u8, __n: usize) [*c]u8;
pub extern fn stpncpy(noalias __dest: [*c]u8, noalias __src: [*c]const u8, __n: usize) [*c]u8;
pub extern fn strlcpy(noalias __dest: [*c]u8, noalias __src: [*c]const u8, __n: usize) usize;
pub extern fn strlcat(noalias __dest: [*c]u8, noalias __src: [*c]const u8, __n: usize) usize;
pub const struct_winsize = extern struct {
    ws_row: c_ushort,
    ws_col: c_ushort,
    ws_xpixel: c_ushort,
    ws_ypixel: c_ushort,
};
pub extern fn ioctl(__fd: c_int, __request: c_ulong, ...) c_int;
pub extern fn stat(noalias __file: [*c]const u8, noalias __buf: [*c]struct_stat) c_int;
pub extern fn fstat(__fd: c_int, __buf: [*c]struct_stat) c_int;
pub extern fn fstatat(__fd: c_int, noalias __file: [*c]const u8, noalias __buf: [*c]struct_stat, __flag: c_int) c_int;
pub extern fn lstat(noalias __file: [*c]const u8, noalias __buf: [*c]struct_stat) c_int;
pub extern fn chmod(__file: [*c]const u8, __mode: __mode_t) c_int;
pub extern fn lchmod(__file: [*c]const u8, __mode: __mode_t) c_int;
pub extern fn fchmod(__fd: c_int, __mode: __mode_t) c_int;
pub extern fn fchmodat(__fd: c_int, __file: [*c]const u8, __mode: __mode_t, __flag: c_int) c_int;
pub extern fn umask(__mask: __mode_t) __mode_t;
pub extern fn mkdir(__path: [*c]const u8, __mode: __mode_t) c_int;
pub extern fn mkdirat(__fd: c_int, __path: [*c]const u8, __mode: __mode_t) c_int;
pub extern fn mknod(__path: [*c]const u8, __mode: __mode_t, __dev: __dev_t) c_int;
pub extern fn mknodat(__fd: c_int, __path: [*c]const u8, __mode: __mode_t, __dev: __dev_t) c_int;
pub extern fn mkfifo(__path: [*c]const u8, __mode: __mode_t) c_int;
pub extern fn mkfifoat(__fd: c_int, __path: [*c]const u8, __mode: __mode_t) c_int;
pub extern fn utimensat(__fd: c_int, __path: [*c]const u8, __times: [*c]const struct_timespec, __flags: c_int) c_int;
pub extern fn futimens(__fd: c_int, __times: [*c]const struct_timespec) c_int;
pub const struct_timezone = extern struct {
    tz_minuteswest: c_int,
    tz_dsttime: c_int,
};
pub extern fn gettimeofday(noalias __tv: [*c]struct_timeval, noalias __tz: ?*anyopaque) c_int;
pub extern fn settimeofday(__tv: [*c]const struct_timeval, __tz: [*c]const struct_timezone) c_int;
pub extern fn adjtime(__delta: [*c]const struct_timeval, __olddelta: [*c]struct_timeval) c_int;
pub const ITIMER_REAL: c_int = 0;
pub const ITIMER_VIRTUAL: c_int = 1;
pub const ITIMER_PROF: c_int = 2;
pub const enum___itimer_which = c_uint;
pub const struct_itimerval = extern struct {
    it_interval: struct_timeval,
    it_value: struct_timeval,
};
pub const __itimer_which_t = c_int;
pub extern fn getitimer(__which: __itimer_which_t, __value: [*c]struct_itimerval) c_int;
pub extern fn setitimer(__which: __itimer_which_t, noalias __new: [*c]const struct_itimerval, noalias __old: [*c]struct_itimerval) c_int;
pub extern fn utimes(__file: [*c]const u8, __tvp: [*c]const struct_timeval) c_int;
pub extern fn lutimes(__file: [*c]const u8, __tvp: [*c]const struct_timeval) c_int;
pub extern fn futimes(__fd: c_int, __tvp: [*c]const struct_timeval) c_int;
pub const cc_t = u8;
pub const speed_t = c_uint;
pub const tcflag_t = c_uint;
const union_unnamed_29 = extern union {
    __ispeed: speed_t,
    c_ispeed: speed_t,
};
const union_unnamed_30 = extern union {
    __ospeed: speed_t,
    c_ospeed: speed_t,
};
pub const struct_termios = extern struct {
    c_iflag: tcflag_t,
    c_oflag: tcflag_t,
    c_cflag: tcflag_t,
    c_lflag: tcflag_t,
    c_line: cc_t,
    c_cc: [32]cc_t,
    unnamed_0: union_unnamed_29,
    unnamed_1: union_unnamed_30,
    pub const cfgetospeed = __root.cfgetospeed;
    pub const cfgetispeed = __root.cfgetispeed;
    pub const cfsetospeed = __root.cfsetospeed;
    pub const cfsetispeed = __root.cfsetispeed;
    pub const cfsetspeed = __root.cfsetspeed;
    pub const cfmakeraw = __root.cfmakeraw;
};
pub extern fn cfgetospeed(__termios_p: [*c]const struct_termios) speed_t;
pub extern fn cfgetispeed(__termios_p: [*c]const struct_termios) speed_t;
pub extern fn cfsetospeed(__termios_p: [*c]struct_termios, __speed: speed_t) c_int;
pub extern fn cfsetispeed(__termios_p: [*c]struct_termios, __speed: speed_t) c_int;
pub extern fn cfsetspeed(__termios_p: [*c]struct_termios, __speed: speed_t) c_int;
pub extern fn tcgetattr(__fd: c_int, __termios_p: [*c]struct_termios) c_int;
pub extern fn tcsetattr(__fd: c_int, __optional_actions: c_int, __termios_p: [*c]const struct_termios) c_int;
pub extern fn cfmakeraw(__termios_p: [*c]struct_termios) void;
pub extern fn tcsendbreak(__fd: c_int, __duration: c_int) c_int;
pub extern fn tcdrain(__fd: c_int) c_int;
pub extern fn tcflush(__fd: c_int, __queue_selector: c_int) c_int;
pub extern fn tcflow(__fd: c_int, __action: c_int) c_int;
pub extern fn tcgetsid(__fd: c_int) __pid_t;
pub const socklen_t = __socklen_t;
pub extern fn access(__name: [*c]const u8, __type: c_int) c_int;
pub extern fn faccessat(__fd: c_int, __file: [*c]const u8, __type: c_int, __flag: c_int) c_int;
pub extern fn lseek(__fd: c_int, __offset: __off_t, __whence: c_int) __off_t;
pub extern fn close(__fd: c_int) c_int;
pub extern fn closefrom(__lowfd: c_int) void;
pub extern fn read(__fd: c_int, __buf: ?*anyopaque, __nbytes: usize) isize;
pub extern fn write(__fd: c_int, __buf: ?*const anyopaque, __n: usize) isize;
pub extern fn pread(__fd: c_int, __buf: ?*anyopaque, __nbytes: usize, __offset: __off_t) isize;
pub extern fn pwrite(__fd: c_int, __buf: ?*const anyopaque, __n: usize, __offset: __off_t) isize;
pub extern fn pipe(__pipedes: [*c]c_int) c_int;
pub extern fn alarm(__seconds: c_uint) c_uint;
pub extern fn sleep(__seconds: c_uint) c_uint;
pub extern fn ualarm(__value: __useconds_t, __interval: __useconds_t) __useconds_t;
pub extern fn usleep(__useconds: __useconds_t) c_int;
pub extern fn pause() c_int;
pub extern fn chown(__file: [*c]const u8, __owner: __uid_t, __group: __gid_t) c_int;
pub extern fn fchown(__fd: c_int, __owner: __uid_t, __group: __gid_t) c_int;
pub extern fn lchown(__file: [*c]const u8, __owner: __uid_t, __group: __gid_t) c_int;
pub extern fn fchownat(__fd: c_int, __file: [*c]const u8, __owner: __uid_t, __group: __gid_t, __flag: c_int) c_int;
pub extern fn chdir(__path: [*c]const u8) c_int;
pub extern fn fchdir(__fd: c_int) c_int;
pub extern fn getcwd(__buf: [*c]u8, __size: usize) [*c]u8;
pub extern fn getwd(__buf: [*c]u8) [*c]u8;
pub extern fn dup(__fd: c_int) c_int;
pub extern fn dup2(__fd: c_int, __fd2: c_int) c_int;
pub extern var __environ: [*c][*c]u8;
pub extern fn execve(__path: [*c]const u8, __argv: [*c]const [*c]u8, __envp: [*c]const [*c]u8) c_int;
pub extern fn fexecve(__fd: c_int, __argv: [*c]const [*c]u8, __envp: [*c]const [*c]u8) c_int;
pub extern fn execv(__path: [*c]const u8, __argv: [*c]const [*c]u8) c_int;
pub extern fn execle(__path: [*c]const u8, __arg: [*c]const u8, ...) c_int;
pub extern fn execl(__path: [*c]const u8, __arg: [*c]const u8, ...) c_int;
pub extern fn execvp(__file: [*c]const u8, __argv: [*c]const [*c]u8) c_int;
pub extern fn execlp(__file: [*c]const u8, __arg: [*c]const u8, ...) c_int;
pub extern fn nice(__inc: c_int) c_int;
pub extern fn _exit(__status: c_int) noreturn;
pub const _PC_LINK_MAX: c_int = 0;
pub const _PC_MAX_CANON: c_int = 1;
pub const _PC_MAX_INPUT: c_int = 2;
pub const _PC_NAME_MAX: c_int = 3;
pub const _PC_PATH_MAX: c_int = 4;
pub const _PC_PIPE_BUF: c_int = 5;
pub const _PC_CHOWN_RESTRICTED: c_int = 6;
pub const _PC_NO_TRUNC: c_int = 7;
pub const _PC_VDISABLE: c_int = 8;
pub const _PC_SYNC_IO: c_int = 9;
pub const _PC_ASYNC_IO: c_int = 10;
pub const _PC_PRIO_IO: c_int = 11;
pub const _PC_SOCK_MAXBUF: c_int = 12;
pub const _PC_FILESIZEBITS: c_int = 13;
pub const _PC_REC_INCR_XFER_SIZE: c_int = 14;
pub const _PC_REC_MAX_XFER_SIZE: c_int = 15;
pub const _PC_REC_MIN_XFER_SIZE: c_int = 16;
pub const _PC_REC_XFER_ALIGN: c_int = 17;
pub const _PC_ALLOC_SIZE_MIN: c_int = 18;
pub const _PC_SYMLINK_MAX: c_int = 19;
pub const _PC_2_SYMLINKS: c_int = 20;
const enum_unnamed_31 = c_uint;
pub const _SC_ARG_MAX: c_int = 0;
pub const _SC_CHILD_MAX: c_int = 1;
pub const _SC_CLK_TCK: c_int = 2;
pub const _SC_NGROUPS_MAX: c_int = 3;
pub const _SC_OPEN_MAX: c_int = 4;
pub const _SC_STREAM_MAX: c_int = 5;
pub const _SC_TZNAME_MAX: c_int = 6;
pub const _SC_JOB_CONTROL: c_int = 7;
pub const _SC_SAVED_IDS: c_int = 8;
pub const _SC_REALTIME_SIGNALS: c_int = 9;
pub const _SC_PRIORITY_SCHEDULING: c_int = 10;
pub const _SC_TIMERS: c_int = 11;
pub const _SC_ASYNCHRONOUS_IO: c_int = 12;
pub const _SC_PRIORITIZED_IO: c_int = 13;
pub const _SC_SYNCHRONIZED_IO: c_int = 14;
pub const _SC_FSYNC: c_int = 15;
pub const _SC_MAPPED_FILES: c_int = 16;
pub const _SC_MEMLOCK: c_int = 17;
pub const _SC_MEMLOCK_RANGE: c_int = 18;
pub const _SC_MEMORY_PROTECTION: c_int = 19;
pub const _SC_MESSAGE_PASSING: c_int = 20;
pub const _SC_SEMAPHORES: c_int = 21;
pub const _SC_SHARED_MEMORY_OBJECTS: c_int = 22;
pub const _SC_AIO_LISTIO_MAX: c_int = 23;
pub const _SC_AIO_MAX: c_int = 24;
pub const _SC_AIO_PRIO_DELTA_MAX: c_int = 25;
pub const _SC_DELAYTIMER_MAX: c_int = 26;
pub const _SC_MQ_OPEN_MAX: c_int = 27;
pub const _SC_MQ_PRIO_MAX: c_int = 28;
pub const _SC_VERSION: c_int = 29;
pub const _SC_PAGESIZE: c_int = 30;
pub const _SC_RTSIG_MAX: c_int = 31;
pub const _SC_SEM_NSEMS_MAX: c_int = 32;
pub const _SC_SEM_VALUE_MAX: c_int = 33;
pub const _SC_SIGQUEUE_MAX: c_int = 34;
pub const _SC_TIMER_MAX: c_int = 35;
pub const _SC_BC_BASE_MAX: c_int = 36;
pub const _SC_BC_DIM_MAX: c_int = 37;
pub const _SC_BC_SCALE_MAX: c_int = 38;
pub const _SC_BC_STRING_MAX: c_int = 39;
pub const _SC_COLL_WEIGHTS_MAX: c_int = 40;
pub const _SC_EQUIV_CLASS_MAX: c_int = 41;
pub const _SC_EXPR_NEST_MAX: c_int = 42;
pub const _SC_LINE_MAX: c_int = 43;
pub const _SC_RE_DUP_MAX: c_int = 44;
pub const _SC_CHARCLASS_NAME_MAX: c_int = 45;
pub const _SC_2_VERSION: c_int = 46;
pub const _SC_2_C_BIND: c_int = 47;
pub const _SC_2_C_DEV: c_int = 48;
pub const _SC_2_FORT_DEV: c_int = 49;
pub const _SC_2_FORT_RUN: c_int = 50;
pub const _SC_2_SW_DEV: c_int = 51;
pub const _SC_2_LOCALEDEF: c_int = 52;
pub const _SC_PII: c_int = 53;
pub const _SC_PII_XTI: c_int = 54;
pub const _SC_PII_SOCKET: c_int = 55;
pub const _SC_PII_INTERNET: c_int = 56;
pub const _SC_PII_OSI: c_int = 57;
pub const _SC_POLL: c_int = 58;
pub const _SC_SELECT: c_int = 59;
pub const _SC_UIO_MAXIOV: c_int = 60;
pub const _SC_IOV_MAX: c_int = 60;
pub const _SC_PII_INTERNET_STREAM: c_int = 61;
pub const _SC_PII_INTERNET_DGRAM: c_int = 62;
pub const _SC_PII_OSI_COTS: c_int = 63;
pub const _SC_PII_OSI_CLTS: c_int = 64;
pub const _SC_PII_OSI_M: c_int = 65;
pub const _SC_T_IOV_MAX: c_int = 66;
pub const _SC_THREADS: c_int = 67;
pub const _SC_THREAD_SAFE_FUNCTIONS: c_int = 68;
pub const _SC_GETGR_R_SIZE_MAX: c_int = 69;
pub const _SC_GETPW_R_SIZE_MAX: c_int = 70;
pub const _SC_LOGIN_NAME_MAX: c_int = 71;
pub const _SC_TTY_NAME_MAX: c_int = 72;
pub const _SC_THREAD_DESTRUCTOR_ITERATIONS: c_int = 73;
pub const _SC_THREAD_KEYS_MAX: c_int = 74;
pub const _SC_THREAD_STACK_MIN: c_int = 75;
pub const _SC_THREAD_THREADS_MAX: c_int = 76;
pub const _SC_THREAD_ATTR_STACKADDR: c_int = 77;
pub const _SC_THREAD_ATTR_STACKSIZE: c_int = 78;
pub const _SC_THREAD_PRIORITY_SCHEDULING: c_int = 79;
pub const _SC_THREAD_PRIO_INHERIT: c_int = 80;
pub const _SC_THREAD_PRIO_PROTECT: c_int = 81;
pub const _SC_THREAD_PROCESS_SHARED: c_int = 82;
pub const _SC_NPROCESSORS_CONF: c_int = 83;
pub const _SC_NPROCESSORS_ONLN: c_int = 84;
pub const _SC_PHYS_PAGES: c_int = 85;
pub const _SC_AVPHYS_PAGES: c_int = 86;
pub const _SC_ATEXIT_MAX: c_int = 87;
pub const _SC_PASS_MAX: c_int = 88;
pub const _SC_XOPEN_VERSION: c_int = 89;
pub const _SC_XOPEN_XCU_VERSION: c_int = 90;
pub const _SC_XOPEN_UNIX: c_int = 91;
pub const _SC_XOPEN_CRYPT: c_int = 92;
pub const _SC_XOPEN_ENH_I18N: c_int = 93;
pub const _SC_XOPEN_SHM: c_int = 94;
pub const _SC_2_CHAR_TERM: c_int = 95;
pub const _SC_2_C_VERSION: c_int = 96;
pub const _SC_2_UPE: c_int = 97;
pub const _SC_XOPEN_XPG2: c_int = 98;
pub const _SC_XOPEN_XPG3: c_int = 99;
pub const _SC_XOPEN_XPG4: c_int = 100;
pub const _SC_CHAR_BIT: c_int = 101;
pub const _SC_CHAR_MAX: c_int = 102;
pub const _SC_CHAR_MIN: c_int = 103;
pub const _SC_INT_MAX: c_int = 104;
pub const _SC_INT_MIN: c_int = 105;
pub const _SC_LONG_BIT: c_int = 106;
pub const _SC_WORD_BIT: c_int = 107;
pub const _SC_MB_LEN_MAX: c_int = 108;
pub const _SC_NZERO: c_int = 109;
pub const _SC_SSIZE_MAX: c_int = 110;
pub const _SC_SCHAR_MAX: c_int = 111;
pub const _SC_SCHAR_MIN: c_int = 112;
pub const _SC_SHRT_MAX: c_int = 113;
pub const _SC_SHRT_MIN: c_int = 114;
pub const _SC_UCHAR_MAX: c_int = 115;
pub const _SC_UINT_MAX: c_int = 116;
pub const _SC_ULONG_MAX: c_int = 117;
pub const _SC_USHRT_MAX: c_int = 118;
pub const _SC_NL_ARGMAX: c_int = 119;
pub const _SC_NL_LANGMAX: c_int = 120;
pub const _SC_NL_MSGMAX: c_int = 121;
pub const _SC_NL_NMAX: c_int = 122;
pub const _SC_NL_SETMAX: c_int = 123;
pub const _SC_NL_TEXTMAX: c_int = 124;
pub const _SC_XBS5_ILP32_OFF32: c_int = 125;
pub const _SC_XBS5_ILP32_OFFBIG: c_int = 126;
pub const _SC_XBS5_LP64_OFF64: c_int = 127;
pub const _SC_XBS5_LPBIG_OFFBIG: c_int = 128;
pub const _SC_XOPEN_LEGACY: c_int = 129;
pub const _SC_XOPEN_REALTIME: c_int = 130;
pub const _SC_XOPEN_REALTIME_THREADS: c_int = 131;
pub const _SC_ADVISORY_INFO: c_int = 132;
pub const _SC_BARRIERS: c_int = 133;
pub const _SC_BASE: c_int = 134;
pub const _SC_C_LANG_SUPPORT: c_int = 135;
pub const _SC_C_LANG_SUPPORT_R: c_int = 136;
pub const _SC_CLOCK_SELECTION: c_int = 137;
pub const _SC_CPUTIME: c_int = 138;
pub const _SC_THREAD_CPUTIME: c_int = 139;
pub const _SC_DEVICE_IO: c_int = 140;
pub const _SC_DEVICE_SPECIFIC: c_int = 141;
pub const _SC_DEVICE_SPECIFIC_R: c_int = 142;
pub const _SC_FD_MGMT: c_int = 143;
pub const _SC_FIFO: c_int = 144;
pub const _SC_PIPE: c_int = 145;
pub const _SC_FILE_ATTRIBUTES: c_int = 146;
pub const _SC_FILE_LOCKING: c_int = 147;
pub const _SC_FILE_SYSTEM: c_int = 148;
pub const _SC_MONOTONIC_CLOCK: c_int = 149;
pub const _SC_MULTI_PROCESS: c_int = 150;
pub const _SC_SINGLE_PROCESS: c_int = 151;
pub const _SC_NETWORKING: c_int = 152;
pub const _SC_READER_WRITER_LOCKS: c_int = 153;
pub const _SC_SPIN_LOCKS: c_int = 154;
pub const _SC_REGEXP: c_int = 155;
pub const _SC_REGEX_VERSION: c_int = 156;
pub const _SC_SHELL: c_int = 157;
pub const _SC_SIGNALS: c_int = 158;
pub const _SC_SPAWN: c_int = 159;
pub const _SC_SPORADIC_SERVER: c_int = 160;
pub const _SC_THREAD_SPORADIC_SERVER: c_int = 161;
pub const _SC_SYSTEM_DATABASE: c_int = 162;
pub const _SC_SYSTEM_DATABASE_R: c_int = 163;
pub const _SC_TIMEOUTS: c_int = 164;
pub const _SC_TYPED_MEMORY_OBJECTS: c_int = 165;
pub const _SC_USER_GROUPS: c_int = 166;
pub const _SC_USER_GROUPS_R: c_int = 167;
pub const _SC_2_PBS: c_int = 168;
pub const _SC_2_PBS_ACCOUNTING: c_int = 169;
pub const _SC_2_PBS_LOCATE: c_int = 170;
pub const _SC_2_PBS_MESSAGE: c_int = 171;
pub const _SC_2_PBS_TRACK: c_int = 172;
pub const _SC_SYMLOOP_MAX: c_int = 173;
pub const _SC_STREAMS: c_int = 174;
pub const _SC_2_PBS_CHECKPOINT: c_int = 175;
pub const _SC_V6_ILP32_OFF32: c_int = 176;
pub const _SC_V6_ILP32_OFFBIG: c_int = 177;
pub const _SC_V6_LP64_OFF64: c_int = 178;
pub const _SC_V6_LPBIG_OFFBIG: c_int = 179;
pub const _SC_HOST_NAME_MAX: c_int = 180;
pub const _SC_TRACE: c_int = 181;
pub const _SC_TRACE_EVENT_FILTER: c_int = 182;
pub const _SC_TRACE_INHERIT: c_int = 183;
pub const _SC_TRACE_LOG: c_int = 184;
pub const _SC_LEVEL1_ICACHE_SIZE: c_int = 185;
pub const _SC_LEVEL1_ICACHE_ASSOC: c_int = 186;
pub const _SC_LEVEL1_ICACHE_LINESIZE: c_int = 187;
pub const _SC_LEVEL1_DCACHE_SIZE: c_int = 188;
pub const _SC_LEVEL1_DCACHE_ASSOC: c_int = 189;
pub const _SC_LEVEL1_DCACHE_LINESIZE: c_int = 190;
pub const _SC_LEVEL2_CACHE_SIZE: c_int = 191;
pub const _SC_LEVEL2_CACHE_ASSOC: c_int = 192;
pub const _SC_LEVEL2_CACHE_LINESIZE: c_int = 193;
pub const _SC_LEVEL3_CACHE_SIZE: c_int = 194;
pub const _SC_LEVEL3_CACHE_ASSOC: c_int = 195;
pub const _SC_LEVEL3_CACHE_LINESIZE: c_int = 196;
pub const _SC_LEVEL4_CACHE_SIZE: c_int = 197;
pub const _SC_LEVEL4_CACHE_ASSOC: c_int = 198;
pub const _SC_LEVEL4_CACHE_LINESIZE: c_int = 199;
pub const _SC_IPV6: c_int = 235;
pub const _SC_RAW_SOCKETS: c_int = 236;
pub const _SC_V7_ILP32_OFF32: c_int = 237;
pub const _SC_V7_ILP32_OFFBIG: c_int = 238;
pub const _SC_V7_LP64_OFF64: c_int = 239;
pub const _SC_V7_LPBIG_OFFBIG: c_int = 240;
pub const _SC_SS_REPL_MAX: c_int = 241;
pub const _SC_TRACE_EVENT_NAME_MAX: c_int = 242;
pub const _SC_TRACE_NAME_MAX: c_int = 243;
pub const _SC_TRACE_SYS_MAX: c_int = 244;
pub const _SC_TRACE_USER_EVENT_MAX: c_int = 245;
pub const _SC_XOPEN_STREAMS: c_int = 246;
pub const _SC_THREAD_ROBUST_PRIO_INHERIT: c_int = 247;
pub const _SC_THREAD_ROBUST_PRIO_PROTECT: c_int = 248;
pub const _SC_MINSIGSTKSZ: c_int = 249;
pub const _SC_SIGSTKSZ: c_int = 250;
const enum_unnamed_32 = c_uint;
pub const _CS_PATH: c_int = 0;
pub const _CS_V6_WIDTH_RESTRICTED_ENVS: c_int = 1;
pub const _CS_GNU_LIBC_VERSION: c_int = 2;
pub const _CS_GNU_LIBPTHREAD_VERSION: c_int = 3;
pub const _CS_V5_WIDTH_RESTRICTED_ENVS: c_int = 4;
pub const _CS_V7_WIDTH_RESTRICTED_ENVS: c_int = 5;
pub const _CS_LFS_CFLAGS: c_int = 1000;
pub const _CS_LFS_LDFLAGS: c_int = 1001;
pub const _CS_LFS_LIBS: c_int = 1002;
pub const _CS_LFS_LINTFLAGS: c_int = 1003;
pub const _CS_LFS64_CFLAGS: c_int = 1004;
pub const _CS_LFS64_LDFLAGS: c_int = 1005;
pub const _CS_LFS64_LIBS: c_int = 1006;
pub const _CS_LFS64_LINTFLAGS: c_int = 1007;
pub const _CS_XBS5_ILP32_OFF32_CFLAGS: c_int = 1100;
pub const _CS_XBS5_ILP32_OFF32_LDFLAGS: c_int = 1101;
pub const _CS_XBS5_ILP32_OFF32_LIBS: c_int = 1102;
pub const _CS_XBS5_ILP32_OFF32_LINTFLAGS: c_int = 1103;
pub const _CS_XBS5_ILP32_OFFBIG_CFLAGS: c_int = 1104;
pub const _CS_XBS5_ILP32_OFFBIG_LDFLAGS: c_int = 1105;
pub const _CS_XBS5_ILP32_OFFBIG_LIBS: c_int = 1106;
pub const _CS_XBS5_ILP32_OFFBIG_LINTFLAGS: c_int = 1107;
pub const _CS_XBS5_LP64_OFF64_CFLAGS: c_int = 1108;
pub const _CS_XBS5_LP64_OFF64_LDFLAGS: c_int = 1109;
pub const _CS_XBS5_LP64_OFF64_LIBS: c_int = 1110;
pub const _CS_XBS5_LP64_OFF64_LINTFLAGS: c_int = 1111;
pub const _CS_XBS5_LPBIG_OFFBIG_CFLAGS: c_int = 1112;
pub const _CS_XBS5_LPBIG_OFFBIG_LDFLAGS: c_int = 1113;
pub const _CS_XBS5_LPBIG_OFFBIG_LIBS: c_int = 1114;
pub const _CS_XBS5_LPBIG_OFFBIG_LINTFLAGS: c_int = 1115;
pub const _CS_POSIX_V6_ILP32_OFF32_CFLAGS: c_int = 1116;
pub const _CS_POSIX_V6_ILP32_OFF32_LDFLAGS: c_int = 1117;
pub const _CS_POSIX_V6_ILP32_OFF32_LIBS: c_int = 1118;
pub const _CS_POSIX_V6_ILP32_OFF32_LINTFLAGS: c_int = 1119;
pub const _CS_POSIX_V6_ILP32_OFFBIG_CFLAGS: c_int = 1120;
pub const _CS_POSIX_V6_ILP32_OFFBIG_LDFLAGS: c_int = 1121;
pub const _CS_POSIX_V6_ILP32_OFFBIG_LIBS: c_int = 1122;
pub const _CS_POSIX_V6_ILP32_OFFBIG_LINTFLAGS: c_int = 1123;
pub const _CS_POSIX_V6_LP64_OFF64_CFLAGS: c_int = 1124;
pub const _CS_POSIX_V6_LP64_OFF64_LDFLAGS: c_int = 1125;
pub const _CS_POSIX_V6_LP64_OFF64_LIBS: c_int = 1126;
pub const _CS_POSIX_V6_LP64_OFF64_LINTFLAGS: c_int = 1127;
pub const _CS_POSIX_V6_LPBIG_OFFBIG_CFLAGS: c_int = 1128;
pub const _CS_POSIX_V6_LPBIG_OFFBIG_LDFLAGS: c_int = 1129;
pub const _CS_POSIX_V6_LPBIG_OFFBIG_LIBS: c_int = 1130;
pub const _CS_POSIX_V6_LPBIG_OFFBIG_LINTFLAGS: c_int = 1131;
pub const _CS_POSIX_V7_ILP32_OFF32_CFLAGS: c_int = 1132;
pub const _CS_POSIX_V7_ILP32_OFF32_LDFLAGS: c_int = 1133;
pub const _CS_POSIX_V7_ILP32_OFF32_LIBS: c_int = 1134;
pub const _CS_POSIX_V7_ILP32_OFF32_LINTFLAGS: c_int = 1135;
pub const _CS_POSIX_V7_ILP32_OFFBIG_CFLAGS: c_int = 1136;
pub const _CS_POSIX_V7_ILP32_OFFBIG_LDFLAGS: c_int = 1137;
pub const _CS_POSIX_V7_ILP32_OFFBIG_LIBS: c_int = 1138;
pub const _CS_POSIX_V7_ILP32_OFFBIG_LINTFLAGS: c_int = 1139;
pub const _CS_POSIX_V7_LP64_OFF64_CFLAGS: c_int = 1140;
pub const _CS_POSIX_V7_LP64_OFF64_LDFLAGS: c_int = 1141;
pub const _CS_POSIX_V7_LP64_OFF64_LIBS: c_int = 1142;
pub const _CS_POSIX_V7_LP64_OFF64_LINTFLAGS: c_int = 1143;
pub const _CS_POSIX_V7_LPBIG_OFFBIG_CFLAGS: c_int = 1144;
pub const _CS_POSIX_V7_LPBIG_OFFBIG_LDFLAGS: c_int = 1145;
pub const _CS_POSIX_V7_LPBIG_OFFBIG_LIBS: c_int = 1146;
pub const _CS_POSIX_V7_LPBIG_OFFBIG_LINTFLAGS: c_int = 1147;
pub const _CS_V6_ENV: c_int = 1148;
pub const _CS_V7_ENV: c_int = 1149;
const enum_unnamed_33 = c_uint;
pub extern fn pathconf(__path: [*c]const u8, __name: c_int) c_long;
pub extern fn fpathconf(__fd: c_int, __name: c_int) c_long;
pub extern fn sysconf(__name: c_int) c_long;
pub extern fn confstr(__name: c_int, __buf: [*c]u8, __len: usize) usize;
pub extern fn getpid() __pid_t;
pub extern fn getppid() __pid_t;
pub extern fn getpgrp() __pid_t;
pub extern fn __getpgid(__pid: __pid_t) __pid_t;
pub extern fn getpgid(__pid: __pid_t) __pid_t;
pub extern fn setpgid(__pid: __pid_t, __pgid: __pid_t) c_int;
pub extern fn setpgrp() c_int;
pub extern fn setsid() __pid_t;
pub extern fn getsid(__pid: __pid_t) __pid_t;
pub extern fn getuid() __uid_t;
pub extern fn geteuid() __uid_t;
pub extern fn getgid() __gid_t;
pub extern fn getegid() __gid_t;
pub extern fn getgroups(__size: c_int, __list: [*c]__gid_t) c_int;
pub extern fn setuid(__uid: __uid_t) c_int;
pub extern fn setreuid(__ruid: __uid_t, __euid: __uid_t) c_int;
pub extern fn seteuid(__uid: __uid_t) c_int;
pub extern fn setgid(__gid: __gid_t) c_int;
pub extern fn setregid(__rgid: __gid_t, __egid: __gid_t) c_int;
pub extern fn setegid(__gid: __gid_t) c_int;
pub extern fn fork() __pid_t;
pub extern fn vfork() __pid_t;
pub extern fn ttyname(__fd: c_int) [*c]u8;
pub extern fn ttyname_r(__fd: c_int, __buf: [*c]u8, __buflen: usize) c_int;
pub extern fn isatty(__fd: c_int) c_int;
pub extern fn ttyslot() c_int;
pub extern fn link(__from: [*c]const u8, __to: [*c]const u8) c_int;
pub extern fn linkat(__fromfd: c_int, __from: [*c]const u8, __tofd: c_int, __to: [*c]const u8, __flags: c_int) c_int;
pub extern fn symlink(__from: [*c]const u8, __to: [*c]const u8) c_int;
pub extern fn readlink(noalias __path: [*c]const u8, noalias __buf: [*c]u8, __len: usize) isize;
pub extern fn symlinkat(__from: [*c]const u8, __tofd: c_int, __to: [*c]const u8) c_int;
pub extern fn readlinkat(__fd: c_int, noalias __path: [*c]const u8, noalias __buf: [*c]u8, __len: usize) isize;
pub extern fn unlink(__name: [*c]const u8) c_int;
pub extern fn unlinkat(__fd: c_int, __name: [*c]const u8, __flag: c_int) c_int;
pub extern fn rmdir(__path: [*c]const u8) c_int;
pub extern fn tcgetpgrp(__fd: c_int) __pid_t;
pub extern fn tcsetpgrp(__fd: c_int, __pgrp_id: __pid_t) c_int;
pub extern fn getlogin() [*c]u8;
pub extern fn getlogin_r(__name: [*c]u8, __name_len: usize) c_int;
pub extern fn setlogin(__name: [*c]const u8) c_int;
pub extern var optarg: [*c]u8;
pub extern var optind: c_int;
pub extern var opterr: c_int;
pub extern var optopt: c_int;
pub extern fn getopt(___argc: c_int, ___argv: [*c]const [*c]u8, __shortopts: [*c]const u8) c_int;
pub extern fn gethostname(__name: [*c]u8, __len: usize) c_int;
pub extern fn sethostname(__name: [*c]const u8, __len: usize) c_int;
pub extern fn sethostid(__id: c_long) c_int;
pub extern fn getdomainname(__name: [*c]u8, __len: usize) c_int;
pub extern fn setdomainname(__name: [*c]const u8, __len: usize) c_int;
pub extern fn vhangup() c_int;
pub extern fn revoke(__file: [*c]const u8) c_int;
pub extern fn profil(__sample_buffer: [*c]c_ushort, __size: usize, __offset: usize, __scale: c_uint) c_int;
pub extern fn acct(__name: [*c]const u8) c_int;
pub extern fn getusershell() [*c]u8;
pub extern fn endusershell() void;
pub extern fn setusershell() void;
pub extern fn daemon(__nochdir: c_int, __noclose: c_int) c_int;
pub extern fn chroot(__path: [*c]const u8) c_int;
pub extern fn getpass(__prompt: [*c]const u8) [*c]u8;
pub extern fn fsync(__fd: c_int) c_int;
pub extern fn gethostid() c_long;
pub extern fn sync() void;
pub extern fn getpagesize() c_int;
pub extern fn getdtablesize() c_int;
pub extern fn truncate(__file: [*c]const u8, __length: __off_t) c_int;
pub extern fn ftruncate(__fd: c_int, __length: __off_t) c_int;
pub extern fn brk(__addr: ?*anyopaque) c_int;
pub extern fn sbrk(__delta: isize) ?*anyopaque;
pub extern fn syscall(__sysno: c_long, ...) c_long;
pub extern fn fdatasync(__fildes: c_int) c_int;
pub extern fn crypt(__key: [*c]const u8, __salt: [*c]const u8) [*c]u8;
pub extern fn swab(noalias __from: ?*const anyopaque, noalias __to: ?*anyopaque, __n: isize) void;
pub extern fn getentropy(__buffer: ?*anyopaque, __length: usize) c_int;
pub const uintattr_t = u32;
pub const struct_tb_cell = extern struct {
    ch: u32,
    fg: uintattr_t,
    bg: uintattr_t,
    pub const cell_cmp = __root.cell_cmp;
    pub const cell_copy = __root.cell_copy;
    pub const cell_set = __root.cell_set;
    pub const cell_reserve_ech = __root.cell_reserve_ech;
    pub const cell_free = __root.cell_free;
    pub const cmp = __root.cell_cmp;
    pub const copy = __root.cell_copy;
    pub const set = __root.cell_set;
    pub const ech = __root.cell_reserve_ech;
};
pub const struct_tb_event = extern struct {
    type: u8,
    mod: u8,
    key: u16,
    ch: u32,
    w: i32,
    h: i32,
    x: i32,
    y: i32,
    pub const tb_peek_event = __root.tb_peek_event;
    pub const tb_poll_event = __root.tb_poll_event;
    pub const wait_event = __root.wait_event;
    pub const extract_event = __root.extract_event;
    pub const extract_esc = __root.extract_esc;
    pub const extract_esc_user = __root.extract_esc_user;
    pub const extract_esc_cap = __root.extract_esc_cap;
    pub const extract_esc_mouse = __root.extract_esc_mouse;
    pub const event = __root.tb_peek_event;
    pub const esc = __root.extract_esc;
    pub const user = __root.extract_esc_user;
    pub const cap = __root.extract_esc_cap;
    pub const mouse = __root.extract_esc_mouse;
};
pub export fn tb_init() c_int {
    return tb_init_file("/dev/tty");
}
pub export fn tb_init_file(arg_path: [*c]const u8) c_int {
    var path = arg_path;
    _ = &path;
    if (global.initialized != 0) return -@as(c_int, 3);
    var ttyfd: c_int = open(path, O_RDWR);
    _ = &ttyfd;
    if (ttyfd < @as(c_int, 0)) {
        global.last_errno = __errno_location().*;
        return -@as(c_int, 4);
    }
    global.ttyfd_open = 1;
    return tb_init_fd(ttyfd);
}
pub export fn tb_init_fd(arg_ttyfd: c_int) c_int {
    var ttyfd = arg_ttyfd;
    _ = &ttyfd;
    return tb_init_rwfd(ttyfd, ttyfd);
}
pub export fn tb_init_rwfd(arg_rfd: c_int, arg_wfd: c_int) c_int {
    var rfd = arg_rfd;
    _ = &rfd;
    var wfd = arg_wfd;
    _ = &wfd;
    var rv: c_int = undefined;
    _ = &rv;
    _ = tb_reset();
    global.ttyfd = if (isatty(rfd) != 0) rfd else if (isatty(wfd) != 0) wfd else -@as(c_int, 1);
    global.rfd = rfd;
    global.wfd = wfd;
    while (true) {
        if ((blk: {
            const tmp = init_term_attrs();
            rv = tmp;
            break :blk tmp;
        }) != TB_OK) break;
        if ((blk: {
            const tmp = init_term_caps();
            rv = tmp;
            break :blk tmp;
        }) != TB_OK) break;
        if ((blk: {
            const tmp = init_cap_trie();
            rv = tmp;
            break :blk tmp;
        }) != TB_OK) break;
        if ((blk: {
            const tmp = init_resize_handler();
            rv = tmp;
            break :blk tmp;
        }) != TB_OK) break;
        if ((blk: {
            const tmp = send_init_escape_codes();
            rv = tmp;
            break :blk tmp;
        }) != TB_OK) break;
        if ((blk: {
            const tmp = send_clear();
            rv = tmp;
            break :blk tmp;
        }) != TB_OK) break;
        if ((blk: {
            const tmp = update_term_size();
            rv = tmp;
            break :blk tmp;
        }) != TB_OK) break;
        if ((blk: {
            const tmp = init_cellbuf();
            rv = tmp;
            break :blk tmp;
        }) != TB_OK) break;
        global.initialized = 1;
        if (!false) break;
    }
    if (rv != TB_OK) {
        _ = tb_deinit();
    }
    return rv;
}
pub export fn tb_shutdown() c_int {
    if (!(global.initialized != 0)) return -@as(c_int, 8);
    _ = tb_deinit();
    return TB_OK;
}
pub export fn tb_width() c_int {
    if (!(global.initialized != 0)) return -@as(c_int, 8);
    return global.width;
}
pub export fn tb_height() c_int {
    if (!(global.initialized != 0)) return -@as(c_int, 8);
    return global.height;
}
pub export fn tb_clear() c_int {
    if (!(global.initialized != 0)) return -@as(c_int, 8);
    return cellbuf_clear(&global.back);
}
pub export fn tb_set_clear_attrs(arg_fg: uintattr_t, arg_bg: uintattr_t) c_int {
    var fg = arg_fg;
    _ = &fg;
    var bg = arg_bg;
    _ = &bg;
    if (!(global.initialized != 0)) return -@as(c_int, 8);
    global.fg = fg;
    global.bg = bg;
    return TB_OK;
}
pub export fn tb_present() c_int {
    if (!(global.initialized != 0)) return -@as(c_int, 8);
    var rv: c_int = undefined;
    _ = &rv;
    global.last_x = -@as(c_int, 1);
    global.last_y = -@as(c_int, 1);
    var x: c_int = undefined;
    _ = &x;
    var y: c_int = undefined;
    _ = &y;
    var i: c_int = undefined;
    _ = &i;
    {
        y = 0;
        while (y < global.front.height) : (y += 1) {
            {
                x = 0;
                while (x < global.front.width) {
                    var back: [*c]struct_tb_cell = undefined;
                    _ = &back;
                    var front: [*c]struct_tb_cell = undefined;
                    _ = &front;
                    if ((blk: {
                        const tmp = cellbuf_get(&global.back, x, y, &back);
                        rv = tmp;
                        break :blk tmp;
                    }) != TB_OK) return rv;
                    if ((blk: {
                        const tmp = cellbuf_get(&global.front, x, y, &front);
                        rv = tmp;
                        break :blk tmp;
                    }) != TB_OK) return rv;
                    var w: c_int = undefined;
                    _ = &w;
                    {
                        w = tb_wcwidth(@bitCast(@as(c_int, @as(wchar_t, @bitCast(@as(c_uint, @truncate(back.*.ch)))))));
                    }
                    if (w < @as(c_int, 1)) {
                        w = 1;
                    }
                    if (cell_cmp(back, front) != @as(c_int, 0)) {
                        _ = cell_copy(front, back);
                        _ = send_attr(back.*.fg, back.*.bg);
                        if ((w > @as(c_int, 1)) and (x >= (global.front.width - (w - @as(c_int, 1))))) {
                            {
                                i = x;
                                while (i < global.front.width) : (i += 1) {
                                    _ = send_char(i, y, ' ');
                                }
                            }
                        } else {
                            {
                                _ = send_char(x, y, back.*.ch);
                            }
                            {
                                i = 1;
                                while (i < w) : (i += 1) {
                                    var front_wide: [*c]struct_tb_cell = undefined;
                                    _ = &front_wide;
                                    var invalid: u32 = @bitCast(@as(c_int, -@as(c_int, 1)));
                                    _ = &invalid;
                                    if ((blk: {
                                        const tmp = cellbuf_get(&global.front, x + i, y, &front_wide);
                                        rv = tmp;
                                        break :blk tmp;
                                    }) != TB_OK) return rv;
                                    if ((blk: {
                                        const tmp = cell_set(front_wide, &invalid, 1, @bitCast(@as(c_int, -@as(c_int, 1))), @bitCast(@as(c_int, -@as(c_int, 1))));
                                        rv = tmp;
                                        break :blk tmp;
                                    }) != TB_OK) return rv;
                                }
                            }
                        }
                    }
                    x += w;
                }
            }
        }
    }
    if ((blk: {
        const tmp = send_cursor_if(global.cursor_x, global.cursor_y);
        rv = tmp;
        break :blk tmp;
    }) != TB_OK) return rv;
    if ((blk: {
        const tmp = bytebuf_flush(&global.out, global.wfd);
        rv = tmp;
        break :blk tmp;
    }) != TB_OK) return rv;
    return TB_OK;
}
pub export fn tb_invalidate() c_int {
    var rv: c_int = undefined;
    _ = &rv;
    if (!(global.initialized != 0)) return -@as(c_int, 8);
    if ((blk: {
        const tmp = resize_cellbufs();
        rv = tmp;
        break :blk tmp;
    }) != TB_OK) return rv;
    return TB_OK;
}
pub export fn tb_set_cursor(arg_cx: c_int, arg_cy: c_int) c_int {
    var cx = arg_cx;
    _ = &cx;
    var cy = arg_cy;
    _ = &cy;
    if (!(global.initialized != 0)) return -@as(c_int, 8);
    var rv: c_int = undefined;
    _ = &rv;
    if (cx < @as(c_int, 0)) {
        cx = 0;
    }
    if (cy < @as(c_int, 0)) {
        cy = 0;
    }
    if (global.cursor_x == -@as(c_int, 1)) {
        if ((blk: {
            const tmp = bytebuf_puts(&global.out, global.caps[TB_CAP_SHOW_CURSOR]);
            rv = tmp;
            break :blk tmp;
        }) != TB_OK) return rv;
    }
    if ((blk: {
        const tmp = send_cursor_if(cx, cy);
        rv = tmp;
        break :blk tmp;
    }) != TB_OK) return rv;
    global.cursor_x = cx;
    global.cursor_y = cy;
    return TB_OK;
}
pub export fn tb_hide_cursor() c_int {
    if (!(global.initialized != 0)) return -@as(c_int, 8);
    var rv: c_int = undefined;
    _ = &rv;
    if (global.cursor_x >= @as(c_int, 0)) {
        if ((blk: {
            const tmp = bytebuf_puts(&global.out, global.caps[TB_CAP_HIDE_CURSOR]);
            rv = tmp;
            break :blk tmp;
        }) != TB_OK) return rv;
    }
    global.cursor_x = -@as(c_int, 1);
    global.cursor_y = -@as(c_int, 1);
    return TB_OK;
}
pub export fn tb_set_cell(arg_x: c_int, arg_y: c_int, arg_ch: u32, arg_fg: uintattr_t, arg_bg: uintattr_t) c_int {
    var x = arg_x;
    _ = &x;
    var y = arg_y;
    _ = &y;
    var ch = arg_ch;
    _ = &ch;
    var fg = arg_fg;
    _ = &fg;
    var bg = arg_bg;
    _ = &bg;
    return tb_set_cell_ex(x, y, &ch, 1, fg, bg);
}
pub export fn tb_set_cell_ex(arg_x: c_int, arg_y: c_int, arg_ch: [*c]u32, arg_nch: usize, arg_fg: uintattr_t, arg_bg: uintattr_t) c_int {
    var x = arg_x;
    _ = &x;
    var y = arg_y;
    _ = &y;
    var ch = arg_ch;
    _ = &ch;
    var nch = arg_nch;
    _ = &nch;
    var fg = arg_fg;
    _ = &fg;
    var bg = arg_bg;
    _ = &bg;
    if (!(global.initialized != 0)) return -@as(c_int, 8);
    var rv: c_int = undefined;
    _ = &rv;
    var cell: [*c]struct_tb_cell = undefined;
    _ = &cell;
    if ((blk: {
        const tmp = cellbuf_get(&global.back, x, y, &cell);
        rv = tmp;
        break :blk tmp;
    }) != TB_OK) return rv;
    if ((blk: {
        const tmp = cell_set(cell, ch, nch, fg, bg);
        rv = tmp;
        break :blk tmp;
    }) != TB_OK) return rv;
    return TB_OK;
}
pub export fn tb_extend_cell(arg_x: c_int, arg_y: c_int, arg_ch: u32) c_int {
    var x = arg_x;
    _ = &x;
    var y = arg_y;
    _ = &y;
    var ch = arg_ch;
    _ = &ch;
    if (!(global.initialized != 0)) return -@as(c_int, 8);
    _ = &x;
    _ = &y;
    _ = &ch;
    return -@as(c_int, 1);
}
pub export fn tb_get_cell(arg_x: c_int, arg_y: c_int, arg_back: c_int, arg_cell: [*c][*c]struct_tb_cell) c_int {
    var x = arg_x;
    _ = &x;
    var y = arg_y;
    _ = &y;
    var back = arg_back;
    _ = &back;
    var cell = arg_cell;
    _ = &cell;
    if (!(global.initialized != 0)) return -@as(c_int, 8);
    return cellbuf_get(if (back != 0) &global.back else &global.front, x, y, cell);
}
pub export fn tb_set_input_mode(arg_mode: c_int) c_int {
    var mode = arg_mode;
    _ = &mode;
    if (!(global.initialized != 0)) return -@as(c_int, 8);
    if (mode == TB_INPUT_CURRENT) return global.input_mode;
    var esc_or_alt: c_int = TB_INPUT_ESC | TB_INPUT_ALT;
    _ = &esc_or_alt;
    if ((mode & esc_or_alt) == @as(c_int, 0)) {
        mode |= TB_INPUT_ESC;
    } else if ((mode & esc_or_alt) == esc_or_alt) {
        mode &= ~TB_INPUT_ALT;
    }
    if ((mode & TB_INPUT_MOUSE) != 0) {
        _ = bytebuf_puts(&global.out, "\x1b[?1000h\x1b[?1002h\x1b[?1015h\x1b[?1006h");
        _ = bytebuf_flush(&global.out, global.wfd);
    } else {
        _ = bytebuf_puts(&global.out, "\x1b[?1006l\x1b[?1015l\x1b[?1002l\x1b[?1000l");
        _ = bytebuf_flush(&global.out, global.wfd);
    }
    global.input_mode = mode;
    return TB_OK;
}
pub export fn tb_set_output_mode(arg_mode: c_int) c_int {
    var mode = arg_mode;
    _ = &mode;
    if (!(global.initialized != 0)) return -@as(c_int, 8);
    while (true) {
        switch (mode) {
            TB_OUTPUT_CURRENT => {
                return global.output_mode;
            },
            TB_OUTPUT_NORMAL, TB_OUTPUT_256, TB_OUTPUT_216, TB_OUTPUT_GRAYSCALE, TB_OUTPUT_TRUECOLOR => {
                global.last_fg = ~global.fg;
                global.last_bg = ~global.bg;
                global.output_mode = mode;
                return TB_OK;
            },
            else => {},
        }
        break;
    }
    return -@as(c_int, 1);
}
pub export fn tb_peek_event(arg_event: [*c]struct_tb_event, arg_timeout_ms: c_int) c_int {
    var event = arg_event;
    _ = &event;
    var timeout_ms = arg_timeout_ms;
    _ = &timeout_ms;
    if (!(global.initialized != 0)) return -@as(c_int, 8);
    return wait_event(event, timeout_ms);
}
pub export fn tb_poll_event(arg_event: [*c]struct_tb_event) c_int {
    var event = arg_event;
    _ = &event;
    if (!(global.initialized != 0)) return -@as(c_int, 8);
    return wait_event(event, -@as(c_int, 1));
}
pub export fn tb_get_fds(arg_ttyfd: [*c]c_int, arg_resizefd: [*c]c_int) c_int {
    var ttyfd = arg_ttyfd;
    _ = &ttyfd;
    var resizefd = arg_resizefd;
    _ = &resizefd;
    if (!(global.initialized != 0)) return -@as(c_int, 8);
    ttyfd.* = global.rfd;
    resizefd.* = global.resize_pipefd[@as(c_int, 0)];
    return TB_OK;
}
pub export fn tb_print(arg_x: c_int, arg_y: c_int, arg_fg: uintattr_t, arg_bg: uintattr_t, arg_str: [*c]const u8) c_int {
    var x = arg_x;
    _ = &x;
    var y = arg_y;
    _ = &y;
    var fg = arg_fg;
    _ = &fg;
    var bg = arg_bg;
    _ = &bg;
    var str = arg_str;
    _ = &str;
    return tb_print_ex(x, y, fg, bg, null, str);
} // ./zig-pkg/N-V-__8AAAUXBQD6Fwpi9m0MBqWXFFaqW5l1lVrJC2Ynj7a-/termbox2.h:2689:5: warning: TODO unable to translate variadic function, demoted to extern
pub extern fn tb_printf(x: c_int, y: c_int, fg: uintattr_t, bg: uintattr_t, fmt: [*c]const u8, ...) c_int;
pub export fn tb_print_ex(arg_x: c_int, arg_y: c_int, arg_fg: uintattr_t, arg_bg: uintattr_t, arg_out_w: [*c]usize, arg_str: [*c]const u8) c_int {
    var x = arg_x;
    _ = &x;
    var y = arg_y;
    _ = &y;
    var fg = arg_fg;
    _ = &fg;
    var bg = arg_bg;
    _ = &bg;
    var out_w = arg_out_w;
    _ = &out_w;
    var str = arg_str;
    _ = &str;
    var rv: c_int = undefined;
    _ = &rv;
    var w: c_int = undefined;
    _ = &w;
    var ix: c_int = undefined;
    _ = &ix;
    var x_prev: c_int = undefined;
    _ = &x_prev;
    var uni: u32 = undefined;
    _ = &uni;
    if (!(global.initialized != 0)) return -@as(c_int, 8);
    if (!(cellbuf_in_bounds(&global.back, x, y) != 0)) {
        return -@as(c_int, 9);
    }
    ix = x;
    x_prev = x;
    if (out_w != null) {
        out_w.* = 0;
    }
    while (@as(c_int, str.*) != 0) {
        rv = tb_utf8_char_to_unicode(&uni, str);
        if (rv < @as(c_int, 0)) {
            uni = 65533;
            str += @as(usize, @bitCast(@as(isize, @intCast(rv * -@as(c_int, 1)))));
        } else if (rv > @as(c_int, 0)) {
            str += @as(usize, @bitCast(@as(isize, @intCast(rv))));
        } else {
            break;
        }
        if (uni == @as(u32, '\n')) {
            x = ix;
            x_prev = x;
            y += 1;
            continue;
        } else if (!(tb_iswprint_ex(uni, &w) != 0)) {
            uni = 65533;
            w = 1;
        }
        if (w < @as(c_int, 0)) {
            return -@as(c_int, 1);
        } else if (w == @as(c_int, 0)) {
            if (cellbuf_in_bounds(&global.back, x_prev, y) != 0) {
                if ((blk: {
                    const tmp = tb_extend_cell(x_prev, y, uni);
                    rv = tmp;
                    break :blk tmp;
                }) != TB_OK) return rv;
            }
        } else {
            if (cellbuf_in_bounds(&global.back, x, y) != 0) {
                if ((blk: {
                    const tmp = tb_set_cell(x, y, uni, fg, bg);
                    rv = tmp;
                    break :blk tmp;
                }) != TB_OK) return rv;
            }
            x_prev = x;
            x += w;
            if (out_w != null) {
                out_w.* +%= @bitCast(@as(c_long, w));
            }
        }
    }
    return TB_OK;
} // ./zig-pkg/N-V-__8AAAUXBQD6Fwpi9m0MBqWXFFaqW5l1lVrJC2Ynj7a-/termbox2.h:2699:5: warning: TODO unable to translate variadic function, demoted to extern
pub extern fn tb_printf_ex(x: c_int, y: c_int, fg: uintattr_t, bg: uintattr_t, out_w: [*c]usize, fmt: [*c]const u8, ...) c_int;
pub export fn tb_send(arg_buf: [*c]const u8, arg_nbuf: usize) c_int {
    var buf = arg_buf;
    _ = &buf;
    var nbuf = arg_nbuf;
    _ = &nbuf;
    return bytebuf_nputs(&global.out, buf, nbuf);
} // ./zig-pkg/N-V-__8AAAUXBQD6Fwpi9m0MBqWXFFaqW5l1lVrJC2Ynj7a-/termbox2.h:2713:5: warning: TODO unable to translate variadic function, demoted to extern
pub extern fn tb_sendf(fmt: [*c]const u8, ...) c_int;
pub export fn tb_set_func(arg_fn_type: c_int, arg_fn: ?*const fn ([*c]struct_tb_event, [*c]usize) callconv(.c) c_int) c_int {
    var fn_type = arg_fn_type;
    _ = &fn_type;
    var @"fn" = arg_fn;
    _ = &@"fn";
    while (true) {
        switch (fn_type) {
            TB_FUNC_EXTRACT_PRE => {
                global.fn_extract_esc_pre = @"fn";
                return TB_OK;
            },
            TB_FUNC_EXTRACT_POST => {
                global.fn_extract_esc_post = @"fn";
                return TB_OK;
            },
            else => {},
        }
        break;
    }
    return -@as(c_int, 1);
}
pub export fn tb_utf8_char_length(arg_c: u8) c_int {
    var c = arg_c;
    _ = &c;
    return utf8_length[@as(u8, c)];
}
pub export fn tb_utf8_char_to_unicode(arg_out: [*c]u32, arg_c: [*c]const u8) c_int {
    var out = arg_out;
    _ = &out;
    var c = arg_c;
    _ = &c;
    if (@as(c_int, c.*) == @as(c_int, '\x00')) return 0;
    var i: c_int = undefined;
    _ = &i;
    var len: u8 = @bitCast(@as(i8, @truncate(tb_utf8_char_length(c.*))));
    _ = &len;
    var mask: u8 = utf8_mask[@bitCast(@as(isize, @intCast(@as(c_int, len) - @as(c_int, 1))))];
    _ = &mask;
    var result: u32 = @bitCast(@as(c_int, @as(c_int, c[@as(c_int, 0)]) & @as(c_int, mask)));
    _ = &result;
    {
        i = 1;
        while ((i < @as(c_int, len)) and (@as(c_int, c[@bitCast(@as(isize, @intCast(i)))]) != @as(c_int, '\x00'))) : (i += 1) {
            result <<= @intCast(@as(u32, 6));
            result |= @bitCast(@as(c_int, @as(c_int, c[@bitCast(@as(isize, @intCast(i)))]) & @as(c_int, 63)));
        }
    }
    if (i != @as(c_int, len)) return i * -@as(c_int, 1);
    out.* = result;
    return len;
}
pub export fn tb_utf8_unicode_to_char(arg_out: [*c]u8, arg_c: u32) c_int {
    var out = arg_out;
    _ = &out;
    var c = arg_c;
    _ = &c;
    var len: c_int = 0;
    _ = &len;
    var first: c_int = undefined;
    _ = &first;
    var i: c_int = undefined;
    _ = &i;
    if (c < @as(u32, 128)) {
        first = 0;
        len = 1;
    } else if (c < @as(u32, 2048)) {
        first = 192;
        len = 2;
    } else if (c < @as(u32, 65536)) {
        first = 224;
        len = 3;
    } else if (c < @as(u32, 2097152)) {
        first = 240;
        len = 4;
    } else if (c < @as(u32, 67108864)) {
        first = 248;
        len = 5;
    } else {
        first = 252;
        len = 6;
    }
    {
        i = len - @as(c_int, 1);
        while (i > @as(c_int, 0)) : (i -= 1) {
            out[@bitCast(@as(isize, @intCast(i)))] = @truncate((c & @as(u32, 63)) | @as(u32, 128));
            c >>= @intCast(@as(u32, 6));
        }
    }
    out[@as(c_int, 0)] = @truncate(c | @as(u32, @bitCast(@as(c_int, first))));
    out[@bitCast(@as(isize, @intCast(len)))] = '\x00';
    return len;
}
pub export fn tb_last_errno() c_int {
    return global.last_errno;
}
pub export fn tb_strerror(arg_err: c_int) [*c]const u8 {
    var err = arg_err;
    _ = &err;
    while (true) {
        switch (err) {
            TB_OK => {
                return "Success";
            },
            -@as(c_int, 2) => {
                return "Not enough input";
            },
            -@as(c_int, 3) => {
                return "Termbox initialized already";
            },
            -@as(c_int, 5) => {
                return "Out of memory";
            },
            -@as(c_int, 6) => {
                return "No event";
            },
            -@as(c_int, 7) => {
                return "No TERM in environment";
            },
            -@as(c_int, 8) => {
                return "Termbox not initialized";
            },
            -@as(c_int, 9) => {
                return "Out of bounds";
            },
            -@as(c_int, 17) => {
                return "Unsupported terminal";
            },
            -@as(c_int, 22) => {
                return "Termcaps collision";
            },
            -@as(c_int, 21) => {
                return "Terminal width/height not received by sscanf() after resize";
            },
            else => {
                _ = strerror_r(global.last_errno, @ptrCast(@alignCast(&global.errbuf)), @sizeOf(@TypeOf(global.errbuf)));
                return @ptrCast(@alignCast(@as([*c]u8, @ptrCast(@alignCast(&global.errbuf)))));
            },
        }
        break;
    }
    return undefined;
}
pub export fn tb_cell_buffer() [*c]struct_tb_cell {
    if (!(global.initialized != 0)) return null;
    return global.back.cells;
}
pub export fn tb_has_truecolor() c_int {
    return 1;
}
pub export fn tb_has_egc() c_int {
    return 0;
}
pub export fn tb_attr_width() c_int {
    return TB_OPT_ATTR_W;
}
pub export fn tb_version() [*c]const u8 {
    return "2.7.0-dev";
}
pub export fn tb_iswprint(arg_ch: u32) c_int {
    var ch = arg_ch;
    _ = &ch;
    return tb_iswprint_ex(ch, null);
}
pub export fn tb_wcwidth(arg_ch: u32) c_int {
    var ch = arg_ch;
    _ = &ch;
    var w: c_int = undefined;
    _ = &w;
    _ = tb_iswprint_ex(ch, &w);
    return w;
}
pub const struct_bytebuf = extern struct {
    buf: [*c]u8,
    len: usize,
    cap: usize,
    pub const bytebuf_puts = __root.bytebuf_puts;
    pub const bytebuf_nputs = __root.bytebuf_nputs;
    pub const bytebuf_shift = __root.bytebuf_shift;
    pub const bytebuf_flush = __root.bytebuf_flush;
    pub const bytebuf_reserve = __root.bytebuf_reserve;
    pub const bytebuf_free = __root.bytebuf_free;
    pub const nputs = __root.bytebuf_nputs;
    pub const shift = __root.bytebuf_shift;
    pub const flush = __root.bytebuf_flush;
    pub const reserve = __root.bytebuf_reserve;
};
pub const struct_cellbuf = extern struct {
    width: c_int,
    height: c_int,
    cells: [*c]struct_tb_cell,
    pub const cellbuf_init = __root.cellbuf_init;
    pub const cellbuf_free = __root.cellbuf_free;
    pub const cellbuf_clear = __root.cellbuf_clear;
    pub const cellbuf_get = __root.cellbuf_get;
    pub const cellbuf_in_bounds = __root.cellbuf_in_bounds;
    pub const cellbuf_resize = __root.cellbuf_resize;
    pub const init = __root.cellbuf_init;
    pub const clear = __root.cellbuf_clear;
    pub const get = __root.cellbuf_get;
    pub const in_bounds = __root.cellbuf_in_bounds;
    pub const resize = __root.cellbuf_resize;
};
pub const struct_cap_trie = extern struct {
    c: u8,
    children: [*c]struct_cap_trie,
    nchildren: usize,
    is_leaf: c_int,
    key: u16,
    mod: u8,
    pub const cap_trie_deinit = __root.cap_trie_deinit;
    pub const deinit = __root.cap_trie_deinit;
};
pub const struct_tb_global = extern struct {
    ttyfd: c_int,
    rfd: c_int,
    wfd: c_int,
    ttyfd_open: c_int,
    resize_pipefd: [2]c_int,
    width: c_int,
    height: c_int,
    cursor_x: c_int,
    cursor_y: c_int,
    last_x: c_int,
    last_y: c_int,
    fg: uintattr_t,
    bg: uintattr_t,
    last_fg: uintattr_t,
    last_bg: uintattr_t,
    input_mode: c_int,
    output_mode: c_int,
    terminfo: [*c]u8,
    nterminfo: usize,
    caps: [38][*c]const u8,
    cap_trie: struct_cap_trie,
    in: struct_bytebuf,
    out: struct_bytebuf,
    back: struct_cellbuf,
    front: struct_cellbuf,
    orig_tios: struct_termios,
    has_orig_tios: c_int,
    last_errno: c_int,
    initialized: c_int,
    fn_extract_esc_pre: ?*const fn ([*c]struct_tb_event, [*c]usize) callconv(.c) c_int,
    fn_extract_esc_post: ?*const fn ([*c]struct_tb_event, [*c]usize) callconv(.c) c_int,
    errbuf: [1024]u8,
};
pub var global: struct_tb_global = struct_tb_global{
    .ttyfd = 0,
    .rfd = 0,
    .wfd = 0,
    .ttyfd_open = 0,
    .resize_pipefd = @import("std").mem.zeroes([2]c_int),
    .width = 0,
    .height = 0,
    .cursor_x = 0,
    .cursor_y = 0,
    .last_x = 0,
    .last_y = 0,
    .fg = 0,
    .bg = 0,
    .last_fg = 0,
    .last_bg = 0,
    .input_mode = 0,
    .output_mode = 0,
    .terminfo = null,
    .nterminfo = 0,
    .caps = @import("std").mem.zeroes([38][*c]const u8),
    .cap_trie = @import("std").mem.zeroes(struct_cap_trie),
    .in = @import("std").mem.zeroes(struct_bytebuf),
    .out = @import("std").mem.zeroes(struct_bytebuf),
    .back = @import("std").mem.zeroes(struct_cellbuf),
    .front = @import("std").mem.zeroes(struct_cellbuf),
    .orig_tios = @import("std").mem.zeroes(struct_termios),
    .has_orig_tios = 0,
    .last_errno = 0,
    .initialized = 0,
    .fn_extract_esc_pre = null,
    .fn_extract_esc_post = null,
    .errbuf = @import("std").mem.zeroes([1024]u8),
};
pub const terminfo_cap_indexes: [38]i16 = [38]i16{
    66,
    68,
    69,
    70,
    71,
    72,
    73,
    74,
    75,
    67,
    216,
    217,
    77,
    59,
    76,
    164,
    82,
    81,
    87,
    61,
    79,
    83,
    148,
    28,
    40,
    16,
    13,
    5,
    39,
    36,
    27,
    26,
    311,
    34,
    89,
    88,
    30,
    32,
};
pub var xterm_caps: [38][*c]const u8 = [38][*c]const u8{
    "\x1bOP",
    "\x1bOQ",
    "\x1bOR",
    "\x1bOS",
    "\x1b[15~",
    "\x1b[17~",
    "\x1b[18~",
    "\x1b[19~",
    "\x1b[20~",
    "\x1b[21~",
    "\x1b[23~",
    "\x1b[24~",
    "\x1b[2~",
    "\x1b[3~",
    "\x1bOH",
    "\x1bOF",
    "\x1b[5~",
    "\x1b[6~",
    "\x1bOA",
    "\x1bOB",
    "\x1bOD",
    "\x1bOC",
    "\x1b[Z",
    "\x1b[?1049h\x1b[22;0;0t",
    "\x1b[?1049l\x1b[23;0;0t",
    "\x1b[?12l\x1b[?25h",
    "\x1b[?25l",
    "\x1b[H\x1b[2J",
    "\x1b(B\x1b[m",
    "\x1b[4m",
    "\x1b[1m",
    "\x1b[5m",
    "\x1b[3m",
    "\x1b[7m",
    "\x1b[?1h\x1b=",
    "\x1b[?1l\x1b>",
    "\x1b[2m",
    "\x1b[8m",
};
pub var linux_caps: [38][*c]const u8 = [38][*c]const u8{
    "\x1b[[A",
    "\x1b[[B",
    "\x1b[[C",
    "\x1b[[D",
    "\x1b[[E",
    "\x1b[17~",
    "\x1b[18~",
    "\x1b[19~",
    "\x1b[20~",
    "\x1b[21~",
    "\x1b[23~",
    "\x1b[24~",
    "\x1b[2~",
    "\x1b[3~",
    "\x1b[1~",
    "\x1b[4~",
    "\x1b[5~",
    "\x1b[6~",
    "\x1b[A",
    "\x1b[B",
    "\x1b[D",
    "\x1b[C",
    "\x1b\t",
    "",
    "",
    "\x1b[?25h\x1b[?0c",
    "\x1b[?25l\x1b[?1c",
    "\x1b[H\x1b[J",
    "\x1b[m\x0f",
    "\x1b[4m",
    "\x1b[1m",
    "\x1b[5m",
    "",
    "\x1b[7m",
    "",
    "",
    "\x1b[2m",
    "",
};
pub var screen_caps: [38][*c]const u8 = [38][*c]const u8{
    "\x1bOP",
    "\x1bOQ",
    "\x1bOR",
    "\x1bOS",
    "\x1b[15~",
    "\x1b[17~",
    "\x1b[18~",
    "\x1b[19~",
    "\x1b[20~",
    "\x1b[21~",
    "\x1b[23~",
    "\x1b[24~",
    "\x1b[2~",
    "\x1b[3~",
    "\x1b[1~",
    "\x1b[4~",
    "\x1b[5~",
    "\x1b[6~",
    "\x1bOA",
    "\x1bOB",
    "\x1bOD",
    "\x1bOC",
    "\x1b[Z",
    "\x1b[?1049h",
    "\x1b[?1049l",
    "\x1b[34h\x1b[?25h",
    "\x1b[?25l",
    "\x1b[H\x1b[J",
    "\x1b[m\x0f",
    "\x1b[4m",
    "\x1b[1m",
    "\x1b[5m",
    "",
    "\x1b[7m",
    "\x1b[?1h\x1b=",
    "\x1b[?1l\x1b>",
    "\x1b[2m",
    "",
};
pub var rxvt_256color_caps: [38][*c]const u8 = [38][*c]const u8{
    "\x1b[11~",
    "\x1b[12~",
    "\x1b[13~",
    "\x1b[14~",
    "\x1b[15~",
    "\x1b[17~",
    "\x1b[18~",
    "\x1b[19~",
    "\x1b[20~",
    "\x1b[21~",
    "\x1b[23~",
    "\x1b[24~",
    "\x1b[2~",
    "\x1b[3~",
    "\x1b[7~",
    "\x1b[8~",
    "\x1b[5~",
    "\x1b[6~",
    "\x1b[A",
    "\x1b[B",
    "\x1b[D",
    "\x1b[C",
    "\x1b[Z",
    "\x1b7\x1b[?47h",
    "\x1b[2J\x1b[?47l\x1b8",
    "\x1b[?25h",
    "\x1b[?25l",
    "\x1b[H\x1b[2J",
    "\x1b[m\x0f",
    "\x1b[4m",
    "\x1b[1m",
    "\x1b[5m",
    "",
    "\x1b[7m",
    "\x1b=",
    "\x1b>",
    "",
    "",
};
pub var rxvt_unicode_caps: [38][*c]const u8 = [38][*c]const u8{
    "\x1b[11~",
    "\x1b[12~",
    "\x1b[13~",
    "\x1b[14~",
    "\x1b[15~",
    "\x1b[17~",
    "\x1b[18~",
    "\x1b[19~",
    "\x1b[20~",
    "\x1b[21~",
    "\x1b[23~",
    "\x1b[24~",
    "\x1b[2~",
    "\x1b[3~",
    "\x1b[7~",
    "\x1b[8~",
    "\x1b[5~",
    "\x1b[6~",
    "\x1b[A",
    "\x1b[B",
    "\x1b[D",
    "\x1b[C",
    "\x1b[Z",
    "\x1b[?1049h",
    "\x1b[r\x1b[?1049l",
    "\x1b[?12l\x1b[?25h",
    "\x1b[?25l",
    "\x1b[H\x1b[2J",
    "\x1b[m\x1b(B",
    "\x1b[4m",
    "\x1b[1m",
    "\x1b[5m",
    "\x1b[3m",
    "\x1b[7m",
    "\x1b=",
    "\x1b>",
    "",
    "",
};
pub var eterm_caps: [38][*c]const u8 = [38][*c]const u8{
    "\x1b[11~",
    "\x1b[12~",
    "\x1b[13~",
    "\x1b[14~",
    "\x1b[15~",
    "\x1b[17~",
    "\x1b[18~",
    "\x1b[19~",
    "\x1b[20~",
    "\x1b[21~",
    "\x1b[23~",
    "\x1b[24~",
    "\x1b[2~",
    "\x1b[3~",
    "\x1b[7~",
    "\x1b[8~",
    "\x1b[5~",
    "\x1b[6~",
    "\x1b[A",
    "\x1b[B",
    "\x1b[D",
    "\x1b[C",
    "",
    "\x1b7\x1b[?47h",
    "\x1b[2J\x1b[?47l\x1b8",
    "\x1b[?25h",
    "\x1b[?25l",
    "\x1b[H\x1b[2J",
    "\x1b[m\x0f",
    "\x1b[4m",
    "\x1b[1m",
    "\x1b[5m",
    "",
    "\x1b[7m",
    "",
    "",
    "",
    "",
};
const struct_unnamed_34 = extern struct {
    name: [*c]const u8,
    caps: [*c][*c]const u8,
    alias: [*c]const u8,
};
pub var builtin_terms: [7]struct_unnamed_34 = [7]struct_unnamed_34{
    struct_unnamed_34{
        .name = "xterm",
        .caps = @ptrCast(@alignCast(&xterm_caps)),
        .alias = "",
    },
    struct_unnamed_34{
        .name = "linux",
        .caps = @ptrCast(@alignCast(&linux_caps)),
        .alias = "",
    },
    struct_unnamed_34{
        .name = "screen",
        .caps = @ptrCast(@alignCast(&screen_caps)),
        .alias = "tmux",
    },
    struct_unnamed_34{
        .name = "rxvt-256color",
        .caps = @ptrCast(@alignCast(&rxvt_256color_caps)),
        .alias = "",
    },
    struct_unnamed_34{
        .name = "rxvt-unicode",
        .caps = @ptrCast(@alignCast(&rxvt_unicode_caps)),
        .alias = "rxvt",
    },
    struct_unnamed_34{
        .name = "Eterm",
        .caps = @ptrCast(@alignCast(&eterm_caps)),
        .alias = "",
    },
    struct_unnamed_34{
        .name = null,
        .caps = null,
        .alias = null,
    },
};
const struct_unnamed_35 = extern struct {
    cap: [*c]const u8,
    key: u16,
    mod: u8,
};
pub var builtin_mod_caps: [313]struct_unnamed_35 = [313]struct_unnamed_35{
    struct_unnamed_35{
        .cap = "\x1b[1;2A",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 18)))),
        .mod = TB_MOD_SHIFT,
    },
    struct_unnamed_35{
        .cap = "\x1b[1;3A",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 18)))),
        .mod = TB_MOD_ALT,
    },
    struct_unnamed_35{
        .cap = "\x1b[1;4A",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 18)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_ALT | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[1;5A",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 18)))),
        .mod = TB_MOD_CTRL,
    },
    struct_unnamed_35{
        .cap = "\x1b[1;6A",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 18)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_CTRL | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[1;7A",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 18)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_CTRL | TB_MOD_ALT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[1;8A",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 18)))),
        .mod = @bitCast(@as(i8, @truncate((TB_MOD_CTRL | TB_MOD_ALT) | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[1;2B",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 19)))),
        .mod = TB_MOD_SHIFT,
    },
    struct_unnamed_35{
        .cap = "\x1b[1;3B",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 19)))),
        .mod = TB_MOD_ALT,
    },
    struct_unnamed_35{
        .cap = "\x1b[1;4B",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 19)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_ALT | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[1;5B",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 19)))),
        .mod = TB_MOD_CTRL,
    },
    struct_unnamed_35{
        .cap = "\x1b[1;6B",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 19)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_CTRL | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[1;7B",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 19)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_CTRL | TB_MOD_ALT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[1;8B",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 19)))),
        .mod = @bitCast(@as(i8, @truncate((TB_MOD_CTRL | TB_MOD_ALT) | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[1;2C",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 21)))),
        .mod = TB_MOD_SHIFT,
    },
    struct_unnamed_35{
        .cap = "\x1b[1;3C",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 21)))),
        .mod = TB_MOD_ALT,
    },
    struct_unnamed_35{
        .cap = "\x1b[1;4C",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 21)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_ALT | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[1;5C",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 21)))),
        .mod = TB_MOD_CTRL,
    },
    struct_unnamed_35{
        .cap = "\x1b[1;6C",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 21)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_CTRL | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[1;7C",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 21)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_CTRL | TB_MOD_ALT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[1;8C",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 21)))),
        .mod = @bitCast(@as(i8, @truncate((TB_MOD_CTRL | TB_MOD_ALT) | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[1;2D",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 20)))),
        .mod = TB_MOD_SHIFT,
    },
    struct_unnamed_35{
        .cap = "\x1b[1;3D",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 20)))),
        .mod = TB_MOD_ALT,
    },
    struct_unnamed_35{
        .cap = "\x1b[1;4D",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 20)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_ALT | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[1;5D",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 20)))),
        .mod = TB_MOD_CTRL,
    },
    struct_unnamed_35{
        .cap = "\x1b[1;6D",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 20)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_CTRL | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[1;7D",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 20)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_CTRL | TB_MOD_ALT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[1;8D",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 20)))),
        .mod = @bitCast(@as(i8, @truncate((TB_MOD_CTRL | TB_MOD_ALT) | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[1;2H",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 14)))),
        .mod = TB_MOD_SHIFT,
    },
    struct_unnamed_35{
        .cap = "\x1b[1;3H",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 14)))),
        .mod = TB_MOD_ALT,
    },
    struct_unnamed_35{
        .cap = "\x1b[1;4H",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 14)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_ALT | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[1;5H",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 14)))),
        .mod = TB_MOD_CTRL,
    },
    struct_unnamed_35{
        .cap = "\x1b[1;6H",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 14)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_CTRL | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[1;7H",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 14)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_CTRL | TB_MOD_ALT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[1;8H",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 14)))),
        .mod = @bitCast(@as(i8, @truncate((TB_MOD_CTRL | TB_MOD_ALT) | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[1;2F",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 15)))),
        .mod = TB_MOD_SHIFT,
    },
    struct_unnamed_35{
        .cap = "\x1b[1;3F",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 15)))),
        .mod = TB_MOD_ALT,
    },
    struct_unnamed_35{
        .cap = "\x1b[1;4F",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 15)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_ALT | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[1;5F",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 15)))),
        .mod = TB_MOD_CTRL,
    },
    struct_unnamed_35{
        .cap = "\x1b[1;6F",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 15)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_CTRL | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[1;7F",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 15)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_CTRL | TB_MOD_ALT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[1;8F",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 15)))),
        .mod = @bitCast(@as(i8, @truncate((TB_MOD_CTRL | TB_MOD_ALT) | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[2;2~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 12)))),
        .mod = TB_MOD_SHIFT,
    },
    struct_unnamed_35{
        .cap = "\x1b[2;3~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 12)))),
        .mod = TB_MOD_ALT,
    },
    struct_unnamed_35{
        .cap = "\x1b[2;4~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 12)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_ALT | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[2;5~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 12)))),
        .mod = TB_MOD_CTRL,
    },
    struct_unnamed_35{
        .cap = "\x1b[2;6~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 12)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_CTRL | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[2;7~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 12)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_CTRL | TB_MOD_ALT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[2;8~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 12)))),
        .mod = @bitCast(@as(i8, @truncate((TB_MOD_CTRL | TB_MOD_ALT) | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[3;2~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 13)))),
        .mod = TB_MOD_SHIFT,
    },
    struct_unnamed_35{
        .cap = "\x1b[3;3~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 13)))),
        .mod = TB_MOD_ALT,
    },
    struct_unnamed_35{
        .cap = "\x1b[3;4~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 13)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_ALT | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[3;5~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 13)))),
        .mod = TB_MOD_CTRL,
    },
    struct_unnamed_35{
        .cap = "\x1b[3;6~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 13)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_CTRL | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[3;7~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 13)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_CTRL | TB_MOD_ALT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[3;8~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 13)))),
        .mod = @bitCast(@as(i8, @truncate((TB_MOD_CTRL | TB_MOD_ALT) | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[5;2~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 16)))),
        .mod = TB_MOD_SHIFT,
    },
    struct_unnamed_35{
        .cap = "\x1b[5;3~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 16)))),
        .mod = TB_MOD_ALT,
    },
    struct_unnamed_35{
        .cap = "\x1b[5;4~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 16)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_ALT | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[5;5~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 16)))),
        .mod = TB_MOD_CTRL,
    },
    struct_unnamed_35{
        .cap = "\x1b[5;6~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 16)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_CTRL | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[5;7~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 16)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_CTRL | TB_MOD_ALT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[5;8~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 16)))),
        .mod = @bitCast(@as(i8, @truncate((TB_MOD_CTRL | TB_MOD_ALT) | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[6;2~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 17)))),
        .mod = TB_MOD_SHIFT,
    },
    struct_unnamed_35{
        .cap = "\x1b[6;3~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 17)))),
        .mod = TB_MOD_ALT,
    },
    struct_unnamed_35{
        .cap = "\x1b[6;4~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 17)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_ALT | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[6;5~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 17)))),
        .mod = TB_MOD_CTRL,
    },
    struct_unnamed_35{
        .cap = "\x1b[6;6~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 17)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_CTRL | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[6;7~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 17)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_CTRL | TB_MOD_ALT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[6;8~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 17)))),
        .mod = @bitCast(@as(i8, @truncate((TB_MOD_CTRL | TB_MOD_ALT) | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[1;2P",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 0)))),
        .mod = TB_MOD_SHIFT,
    },
    struct_unnamed_35{
        .cap = "\x1b[1;3P",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 0)))),
        .mod = TB_MOD_ALT,
    },
    struct_unnamed_35{
        .cap = "\x1b[1;4P",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 0)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_ALT | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[1;5P",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 0)))),
        .mod = TB_MOD_CTRL,
    },
    struct_unnamed_35{
        .cap = "\x1b[1;6P",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 0)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_CTRL | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[1;7P",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 0)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_CTRL | TB_MOD_ALT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[1;8P",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 0)))),
        .mod = @bitCast(@as(i8, @truncate((TB_MOD_CTRL | TB_MOD_ALT) | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[1;2Q",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 1)))),
        .mod = TB_MOD_SHIFT,
    },
    struct_unnamed_35{
        .cap = "\x1b[1;3Q",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 1)))),
        .mod = TB_MOD_ALT,
    },
    struct_unnamed_35{
        .cap = "\x1b[1;4Q",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 1)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_ALT | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[1;5Q",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 1)))),
        .mod = TB_MOD_CTRL,
    },
    struct_unnamed_35{
        .cap = "\x1b[1;6Q",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 1)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_CTRL | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[1;7Q",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 1)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_CTRL | TB_MOD_ALT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[1;8Q",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 1)))),
        .mod = @bitCast(@as(i8, @truncate((TB_MOD_CTRL | TB_MOD_ALT) | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[1;2R",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 2)))),
        .mod = TB_MOD_SHIFT,
    },
    struct_unnamed_35{
        .cap = "\x1b[1;3R",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 2)))),
        .mod = TB_MOD_ALT,
    },
    struct_unnamed_35{
        .cap = "\x1b[1;4R",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 2)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_ALT | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[1;5R",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 2)))),
        .mod = TB_MOD_CTRL,
    },
    struct_unnamed_35{
        .cap = "\x1b[1;6R",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 2)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_CTRL | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[1;7R",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 2)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_CTRL | TB_MOD_ALT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[1;8R",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 2)))),
        .mod = @bitCast(@as(i8, @truncate((TB_MOD_CTRL | TB_MOD_ALT) | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[1;2S",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 3)))),
        .mod = TB_MOD_SHIFT,
    },
    struct_unnamed_35{
        .cap = "\x1b[1;3S",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 3)))),
        .mod = TB_MOD_ALT,
    },
    struct_unnamed_35{
        .cap = "\x1b[1;4S",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 3)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_ALT | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[1;5S",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 3)))),
        .mod = TB_MOD_CTRL,
    },
    struct_unnamed_35{
        .cap = "\x1b[1;6S",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 3)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_CTRL | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[1;7S",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 3)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_CTRL | TB_MOD_ALT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[1;8S",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 3)))),
        .mod = @bitCast(@as(i8, @truncate((TB_MOD_CTRL | TB_MOD_ALT) | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[15;2~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 4)))),
        .mod = TB_MOD_SHIFT,
    },
    struct_unnamed_35{
        .cap = "\x1b[15;3~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 4)))),
        .mod = TB_MOD_ALT,
    },
    struct_unnamed_35{
        .cap = "\x1b[15;4~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 4)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_ALT | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[15;5~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 4)))),
        .mod = TB_MOD_CTRL,
    },
    struct_unnamed_35{
        .cap = "\x1b[15;6~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 4)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_CTRL | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[15;7~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 4)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_CTRL | TB_MOD_ALT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[15;8~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 4)))),
        .mod = @bitCast(@as(i8, @truncate((TB_MOD_CTRL | TB_MOD_ALT) | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[17;2~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 5)))),
        .mod = TB_MOD_SHIFT,
    },
    struct_unnamed_35{
        .cap = "\x1b[17;3~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 5)))),
        .mod = TB_MOD_ALT,
    },
    struct_unnamed_35{
        .cap = "\x1b[17;4~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 5)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_ALT | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[17;5~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 5)))),
        .mod = TB_MOD_CTRL,
    },
    struct_unnamed_35{
        .cap = "\x1b[17;6~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 5)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_CTRL | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[17;7~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 5)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_CTRL | TB_MOD_ALT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[17;8~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 5)))),
        .mod = @bitCast(@as(i8, @truncate((TB_MOD_CTRL | TB_MOD_ALT) | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[18;2~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 6)))),
        .mod = TB_MOD_SHIFT,
    },
    struct_unnamed_35{
        .cap = "\x1b[18;3~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 6)))),
        .mod = TB_MOD_ALT,
    },
    struct_unnamed_35{
        .cap = "\x1b[18;4~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 6)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_ALT | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[18;5~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 6)))),
        .mod = TB_MOD_CTRL,
    },
    struct_unnamed_35{
        .cap = "\x1b[18;6~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 6)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_CTRL | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[18;7~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 6)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_CTRL | TB_MOD_ALT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[18;8~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 6)))),
        .mod = @bitCast(@as(i8, @truncate((TB_MOD_CTRL | TB_MOD_ALT) | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[19;2~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 7)))),
        .mod = TB_MOD_SHIFT,
    },
    struct_unnamed_35{
        .cap = "\x1b[19;3~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 7)))),
        .mod = TB_MOD_ALT,
    },
    struct_unnamed_35{
        .cap = "\x1b[19;4~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 7)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_ALT | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[19;5~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 7)))),
        .mod = TB_MOD_CTRL,
    },
    struct_unnamed_35{
        .cap = "\x1b[19;6~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 7)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_CTRL | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[19;7~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 7)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_CTRL | TB_MOD_ALT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[19;8~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 7)))),
        .mod = @bitCast(@as(i8, @truncate((TB_MOD_CTRL | TB_MOD_ALT) | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[20;2~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 8)))),
        .mod = TB_MOD_SHIFT,
    },
    struct_unnamed_35{
        .cap = "\x1b[20;3~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 8)))),
        .mod = TB_MOD_ALT,
    },
    struct_unnamed_35{
        .cap = "\x1b[20;4~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 8)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_ALT | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[20;5~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 8)))),
        .mod = TB_MOD_CTRL,
    },
    struct_unnamed_35{
        .cap = "\x1b[20;6~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 8)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_CTRL | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[20;7~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 8)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_CTRL | TB_MOD_ALT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[20;8~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 8)))),
        .mod = @bitCast(@as(i8, @truncate((TB_MOD_CTRL | TB_MOD_ALT) | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[21;2~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 9)))),
        .mod = TB_MOD_SHIFT,
    },
    struct_unnamed_35{
        .cap = "\x1b[21;3~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 9)))),
        .mod = TB_MOD_ALT,
    },
    struct_unnamed_35{
        .cap = "\x1b[21;4~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 9)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_ALT | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[21;5~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 9)))),
        .mod = TB_MOD_CTRL,
    },
    struct_unnamed_35{
        .cap = "\x1b[21;6~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 9)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_CTRL | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[21;7~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 9)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_CTRL | TB_MOD_ALT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[21;8~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 9)))),
        .mod = @bitCast(@as(i8, @truncate((TB_MOD_CTRL | TB_MOD_ALT) | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[23;2~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 10)))),
        .mod = TB_MOD_SHIFT,
    },
    struct_unnamed_35{
        .cap = "\x1b[23;3~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 10)))),
        .mod = TB_MOD_ALT,
    },
    struct_unnamed_35{
        .cap = "\x1b[23;4~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 10)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_ALT | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[23;5~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 10)))),
        .mod = TB_MOD_CTRL,
    },
    struct_unnamed_35{
        .cap = "\x1b[23;6~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 10)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_CTRL | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[23;7~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 10)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_CTRL | TB_MOD_ALT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[23;8~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 10)))),
        .mod = @bitCast(@as(i8, @truncate((TB_MOD_CTRL | TB_MOD_ALT) | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[24;2~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 11)))),
        .mod = TB_MOD_SHIFT,
    },
    struct_unnamed_35{
        .cap = "\x1b[24;3~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 11)))),
        .mod = TB_MOD_ALT,
    },
    struct_unnamed_35{
        .cap = "\x1b[24;4~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 11)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_ALT | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[24;5~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 11)))),
        .mod = TB_MOD_CTRL,
    },
    struct_unnamed_35{
        .cap = "\x1b[24;6~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 11)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_CTRL | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[24;7~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 11)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_CTRL | TB_MOD_ALT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[24;8~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 11)))),
        .mod = @bitCast(@as(i8, @truncate((TB_MOD_CTRL | TB_MOD_ALT) | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[a",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 18)))),
        .mod = TB_MOD_SHIFT,
    },
    struct_unnamed_35{
        .cap = "\x1b\x1b[A",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 18)))),
        .mod = TB_MOD_ALT,
    },
    struct_unnamed_35{
        .cap = "\x1b\x1b[a",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 18)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_ALT | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1bOa",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 18)))),
        .mod = TB_MOD_CTRL,
    },
    struct_unnamed_35{
        .cap = "\x1b\x1bOa",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 18)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_CTRL | TB_MOD_ALT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[b",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 19)))),
        .mod = TB_MOD_SHIFT,
    },
    struct_unnamed_35{
        .cap = "\x1b\x1b[B",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 19)))),
        .mod = TB_MOD_ALT,
    },
    struct_unnamed_35{
        .cap = "\x1b\x1b[b",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 19)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_ALT | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1bOb",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 19)))),
        .mod = TB_MOD_CTRL,
    },
    struct_unnamed_35{
        .cap = "\x1b\x1bOb",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 19)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_CTRL | TB_MOD_ALT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[c",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 21)))),
        .mod = TB_MOD_SHIFT,
    },
    struct_unnamed_35{
        .cap = "\x1b\x1b[C",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 21)))),
        .mod = TB_MOD_ALT,
    },
    struct_unnamed_35{
        .cap = "\x1b\x1b[c",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 21)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_ALT | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1bOc",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 21)))),
        .mod = TB_MOD_CTRL,
    },
    struct_unnamed_35{
        .cap = "\x1b\x1bOc",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 21)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_CTRL | TB_MOD_ALT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[d",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 20)))),
        .mod = TB_MOD_SHIFT,
    },
    struct_unnamed_35{
        .cap = "\x1b\x1b[D",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 20)))),
        .mod = TB_MOD_ALT,
    },
    struct_unnamed_35{
        .cap = "\x1b\x1b[d",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 20)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_ALT | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1bOd",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 20)))),
        .mod = TB_MOD_CTRL,
    },
    struct_unnamed_35{
        .cap = "\x1b\x1bOd",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 20)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_CTRL | TB_MOD_ALT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[7$",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 14)))),
        .mod = TB_MOD_SHIFT,
    },
    struct_unnamed_35{
        .cap = "\x1b\x1b[7~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 14)))),
        .mod = TB_MOD_ALT,
    },
    struct_unnamed_35{
        .cap = "\x1b\x1b[7$",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 14)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_ALT | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[7^",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 14)))),
        .mod = TB_MOD_CTRL,
    },
    struct_unnamed_35{
        .cap = "\x1b[7@",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 14)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_CTRL | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b\x1b[7^",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 14)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_CTRL | TB_MOD_ALT))),
    },
    struct_unnamed_35{
        .cap = "\x1b\x1b[7@",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 14)))),
        .mod = @bitCast(@as(i8, @truncate((TB_MOD_CTRL | TB_MOD_ALT) | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b\x1b[8~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 15)))),
        .mod = TB_MOD_ALT,
    },
    struct_unnamed_35{
        .cap = "\x1b\x1b[8$",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 15)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_ALT | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[8^",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 15)))),
        .mod = TB_MOD_CTRL,
    },
    struct_unnamed_35{
        .cap = "\x1b\x1b[8^",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 15)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_CTRL | TB_MOD_ALT))),
    },
    struct_unnamed_35{
        .cap = "\x1b\x1b[8@",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 15)))),
        .mod = @bitCast(@as(i8, @truncate((TB_MOD_CTRL | TB_MOD_ALT) | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[8@",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 15)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_CTRL | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[8$",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 15)))),
        .mod = TB_MOD_SHIFT,
    },
    struct_unnamed_35{
        .cap = "\x1b\x1b[2~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 12)))),
        .mod = TB_MOD_ALT,
    },
    struct_unnamed_35{
        .cap = "\x1b\x1b[2$",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 12)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_ALT | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[2^",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 12)))),
        .mod = TB_MOD_CTRL,
    },
    struct_unnamed_35{
        .cap = "\x1b\x1b[2^",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 12)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_CTRL | TB_MOD_ALT))),
    },
    struct_unnamed_35{
        .cap = "\x1b\x1b[2@",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 12)))),
        .mod = @bitCast(@as(i8, @truncate((TB_MOD_CTRL | TB_MOD_ALT) | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[2@",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 12)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_CTRL | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[2$",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 12)))),
        .mod = TB_MOD_SHIFT,
    },
    struct_unnamed_35{
        .cap = "\x1b\x1b[3~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 13)))),
        .mod = TB_MOD_ALT,
    },
    struct_unnamed_35{
        .cap = "\x1b\x1b[3$",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 13)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_ALT | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[3^",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 13)))),
        .mod = TB_MOD_CTRL,
    },
    struct_unnamed_35{
        .cap = "\x1b\x1b[3^",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 13)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_CTRL | TB_MOD_ALT))),
    },
    struct_unnamed_35{
        .cap = "\x1b\x1b[3@",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 13)))),
        .mod = @bitCast(@as(i8, @truncate((TB_MOD_CTRL | TB_MOD_ALT) | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[3@",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 13)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_CTRL | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[3$",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 13)))),
        .mod = TB_MOD_SHIFT,
    },
    struct_unnamed_35{
        .cap = "\x1b\x1b[5~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 16)))),
        .mod = TB_MOD_ALT,
    },
    struct_unnamed_35{
        .cap = "\x1b\x1b[5$",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 16)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_ALT | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[5^",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 16)))),
        .mod = TB_MOD_CTRL,
    },
    struct_unnamed_35{
        .cap = "\x1b\x1b[5^",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 16)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_CTRL | TB_MOD_ALT))),
    },
    struct_unnamed_35{
        .cap = "\x1b\x1b[5@",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 16)))),
        .mod = @bitCast(@as(i8, @truncate((TB_MOD_CTRL | TB_MOD_ALT) | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[5@",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 16)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_CTRL | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[5$",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 16)))),
        .mod = TB_MOD_SHIFT,
    },
    struct_unnamed_35{
        .cap = "\x1b\x1b[6~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 17)))),
        .mod = TB_MOD_ALT,
    },
    struct_unnamed_35{
        .cap = "\x1b\x1b[6$",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 17)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_ALT | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[6^",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 17)))),
        .mod = TB_MOD_CTRL,
    },
    struct_unnamed_35{
        .cap = "\x1b\x1b[6^",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 17)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_CTRL | TB_MOD_ALT))),
    },
    struct_unnamed_35{
        .cap = "\x1b\x1b[6@",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 17)))),
        .mod = @bitCast(@as(i8, @truncate((TB_MOD_CTRL | TB_MOD_ALT) | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[6@",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 17)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_CTRL | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[6$",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 17)))),
        .mod = TB_MOD_SHIFT,
    },
    struct_unnamed_35{
        .cap = "\x1b\x1b[11~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 0)))),
        .mod = TB_MOD_ALT,
    },
    struct_unnamed_35{
        .cap = "\x1b\x1b[23~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 0)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_ALT | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[11^",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 0)))),
        .mod = TB_MOD_CTRL,
    },
    struct_unnamed_35{
        .cap = "\x1b\x1b[11^",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 0)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_CTRL | TB_MOD_ALT))),
    },
    struct_unnamed_35{
        .cap = "\x1b\x1b[23^",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 0)))),
        .mod = @bitCast(@as(i8, @truncate((TB_MOD_CTRL | TB_MOD_ALT) | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[23^",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 0)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_CTRL | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[23~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 0)))),
        .mod = TB_MOD_SHIFT,
    },
    struct_unnamed_35{
        .cap = "\x1b\x1b[12~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 1)))),
        .mod = TB_MOD_ALT,
    },
    struct_unnamed_35{
        .cap = "\x1b\x1b[24~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 1)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_ALT | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[12^",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 1)))),
        .mod = TB_MOD_CTRL,
    },
    struct_unnamed_35{
        .cap = "\x1b\x1b[12^",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 1)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_CTRL | TB_MOD_ALT))),
    },
    struct_unnamed_35{
        .cap = "\x1b\x1b[24^",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 1)))),
        .mod = @bitCast(@as(i8, @truncate((TB_MOD_CTRL | TB_MOD_ALT) | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[24^",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 1)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_CTRL | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[24~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 1)))),
        .mod = TB_MOD_SHIFT,
    },
    struct_unnamed_35{
        .cap = "\x1b\x1b[13~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 2)))),
        .mod = TB_MOD_ALT,
    },
    struct_unnamed_35{
        .cap = "\x1b\x1b[25~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 2)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_ALT | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[13^",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 2)))),
        .mod = TB_MOD_CTRL,
    },
    struct_unnamed_35{
        .cap = "\x1b\x1b[13^",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 2)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_CTRL | TB_MOD_ALT))),
    },
    struct_unnamed_35{
        .cap = "\x1b\x1b[25^",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 2)))),
        .mod = @bitCast(@as(i8, @truncate((TB_MOD_CTRL | TB_MOD_ALT) | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[25^",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 2)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_CTRL | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[25~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 2)))),
        .mod = TB_MOD_SHIFT,
    },
    struct_unnamed_35{
        .cap = "\x1b\x1b[14~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 3)))),
        .mod = TB_MOD_ALT,
    },
    struct_unnamed_35{
        .cap = "\x1b\x1b[26~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 3)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_ALT | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[14^",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 3)))),
        .mod = TB_MOD_CTRL,
    },
    struct_unnamed_35{
        .cap = "\x1b\x1b[14^",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 3)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_CTRL | TB_MOD_ALT))),
    },
    struct_unnamed_35{
        .cap = "\x1b\x1b[26^",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 3)))),
        .mod = @bitCast(@as(i8, @truncate((TB_MOD_CTRL | TB_MOD_ALT) | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[26^",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 3)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_CTRL | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[26~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 3)))),
        .mod = TB_MOD_SHIFT,
    },
    struct_unnamed_35{
        .cap = "\x1b\x1b[15~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 4)))),
        .mod = TB_MOD_ALT,
    },
    struct_unnamed_35{
        .cap = "\x1b\x1b[28~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 4)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_ALT | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[15^",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 4)))),
        .mod = TB_MOD_CTRL,
    },
    struct_unnamed_35{
        .cap = "\x1b\x1b[15^",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 4)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_CTRL | TB_MOD_ALT))),
    },
    struct_unnamed_35{
        .cap = "\x1b\x1b[28^",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 4)))),
        .mod = @bitCast(@as(i8, @truncate((TB_MOD_CTRL | TB_MOD_ALT) | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[28^",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 4)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_CTRL | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[28~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 4)))),
        .mod = TB_MOD_SHIFT,
    },
    struct_unnamed_35{
        .cap = "\x1b\x1b[17~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 5)))),
        .mod = TB_MOD_ALT,
    },
    struct_unnamed_35{
        .cap = "\x1b\x1b[29~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 5)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_ALT | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[17^",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 5)))),
        .mod = TB_MOD_CTRL,
    },
    struct_unnamed_35{
        .cap = "\x1b\x1b[17^",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 5)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_CTRL | TB_MOD_ALT))),
    },
    struct_unnamed_35{
        .cap = "\x1b\x1b[29^",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 5)))),
        .mod = @bitCast(@as(i8, @truncate((TB_MOD_CTRL | TB_MOD_ALT) | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[29^",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 5)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_CTRL | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[29~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 5)))),
        .mod = TB_MOD_SHIFT,
    },
    struct_unnamed_35{
        .cap = "\x1b\x1b[18~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 6)))),
        .mod = TB_MOD_ALT,
    },
    struct_unnamed_35{
        .cap = "\x1b\x1b[31~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 6)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_ALT | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[18^",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 6)))),
        .mod = TB_MOD_CTRL,
    },
    struct_unnamed_35{
        .cap = "\x1b\x1b[18^",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 6)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_CTRL | TB_MOD_ALT))),
    },
    struct_unnamed_35{
        .cap = "\x1b\x1b[31^",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 6)))),
        .mod = @bitCast(@as(i8, @truncate((TB_MOD_CTRL | TB_MOD_ALT) | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[31^",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 6)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_CTRL | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[31~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 6)))),
        .mod = TB_MOD_SHIFT,
    },
    struct_unnamed_35{
        .cap = "\x1b\x1b[19~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 7)))),
        .mod = TB_MOD_ALT,
    },
    struct_unnamed_35{
        .cap = "\x1b\x1b[32~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 7)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_ALT | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[19^",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 7)))),
        .mod = TB_MOD_CTRL,
    },
    struct_unnamed_35{
        .cap = "\x1b\x1b[19^",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 7)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_CTRL | TB_MOD_ALT))),
    },
    struct_unnamed_35{
        .cap = "\x1b\x1b[32^",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 7)))),
        .mod = @bitCast(@as(i8, @truncate((TB_MOD_CTRL | TB_MOD_ALT) | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[32^",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 7)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_CTRL | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[32~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 7)))),
        .mod = TB_MOD_SHIFT,
    },
    struct_unnamed_35{
        .cap = "\x1b\x1b[20~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 8)))),
        .mod = TB_MOD_ALT,
    },
    struct_unnamed_35{
        .cap = "\x1b\x1b[33~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 8)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_ALT | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[20^",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 8)))),
        .mod = TB_MOD_CTRL,
    },
    struct_unnamed_35{
        .cap = "\x1b\x1b[20^",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 8)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_CTRL | TB_MOD_ALT))),
    },
    struct_unnamed_35{
        .cap = "\x1b\x1b[33^",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 8)))),
        .mod = @bitCast(@as(i8, @truncate((TB_MOD_CTRL | TB_MOD_ALT) | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[33^",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 8)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_CTRL | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[33~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 8)))),
        .mod = TB_MOD_SHIFT,
    },
    struct_unnamed_35{
        .cap = "\x1b\x1b[21~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 9)))),
        .mod = TB_MOD_ALT,
    },
    struct_unnamed_35{
        .cap = "\x1b\x1b[34~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 9)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_ALT | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[21^",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 9)))),
        .mod = TB_MOD_CTRL,
    },
    struct_unnamed_35{
        .cap = "\x1b\x1b[21^",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 9)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_CTRL | TB_MOD_ALT))),
    },
    struct_unnamed_35{
        .cap = "\x1b\x1b[34^",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 9)))),
        .mod = @bitCast(@as(i8, @truncate((TB_MOD_CTRL | TB_MOD_ALT) | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[34^",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 9)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_CTRL | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[34~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 9)))),
        .mod = TB_MOD_SHIFT,
    },
    struct_unnamed_35{
        .cap = "\x1b\x1b[23~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 10)))),
        .mod = TB_MOD_ALT,
    },
    struct_unnamed_35{
        .cap = "\x1b\x1b[23$",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 10)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_ALT | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[23^",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 10)))),
        .mod = TB_MOD_CTRL,
    },
    struct_unnamed_35{
        .cap = "\x1b\x1b[23^",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 10)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_CTRL | TB_MOD_ALT))),
    },
    struct_unnamed_35{
        .cap = "\x1b\x1b[23@",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 10)))),
        .mod = @bitCast(@as(i8, @truncate((TB_MOD_CTRL | TB_MOD_ALT) | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[23@",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 10)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_CTRL | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[23$",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 10)))),
        .mod = TB_MOD_SHIFT,
    },
    struct_unnamed_35{
        .cap = "\x1b\x1b[24~",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 11)))),
        .mod = TB_MOD_ALT,
    },
    struct_unnamed_35{
        .cap = "\x1b\x1b[24$",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 11)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_ALT | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[24^",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 11)))),
        .mod = TB_MOD_CTRL,
    },
    struct_unnamed_35{
        .cap = "\x1b\x1b[24^",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 11)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_CTRL | TB_MOD_ALT))),
    },
    struct_unnamed_35{
        .cap = "\x1b\x1b[24@",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 11)))),
        .mod = @bitCast(@as(i8, @truncate((TB_MOD_CTRL | TB_MOD_ALT) | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[24@",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 11)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_CTRL | TB_MOD_SHIFT))),
    },
    struct_unnamed_35{
        .cap = "\x1b[24$",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 11)))),
        .mod = TB_MOD_SHIFT,
    },
    struct_unnamed_35{
        .cap = "\x1b[A",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 18)))),
        .mod = TB_MOD_SHIFT,
    },
    struct_unnamed_35{
        .cap = "\x1b[B",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 19)))),
        .mod = TB_MOD_SHIFT,
    },
    struct_unnamed_35{
        .cap = "\x1b[C",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 21)))),
        .mod = TB_MOD_SHIFT,
    },
    struct_unnamed_35{
        .cap = "\x1b[D",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 20)))),
        .mod = TB_MOD_SHIFT,
    },
    struct_unnamed_35{
        .cap = "\x1bOA",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 18)))),
        .mod = TB_MOD_CTRL,
    },
    struct_unnamed_35{
        .cap = "\x1b\x1bOA",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 18)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_CTRL | TB_MOD_ALT))),
    },
    struct_unnamed_35{
        .cap = "\x1bOB",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 19)))),
        .mod = TB_MOD_CTRL,
    },
    struct_unnamed_35{
        .cap = "\x1b\x1bOB",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 19)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_CTRL | TB_MOD_ALT))),
    },
    struct_unnamed_35{
        .cap = "\x1bOC",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 21)))),
        .mod = TB_MOD_CTRL,
    },
    struct_unnamed_35{
        .cap = "\x1b\x1bOC",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 21)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_CTRL | TB_MOD_ALT))),
    },
    struct_unnamed_35{
        .cap = "\x1bOD",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 20)))),
        .mod = TB_MOD_CTRL,
    },
    struct_unnamed_35{
        .cap = "\x1b\x1bOD",
        .key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 20)))),
        .mod = @bitCast(@as(i8, @truncate(TB_MOD_CTRL | TB_MOD_ALT))),
    },
    struct_unnamed_35{
        .cap = null,
        .key = 0,
        .mod = 0,
    },
};
pub const utf8_length: [256]u8 = [256]u8{
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    5,
    5,
    5,
    5,
    6,
    6,
    1,
    1,
};
pub const utf8_mask: [6]u8 = [6]u8{
    127,
    31,
    15,
    7,
    3,
    1,
};
const struct_unnamed_36 = extern struct {
    range_start: u32,
    range_end: u32,
    width: c_int,
};
pub var wcwidth_table: [2143]struct_unnamed_36 = [2143]struct_unnamed_36{
    struct_unnamed_36{
        .range_start = 1,
        .range_end = 31,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 32,
        .range_end = 126,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 127,
        .range_end = 159,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 160,
        .range_end = 767,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 768,
        .range_end = 879,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 880,
        .range_end = 887,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 888,
        .range_end = 889,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 890,
        .range_end = 895,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 896,
        .range_end = 899,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 900,
        .range_end = 906,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 907,
        .range_end = 907,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 908,
        .range_end = 908,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 909,
        .range_end = 909,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 910,
        .range_end = 929,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 930,
        .range_end = 930,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 931,
        .range_end = 1154,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 1155,
        .range_end = 1161,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 1162,
        .range_end = 1327,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 1328,
        .range_end = 1328,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 1329,
        .range_end = 1366,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 1367,
        .range_end = 1368,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 1369,
        .range_end = 1418,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 1419,
        .range_end = 1420,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 1421,
        .range_end = 1423,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 1424,
        .range_end = 1424,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 1425,
        .range_end = 1469,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 1470,
        .range_end = 1470,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 1471,
        .range_end = 1471,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 1472,
        .range_end = 1472,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 1473,
        .range_end = 1474,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 1475,
        .range_end = 1475,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 1476,
        .range_end = 1477,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 1478,
        .range_end = 1478,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 1479,
        .range_end = 1479,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 1480,
        .range_end = 1487,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 1488,
        .range_end = 1514,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 1515,
        .range_end = 1518,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 1519,
        .range_end = 1524,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 1525,
        .range_end = 1535,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 1536,
        .range_end = 1551,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 1552,
        .range_end = 1562,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 1563,
        .range_end = 1563,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 1564,
        .range_end = 1564,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 1565,
        .range_end = 1610,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 1611,
        .range_end = 1631,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 1632,
        .range_end = 1647,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 1648,
        .range_end = 1648,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 1649,
        .range_end = 1749,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 1750,
        .range_end = 1756,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 1757,
        .range_end = 1758,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 1759,
        .range_end = 1764,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 1765,
        .range_end = 1766,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 1767,
        .range_end = 1768,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 1769,
        .range_end = 1769,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 1770,
        .range_end = 1773,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 1774,
        .range_end = 1805,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 1806,
        .range_end = 1806,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 1807,
        .range_end = 1808,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 1809,
        .range_end = 1809,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 1810,
        .range_end = 1839,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 1840,
        .range_end = 1866,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 1867,
        .range_end = 1868,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 1869,
        .range_end = 1957,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 1958,
        .range_end = 1968,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 1969,
        .range_end = 1969,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 1970,
        .range_end = 1983,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 1984,
        .range_end = 2026,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 2027,
        .range_end = 2035,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 2036,
        .range_end = 2042,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 2043,
        .range_end = 2044,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 2045,
        .range_end = 2045,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 2046,
        .range_end = 2069,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 2070,
        .range_end = 2073,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 2074,
        .range_end = 2074,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 2075,
        .range_end = 2083,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 2084,
        .range_end = 2084,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 2085,
        .range_end = 2087,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 2088,
        .range_end = 2088,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 2089,
        .range_end = 2093,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 2094,
        .range_end = 2095,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 2096,
        .range_end = 2110,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 2111,
        .range_end = 2111,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 2112,
        .range_end = 2136,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 2137,
        .range_end = 2139,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 2140,
        .range_end = 2141,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 2142,
        .range_end = 2142,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 2143,
        .range_end = 2143,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 2144,
        .range_end = 2154,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 2155,
        .range_end = 2159,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 2160,
        .range_end = 2190,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 2191,
        .range_end = 2191,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 2192,
        .range_end = 2193,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 2194,
        .range_end = 2198,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 2199,
        .range_end = 2207,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 2208,
        .range_end = 2249,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 2250,
        .range_end = 2273,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 2274,
        .range_end = 2274,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 2275,
        .range_end = 2306,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 2307,
        .range_end = 2361,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 2362,
        .range_end = 2362,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 2363,
        .range_end = 2363,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 2364,
        .range_end = 2364,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 2365,
        .range_end = 2368,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 2369,
        .range_end = 2376,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 2377,
        .range_end = 2380,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 2381,
        .range_end = 2381,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 2382,
        .range_end = 2384,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 2385,
        .range_end = 2391,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 2392,
        .range_end = 2401,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 2402,
        .range_end = 2403,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 2404,
        .range_end = 2432,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 2433,
        .range_end = 2433,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 2434,
        .range_end = 2435,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 2436,
        .range_end = 2436,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 2437,
        .range_end = 2444,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 2445,
        .range_end = 2446,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 2447,
        .range_end = 2448,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 2449,
        .range_end = 2450,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 2451,
        .range_end = 2472,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 2473,
        .range_end = 2473,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 2474,
        .range_end = 2480,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 2481,
        .range_end = 2481,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 2482,
        .range_end = 2482,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 2483,
        .range_end = 2485,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 2486,
        .range_end = 2489,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 2490,
        .range_end = 2491,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 2492,
        .range_end = 2492,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 2493,
        .range_end = 2496,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 2497,
        .range_end = 2500,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 2501,
        .range_end = 2502,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 2503,
        .range_end = 2504,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 2505,
        .range_end = 2506,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 2507,
        .range_end = 2508,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 2509,
        .range_end = 2509,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 2510,
        .range_end = 2510,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 2511,
        .range_end = 2518,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 2519,
        .range_end = 2519,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 2520,
        .range_end = 2523,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 2524,
        .range_end = 2525,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 2526,
        .range_end = 2526,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 2527,
        .range_end = 2529,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 2530,
        .range_end = 2531,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 2532,
        .range_end = 2533,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 2534,
        .range_end = 2557,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 2558,
        .range_end = 2558,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 2559,
        .range_end = 2560,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 2561,
        .range_end = 2562,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 2563,
        .range_end = 2563,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 2564,
        .range_end = 2564,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 2565,
        .range_end = 2570,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 2571,
        .range_end = 2574,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 2575,
        .range_end = 2576,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 2577,
        .range_end = 2578,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 2579,
        .range_end = 2600,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 2601,
        .range_end = 2601,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 2602,
        .range_end = 2608,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 2609,
        .range_end = 2609,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 2610,
        .range_end = 2611,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 2612,
        .range_end = 2612,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 2613,
        .range_end = 2614,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 2615,
        .range_end = 2615,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 2616,
        .range_end = 2617,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 2618,
        .range_end = 2619,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 2620,
        .range_end = 2620,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 2621,
        .range_end = 2621,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 2622,
        .range_end = 2624,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 2625,
        .range_end = 2626,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 2627,
        .range_end = 2630,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 2631,
        .range_end = 2632,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 2633,
        .range_end = 2634,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 2635,
        .range_end = 2637,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 2638,
        .range_end = 2640,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 2641,
        .range_end = 2641,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 2642,
        .range_end = 2648,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 2649,
        .range_end = 2652,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 2653,
        .range_end = 2653,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 2654,
        .range_end = 2654,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 2655,
        .range_end = 2661,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 2662,
        .range_end = 2671,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 2672,
        .range_end = 2673,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 2674,
        .range_end = 2676,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 2677,
        .range_end = 2677,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 2678,
        .range_end = 2678,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 2679,
        .range_end = 2688,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 2689,
        .range_end = 2690,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 2691,
        .range_end = 2691,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 2692,
        .range_end = 2692,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 2693,
        .range_end = 2701,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 2702,
        .range_end = 2702,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 2703,
        .range_end = 2705,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 2706,
        .range_end = 2706,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 2707,
        .range_end = 2728,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 2729,
        .range_end = 2729,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 2730,
        .range_end = 2736,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 2737,
        .range_end = 2737,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 2738,
        .range_end = 2739,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 2740,
        .range_end = 2740,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 2741,
        .range_end = 2745,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 2746,
        .range_end = 2747,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 2748,
        .range_end = 2748,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 2749,
        .range_end = 2752,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 2753,
        .range_end = 2757,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 2758,
        .range_end = 2758,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 2759,
        .range_end = 2760,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 2761,
        .range_end = 2761,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 2762,
        .range_end = 2762,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 2763,
        .range_end = 2764,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 2765,
        .range_end = 2765,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 2766,
        .range_end = 2767,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 2768,
        .range_end = 2768,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 2769,
        .range_end = 2783,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 2784,
        .range_end = 2785,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 2786,
        .range_end = 2787,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 2788,
        .range_end = 2789,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 2790,
        .range_end = 2801,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 2802,
        .range_end = 2808,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 2809,
        .range_end = 2809,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 2810,
        .range_end = 2815,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 2816,
        .range_end = 2816,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 2817,
        .range_end = 2817,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 2818,
        .range_end = 2819,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 2820,
        .range_end = 2820,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 2821,
        .range_end = 2828,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 2829,
        .range_end = 2830,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 2831,
        .range_end = 2832,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 2833,
        .range_end = 2834,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 2835,
        .range_end = 2856,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 2857,
        .range_end = 2857,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 2858,
        .range_end = 2864,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 2865,
        .range_end = 2865,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 2866,
        .range_end = 2867,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 2868,
        .range_end = 2868,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 2869,
        .range_end = 2873,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 2874,
        .range_end = 2875,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 2876,
        .range_end = 2876,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 2877,
        .range_end = 2878,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 2879,
        .range_end = 2879,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 2880,
        .range_end = 2880,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 2881,
        .range_end = 2884,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 2885,
        .range_end = 2886,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 2887,
        .range_end = 2888,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 2889,
        .range_end = 2890,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 2891,
        .range_end = 2892,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 2893,
        .range_end = 2893,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 2894,
        .range_end = 2900,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 2901,
        .range_end = 2902,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 2903,
        .range_end = 2903,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 2904,
        .range_end = 2907,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 2908,
        .range_end = 2909,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 2910,
        .range_end = 2910,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 2911,
        .range_end = 2913,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 2914,
        .range_end = 2915,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 2916,
        .range_end = 2917,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 2918,
        .range_end = 2935,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 2936,
        .range_end = 2945,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 2946,
        .range_end = 2946,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 2947,
        .range_end = 2947,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 2948,
        .range_end = 2948,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 2949,
        .range_end = 2954,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 2955,
        .range_end = 2957,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 2958,
        .range_end = 2960,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 2961,
        .range_end = 2961,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 2962,
        .range_end = 2965,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 2966,
        .range_end = 2968,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 2969,
        .range_end = 2970,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 2971,
        .range_end = 2971,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 2972,
        .range_end = 2972,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 2973,
        .range_end = 2973,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 2974,
        .range_end = 2975,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 2976,
        .range_end = 2978,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 2979,
        .range_end = 2980,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 2981,
        .range_end = 2983,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 2984,
        .range_end = 2986,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 2987,
        .range_end = 2989,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 2990,
        .range_end = 3001,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 3002,
        .range_end = 3005,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 3006,
        .range_end = 3007,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 3008,
        .range_end = 3008,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 3009,
        .range_end = 3010,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 3011,
        .range_end = 3013,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 3014,
        .range_end = 3016,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 3017,
        .range_end = 3017,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 3018,
        .range_end = 3020,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 3021,
        .range_end = 3021,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 3022,
        .range_end = 3023,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 3024,
        .range_end = 3024,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 3025,
        .range_end = 3030,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 3031,
        .range_end = 3031,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 3032,
        .range_end = 3045,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 3046,
        .range_end = 3066,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 3067,
        .range_end = 3071,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 3072,
        .range_end = 3072,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 3073,
        .range_end = 3075,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 3076,
        .range_end = 3076,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 3077,
        .range_end = 3084,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 3085,
        .range_end = 3085,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 3086,
        .range_end = 3088,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 3089,
        .range_end = 3089,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 3090,
        .range_end = 3112,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 3113,
        .range_end = 3113,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 3114,
        .range_end = 3129,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 3130,
        .range_end = 3131,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 3132,
        .range_end = 3132,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 3133,
        .range_end = 3133,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 3134,
        .range_end = 3136,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 3137,
        .range_end = 3140,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 3141,
        .range_end = 3141,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 3142,
        .range_end = 3144,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 3145,
        .range_end = 3145,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 3146,
        .range_end = 3149,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 3150,
        .range_end = 3156,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 3157,
        .range_end = 3158,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 3159,
        .range_end = 3159,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 3160,
        .range_end = 3162,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 3163,
        .range_end = 3164,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 3165,
        .range_end = 3165,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 3166,
        .range_end = 3167,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 3168,
        .range_end = 3169,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 3170,
        .range_end = 3171,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 3172,
        .range_end = 3173,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 3174,
        .range_end = 3183,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 3184,
        .range_end = 3190,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 3191,
        .range_end = 3200,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 3201,
        .range_end = 3201,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 3202,
        .range_end = 3212,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 3213,
        .range_end = 3213,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 3214,
        .range_end = 3216,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 3217,
        .range_end = 3217,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 3218,
        .range_end = 3240,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 3241,
        .range_end = 3241,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 3242,
        .range_end = 3251,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 3252,
        .range_end = 3252,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 3253,
        .range_end = 3257,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 3258,
        .range_end = 3259,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 3260,
        .range_end = 3260,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 3261,
        .range_end = 3262,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 3263,
        .range_end = 3263,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 3264,
        .range_end = 3268,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 3269,
        .range_end = 3269,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 3270,
        .range_end = 3270,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 3271,
        .range_end = 3272,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 3273,
        .range_end = 3273,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 3274,
        .range_end = 3275,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 3276,
        .range_end = 3277,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 3278,
        .range_end = 3284,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 3285,
        .range_end = 3286,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 3287,
        .range_end = 3292,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 3293,
        .range_end = 3294,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 3295,
        .range_end = 3295,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 3296,
        .range_end = 3297,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 3298,
        .range_end = 3299,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 3300,
        .range_end = 3301,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 3302,
        .range_end = 3311,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 3312,
        .range_end = 3312,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 3313,
        .range_end = 3315,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 3316,
        .range_end = 3327,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 3328,
        .range_end = 3329,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 3330,
        .range_end = 3340,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 3341,
        .range_end = 3341,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 3342,
        .range_end = 3344,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 3345,
        .range_end = 3345,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 3346,
        .range_end = 3386,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 3387,
        .range_end = 3388,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 3389,
        .range_end = 3392,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 3393,
        .range_end = 3396,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 3397,
        .range_end = 3397,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 3398,
        .range_end = 3400,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 3401,
        .range_end = 3401,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 3402,
        .range_end = 3404,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 3405,
        .range_end = 3405,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 3406,
        .range_end = 3407,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 3408,
        .range_end = 3411,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 3412,
        .range_end = 3425,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 3426,
        .range_end = 3427,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 3428,
        .range_end = 3429,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 3430,
        .range_end = 3455,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 3456,
        .range_end = 3456,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 3457,
        .range_end = 3457,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 3458,
        .range_end = 3459,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 3460,
        .range_end = 3460,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 3461,
        .range_end = 3478,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 3479,
        .range_end = 3481,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 3482,
        .range_end = 3505,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 3506,
        .range_end = 3506,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 3507,
        .range_end = 3515,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 3516,
        .range_end = 3516,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 3517,
        .range_end = 3517,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 3518,
        .range_end = 3519,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 3520,
        .range_end = 3526,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 3527,
        .range_end = 3529,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 3530,
        .range_end = 3530,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 3531,
        .range_end = 3534,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 3535,
        .range_end = 3537,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 3538,
        .range_end = 3540,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 3541,
        .range_end = 3541,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 3542,
        .range_end = 3542,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 3543,
        .range_end = 3543,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 3544,
        .range_end = 3551,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 3552,
        .range_end = 3557,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 3558,
        .range_end = 3567,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 3568,
        .range_end = 3569,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 3570,
        .range_end = 3572,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 3573,
        .range_end = 3584,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 3585,
        .range_end = 3632,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 3633,
        .range_end = 3633,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 3634,
        .range_end = 3635,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 3636,
        .range_end = 3642,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 3643,
        .range_end = 3646,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 3647,
        .range_end = 3654,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 3655,
        .range_end = 3662,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 3663,
        .range_end = 3675,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 3676,
        .range_end = 3712,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 3713,
        .range_end = 3714,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 3715,
        .range_end = 3715,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 3716,
        .range_end = 3716,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 3717,
        .range_end = 3717,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 3718,
        .range_end = 3722,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 3723,
        .range_end = 3723,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 3724,
        .range_end = 3747,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 3748,
        .range_end = 3748,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 3749,
        .range_end = 3749,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 3750,
        .range_end = 3750,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 3751,
        .range_end = 3760,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 3761,
        .range_end = 3761,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 3762,
        .range_end = 3763,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 3764,
        .range_end = 3772,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 3773,
        .range_end = 3773,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 3774,
        .range_end = 3775,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 3776,
        .range_end = 3780,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 3781,
        .range_end = 3781,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 3782,
        .range_end = 3782,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 3783,
        .range_end = 3783,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 3784,
        .range_end = 3790,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 3791,
        .range_end = 3791,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 3792,
        .range_end = 3801,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 3802,
        .range_end = 3803,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 3804,
        .range_end = 3807,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 3808,
        .range_end = 3839,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 3840,
        .range_end = 3863,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 3864,
        .range_end = 3865,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 3866,
        .range_end = 3892,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 3893,
        .range_end = 3893,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 3894,
        .range_end = 3894,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 3895,
        .range_end = 3895,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 3896,
        .range_end = 3896,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 3897,
        .range_end = 3897,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 3898,
        .range_end = 3911,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 3912,
        .range_end = 3912,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 3913,
        .range_end = 3948,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 3949,
        .range_end = 3952,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 3953,
        .range_end = 3966,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 3967,
        .range_end = 3967,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 3968,
        .range_end = 3972,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 3973,
        .range_end = 3973,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 3974,
        .range_end = 3975,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 3976,
        .range_end = 3980,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 3981,
        .range_end = 3991,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 3992,
        .range_end = 3992,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 3993,
        .range_end = 4028,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 4029,
        .range_end = 4029,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 4030,
        .range_end = 4037,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 4038,
        .range_end = 4038,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 4039,
        .range_end = 4044,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 4045,
        .range_end = 4045,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 4046,
        .range_end = 4058,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 4059,
        .range_end = 4095,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 4096,
        .range_end = 4140,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 4141,
        .range_end = 4144,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 4145,
        .range_end = 4145,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 4146,
        .range_end = 4151,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 4152,
        .range_end = 4152,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 4153,
        .range_end = 4154,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 4155,
        .range_end = 4156,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 4157,
        .range_end = 4158,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 4159,
        .range_end = 4183,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 4184,
        .range_end = 4185,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 4186,
        .range_end = 4189,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 4190,
        .range_end = 4192,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 4193,
        .range_end = 4208,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 4209,
        .range_end = 4212,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 4213,
        .range_end = 4225,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 4226,
        .range_end = 4226,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 4227,
        .range_end = 4228,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 4229,
        .range_end = 4230,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 4231,
        .range_end = 4236,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 4237,
        .range_end = 4237,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 4238,
        .range_end = 4252,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 4253,
        .range_end = 4253,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 4254,
        .range_end = 4293,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 4294,
        .range_end = 4294,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 4295,
        .range_end = 4295,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 4296,
        .range_end = 4300,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 4301,
        .range_end = 4301,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 4302,
        .range_end = 4303,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 4304,
        .range_end = 4351,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 4352,
        .range_end = 4447,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 4448,
        .range_end = 4607,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 4608,
        .range_end = 4680,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 4681,
        .range_end = 4681,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 4682,
        .range_end = 4685,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 4686,
        .range_end = 4687,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 4688,
        .range_end = 4694,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 4695,
        .range_end = 4695,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 4696,
        .range_end = 4696,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 4697,
        .range_end = 4697,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 4698,
        .range_end = 4701,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 4702,
        .range_end = 4703,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 4704,
        .range_end = 4744,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 4745,
        .range_end = 4745,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 4746,
        .range_end = 4749,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 4750,
        .range_end = 4751,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 4752,
        .range_end = 4784,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 4785,
        .range_end = 4785,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 4786,
        .range_end = 4789,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 4790,
        .range_end = 4791,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 4792,
        .range_end = 4798,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 4799,
        .range_end = 4799,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 4800,
        .range_end = 4800,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 4801,
        .range_end = 4801,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 4802,
        .range_end = 4805,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 4806,
        .range_end = 4807,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 4808,
        .range_end = 4822,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 4823,
        .range_end = 4823,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 4824,
        .range_end = 4880,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 4881,
        .range_end = 4881,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 4882,
        .range_end = 4885,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 4886,
        .range_end = 4887,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 4888,
        .range_end = 4954,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 4955,
        .range_end = 4956,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 4957,
        .range_end = 4959,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 4960,
        .range_end = 4988,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 4989,
        .range_end = 4991,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 4992,
        .range_end = 5017,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 5018,
        .range_end = 5023,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 5024,
        .range_end = 5109,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 5110,
        .range_end = 5111,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 5112,
        .range_end = 5117,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 5118,
        .range_end = 5119,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 5120,
        .range_end = 5788,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 5789,
        .range_end = 5791,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 5792,
        .range_end = 5880,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 5881,
        .range_end = 5887,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 5888,
        .range_end = 5905,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 5906,
        .range_end = 5908,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 5909,
        .range_end = 5909,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 5910,
        .range_end = 5918,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 5919,
        .range_end = 5937,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 5938,
        .range_end = 5939,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 5940,
        .range_end = 5942,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 5943,
        .range_end = 5951,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 5952,
        .range_end = 5969,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 5970,
        .range_end = 5971,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 5972,
        .range_end = 5983,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 5984,
        .range_end = 5996,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 5997,
        .range_end = 5997,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 5998,
        .range_end = 6000,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 6001,
        .range_end = 6001,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 6002,
        .range_end = 6003,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 6004,
        .range_end = 6015,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 6016,
        .range_end = 6067,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 6068,
        .range_end = 6069,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 6070,
        .range_end = 6070,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 6071,
        .range_end = 6077,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 6078,
        .range_end = 6085,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 6086,
        .range_end = 6086,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 6087,
        .range_end = 6088,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 6089,
        .range_end = 6099,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 6100,
        .range_end = 6108,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 6109,
        .range_end = 6109,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 6110,
        .range_end = 6111,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 6112,
        .range_end = 6121,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 6122,
        .range_end = 6127,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 6128,
        .range_end = 6137,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 6138,
        .range_end = 6143,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 6144,
        .range_end = 6154,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 6155,
        .range_end = 6159,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 6160,
        .range_end = 6169,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 6170,
        .range_end = 6175,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 6176,
        .range_end = 6264,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 6265,
        .range_end = 6271,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 6272,
        .range_end = 6276,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 6277,
        .range_end = 6278,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 6279,
        .range_end = 6312,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 6313,
        .range_end = 6313,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 6314,
        .range_end = 6314,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 6315,
        .range_end = 6319,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 6320,
        .range_end = 6389,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 6390,
        .range_end = 6399,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 6400,
        .range_end = 6430,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 6431,
        .range_end = 6431,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 6432,
        .range_end = 6434,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 6435,
        .range_end = 6438,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 6439,
        .range_end = 6440,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 6441,
        .range_end = 6443,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 6444,
        .range_end = 6447,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 6448,
        .range_end = 6449,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 6450,
        .range_end = 6450,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 6451,
        .range_end = 6456,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 6457,
        .range_end = 6459,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 6460,
        .range_end = 6463,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 6464,
        .range_end = 6464,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 6465,
        .range_end = 6467,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 6468,
        .range_end = 6509,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 6510,
        .range_end = 6511,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 6512,
        .range_end = 6516,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 6517,
        .range_end = 6527,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 6528,
        .range_end = 6571,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 6572,
        .range_end = 6575,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 6576,
        .range_end = 6601,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 6602,
        .range_end = 6607,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 6608,
        .range_end = 6618,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 6619,
        .range_end = 6621,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 6622,
        .range_end = 6678,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 6679,
        .range_end = 6680,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 6681,
        .range_end = 6682,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 6683,
        .range_end = 6683,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 6684,
        .range_end = 6685,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 6686,
        .range_end = 6741,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 6742,
        .range_end = 6742,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 6743,
        .range_end = 6743,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 6744,
        .range_end = 6750,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 6751,
        .range_end = 6751,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 6752,
        .range_end = 6752,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 6753,
        .range_end = 6753,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 6754,
        .range_end = 6754,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 6755,
        .range_end = 6756,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 6757,
        .range_end = 6764,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 6765,
        .range_end = 6770,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 6771,
        .range_end = 6780,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 6781,
        .range_end = 6782,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 6783,
        .range_end = 6783,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 6784,
        .range_end = 6793,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 6794,
        .range_end = 6799,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 6800,
        .range_end = 6809,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 6810,
        .range_end = 6815,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 6816,
        .range_end = 6829,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 6830,
        .range_end = 6831,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 6832,
        .range_end = 6862,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 6863,
        .range_end = 6911,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 6912,
        .range_end = 6915,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 6916,
        .range_end = 6963,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 6964,
        .range_end = 6964,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 6965,
        .range_end = 6965,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 6966,
        .range_end = 6970,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 6971,
        .range_end = 6971,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 6972,
        .range_end = 6972,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 6973,
        .range_end = 6977,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 6978,
        .range_end = 6978,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 6979,
        .range_end = 6988,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 6989,
        .range_end = 6989,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 6990,
        .range_end = 7018,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 7019,
        .range_end = 7027,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 7028,
        .range_end = 7039,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 7040,
        .range_end = 7041,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 7042,
        .range_end = 7073,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 7074,
        .range_end = 7077,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 7078,
        .range_end = 7079,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 7080,
        .range_end = 7081,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 7082,
        .range_end = 7082,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 7083,
        .range_end = 7085,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 7086,
        .range_end = 7141,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 7142,
        .range_end = 7142,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 7143,
        .range_end = 7143,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 7144,
        .range_end = 7145,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 7146,
        .range_end = 7148,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 7149,
        .range_end = 7149,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 7150,
        .range_end = 7150,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 7151,
        .range_end = 7153,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 7154,
        .range_end = 7155,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 7156,
        .range_end = 7163,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 7164,
        .range_end = 7211,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 7212,
        .range_end = 7219,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 7220,
        .range_end = 7221,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 7222,
        .range_end = 7223,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 7224,
        .range_end = 7226,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 7227,
        .range_end = 7241,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 7242,
        .range_end = 7244,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 7245,
        .range_end = 7306,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 7307,
        .range_end = 7311,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 7312,
        .range_end = 7354,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 7355,
        .range_end = 7356,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 7357,
        .range_end = 7367,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 7368,
        .range_end = 7375,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 7376,
        .range_end = 7378,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 7379,
        .range_end = 7379,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 7380,
        .range_end = 7392,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 7393,
        .range_end = 7393,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 7394,
        .range_end = 7400,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 7401,
        .range_end = 7404,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 7405,
        .range_end = 7405,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 7406,
        .range_end = 7411,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 7412,
        .range_end = 7412,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 7413,
        .range_end = 7415,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 7416,
        .range_end = 7417,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 7418,
        .range_end = 7418,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 7419,
        .range_end = 7423,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 7424,
        .range_end = 7615,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 7616,
        .range_end = 7679,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 7680,
        .range_end = 7957,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 7958,
        .range_end = 7959,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 7960,
        .range_end = 7965,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 7966,
        .range_end = 7967,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 7968,
        .range_end = 8005,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 8006,
        .range_end = 8007,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 8008,
        .range_end = 8013,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 8014,
        .range_end = 8015,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 8016,
        .range_end = 8023,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 8024,
        .range_end = 8024,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 8025,
        .range_end = 8025,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 8026,
        .range_end = 8026,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 8027,
        .range_end = 8027,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 8028,
        .range_end = 8028,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 8029,
        .range_end = 8029,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 8030,
        .range_end = 8030,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 8031,
        .range_end = 8061,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 8062,
        .range_end = 8063,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 8064,
        .range_end = 8116,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 8117,
        .range_end = 8117,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 8118,
        .range_end = 8132,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 8133,
        .range_end = 8133,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 8134,
        .range_end = 8147,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 8148,
        .range_end = 8149,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 8150,
        .range_end = 8155,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 8156,
        .range_end = 8156,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 8157,
        .range_end = 8175,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 8176,
        .range_end = 8177,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 8178,
        .range_end = 8180,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 8181,
        .range_end = 8181,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 8182,
        .range_end = 8190,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 8191,
        .range_end = 8191,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 8192,
        .range_end = 8202,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 8203,
        .range_end = 8207,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 8208,
        .range_end = 8231,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 8232,
        .range_end = 8233,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 8234,
        .range_end = 8238,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 8239,
        .range_end = 8287,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 8288,
        .range_end = 8292,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 8293,
        .range_end = 8293,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 8294,
        .range_end = 8303,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 8304,
        .range_end = 8305,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 8306,
        .range_end = 8307,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 8308,
        .range_end = 8334,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 8335,
        .range_end = 8335,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 8336,
        .range_end = 8348,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 8349,
        .range_end = 8351,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 8352,
        .range_end = 8384,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 8385,
        .range_end = 8399,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 8400,
        .range_end = 8432,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 8433,
        .range_end = 8447,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 8448,
        .range_end = 8587,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 8588,
        .range_end = 8591,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 8592,
        .range_end = 8985,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 8986,
        .range_end = 8987,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 8988,
        .range_end = 9000,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 9001,
        .range_end = 9002,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 9003,
        .range_end = 9192,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 9193,
        .range_end = 9196,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 9197,
        .range_end = 9199,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 9200,
        .range_end = 9200,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 9201,
        .range_end = 9202,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 9203,
        .range_end = 9203,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 9204,
        .range_end = 9257,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 9258,
        .range_end = 9279,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 9280,
        .range_end = 9290,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 9291,
        .range_end = 9311,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 9312,
        .range_end = 9724,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 9725,
        .range_end = 9726,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 9727,
        .range_end = 9747,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 9748,
        .range_end = 9749,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 9750,
        .range_end = 9775,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 9776,
        .range_end = 9783,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 9784,
        .range_end = 9799,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 9800,
        .range_end = 9811,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 9812,
        .range_end = 9854,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 9855,
        .range_end = 9855,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 9856,
        .range_end = 9865,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 9866,
        .range_end = 9871,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 9872,
        .range_end = 9874,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 9875,
        .range_end = 9875,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 9876,
        .range_end = 9888,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 9889,
        .range_end = 9889,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 9890,
        .range_end = 9897,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 9898,
        .range_end = 9899,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 9900,
        .range_end = 9916,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 9917,
        .range_end = 9918,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 9919,
        .range_end = 9923,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 9924,
        .range_end = 9925,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 9926,
        .range_end = 9933,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 9934,
        .range_end = 9934,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 9935,
        .range_end = 9939,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 9940,
        .range_end = 9940,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 9941,
        .range_end = 9961,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 9962,
        .range_end = 9962,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 9963,
        .range_end = 9969,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 9970,
        .range_end = 9971,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 9972,
        .range_end = 9972,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 9973,
        .range_end = 9973,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 9974,
        .range_end = 9977,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 9978,
        .range_end = 9978,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 9979,
        .range_end = 9980,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 9981,
        .range_end = 9981,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 9982,
        .range_end = 9988,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 9989,
        .range_end = 9989,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 9990,
        .range_end = 9993,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 9994,
        .range_end = 9995,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 9996,
        .range_end = 10023,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 10024,
        .range_end = 10024,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 10025,
        .range_end = 10059,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 10060,
        .range_end = 10060,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 10061,
        .range_end = 10061,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 10062,
        .range_end = 10062,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 10063,
        .range_end = 10066,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 10067,
        .range_end = 10069,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 10070,
        .range_end = 10070,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 10071,
        .range_end = 10071,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 10072,
        .range_end = 10132,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 10133,
        .range_end = 10135,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 10136,
        .range_end = 10159,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 10160,
        .range_end = 10160,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 10161,
        .range_end = 10174,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 10175,
        .range_end = 10175,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 10176,
        .range_end = 11034,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 11035,
        .range_end = 11036,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 11037,
        .range_end = 11087,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 11088,
        .range_end = 11088,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 11089,
        .range_end = 11092,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 11093,
        .range_end = 11093,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 11094,
        .range_end = 11123,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 11124,
        .range_end = 11125,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 11126,
        .range_end = 11157,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 11158,
        .range_end = 11158,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 11159,
        .range_end = 11502,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 11503,
        .range_end = 11505,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 11506,
        .range_end = 11507,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 11508,
        .range_end = 11512,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 11513,
        .range_end = 11557,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 11558,
        .range_end = 11558,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 11559,
        .range_end = 11559,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 11560,
        .range_end = 11564,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 11565,
        .range_end = 11565,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 11566,
        .range_end = 11567,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 11568,
        .range_end = 11623,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 11624,
        .range_end = 11630,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 11631,
        .range_end = 11632,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 11633,
        .range_end = 11646,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 11647,
        .range_end = 11647,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 11648,
        .range_end = 11670,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 11671,
        .range_end = 11679,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 11680,
        .range_end = 11686,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 11687,
        .range_end = 11687,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 11688,
        .range_end = 11694,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 11695,
        .range_end = 11695,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 11696,
        .range_end = 11702,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 11703,
        .range_end = 11703,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 11704,
        .range_end = 11710,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 11711,
        .range_end = 11711,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 11712,
        .range_end = 11718,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 11719,
        .range_end = 11719,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 11720,
        .range_end = 11726,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 11727,
        .range_end = 11727,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 11728,
        .range_end = 11734,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 11735,
        .range_end = 11735,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 11736,
        .range_end = 11742,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 11743,
        .range_end = 11743,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 11744,
        .range_end = 11775,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 11776,
        .range_end = 11869,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 11870,
        .range_end = 11903,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 11904,
        .range_end = 11929,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 11930,
        .range_end = 11930,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 11931,
        .range_end = 12019,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 12020,
        .range_end = 12031,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 12032,
        .range_end = 12245,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 12246,
        .range_end = 12271,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 12272,
        .range_end = 12329,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 12330,
        .range_end = 12333,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 12334,
        .range_end = 12350,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 12351,
        .range_end = 12351,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 12352,
        .range_end = 12352,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 12353,
        .range_end = 12438,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 12439,
        .range_end = 12440,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 12441,
        .range_end = 12442,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 12443,
        .range_end = 12543,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 12544,
        .range_end = 12548,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 12549,
        .range_end = 12591,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 12592,
        .range_end = 12592,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 12593,
        .range_end = 12643,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 12644,
        .range_end = 12644,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 12645,
        .range_end = 12686,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 12687,
        .range_end = 12687,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 12688,
        .range_end = 12773,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 12774,
        .range_end = 12782,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 12783,
        .range_end = 12830,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 12831,
        .range_end = 12831,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 12832,
        .range_end = 42124,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 42125,
        .range_end = 42127,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 42128,
        .range_end = 42182,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 42183,
        .range_end = 42191,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 42192,
        .range_end = 42539,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 42540,
        .range_end = 42559,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 42560,
        .range_end = 42606,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 42607,
        .range_end = 42610,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 42611,
        .range_end = 42611,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 42612,
        .range_end = 42621,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 42622,
        .range_end = 42653,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 42654,
        .range_end = 42655,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 42656,
        .range_end = 42735,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 42736,
        .range_end = 42737,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 42738,
        .range_end = 42743,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 42744,
        .range_end = 42751,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 42752,
        .range_end = 42957,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 42958,
        .range_end = 42959,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 42960,
        .range_end = 42961,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 42962,
        .range_end = 42962,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 42963,
        .range_end = 42963,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 42964,
        .range_end = 42964,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 42965,
        .range_end = 42972,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 42973,
        .range_end = 42993,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 42994,
        .range_end = 43009,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 43010,
        .range_end = 43010,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 43011,
        .range_end = 43013,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 43014,
        .range_end = 43014,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 43015,
        .range_end = 43018,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 43019,
        .range_end = 43019,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 43020,
        .range_end = 43044,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 43045,
        .range_end = 43046,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 43047,
        .range_end = 43051,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 43052,
        .range_end = 43052,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 43053,
        .range_end = 43055,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 43056,
        .range_end = 43065,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 43066,
        .range_end = 43071,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 43072,
        .range_end = 43127,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 43128,
        .range_end = 43135,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 43136,
        .range_end = 43203,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 43204,
        .range_end = 43205,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 43206,
        .range_end = 43213,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 43214,
        .range_end = 43225,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 43226,
        .range_end = 43231,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 43232,
        .range_end = 43249,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 43250,
        .range_end = 43262,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 43263,
        .range_end = 43263,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 43264,
        .range_end = 43301,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 43302,
        .range_end = 43309,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 43310,
        .range_end = 43334,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 43335,
        .range_end = 43345,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 43346,
        .range_end = 43347,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 43348,
        .range_end = 43358,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 43359,
        .range_end = 43359,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 43360,
        .range_end = 43388,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 43389,
        .range_end = 43391,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 43392,
        .range_end = 43394,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 43395,
        .range_end = 43442,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 43443,
        .range_end = 43443,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 43444,
        .range_end = 43445,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 43446,
        .range_end = 43449,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 43450,
        .range_end = 43451,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 43452,
        .range_end = 43453,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 43454,
        .range_end = 43469,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 43470,
        .range_end = 43470,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 43471,
        .range_end = 43481,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 43482,
        .range_end = 43485,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 43486,
        .range_end = 43492,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 43493,
        .range_end = 43493,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 43494,
        .range_end = 43518,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 43519,
        .range_end = 43519,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 43520,
        .range_end = 43560,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 43561,
        .range_end = 43566,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 43567,
        .range_end = 43568,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 43569,
        .range_end = 43570,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 43571,
        .range_end = 43572,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 43573,
        .range_end = 43574,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 43575,
        .range_end = 43583,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 43584,
        .range_end = 43586,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 43587,
        .range_end = 43587,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 43588,
        .range_end = 43595,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 43596,
        .range_end = 43596,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 43597,
        .range_end = 43597,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 43598,
        .range_end = 43599,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 43600,
        .range_end = 43609,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 43610,
        .range_end = 43611,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 43612,
        .range_end = 43643,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 43644,
        .range_end = 43644,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 43645,
        .range_end = 43695,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 43696,
        .range_end = 43696,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 43697,
        .range_end = 43697,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 43698,
        .range_end = 43700,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 43701,
        .range_end = 43702,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 43703,
        .range_end = 43704,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 43705,
        .range_end = 43709,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 43710,
        .range_end = 43711,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 43712,
        .range_end = 43712,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 43713,
        .range_end = 43713,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 43714,
        .range_end = 43714,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 43715,
        .range_end = 43738,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 43739,
        .range_end = 43755,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 43756,
        .range_end = 43757,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 43758,
        .range_end = 43765,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 43766,
        .range_end = 43766,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 43767,
        .range_end = 43776,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 43777,
        .range_end = 43782,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 43783,
        .range_end = 43784,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 43785,
        .range_end = 43790,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 43791,
        .range_end = 43792,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 43793,
        .range_end = 43798,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 43799,
        .range_end = 43807,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 43808,
        .range_end = 43814,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 43815,
        .range_end = 43815,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 43816,
        .range_end = 43822,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 43823,
        .range_end = 43823,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 43824,
        .range_end = 43883,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 43884,
        .range_end = 43887,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 43888,
        .range_end = 44004,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 44005,
        .range_end = 44005,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 44006,
        .range_end = 44007,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 44008,
        .range_end = 44008,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 44009,
        .range_end = 44012,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 44013,
        .range_end = 44013,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 44014,
        .range_end = 44015,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 44016,
        .range_end = 44025,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 44026,
        .range_end = 44031,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 44032,
        .range_end = 55203,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 55204,
        .range_end = 55215,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 55216,
        .range_end = 55238,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 55239,
        .range_end = 55242,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 55243,
        .range_end = 55291,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 55292,
        .range_end = 57343,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 57344,
        .range_end = 63743,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 63744,
        .range_end = 64109,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 64110,
        .range_end = 64111,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 64112,
        .range_end = 64217,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 64218,
        .range_end = 64255,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 64256,
        .range_end = 64262,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 64263,
        .range_end = 64274,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 64275,
        .range_end = 64279,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 64280,
        .range_end = 64284,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 64285,
        .range_end = 64285,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 64286,
        .range_end = 64286,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 64287,
        .range_end = 64310,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 64311,
        .range_end = 64311,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 64312,
        .range_end = 64316,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 64317,
        .range_end = 64317,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 64318,
        .range_end = 64318,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 64319,
        .range_end = 64319,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 64320,
        .range_end = 64321,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 64322,
        .range_end = 64322,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 64323,
        .range_end = 64324,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 64325,
        .range_end = 64325,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 64326,
        .range_end = 64450,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 64451,
        .range_end = 64466,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 64467,
        .range_end = 64911,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 64912,
        .range_end = 64913,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 64914,
        .range_end = 64967,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 64968,
        .range_end = 64974,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 64975,
        .range_end = 64975,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 64976,
        .range_end = 65007,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 65008,
        .range_end = 65023,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 65024,
        .range_end = 65039,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 65040,
        .range_end = 65049,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 65050,
        .range_end = 65055,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 65056,
        .range_end = 65071,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 65072,
        .range_end = 65106,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 65107,
        .range_end = 65107,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 65108,
        .range_end = 65126,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 65127,
        .range_end = 65127,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 65128,
        .range_end = 65131,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 65132,
        .range_end = 65135,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 65136,
        .range_end = 65140,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 65141,
        .range_end = 65141,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 65142,
        .range_end = 65276,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 65277,
        .range_end = 65278,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 65279,
        .range_end = 65279,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 65280,
        .range_end = 65280,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 65281,
        .range_end = 65376,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 65377,
        .range_end = 65439,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 65440,
        .range_end = 65440,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 65441,
        .range_end = 65470,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 65471,
        .range_end = 65473,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 65474,
        .range_end = 65479,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 65480,
        .range_end = 65481,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 65482,
        .range_end = 65487,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 65488,
        .range_end = 65489,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 65490,
        .range_end = 65495,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 65496,
        .range_end = 65497,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 65498,
        .range_end = 65500,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 65501,
        .range_end = 65503,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 65504,
        .range_end = 65510,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 65511,
        .range_end = 65511,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 65512,
        .range_end = 65518,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 65519,
        .range_end = 65528,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 65529,
        .range_end = 65533,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 65534,
        .range_end = 65535,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 65536,
        .range_end = 65547,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 65548,
        .range_end = 65548,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 65549,
        .range_end = 65574,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 65575,
        .range_end = 65575,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 65576,
        .range_end = 65594,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 65595,
        .range_end = 65595,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 65596,
        .range_end = 65597,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 65598,
        .range_end = 65598,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 65599,
        .range_end = 65613,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 65614,
        .range_end = 65615,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 65616,
        .range_end = 65629,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 65630,
        .range_end = 65663,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 65664,
        .range_end = 65786,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 65787,
        .range_end = 65791,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 65792,
        .range_end = 65794,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 65795,
        .range_end = 65798,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 65799,
        .range_end = 65843,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 65844,
        .range_end = 65846,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 65847,
        .range_end = 65934,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 65935,
        .range_end = 65935,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 65936,
        .range_end = 65948,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 65949,
        .range_end = 65951,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 65952,
        .range_end = 65952,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 65953,
        .range_end = 65999,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 66000,
        .range_end = 66044,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 66045,
        .range_end = 66045,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 66046,
        .range_end = 66175,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 66176,
        .range_end = 66204,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 66205,
        .range_end = 66207,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 66208,
        .range_end = 66256,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 66257,
        .range_end = 66271,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 66272,
        .range_end = 66272,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 66273,
        .range_end = 66299,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 66300,
        .range_end = 66303,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 66304,
        .range_end = 66339,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 66340,
        .range_end = 66348,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 66349,
        .range_end = 66378,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 66379,
        .range_end = 66383,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 66384,
        .range_end = 66421,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 66422,
        .range_end = 66426,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 66427,
        .range_end = 66431,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 66432,
        .range_end = 66461,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 66462,
        .range_end = 66462,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 66463,
        .range_end = 66499,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 66500,
        .range_end = 66503,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 66504,
        .range_end = 66517,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 66518,
        .range_end = 66559,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 66560,
        .range_end = 66717,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 66718,
        .range_end = 66719,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 66720,
        .range_end = 66729,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 66730,
        .range_end = 66735,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 66736,
        .range_end = 66771,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 66772,
        .range_end = 66775,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 66776,
        .range_end = 66811,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 66812,
        .range_end = 66815,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 66816,
        .range_end = 66855,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 66856,
        .range_end = 66863,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 66864,
        .range_end = 66915,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 66916,
        .range_end = 66926,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 66927,
        .range_end = 66938,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 66939,
        .range_end = 66939,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 66940,
        .range_end = 66954,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 66955,
        .range_end = 66955,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 66956,
        .range_end = 66962,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 66963,
        .range_end = 66963,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 66964,
        .range_end = 66965,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 66966,
        .range_end = 66966,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 66967,
        .range_end = 66977,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 66978,
        .range_end = 66978,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 66979,
        .range_end = 66993,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 66994,
        .range_end = 66994,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 66995,
        .range_end = 67001,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 67002,
        .range_end = 67002,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 67003,
        .range_end = 67004,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 67005,
        .range_end = 67007,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 67008,
        .range_end = 67059,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 67060,
        .range_end = 67071,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 67072,
        .range_end = 67382,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 67383,
        .range_end = 67391,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 67392,
        .range_end = 67413,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 67414,
        .range_end = 67423,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 67424,
        .range_end = 67431,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 67432,
        .range_end = 67455,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 67456,
        .range_end = 67461,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 67462,
        .range_end = 67462,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 67463,
        .range_end = 67504,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 67505,
        .range_end = 67505,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 67506,
        .range_end = 67514,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 67515,
        .range_end = 67583,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 67584,
        .range_end = 67589,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 67590,
        .range_end = 67591,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 67592,
        .range_end = 67592,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 67593,
        .range_end = 67593,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 67594,
        .range_end = 67637,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 67638,
        .range_end = 67638,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 67639,
        .range_end = 67640,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 67641,
        .range_end = 67643,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 67644,
        .range_end = 67644,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 67645,
        .range_end = 67646,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 67647,
        .range_end = 67669,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 67670,
        .range_end = 67670,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 67671,
        .range_end = 67742,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 67743,
        .range_end = 67750,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 67751,
        .range_end = 67759,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 67760,
        .range_end = 67807,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 67808,
        .range_end = 67826,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 67827,
        .range_end = 67827,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 67828,
        .range_end = 67829,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 67830,
        .range_end = 67834,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 67835,
        .range_end = 67867,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 67868,
        .range_end = 67870,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 67871,
        .range_end = 67897,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 67898,
        .range_end = 67902,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 67903,
        .range_end = 67903,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 67904,
        .range_end = 67967,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 67968,
        .range_end = 68023,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 68024,
        .range_end = 68027,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 68028,
        .range_end = 68047,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 68048,
        .range_end = 68049,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 68050,
        .range_end = 68096,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 68097,
        .range_end = 68099,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 68100,
        .range_end = 68100,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 68101,
        .range_end = 68102,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 68103,
        .range_end = 68107,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 68108,
        .range_end = 68111,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 68112,
        .range_end = 68115,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 68116,
        .range_end = 68116,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 68117,
        .range_end = 68119,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 68120,
        .range_end = 68120,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 68121,
        .range_end = 68149,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 68150,
        .range_end = 68151,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 68152,
        .range_end = 68154,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 68155,
        .range_end = 68158,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 68159,
        .range_end = 68159,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 68160,
        .range_end = 68168,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 68169,
        .range_end = 68175,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 68176,
        .range_end = 68184,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 68185,
        .range_end = 68191,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 68192,
        .range_end = 68255,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 68256,
        .range_end = 68287,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 68288,
        .range_end = 68324,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 68325,
        .range_end = 68326,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 68327,
        .range_end = 68330,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 68331,
        .range_end = 68342,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 68343,
        .range_end = 68351,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 68352,
        .range_end = 68405,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 68406,
        .range_end = 68408,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 68409,
        .range_end = 68437,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 68438,
        .range_end = 68439,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 68440,
        .range_end = 68466,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 68467,
        .range_end = 68471,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 68472,
        .range_end = 68497,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 68498,
        .range_end = 68504,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 68505,
        .range_end = 68508,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 68509,
        .range_end = 68520,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 68521,
        .range_end = 68527,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 68528,
        .range_end = 68607,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 68608,
        .range_end = 68680,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 68681,
        .range_end = 68735,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 68736,
        .range_end = 68786,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 68787,
        .range_end = 68799,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 68800,
        .range_end = 68850,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 68851,
        .range_end = 68857,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 68858,
        .range_end = 68899,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 68900,
        .range_end = 68903,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 68904,
        .range_end = 68911,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 68912,
        .range_end = 68921,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 68922,
        .range_end = 68927,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 68928,
        .range_end = 68965,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 68966,
        .range_end = 68968,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 68969,
        .range_end = 68973,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 68974,
        .range_end = 68997,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 68998,
        .range_end = 69005,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 69006,
        .range_end = 69007,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 69008,
        .range_end = 69215,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 69216,
        .range_end = 69246,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 69247,
        .range_end = 69247,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 69248,
        .range_end = 69289,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 69290,
        .range_end = 69290,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 69291,
        .range_end = 69292,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 69293,
        .range_end = 69293,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 69294,
        .range_end = 69295,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 69296,
        .range_end = 69297,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 69298,
        .range_end = 69313,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 69314,
        .range_end = 69316,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 69317,
        .range_end = 69371,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 69372,
        .range_end = 69375,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 69376,
        .range_end = 69415,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 69416,
        .range_end = 69423,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 69424,
        .range_end = 69445,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 69446,
        .range_end = 69456,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 69457,
        .range_end = 69465,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 69466,
        .range_end = 69487,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 69488,
        .range_end = 69505,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 69506,
        .range_end = 69509,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 69510,
        .range_end = 69513,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 69514,
        .range_end = 69551,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 69552,
        .range_end = 69579,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 69580,
        .range_end = 69599,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 69600,
        .range_end = 69622,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 69623,
        .range_end = 69631,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 69632,
        .range_end = 69632,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 69633,
        .range_end = 69633,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 69634,
        .range_end = 69687,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 69688,
        .range_end = 69702,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 69703,
        .range_end = 69709,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 69710,
        .range_end = 69713,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 69714,
        .range_end = 69743,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 69744,
        .range_end = 69744,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 69745,
        .range_end = 69746,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 69747,
        .range_end = 69748,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 69749,
        .range_end = 69749,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 69750,
        .range_end = 69758,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 69759,
        .range_end = 69761,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 69762,
        .range_end = 69810,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 69811,
        .range_end = 69814,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 69815,
        .range_end = 69816,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 69817,
        .range_end = 69818,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 69819,
        .range_end = 69825,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 69826,
        .range_end = 69826,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 69827,
        .range_end = 69836,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 69837,
        .range_end = 69837,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 69838,
        .range_end = 69839,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 69840,
        .range_end = 69864,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 69865,
        .range_end = 69871,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 69872,
        .range_end = 69881,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 69882,
        .range_end = 69887,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 69888,
        .range_end = 69890,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 69891,
        .range_end = 69926,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 69927,
        .range_end = 69931,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 69932,
        .range_end = 69932,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 69933,
        .range_end = 69940,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 69941,
        .range_end = 69941,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 69942,
        .range_end = 69959,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 69960,
        .range_end = 69967,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 69968,
        .range_end = 70002,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 70003,
        .range_end = 70003,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 70004,
        .range_end = 70006,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 70007,
        .range_end = 70015,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 70016,
        .range_end = 70017,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 70018,
        .range_end = 70069,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 70070,
        .range_end = 70078,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 70079,
        .range_end = 70088,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 70089,
        .range_end = 70092,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 70093,
        .range_end = 70094,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 70095,
        .range_end = 70095,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 70096,
        .range_end = 70111,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 70112,
        .range_end = 70112,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 70113,
        .range_end = 70132,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 70133,
        .range_end = 70143,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 70144,
        .range_end = 70161,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 70162,
        .range_end = 70162,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 70163,
        .range_end = 70190,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 70191,
        .range_end = 70193,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 70194,
        .range_end = 70195,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 70196,
        .range_end = 70196,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 70197,
        .range_end = 70197,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 70198,
        .range_end = 70199,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 70200,
        .range_end = 70205,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 70206,
        .range_end = 70206,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 70207,
        .range_end = 70208,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 70209,
        .range_end = 70209,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 70210,
        .range_end = 70271,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 70272,
        .range_end = 70278,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 70279,
        .range_end = 70279,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 70280,
        .range_end = 70280,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 70281,
        .range_end = 70281,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 70282,
        .range_end = 70285,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 70286,
        .range_end = 70286,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 70287,
        .range_end = 70301,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 70302,
        .range_end = 70302,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 70303,
        .range_end = 70313,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 70314,
        .range_end = 70319,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 70320,
        .range_end = 70366,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 70367,
        .range_end = 70367,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 70368,
        .range_end = 70370,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 70371,
        .range_end = 70378,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 70379,
        .range_end = 70383,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 70384,
        .range_end = 70393,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 70394,
        .range_end = 70399,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 70400,
        .range_end = 70401,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 70402,
        .range_end = 70403,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 70404,
        .range_end = 70404,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 70405,
        .range_end = 70412,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 70413,
        .range_end = 70414,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 70415,
        .range_end = 70416,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 70417,
        .range_end = 70418,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 70419,
        .range_end = 70440,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 70441,
        .range_end = 70441,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 70442,
        .range_end = 70448,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 70449,
        .range_end = 70449,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 70450,
        .range_end = 70451,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 70452,
        .range_end = 70452,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 70453,
        .range_end = 70457,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 70458,
        .range_end = 70458,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 70459,
        .range_end = 70460,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 70461,
        .range_end = 70463,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 70464,
        .range_end = 70464,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 70465,
        .range_end = 70468,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 70469,
        .range_end = 70470,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 70471,
        .range_end = 70472,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 70473,
        .range_end = 70474,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 70475,
        .range_end = 70477,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 70478,
        .range_end = 70479,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 70480,
        .range_end = 70480,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 70481,
        .range_end = 70486,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 70487,
        .range_end = 70487,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 70488,
        .range_end = 70492,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 70493,
        .range_end = 70499,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 70500,
        .range_end = 70501,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 70502,
        .range_end = 70508,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 70509,
        .range_end = 70511,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 70512,
        .range_end = 70516,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 70517,
        .range_end = 70527,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 70528,
        .range_end = 70537,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 70538,
        .range_end = 70538,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 70539,
        .range_end = 70539,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 70540,
        .range_end = 70541,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 70542,
        .range_end = 70542,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 70543,
        .range_end = 70543,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 70544,
        .range_end = 70581,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 70582,
        .range_end = 70582,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 70583,
        .range_end = 70586,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 70587,
        .range_end = 70592,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 70593,
        .range_end = 70593,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 70594,
        .range_end = 70594,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 70595,
        .range_end = 70596,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 70597,
        .range_end = 70597,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 70598,
        .range_end = 70598,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 70599,
        .range_end = 70602,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 70603,
        .range_end = 70603,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 70604,
        .range_end = 70605,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 70606,
        .range_end = 70606,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 70607,
        .range_end = 70607,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 70608,
        .range_end = 70608,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 70609,
        .range_end = 70609,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 70610,
        .range_end = 70610,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 70611,
        .range_end = 70613,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 70614,
        .range_end = 70614,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 70615,
        .range_end = 70616,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 70617,
        .range_end = 70624,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 70625,
        .range_end = 70626,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 70627,
        .range_end = 70655,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 70656,
        .range_end = 70711,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 70712,
        .range_end = 70719,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 70720,
        .range_end = 70721,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 70722,
        .range_end = 70724,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 70725,
        .range_end = 70725,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 70726,
        .range_end = 70726,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 70727,
        .range_end = 70747,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 70748,
        .range_end = 70748,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 70749,
        .range_end = 70749,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 70750,
        .range_end = 70750,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 70751,
        .range_end = 70753,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 70754,
        .range_end = 70783,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 70784,
        .range_end = 70834,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 70835,
        .range_end = 70840,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 70841,
        .range_end = 70841,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 70842,
        .range_end = 70842,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 70843,
        .range_end = 70846,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 70847,
        .range_end = 70848,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 70849,
        .range_end = 70849,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 70850,
        .range_end = 70851,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 70852,
        .range_end = 70855,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 70856,
        .range_end = 70863,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 70864,
        .range_end = 70873,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 70874,
        .range_end = 71039,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 71040,
        .range_end = 71089,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 71090,
        .range_end = 71093,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 71094,
        .range_end = 71095,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 71096,
        .range_end = 71099,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 71100,
        .range_end = 71101,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 71102,
        .range_end = 71102,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 71103,
        .range_end = 71104,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 71105,
        .range_end = 71131,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 71132,
        .range_end = 71133,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 71134,
        .range_end = 71167,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 71168,
        .range_end = 71218,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 71219,
        .range_end = 71226,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 71227,
        .range_end = 71228,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 71229,
        .range_end = 71229,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 71230,
        .range_end = 71230,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 71231,
        .range_end = 71232,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 71233,
        .range_end = 71236,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 71237,
        .range_end = 71247,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 71248,
        .range_end = 71257,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 71258,
        .range_end = 71263,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 71264,
        .range_end = 71276,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 71277,
        .range_end = 71295,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 71296,
        .range_end = 71338,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 71339,
        .range_end = 71339,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 71340,
        .range_end = 71340,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 71341,
        .range_end = 71341,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 71342,
        .range_end = 71343,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 71344,
        .range_end = 71349,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 71350,
        .range_end = 71350,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 71351,
        .range_end = 71351,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 71352,
        .range_end = 71353,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 71354,
        .range_end = 71359,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 71360,
        .range_end = 71369,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 71370,
        .range_end = 71375,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 71376,
        .range_end = 71395,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 71396,
        .range_end = 71423,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 71424,
        .range_end = 71450,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 71451,
        .range_end = 71452,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 71453,
        .range_end = 71453,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 71454,
        .range_end = 71454,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 71455,
        .range_end = 71455,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 71456,
        .range_end = 71457,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 71458,
        .range_end = 71461,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 71462,
        .range_end = 71462,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 71463,
        .range_end = 71467,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 71468,
        .range_end = 71471,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 71472,
        .range_end = 71494,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 71495,
        .range_end = 71679,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 71680,
        .range_end = 71726,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 71727,
        .range_end = 71735,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 71736,
        .range_end = 71736,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 71737,
        .range_end = 71738,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 71739,
        .range_end = 71739,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 71740,
        .range_end = 71839,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 71840,
        .range_end = 71922,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 71923,
        .range_end = 71934,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 71935,
        .range_end = 71942,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 71943,
        .range_end = 71944,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 71945,
        .range_end = 71945,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 71946,
        .range_end = 71947,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 71948,
        .range_end = 71955,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 71956,
        .range_end = 71956,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 71957,
        .range_end = 71958,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 71959,
        .range_end = 71959,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 71960,
        .range_end = 71989,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 71990,
        .range_end = 71990,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 71991,
        .range_end = 71992,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 71993,
        .range_end = 71994,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 71995,
        .range_end = 71996,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 71997,
        .range_end = 71997,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 71998,
        .range_end = 71998,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 71999,
        .range_end = 72002,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 72003,
        .range_end = 72003,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 72004,
        .range_end = 72006,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 72007,
        .range_end = 72015,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 72016,
        .range_end = 72025,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 72026,
        .range_end = 72095,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 72096,
        .range_end = 72103,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 72104,
        .range_end = 72105,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 72106,
        .range_end = 72147,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 72148,
        .range_end = 72151,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 72152,
        .range_end = 72153,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 72154,
        .range_end = 72155,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 72156,
        .range_end = 72159,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 72160,
        .range_end = 72160,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 72161,
        .range_end = 72164,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 72165,
        .range_end = 72191,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 72192,
        .range_end = 72192,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 72193,
        .range_end = 72202,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 72203,
        .range_end = 72242,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 72243,
        .range_end = 72248,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 72249,
        .range_end = 72250,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 72251,
        .range_end = 72254,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 72255,
        .range_end = 72262,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 72263,
        .range_end = 72263,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 72264,
        .range_end = 72271,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 72272,
        .range_end = 72272,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 72273,
        .range_end = 72278,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 72279,
        .range_end = 72280,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 72281,
        .range_end = 72283,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 72284,
        .range_end = 72329,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 72330,
        .range_end = 72342,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 72343,
        .range_end = 72343,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 72344,
        .range_end = 72345,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 72346,
        .range_end = 72354,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 72355,
        .range_end = 72367,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 72368,
        .range_end = 72440,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 72441,
        .range_end = 72447,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 72448,
        .range_end = 72457,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 72458,
        .range_end = 72639,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 72640,
        .range_end = 72673,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 72674,
        .range_end = 72687,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 72688,
        .range_end = 72697,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 72698,
        .range_end = 72703,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 72704,
        .range_end = 72712,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 72713,
        .range_end = 72713,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 72714,
        .range_end = 72751,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 72752,
        .range_end = 72758,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 72759,
        .range_end = 72759,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 72760,
        .range_end = 72765,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 72766,
        .range_end = 72766,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 72767,
        .range_end = 72767,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 72768,
        .range_end = 72773,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 72774,
        .range_end = 72783,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 72784,
        .range_end = 72812,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 72813,
        .range_end = 72815,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 72816,
        .range_end = 72847,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 72848,
        .range_end = 72849,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 72850,
        .range_end = 72871,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 72872,
        .range_end = 72872,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 72873,
        .range_end = 72873,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 72874,
        .range_end = 72880,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 72881,
        .range_end = 72881,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 72882,
        .range_end = 72883,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 72884,
        .range_end = 72884,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 72885,
        .range_end = 72886,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 72887,
        .range_end = 72959,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 72960,
        .range_end = 72966,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 72967,
        .range_end = 72967,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 72968,
        .range_end = 72969,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 72970,
        .range_end = 72970,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 72971,
        .range_end = 73008,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 73009,
        .range_end = 73014,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 73015,
        .range_end = 73017,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 73018,
        .range_end = 73018,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 73019,
        .range_end = 73019,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 73020,
        .range_end = 73021,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 73022,
        .range_end = 73022,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 73023,
        .range_end = 73029,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 73030,
        .range_end = 73030,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 73031,
        .range_end = 73031,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 73032,
        .range_end = 73039,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 73040,
        .range_end = 73049,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 73050,
        .range_end = 73055,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 73056,
        .range_end = 73061,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 73062,
        .range_end = 73062,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 73063,
        .range_end = 73064,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 73065,
        .range_end = 73065,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 73066,
        .range_end = 73102,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 73103,
        .range_end = 73103,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 73104,
        .range_end = 73105,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 73106,
        .range_end = 73106,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 73107,
        .range_end = 73108,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 73109,
        .range_end = 73109,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 73110,
        .range_end = 73110,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 73111,
        .range_end = 73111,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 73112,
        .range_end = 73112,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 73113,
        .range_end = 73119,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 73120,
        .range_end = 73129,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 73130,
        .range_end = 73439,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 73440,
        .range_end = 73458,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 73459,
        .range_end = 73460,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 73461,
        .range_end = 73464,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 73465,
        .range_end = 73471,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 73472,
        .range_end = 73473,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 73474,
        .range_end = 73488,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 73489,
        .range_end = 73489,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 73490,
        .range_end = 73525,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 73526,
        .range_end = 73530,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 73531,
        .range_end = 73533,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 73534,
        .range_end = 73535,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 73536,
        .range_end = 73536,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 73537,
        .range_end = 73537,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 73538,
        .range_end = 73538,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 73539,
        .range_end = 73561,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 73562,
        .range_end = 73562,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 73563,
        .range_end = 73647,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 73648,
        .range_end = 73648,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 73649,
        .range_end = 73663,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 73664,
        .range_end = 73713,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 73714,
        .range_end = 73726,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 73727,
        .range_end = 74649,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 74650,
        .range_end = 74751,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 74752,
        .range_end = 74862,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 74863,
        .range_end = 74863,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 74864,
        .range_end = 74868,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 74869,
        .range_end = 74879,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 74880,
        .range_end = 75075,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 75076,
        .range_end = 77711,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 77712,
        .range_end = 77810,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 77811,
        .range_end = 77823,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 77824,
        .range_end = 78911,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 78912,
        .range_end = 78912,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 78913,
        .range_end = 78918,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 78919,
        .range_end = 78933,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 78934,
        .range_end = 78943,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 78944,
        .range_end = 82938,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 82939,
        .range_end = 82943,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 82944,
        .range_end = 83526,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 83527,
        .range_end = 90367,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 90368,
        .range_end = 90397,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 90398,
        .range_end = 90409,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 90410,
        .range_end = 90412,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 90413,
        .range_end = 90415,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 90416,
        .range_end = 90425,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 90426,
        .range_end = 92159,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 92160,
        .range_end = 92728,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 92729,
        .range_end = 92735,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 92736,
        .range_end = 92766,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 92767,
        .range_end = 92767,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 92768,
        .range_end = 92777,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 92778,
        .range_end = 92781,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 92782,
        .range_end = 92862,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 92863,
        .range_end = 92863,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 92864,
        .range_end = 92873,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 92874,
        .range_end = 92879,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 92880,
        .range_end = 92909,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 92910,
        .range_end = 92911,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 92912,
        .range_end = 92916,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 92917,
        .range_end = 92917,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 92918,
        .range_end = 92927,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 92928,
        .range_end = 92975,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 92976,
        .range_end = 92982,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 92983,
        .range_end = 92997,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 92998,
        .range_end = 93007,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 93008,
        .range_end = 93017,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 93018,
        .range_end = 93018,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 93019,
        .range_end = 93025,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 93026,
        .range_end = 93026,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 93027,
        .range_end = 93047,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 93048,
        .range_end = 93052,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 93053,
        .range_end = 93071,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 93072,
        .range_end = 93503,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 93504,
        .range_end = 93561,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 93562,
        .range_end = 93759,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 93760,
        .range_end = 93850,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 93851,
        .range_end = 93951,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 93952,
        .range_end = 94026,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 94027,
        .range_end = 94030,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 94031,
        .range_end = 94031,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 94032,
        .range_end = 94087,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 94088,
        .range_end = 94094,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 94095,
        .range_end = 94098,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 94099,
        .range_end = 94111,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 94112,
        .range_end = 94175,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 94176,
        .range_end = 94179,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 94180,
        .range_end = 94180,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 94181,
        .range_end = 94191,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 94192,
        .range_end = 94193,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 94194,
        .range_end = 94207,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 94208,
        .range_end = 100343,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 100344,
        .range_end = 100351,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 100352,
        .range_end = 101589,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 101590,
        .range_end = 101630,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 101631,
        .range_end = 101640,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 101641,
        .range_end = 110575,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 110576,
        .range_end = 110579,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 110580,
        .range_end = 110580,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 110581,
        .range_end = 110587,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 110588,
        .range_end = 110588,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 110589,
        .range_end = 110590,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 110591,
        .range_end = 110591,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 110592,
        .range_end = 110882,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 110883,
        .range_end = 110897,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 110898,
        .range_end = 110898,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 110899,
        .range_end = 110927,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 110928,
        .range_end = 110930,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 110931,
        .range_end = 110932,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 110933,
        .range_end = 110933,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 110934,
        .range_end = 110947,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 110948,
        .range_end = 110951,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 110952,
        .range_end = 110959,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 110960,
        .range_end = 111355,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 111356,
        .range_end = 113663,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 113664,
        .range_end = 113770,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 113771,
        .range_end = 113775,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 113776,
        .range_end = 113788,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 113789,
        .range_end = 113791,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 113792,
        .range_end = 113800,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 113801,
        .range_end = 113807,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 113808,
        .range_end = 113817,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 113818,
        .range_end = 113819,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 113820,
        .range_end = 113820,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 113821,
        .range_end = 113822,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 113823,
        .range_end = 113823,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 113824,
        .range_end = 113827,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 113828,
        .range_end = 117759,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 117760,
        .range_end = 118009,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 118010,
        .range_end = 118015,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 118016,
        .range_end = 118451,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 118452,
        .range_end = 118527,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 118528,
        .range_end = 118573,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 118574,
        .range_end = 118575,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 118576,
        .range_end = 118598,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 118599,
        .range_end = 118607,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 118608,
        .range_end = 118723,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 118724,
        .range_end = 118783,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 118784,
        .range_end = 119029,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 119030,
        .range_end = 119039,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 119040,
        .range_end = 119078,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 119079,
        .range_end = 119080,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 119081,
        .range_end = 119142,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 119143,
        .range_end = 119145,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 119146,
        .range_end = 119154,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 119155,
        .range_end = 119170,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 119171,
        .range_end = 119172,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 119173,
        .range_end = 119179,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 119180,
        .range_end = 119209,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 119210,
        .range_end = 119213,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 119214,
        .range_end = 119274,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 119275,
        .range_end = 119295,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 119296,
        .range_end = 119361,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 119362,
        .range_end = 119364,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 119365,
        .range_end = 119365,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 119366,
        .range_end = 119487,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 119488,
        .range_end = 119507,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 119508,
        .range_end = 119519,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 119520,
        .range_end = 119539,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 119540,
        .range_end = 119551,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 119552,
        .range_end = 119638,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 119639,
        .range_end = 119647,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 119648,
        .range_end = 119670,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 119671,
        .range_end = 119672,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 119673,
        .range_end = 119807,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 119808,
        .range_end = 119892,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 119893,
        .range_end = 119893,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 119894,
        .range_end = 119964,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 119965,
        .range_end = 119965,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 119966,
        .range_end = 119967,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 119968,
        .range_end = 119969,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 119970,
        .range_end = 119970,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 119971,
        .range_end = 119972,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 119973,
        .range_end = 119974,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 119975,
        .range_end = 119976,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 119977,
        .range_end = 119980,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 119981,
        .range_end = 119981,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 119982,
        .range_end = 119993,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 119994,
        .range_end = 119994,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 119995,
        .range_end = 119995,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 119996,
        .range_end = 119996,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 119997,
        .range_end = 120003,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 120004,
        .range_end = 120004,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 120005,
        .range_end = 120069,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 120070,
        .range_end = 120070,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 120071,
        .range_end = 120074,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 120075,
        .range_end = 120076,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 120077,
        .range_end = 120084,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 120085,
        .range_end = 120085,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 120086,
        .range_end = 120092,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 120093,
        .range_end = 120093,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 120094,
        .range_end = 120121,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 120122,
        .range_end = 120122,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 120123,
        .range_end = 120126,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 120127,
        .range_end = 120127,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 120128,
        .range_end = 120132,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 120133,
        .range_end = 120133,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 120134,
        .range_end = 120134,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 120135,
        .range_end = 120137,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 120138,
        .range_end = 120144,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 120145,
        .range_end = 120145,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 120146,
        .range_end = 120485,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 120486,
        .range_end = 120487,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 120488,
        .range_end = 120779,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 120780,
        .range_end = 120781,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 120782,
        .range_end = 121343,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 121344,
        .range_end = 121398,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 121399,
        .range_end = 121402,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 121403,
        .range_end = 121452,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 121453,
        .range_end = 121460,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 121461,
        .range_end = 121461,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 121462,
        .range_end = 121475,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 121476,
        .range_end = 121476,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 121477,
        .range_end = 121483,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 121484,
        .range_end = 121498,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 121499,
        .range_end = 121503,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 121504,
        .range_end = 121504,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 121505,
        .range_end = 121519,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 121520,
        .range_end = 122623,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 122624,
        .range_end = 122654,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 122655,
        .range_end = 122660,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 122661,
        .range_end = 122666,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 122667,
        .range_end = 122879,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 122880,
        .range_end = 122886,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 122887,
        .range_end = 122887,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 122888,
        .range_end = 122904,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 122905,
        .range_end = 122906,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 122907,
        .range_end = 122913,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 122914,
        .range_end = 122914,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 122915,
        .range_end = 122916,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 122917,
        .range_end = 122917,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 122918,
        .range_end = 122922,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 122923,
        .range_end = 122927,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 122928,
        .range_end = 122989,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 122990,
        .range_end = 123022,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 123023,
        .range_end = 123023,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 123024,
        .range_end = 123135,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 123136,
        .range_end = 123180,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 123181,
        .range_end = 123183,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 123184,
        .range_end = 123190,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 123191,
        .range_end = 123197,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 123198,
        .range_end = 123199,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 123200,
        .range_end = 123209,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 123210,
        .range_end = 123213,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 123214,
        .range_end = 123215,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 123216,
        .range_end = 123535,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 123536,
        .range_end = 123565,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 123566,
        .range_end = 123566,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 123567,
        .range_end = 123583,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 123584,
        .range_end = 123627,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 123628,
        .range_end = 123631,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 123632,
        .range_end = 123641,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 123642,
        .range_end = 123646,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 123647,
        .range_end = 123647,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 123648,
        .range_end = 124111,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 124112,
        .range_end = 124139,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 124140,
        .range_end = 124143,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 124144,
        .range_end = 124153,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 124154,
        .range_end = 124367,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 124368,
        .range_end = 124397,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 124398,
        .range_end = 124399,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 124400,
        .range_end = 124410,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 124411,
        .range_end = 124414,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 124415,
        .range_end = 124415,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 124416,
        .range_end = 124895,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 124896,
        .range_end = 124902,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 124903,
        .range_end = 124903,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 124904,
        .range_end = 124907,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 124908,
        .range_end = 124908,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 124909,
        .range_end = 124910,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 124911,
        .range_end = 124911,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 124912,
        .range_end = 124926,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 124927,
        .range_end = 124927,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 124928,
        .range_end = 125124,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 125125,
        .range_end = 125126,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 125127,
        .range_end = 125135,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 125136,
        .range_end = 125142,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 125143,
        .range_end = 125183,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 125184,
        .range_end = 125251,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 125252,
        .range_end = 125258,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 125259,
        .range_end = 125259,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 125260,
        .range_end = 125263,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 125264,
        .range_end = 125273,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 125274,
        .range_end = 125277,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 125278,
        .range_end = 125279,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 125280,
        .range_end = 126064,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 126065,
        .range_end = 126132,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 126133,
        .range_end = 126208,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 126209,
        .range_end = 126269,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 126270,
        .range_end = 126463,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 126464,
        .range_end = 126467,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 126468,
        .range_end = 126468,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 126469,
        .range_end = 126495,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 126496,
        .range_end = 126496,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 126497,
        .range_end = 126498,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 126499,
        .range_end = 126499,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 126500,
        .range_end = 126500,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 126501,
        .range_end = 126502,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 126503,
        .range_end = 126503,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 126504,
        .range_end = 126504,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 126505,
        .range_end = 126514,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 126515,
        .range_end = 126515,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 126516,
        .range_end = 126519,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 126520,
        .range_end = 126520,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 126521,
        .range_end = 126521,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 126522,
        .range_end = 126522,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 126523,
        .range_end = 126523,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 126524,
        .range_end = 126529,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 126530,
        .range_end = 126530,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 126531,
        .range_end = 126534,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 126535,
        .range_end = 126535,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 126536,
        .range_end = 126536,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 126537,
        .range_end = 126537,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 126538,
        .range_end = 126538,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 126539,
        .range_end = 126539,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 126540,
        .range_end = 126540,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 126541,
        .range_end = 126543,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 126544,
        .range_end = 126544,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 126545,
        .range_end = 126546,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 126547,
        .range_end = 126547,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 126548,
        .range_end = 126548,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 126549,
        .range_end = 126550,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 126551,
        .range_end = 126551,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 126552,
        .range_end = 126552,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 126553,
        .range_end = 126553,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 126554,
        .range_end = 126554,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 126555,
        .range_end = 126555,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 126556,
        .range_end = 126556,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 126557,
        .range_end = 126557,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 126558,
        .range_end = 126558,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 126559,
        .range_end = 126559,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 126560,
        .range_end = 126560,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 126561,
        .range_end = 126562,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 126563,
        .range_end = 126563,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 126564,
        .range_end = 126564,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 126565,
        .range_end = 126566,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 126567,
        .range_end = 126570,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 126571,
        .range_end = 126571,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 126572,
        .range_end = 126578,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 126579,
        .range_end = 126579,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 126580,
        .range_end = 126583,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 126584,
        .range_end = 126584,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 126585,
        .range_end = 126588,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 126589,
        .range_end = 126589,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 126590,
        .range_end = 126590,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 126591,
        .range_end = 126591,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 126592,
        .range_end = 126601,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 126602,
        .range_end = 126602,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 126603,
        .range_end = 126619,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 126620,
        .range_end = 126624,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 126625,
        .range_end = 126627,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 126628,
        .range_end = 126628,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 126629,
        .range_end = 126633,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 126634,
        .range_end = 126634,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 126635,
        .range_end = 126651,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 126652,
        .range_end = 126703,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 126704,
        .range_end = 126705,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 126706,
        .range_end = 126975,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 126976,
        .range_end = 126979,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 126980,
        .range_end = 126980,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 126981,
        .range_end = 127019,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 127020,
        .range_end = 127023,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 127024,
        .range_end = 127123,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 127124,
        .range_end = 127135,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 127136,
        .range_end = 127150,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 127151,
        .range_end = 127152,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 127153,
        .range_end = 127167,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 127168,
        .range_end = 127168,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 127169,
        .range_end = 127182,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 127183,
        .range_end = 127183,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 127184,
        .range_end = 127184,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 127185,
        .range_end = 127221,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 127222,
        .range_end = 127231,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 127232,
        .range_end = 127373,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 127374,
        .range_end = 127374,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 127375,
        .range_end = 127376,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 127377,
        .range_end = 127386,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 127387,
        .range_end = 127405,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 127406,
        .range_end = 127461,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 127462,
        .range_end = 127487,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 127488,
        .range_end = 127490,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 127491,
        .range_end = 127503,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 127504,
        .range_end = 127547,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 127548,
        .range_end = 127551,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 127552,
        .range_end = 127560,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 127561,
        .range_end = 127567,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 127568,
        .range_end = 127569,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 127570,
        .range_end = 127583,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 127584,
        .range_end = 127589,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 127590,
        .range_end = 127743,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 127744,
        .range_end = 127776,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 127777,
        .range_end = 127788,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 127789,
        .range_end = 127797,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 127798,
        .range_end = 127798,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 127799,
        .range_end = 127868,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 127869,
        .range_end = 127869,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 127870,
        .range_end = 127891,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 127892,
        .range_end = 127903,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 127904,
        .range_end = 127946,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 127947,
        .range_end = 127950,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 127951,
        .range_end = 127955,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 127956,
        .range_end = 127967,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 127968,
        .range_end = 127984,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 127985,
        .range_end = 127987,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 127988,
        .range_end = 127988,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 127989,
        .range_end = 127991,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 127992,
        .range_end = 128062,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 128063,
        .range_end = 128063,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 128064,
        .range_end = 128064,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 128065,
        .range_end = 128065,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 128066,
        .range_end = 128252,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 128253,
        .range_end = 128254,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 128255,
        .range_end = 128317,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 128318,
        .range_end = 128330,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 128331,
        .range_end = 128334,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 128335,
        .range_end = 128335,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 128336,
        .range_end = 128359,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 128360,
        .range_end = 128377,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 128378,
        .range_end = 128378,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 128379,
        .range_end = 128404,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 128405,
        .range_end = 128406,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 128407,
        .range_end = 128419,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 128420,
        .range_end = 128420,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 128421,
        .range_end = 128506,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 128507,
        .range_end = 128591,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 128592,
        .range_end = 128639,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 128640,
        .range_end = 128709,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 128710,
        .range_end = 128715,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 128716,
        .range_end = 128716,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 128717,
        .range_end = 128719,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 128720,
        .range_end = 128722,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 128723,
        .range_end = 128724,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 128725,
        .range_end = 128727,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 128728,
        .range_end = 128731,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 128732,
        .range_end = 128735,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 128736,
        .range_end = 128746,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 128747,
        .range_end = 128748,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 128749,
        .range_end = 128751,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 128752,
        .range_end = 128755,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 128756,
        .range_end = 128764,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 128765,
        .range_end = 128767,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 128768,
        .range_end = 128886,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 128887,
        .range_end = 128890,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 128891,
        .range_end = 128985,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 128986,
        .range_end = 128991,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 128992,
        .range_end = 129003,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 129004,
        .range_end = 129007,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 129008,
        .range_end = 129008,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 129009,
        .range_end = 129023,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 129024,
        .range_end = 129035,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 129036,
        .range_end = 129039,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 129040,
        .range_end = 129095,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 129096,
        .range_end = 129103,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 129104,
        .range_end = 129113,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 129114,
        .range_end = 129119,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 129120,
        .range_end = 129159,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 129160,
        .range_end = 129167,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 129168,
        .range_end = 129197,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 129198,
        .range_end = 129199,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 129200,
        .range_end = 129211,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 129212,
        .range_end = 129215,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 129216,
        .range_end = 129217,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 129218,
        .range_end = 129279,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 129280,
        .range_end = 129291,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 129292,
        .range_end = 129338,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 129339,
        .range_end = 129339,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 129340,
        .range_end = 129349,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 129350,
        .range_end = 129350,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 129351,
        .range_end = 129535,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 129536,
        .range_end = 129619,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 129620,
        .range_end = 129631,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 129632,
        .range_end = 129645,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 129646,
        .range_end = 129647,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 129648,
        .range_end = 129660,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 129661,
        .range_end = 129663,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 129664,
        .range_end = 129673,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 129674,
        .range_end = 129678,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 129679,
        .range_end = 129734,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 129735,
        .range_end = 129741,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 129742,
        .range_end = 129756,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 129757,
        .range_end = 129758,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 129759,
        .range_end = 129769,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 129770,
        .range_end = 129775,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 129776,
        .range_end = 129784,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 129785,
        .range_end = 129791,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 129792,
        .range_end = 129938,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 129939,
        .range_end = 129939,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 129940,
        .range_end = 130041,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 130042,
        .range_end = 131071,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 131072,
        .range_end = 173791,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 173792,
        .range_end = 173823,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 173824,
        .range_end = 177977,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 177978,
        .range_end = 177983,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 177984,
        .range_end = 178205,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 178206,
        .range_end = 178207,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 178208,
        .range_end = 183969,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 183970,
        .range_end = 183983,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 183984,
        .range_end = 191456,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 191457,
        .range_end = 191471,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 191472,
        .range_end = 192093,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 192094,
        .range_end = 194559,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 194560,
        .range_end = 195101,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 195102,
        .range_end = 196607,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 196608,
        .range_end = 201546,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 201547,
        .range_end = 201551,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 201552,
        .range_end = 205743,
        .width = 2,
    },
    struct_unnamed_36{
        .range_start = 205744,
        .range_end = 917504,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 917505,
        .range_end = 917505,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 917506,
        .range_end = 917535,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 917536,
        .range_end = 917631,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 917632,
        .range_end = 917759,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 917760,
        .range_end = 917999,
        .width = 0,
    },
    struct_unnamed_36{
        .range_start = 918000,
        .range_end = 983039,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 983040,
        .range_end = 1048573,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 1048574,
        .range_end = 1048575,
        .width = -@as(c_int, 1),
    },
    struct_unnamed_36{
        .range_start = 1048576,
        .range_end = 1114109,
        .width = 1,
    },
    struct_unnamed_36{
        .range_start = 1114110,
        .range_end = 1114111,
        .width = -@as(c_int, 1),
    },
};
pub fn tb_reset() callconv(.c) c_int {
    var ttyfd_open: c_int = global.ttyfd_open;
    _ = &ttyfd_open;
    _ = memset(@ptrCast(@alignCast(&global)), 0, @sizeOf(@TypeOf(global)));
    global.ttyfd = -@as(c_int, 1);
    global.rfd = -@as(c_int, 1);
    global.wfd = -@as(c_int, 1);
    global.ttyfd_open = ttyfd_open;
    global.resize_pipefd[@as(c_int, 0)] = -@as(c_int, 1);
    global.resize_pipefd[@as(c_int, 1)] = -@as(c_int, 1);
    global.width = -@as(c_int, 1);
    global.height = -@as(c_int, 1);
    global.cursor_x = -@as(c_int, 1);
    global.cursor_y = -@as(c_int, 1);
    global.last_x = -@as(c_int, 1);
    global.last_y = -@as(c_int, 1);
    global.fg = TB_DEFAULT;
    global.bg = TB_DEFAULT;
    global.last_fg = ~global.fg;
    global.last_bg = ~global.bg;
    global.input_mode = TB_INPUT_ESC;
    global.output_mode = TB_OUTPUT_NORMAL;
    return TB_OK;
}
pub export fn tb_printf_inner(arg_x: c_int, arg_y: c_int, arg_fg: uintattr_t, arg_bg: uintattr_t, arg_out_w: [*c]usize, arg_fmt: [*c]const u8, arg_vl: [*c]struct___va_list_tag_26) c_int {
    var x = arg_x;
    _ = &x;
    var y = arg_y;
    _ = &y;
    var fg = arg_fg;
    _ = &fg;
    var bg = arg_bg;
    _ = &bg;
    var out_w = arg_out_w;
    _ = &out_w;
    var fmt = arg_fmt;
    _ = &fmt;
    var vl = arg_vl;
    _ = &vl;
    var rv: c_int = undefined;
    _ = &rv;
    var buf: [4096]u8 = undefined;
    _ = &buf;
    rv = vsnprintf(@ptrCast(@alignCast(&buf)), @sizeOf(@TypeOf(buf)), fmt, vl);
    if ((rv < @as(c_int, 0)) or (rv >= @as(c_int, @bitCast(@as(c_uint, @truncate(@sizeOf(@TypeOf(buf)))))))) {
        return -@as(c_int, 1);
    }
    return tb_print_ex(x, y, fg, bg, out_w, @ptrCast(@alignCast(&buf)));
}
pub fn init_term_attrs() callconv(.c) c_int {
    if (global.ttyfd < @as(c_int, 0)) return TB_OK;
    if (tcgetattr(global.ttyfd, &global.orig_tios) != @as(c_int, 0)) {
        global.last_errno = __errno_location().*;
        return -@as(c_int, 15);
    }
    var tios: struct_termios = undefined;
    _ = &tios;
    _ = memcpy(@ptrCast(@alignCast(&tios)), @ptrCast(@alignCast(&global.orig_tios)), @sizeOf(@TypeOf(tios)));
    global.has_orig_tios = 1;
    cfmakeraw(&tios);
    tios.c_cc[VMIN] = 1;
    tios.c_cc[VTIME] = 0;
    if (tcsetattr(global.ttyfd, TCSAFLUSH, &tios) != @as(c_int, 0)) {
        global.last_errno = __errno_location().*;
        return -@as(c_int, 16);
    }
    return TB_OK;
}
pub fn init_term_caps() callconv(.c) c_int {
    if (load_terminfo() == TB_OK) {
        return parse_terminfo_caps();
    }
    return load_builtin_caps();
}
pub fn init_cap_trie() callconv(.c) c_int {
    var rv: c_int = undefined;
    _ = &rv;
    var i: c_int = undefined;
    _ = &i;
    {
        i = 0;
        while (i < TB_CAP__COUNT_KEYS) : (i += 1) {
            rv = cap_trie_add(global.caps[@bitCast(@as(isize, @intCast(i)))], @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - i))), 0);
            if ((rv != TB_OK) and (rv != -@as(c_int, 22))) return rv;
        }
    }
    {
        i = 0;
        while (@as(?*anyopaque, @ptrCast(@alignCast(@constCast(builtin_mod_caps[@bitCast(@as(isize, @intCast(i)))].cap)))) != @as(?*anyopaque, null)) : (i += 1) {
            rv = cap_trie_add(builtin_mod_caps[@bitCast(@as(isize, @intCast(i)))].cap, builtin_mod_caps[@bitCast(@as(isize, @intCast(i)))].key, builtin_mod_caps[@bitCast(@as(isize, @intCast(i)))].mod);
            if ((rv != TB_OK) and (rv != -@as(c_int, 22))) return rv;
        }
    }
    return TB_OK;
}
pub fn cap_trie_add(arg_cap: [*c]const u8, arg_key: u16, arg_mod: u8) callconv(.c) c_int {
    var cap = arg_cap;
    _ = &cap;
    var key = arg_key;
    _ = &key;
    var mod = arg_mod;
    _ = &mod;
    var next: [*c]struct_cap_trie = undefined;
    _ = &next;
    var node: [*c]struct_cap_trie = &global.cap_trie;
    _ = &node;
    var i: usize = undefined;
    _ = &i;
    var j: usize = undefined;
    _ = &j;
    if (!(cap != null) or (strlen(cap) <= @as(usize, 0))) return TB_OK;
    {
        i = 0;
        while (@as(c_int, cap[i]) != @as(c_int, '\x00')) : (i +%= 1) {
            var c: u8 = cap[i];
            _ = &c;
            next = null;
            {
                j = 0;
                while (j < node.*.nchildren) : (j +%= 1) {
                    if (@as(c_int, node.*.children[j].c) == @as(c_int, c)) {
                        next = &node.*.children[j];
                        break;
                    }
                }
            }
            if (!(next != null)) {
                node.*.nchildren +%= 1;
                node.*.children = @ptrCast(@alignCast(realloc(@ptrCast(@alignCast(node.*.children)), @sizeOf(@TypeOf(node.*)) *% node.*.nchildren)));
                if (!(node.*.children != null)) {
                    return -@as(c_int, 5);
                }
                next = &node.*.children[node.*.nchildren -% @as(usize, 1)];
                _ = memset(@ptrCast(@alignCast(next)), 0, @sizeOf(@TypeOf(next.*)));
                next.*.c = c;
            }
            node = next;
        }
    }
    if (node.*.is_leaf != 0) {
        return -@as(c_int, 22);
    }
    node.*.is_leaf = 1;
    node.*.key = key;
    node.*.mod = mod;
    return TB_OK;
}
pub fn cap_trie_find(arg_buf: [*c]const u8, arg_nbuf: usize, arg_last: [*c][*c]struct_cap_trie, arg_depth: [*c]usize) callconv(.c) c_int {
    var buf = arg_buf;
    _ = &buf;
    var nbuf = arg_nbuf;
    _ = &nbuf;
    var last = arg_last;
    _ = &last;
    var depth = arg_depth;
    _ = &depth;
    var next: [*c]struct_cap_trie = undefined;
    _ = &next;
    var node: [*c]struct_cap_trie = &global.cap_trie;
    _ = &node;
    var i: usize = undefined;
    _ = &i;
    var j: usize = undefined;
    _ = &j;
    last.* = node;
    depth.* = 0;
    {
        i = 0;
        while (i < nbuf) : (i +%= 1) {
            var c: u8 = buf[i];
            _ = &c;
            next = null;
            {
                j = 0;
                while (j < node.*.nchildren) : (j +%= 1) {
                    if (@as(c_int, node.*.children[j].c) == @as(c_int, c)) {
                        next = &node.*.children[j];
                        break;
                    }
                }
            }
            if (!(next != null)) {
                return TB_OK;
            }
            node = next;
            last.* = node;
            depth.* +%= 1;
            if ((node.*.is_leaf != 0) and (node.*.nchildren < @as(usize, 1))) {
                break;
            }
        }
    }
    return TB_OK;
}
pub fn cap_trie_deinit(arg_node: [*c]struct_cap_trie) callconv(.c) c_int {
    var node = arg_node;
    _ = &node;
    var j: usize = undefined;
    _ = &j;
    {
        j = 0;
        while (j < node.*.nchildren) : (j +%= 1) {
            _ = cap_trie_deinit(&node.*.children[j]);
        }
    }
    if (node.*.children != null) {
        free(@ptrCast(@alignCast(node.*.children)));
    }
    _ = memset(@ptrCast(@alignCast(node)), 0, @sizeOf(@TypeOf(node.*)));
    return TB_OK;
}
pub fn init_resize_handler() callconv(.c) c_int {
    if (pipe(@ptrCast(@alignCast(&global.resize_pipefd))) != @as(c_int, 0)) {
        global.last_errno = __errno_location().*;
        return -@as(c_int, 12);
    }
    var sa: struct_sigaction = undefined;
    _ = &sa;
    _ = memset(@ptrCast(@alignCast(&sa)), 0, @sizeOf(@TypeOf(sa)));
    sa.__sigaction_handler.sa_handler = handle_resize;
    if (sigaction(SIGWINCH, &sa, null) != @as(c_int, 0)) {
        global.last_errno = __errno_location().*;
        return -@as(c_int, 13);
    }
    return TB_OK;
}
pub fn send_init_escape_codes() callconv(.c) c_int {
    var rv: c_int = undefined;
    _ = &rv;
    if ((blk: {
        const tmp = bytebuf_puts(&global.out, global.caps[TB_CAP_ENTER_CA]);
        rv = tmp;
        break :blk tmp;
    }) != TB_OK) return rv;
    if ((blk: {
        const tmp = bytebuf_puts(&global.out, global.caps[TB_CAP_ENTER_KEYPAD]);
        rv = tmp;
        break :blk tmp;
    }) != TB_OK) return rv;
    if ((blk: {
        const tmp = bytebuf_puts(&global.out, global.caps[TB_CAP_HIDE_CURSOR]);
        rv = tmp;
        break :blk tmp;
    }) != TB_OK) return rv;
    return TB_OK;
}
pub fn send_clear() callconv(.c) c_int {
    var rv: c_int = undefined;
    _ = &rv;
    if ((blk: {
        const tmp = send_attr(global.fg, global.bg);
        rv = tmp;
        break :blk tmp;
    }) != TB_OK) return rv;
    if ((blk: {
        const tmp = bytebuf_puts(&global.out, global.caps[TB_CAP_CLEAR_SCREEN]);
        rv = tmp;
        break :blk tmp;
    }) != TB_OK) return rv;
    if ((blk: {
        const tmp = send_cursor_if(global.cursor_x, global.cursor_y);
        rv = tmp;
        break :blk tmp;
    }) != TB_OK) return rv;
    if ((blk: {
        const tmp = bytebuf_flush(&global.out, global.wfd);
        rv = tmp;
        break :blk tmp;
    }) != TB_OK) return rv;
    global.last_x = -@as(c_int, 1);
    global.last_y = -@as(c_int, 1);
    return TB_OK;
}
pub fn update_term_size() callconv(.c) c_int {
    var rv: c_int = undefined;
    _ = &rv;
    var ioctl_errno: c_int = undefined;
    _ = &ioctl_errno;
    if (global.ttyfd < @as(c_int, 0)) return TB_OK;
    var sz: struct_winsize = undefined;
    _ = &sz;
    _ = memset(@ptrCast(@alignCast(&sz)), 0, @sizeOf(@TypeOf(sz)));
    if (ioctl(global.ttyfd, TIOCGWINSZ, &sz) == @as(c_int, 0)) {
        global.width = sz.ws_col;
        global.height = sz.ws_row;
        return TB_OK;
    }
    ioctl_errno = __errno_location().*;
    if ((blk: {
        const tmp = update_term_size_via_esc();
        rv = tmp;
        break :blk tmp;
    }) == TB_OK) return rv;
    global.last_errno = ioctl_errno;
    return -@as(c_int, 11);
}
pub fn update_term_size_via_esc() callconv(.c) c_int {
    var move_and_report: [16:0]u8 = "\x1b[9999;9999H\x1b[6n".*;
    _ = &move_and_report;
    var write_rv: isize = write(global.wfd, @ptrCast(@alignCast(@as([*c]u8, @ptrCast(@alignCast(&move_and_report))))), strlen(@ptrCast(@alignCast(&move_and_report))));
    _ = &write_rv;
    if (write_rv != @as(isize, @bitCast(@as(c_ulong, @truncate(strlen(@ptrCast(@alignCast(&move_and_report)))))))) {
        return -@as(c_int, 18);
    }
    var fds: fd_set = undefined;
    _ = &fds;
    _ = memset(@ptrCast(@alignCast(&fds)), 0, @sizeOf(fd_set));
    (&fds).*.fds_bits[@bitCast(@as(isize, @intCast(@divTrunc(global.rfd, @as(c_int, 8) * @as(c_int, @bitCast(@as(c_uint, @truncate(@sizeOf(__fd_mask)))))))))] |= @as(__fd_mask, @bitCast(@as(c_ulong, @truncate(@as(c_ulong, 1) << @intCast(@as(c_ulong, @bitCast(@as(c_long, __helpers.signedRemainder(global.rfd, @as(c_int, 8) * @as(c_int, @bitCast(@as(c_uint, @truncate(@sizeOf(__fd_mask))))))))))))));
    var timeout: struct_timeval = undefined;
    _ = &timeout;
    timeout.tv_sec = 0;
    timeout.tv_usec = TB_RESIZE_FALLBACK_MS * @as(c_int, 1000);
    var select_rv: c_int = select(global.rfd + @as(c_int, 1), &fds, null, null, &timeout);
    _ = &select_rv;
    if (select_rv != @as(c_int, 1)) {
        global.last_errno = __errno_location().*;
        return -@as(c_int, 19);
    }
    var buf: [64]u8 = undefined;
    _ = &buf;
    var read_rv: isize = read(global.rfd, @ptrCast(@alignCast(@as([*c]u8, @ptrCast(@alignCast(&buf))))), @sizeOf(@TypeOf(buf)) -% @as(c_ulong, 1));
    _ = &read_rv;
    if (read_rv < @as(isize, 1)) {
        global.last_errno = __errno_location().*;
        return -@as(c_int, 20);
    }
    buf[@bitCast(@as(isize, @intCast(read_rv)))] = '\x00';
    var rw: c_int = undefined;
    _ = &rw;
    var rh: c_int = undefined;
    _ = &rh;
    if (sscanf(@ptrCast(@alignCast(&buf)), "\x1b[%d;%dR", &rh, &rw) != @as(c_int, 2)) {
        return -@as(c_int, 21);
    }
    global.width = rw;
    global.height = rh;
    return TB_OK;
}
pub fn init_cellbuf() callconv(.c) c_int {
    var rv: c_int = undefined;
    _ = &rv;
    if ((blk: {
        const tmp = cellbuf_init(&global.back, global.width, global.height);
        rv = tmp;
        break :blk tmp;
    }) != TB_OK) return rv;
    if ((blk: {
        const tmp = cellbuf_init(&global.front, global.width, global.height);
        rv = tmp;
        break :blk tmp;
    }) != TB_OK) return rv;
    if ((blk: {
        const tmp = cellbuf_clear(&global.back);
        rv = tmp;
        break :blk tmp;
    }) != TB_OK) return rv;
    if ((blk: {
        const tmp = cellbuf_clear(&global.front);
        rv = tmp;
        break :blk tmp;
    }) != TB_OK) return rv;
    return TB_OK;
}
pub fn tb_deinit() callconv(.c) c_int {
    if ((@as(?*anyopaque, @ptrCast(@alignCast(@constCast(global.caps[@as(c_int, 0)])))) != @as(?*anyopaque, null)) and (global.wfd >= @as(c_int, 0))) {
        _ = bytebuf_puts(&global.out, global.caps[TB_CAP_SHOW_CURSOR]);
        _ = bytebuf_puts(&global.out, global.caps[TB_CAP_SGR0]);
        _ = bytebuf_puts(&global.out, global.caps[TB_CAP_CLEAR_SCREEN]);
        _ = bytebuf_puts(&global.out, global.caps[TB_CAP_EXIT_CA]);
        _ = bytebuf_puts(&global.out, global.caps[TB_CAP_EXIT_KEYPAD]);
        _ = bytebuf_puts(&global.out, "\x1b[?1006l\x1b[?1015l\x1b[?1002l\x1b[?1000l");
        _ = bytebuf_flush(&global.out, global.wfd);
    }
    if (global.ttyfd >= @as(c_int, 0)) {
        if (global.has_orig_tios != 0) {
            _ = tcsetattr(global.ttyfd, TCSAFLUSH, &global.orig_tios);
        }
        if (global.ttyfd_open != 0) {
            _ = close(global.ttyfd);
            global.ttyfd_open = 0;
        }
    }
    var sa: struct_sigaction = undefined;
    _ = &sa;
    _ = memset(@ptrCast(@alignCast(&sa)), 0, @sizeOf(@TypeOf(sa)));
    sa.__sigaction_handler.sa_handler = null;
    _ = sigaction(SIGWINCH, &sa, null);
    if (global.resize_pipefd[@as(c_int, 0)] >= @as(c_int, 0)) {
        _ = close(global.resize_pipefd[@as(c_int, 0)]);
    }
    if (global.resize_pipefd[@as(c_int, 1)] >= @as(c_int, 0)) {
        _ = close(global.resize_pipefd[@as(c_int, 1)]);
    }
    _ = cellbuf_free(&global.back);
    _ = cellbuf_free(&global.front);
    _ = bytebuf_free(&global.in);
    _ = bytebuf_free(&global.out);
    if (global.terminfo != null) {
        free(@ptrCast(@alignCast(global.terminfo)));
    }
    _ = cap_trie_deinit(&global.cap_trie);
    _ = tb_reset();
    return TB_OK;
}
pub fn load_terminfo() callconv(.c) c_int {
    var rv: c_int = undefined;
    _ = &rv;
    var tmp: [4096]u8 = undefined;
    _ = &tmp;
    var term: [*c]const u8 = getenv("TERM");
    _ = &term;
    if (!(term != null)) return -@as(c_int, 1);
    var terminfo: [*c]const u8 = getenv("TERMINFO");
    _ = &terminfo;
    if (terminfo != null) if ((blk: {
        const tmp_1 = load_terminfo_from_path(terminfo, term);
        rv = tmp_1;
        break :blk tmp_1;
    }) == TB_OK) return rv;
    var home: [*c]const u8 = getenv("HOME");
    _ = &home;
    if (home != null) {
        while (true) {
            rv = snprintf(@ptrCast(@alignCast(&tmp)), @sizeOf(@TypeOf(tmp)), "%s/.terminfo", home);
            if ((rv < @as(c_int, 0)) or (rv >= @as(c_int, @bitCast(@as(c_uint, @truncate(@sizeOf(@TypeOf(tmp)))))))) return -@as(c_int, 1);
            if (!false) break;
        }
        if ((blk: {
            const tmp_1 = load_terminfo_from_path(@ptrCast(@alignCast(&tmp)), term);
            rv = tmp_1;
            break :blk tmp_1;
        }) == TB_OK) return rv;
    }
    var dirs: [*c]const u8 = getenv("TERMINFO_DIRS");
    _ = &dirs;
    if (dirs != null) {
        while (true) {
            rv = snprintf(@ptrCast(@alignCast(&tmp)), @sizeOf(@TypeOf(tmp)), "%s", dirs);
            if ((rv < @as(c_int, 0)) or (rv >= @as(c_int, @bitCast(@as(c_uint, @truncate(@sizeOf(@TypeOf(tmp)))))))) return -@as(c_int, 1);
            if (!false) break;
        }
        var dir: [*c]u8 = strtok(@ptrCast(@alignCast(&tmp)), ":");
        _ = &dir;
        while (dir != null) {
            var cdir: [*c]const u8 = dir;
            _ = &cdir;
            if (@as(c_int, cdir.*) != @as(c_int, '\x00')) {
                if ((blk: {
                    const tmp_1 = load_terminfo_from_path(cdir, term);
                    rv = tmp_1;
                    break :blk tmp_1;
                }) == TB_OK) return rv;
            }
            dir = strtok(null, ":");
        }
    }
    if ((blk: {
        const tmp_1 = load_terminfo_from_path("/usr/local/etc/terminfo", term);
        rv = tmp_1;
        break :blk tmp_1;
    }) == TB_OK) return rv;
    if ((blk: {
        const tmp_1 = load_terminfo_from_path("/usr/local/share/terminfo", term);
        rv = tmp_1;
        break :blk tmp_1;
    }) == TB_OK) return rv;
    if ((blk: {
        const tmp_1 = load_terminfo_from_path("/usr/local/lib/terminfo", term);
        rv = tmp_1;
        break :blk tmp_1;
    }) == TB_OK) return rv;
    if ((blk: {
        const tmp_1 = load_terminfo_from_path("/etc/terminfo", term);
        rv = tmp_1;
        break :blk tmp_1;
    }) == TB_OK) return rv;
    if ((blk: {
        const tmp_1 = load_terminfo_from_path("/usr/share/terminfo", term);
        rv = tmp_1;
        break :blk tmp_1;
    }) == TB_OK) return rv;
    if ((blk: {
        const tmp_1 = load_terminfo_from_path("/usr/lib/terminfo", term);
        rv = tmp_1;
        break :blk tmp_1;
    }) == TB_OK) return rv;
    if ((blk: {
        const tmp_1 = load_terminfo_from_path("/usr/share/lib/terminfo", term);
        rv = tmp_1;
        break :blk tmp_1;
    }) == TB_OK) return rv;
    if ((blk: {
        const tmp_1 = load_terminfo_from_path("/lib/terminfo", term);
        rv = tmp_1;
        break :blk tmp_1;
    }) == TB_OK) return rv;
    return -@as(c_int, 1);
}
pub fn load_terminfo_from_path(arg_path: [*c]const u8, arg_term: [*c]const u8) callconv(.c) c_int {
    var path = arg_path;
    _ = &path;
    var term = arg_term;
    _ = &term;
    var rv: c_int = undefined;
    _ = &rv;
    var tmp: [4096]u8 = undefined;
    _ = &tmp;
    while (true) {
        rv = snprintf(@ptrCast(@alignCast(&tmp)), @sizeOf(@TypeOf(tmp)), "%s/%c/%s", path, term[@as(c_int, 0)], term);
        if ((rv < @as(c_int, 0)) or (rv >= @as(c_int, @bitCast(@as(c_uint, @truncate(@sizeOf(@TypeOf(tmp)))))))) return -@as(c_int, 1);
        if (!false) break;
    }
    if ((blk: {
        const tmp_1 = read_terminfo_path(@ptrCast(@alignCast(&tmp)));
        rv = tmp_1;
        break :blk tmp_1;
    }) == TB_OK) return rv;
    return -@as(c_int, 1);
}
pub fn read_terminfo_path(arg_path: [*c]const u8) callconv(.c) c_int {
    var path = arg_path;
    _ = &path;
    var fp: ?*FILE = fopen(path, "rb");
    _ = &fp;
    if (!(fp != null)) return -@as(c_int, 1);
    _ = fseek(fp, 0, SEEK_END);
    var fsize: usize = @bitCast(@as(c_long, ftell(fp)));
    _ = &fsize;
    _ = fseek(fp, 0, SEEK_SET);
    var data: [*c]u8 = @ptrCast(@alignCast(malloc(fsize)));
    _ = &data;
    if (!(data != null)) {
        _ = fclose(fp);
        return -@as(c_int, 1);
    }
    if (fread(@ptrCast(@alignCast(data)), 1, fsize, fp) != fsize) {
        _ = fclose(fp);
        free(@ptrCast(@alignCast(data)));
        return -@as(c_int, 1);
    }
    global.terminfo = data;
    global.nterminfo = fsize;
    _ = fclose(fp);
    return TB_OK;
}
pub fn parse_terminfo_caps() callconv(.c) c_int {
    if (global.nterminfo < @as(usize, @bitCast(@as(c_long, @as(c_int, 6) * @as(c_int, @bitCast(@as(c_uint, @truncate(@sizeOf(i16))))))))) return -@as(c_int, 1);
    var magic_number: i16 = undefined;
    _ = &magic_number;
    var nbytes_names: i16 = undefined;
    _ = &nbytes_names;
    var nbytes_bools: i16 = undefined;
    _ = &nbytes_bools;
    var num_ints: i16 = undefined;
    _ = &num_ints;
    var num_offsets: i16 = undefined;
    _ = &num_offsets;
    var nbytes_strings: i16 = undefined;
    _ = &nbytes_strings;
    var nbytes_header: usize = @as(c_ulong, 6) *% @sizeOf(i16);
    _ = &nbytes_header;
    _ = get_terminfo_int16(@bitCast(@as(c_uint, @truncate(@as(c_ulong, 0) *% @sizeOf(i16)))), &magic_number);
    _ = get_terminfo_int16(@bitCast(@as(c_uint, @truncate(@as(c_ulong, 1) *% @sizeOf(i16)))), &nbytes_names);
    _ = get_terminfo_int16(@bitCast(@as(c_uint, @truncate(@as(c_ulong, 2) *% @sizeOf(i16)))), &nbytes_bools);
    _ = get_terminfo_int16(@bitCast(@as(c_uint, @truncate(@as(c_ulong, 3) *% @sizeOf(i16)))), &num_ints);
    _ = get_terminfo_int16(@bitCast(@as(c_uint, @truncate(@as(c_ulong, 4) *% @sizeOf(i16)))), &num_offsets);
    _ = get_terminfo_int16(@bitCast(@as(c_uint, @truncate(@as(c_ulong, 5) *% @sizeOf(i16)))), &nbytes_strings);
    const bytes_per_int: c_int = if (@as(c_int, magic_number) == @as(c_int, 542)) @as(c_int, 4) else @as(c_int, 2);
    _ = &bytes_per_int;
    const align_offset: c_int = if (__helpers.signedRemainder(@as(c_int, nbytes_names) + @as(c_int, nbytes_bools), @as(c_int, 2)) != @as(c_int, 0)) @as(c_int, 1) else @as(c_int, 0);
    _ = &align_offset;
    const pos_str_offsets: c_int = @bitCast(@as(c_uint, @truncate((((nbytes_header +% @as(usize, @bitCast(@as(c_long, @as(c_int, nbytes_names))))) +% @as(usize, @bitCast(@as(c_long, @as(c_int, nbytes_bools))))) +% @as(usize, @bitCast(@as(c_long, align_offset)))) +% @as(usize, @bitCast(@as(c_long, @as(c_int, num_ints) * bytes_per_int))))));
    _ = &pos_str_offsets;
    const pos_str_table: c_int = @bitCast(@as(c_uint, @truncate(@as(c_ulong, @bitCast(@as(c_long, pos_str_offsets))) +% (@as(c_ulong, @bitCast(@as(c_long, @as(c_int, num_offsets)))) *% @sizeOf(i16)))));
    _ = &pos_str_table;
    var i: c_int = undefined;
    _ = &i;
    {
        i = 0;
        while (i < TB_CAP__COUNT) : (i += 1) {
            var cap: [*c]const u8 = get_terminfo_string(@truncate(pos_str_offsets), num_offsets, @truncate(pos_str_table), nbytes_strings, terminfo_cap_indexes[@bitCast(@as(isize, @intCast(i)))]);
            _ = &cap;
            if (!(cap != null)) {
                return -@as(c_int, 1);
            }
            global.caps[@bitCast(@as(isize, @intCast(i)))] = cap;
        }
    }
    return TB_OK;
}
pub fn load_builtin_caps() callconv(.c) c_int {
    var i: c_int = undefined;
    _ = &i;
    var j: c_int = undefined;
    _ = &j;
    var term: [*c]const u8 = getenv("TERM");
    _ = &term;
    if (!(term != null)) return -@as(c_int, 7);
    {
        i = 0;
        while (@as(?*anyopaque, @ptrCast(@alignCast(@constCast(builtin_terms[@bitCast(@as(isize, @intCast(i)))].name)))) != @as(?*anyopaque, null)) : (i += 1) {
            if (strcmp(term, builtin_terms[@bitCast(@as(isize, @intCast(i)))].name) == @as(c_int, 0)) {
                {
                    j = 0;
                    while (j < TB_CAP__COUNT) : (j += 1) {
                        global.caps[@bitCast(@as(isize, @intCast(j)))] = builtin_terms[@bitCast(@as(isize, @intCast(i)))].caps[@bitCast(@as(isize, @intCast(j)))];
                    }
                }
                return TB_OK;
            }
        }
    }
    {
        i = 0;
        while (@as(?*anyopaque, @ptrCast(@alignCast(@constCast(builtin_terms[@bitCast(@as(isize, @intCast(i)))].name)))) != @as(?*anyopaque, null)) : (i += 1) {
            if ((@as(?*anyopaque, @ptrCast(@alignCast(strstr(term, builtin_terms[@bitCast(@as(isize, @intCast(i)))].name)))) != @as(?*anyopaque, null)) or ((@as(c_int, builtin_terms[@bitCast(@as(isize, @intCast(i)))].alias.*) != @as(c_int, '\x00')) and (@as(?*anyopaque, @ptrCast(@alignCast(strstr(term, builtin_terms[@bitCast(@as(isize, @intCast(i)))].alias)))) != @as(?*anyopaque, null)))) {
                {
                    j = 0;
                    while (j < TB_CAP__COUNT) : (j += 1) {
                        global.caps[@bitCast(@as(isize, @intCast(j)))] = builtin_terms[@bitCast(@as(isize, @intCast(i)))].caps[@bitCast(@as(isize, @intCast(j)))];
                    }
                }
                return TB_OK;
            }
        }
    }
    return -@as(c_int, 17);
}
pub fn get_terminfo_string(arg_offsets_pos: i16, arg_offsets_len: i16, arg_table_pos: i16, arg_table_size: i16, arg_index_1: i16) callconv(.c) [*c]const u8 {
    var offsets_pos = arg_offsets_pos;
    _ = &offsets_pos;
    var offsets_len = arg_offsets_len;
    _ = &offsets_len;
    var table_pos = arg_table_pos;
    _ = &table_pos;
    var table_size = arg_table_size;
    _ = &table_size;
    var index_1 = arg_index_1;
    _ = &index_1;
    if (@as(c_int, index_1) >= @as(c_int, offsets_len)) {
        return "";
    }
    var table_offset: i16 = undefined;
    _ = &table_offset;
    var table_offset_offset: c_int = @as(c_int, offsets_pos) + (@as(c_int, index_1) * @as(c_int, @bitCast(@as(c_uint, @truncate(@sizeOf(i16))))));
    _ = &table_offset_offset;
    if (get_terminfo_int16(table_offset_offset, &table_offset) != TB_OK) {
        return null;
    }
    if ((@as(c_int, table_offset) < @as(c_int, 0)) or (@as(c_int, table_offset) >= @as(c_int, table_size))) {
        return "";
    }
    var str_offset: c_int = @as(c_int, table_pos) + @as(c_int, table_offset);
    _ = &str_offset;
    if (str_offset >= @as(c_int, @bitCast(@as(c_uint, @truncate(global.nterminfo))))) {
        return null;
    }
    return @ptrCast(@alignCast(global.terminfo + @as(usize, @bitCast(@as(isize, @intCast(str_offset))))));
}
pub fn get_terminfo_int16(arg_offset: c_int, arg_val: [*c]i16) callconv(.c) c_int {
    var offset = arg_offset;
    _ = &offset;
    var val = arg_val;
    _ = &val;
    if ((offset < @as(c_int, 0)) or ((@as(c_ulong, @bitCast(@as(c_long, offset))) +% @sizeOf(i16)) > global.nterminfo)) {
        val.* = @truncate(-@as(c_int, 1));
        return -@as(c_int, 1);
    }
    _ = memcpy(@ptrCast(@alignCast(val)), @ptrCast(@alignCast(global.terminfo + @as(usize, @bitCast(@as(isize, @intCast(offset)))))), @sizeOf(i16));
    return TB_OK;
}
pub fn wait_event(arg_event: [*c]struct_tb_event, arg_timeout: c_int) callconv(.c) c_int {
    var event = arg_event;
    _ = &event;
    var timeout = arg_timeout;
    _ = &timeout;
    var rv: c_int = undefined;
    _ = &rv;
    var buf: [64]u8 = undefined;
    _ = &buf;
    _ = memset(@ptrCast(@alignCast(event)), 0, @sizeOf(@TypeOf(event.*)));
    if ((blk: {
        const tmp = extract_event(event);
        rv = tmp;
        break :blk tmp;
    }) == TB_OK) return rv;
    var fds: fd_set = undefined;
    _ = &fds;
    var tv: struct_timeval = undefined;
    _ = &tv;
    tv.tv_sec = @divTrunc(timeout, @as(c_int, 1000));
    tv.tv_usec = (@as(__time_t, timeout) - (tv.tv_sec * @as(__time_t, 1000))) * @as(__time_t, 1000);
    while (true) {
        _ = memset(@ptrCast(@alignCast(&fds)), 0, @sizeOf(fd_set));
        (&fds).*.fds_bits[@bitCast(@as(isize, @intCast(@divTrunc(global.rfd, @as(c_int, 8) * @as(c_int, @bitCast(@as(c_uint, @truncate(@sizeOf(__fd_mask)))))))))] |= @as(__fd_mask, @bitCast(@as(c_ulong, @truncate(@as(c_ulong, 1) << @intCast(@as(c_ulong, @bitCast(@as(c_long, __helpers.signedRemainder(global.rfd, @as(c_int, 8) * @as(c_int, @bitCast(@as(c_uint, @truncate(@sizeOf(__fd_mask))))))))))))));
        (&fds).*.fds_bits[@bitCast(@as(isize, @intCast(@divTrunc(global.resize_pipefd[@as(c_int, 0)], @as(c_int, 8) * @as(c_int, @bitCast(@as(c_uint, @truncate(@sizeOf(__fd_mask)))))))))] |= @as(__fd_mask, @bitCast(@as(c_ulong, @truncate(@as(c_ulong, 1) << @intCast(@as(c_ulong, @bitCast(@as(c_long, __helpers.signedRemainder(global.resize_pipefd[@as(c_int, 0)], @as(c_int, 8) * @as(c_int, @bitCast(@as(c_uint, @truncate(@sizeOf(__fd_mask))))))))))))));
        var maxfd: c_int = if (global.resize_pipefd[@as(c_int, 0)] > global.rfd) global.resize_pipefd[@as(c_int, 0)] else global.rfd;
        _ = &maxfd;
        var select_rv: c_int = select(maxfd + @as(c_int, 1), &fds, null, null, @ptrCast(@alignCast(if (timeout < @as(c_int, 0)) @as(?*anyopaque, null) else @as(?*anyopaque, @ptrCast(@alignCast(&tv))))));
        _ = &select_rv;
        if (select_rv < @as(c_int, 0)) {
            global.last_errno = __errno_location().*;
            return -@as(c_int, 14);
        } else if (select_rv == @as(c_int, 0)) {
            return -@as(c_int, 6);
        }
        var tty_has_events: c_int = @intFromBool(((&fds).*.fds_bits[@bitCast(@as(isize, @intCast(@divTrunc(global.rfd, @as(c_int, 8) * @as(c_int, @bitCast(@as(c_uint, @truncate(@sizeOf(__fd_mask)))))))))] & @as(__fd_mask, @bitCast(@as(c_ulong, @truncate(@as(c_ulong, 1) << @intCast(@as(c_ulong, @bitCast(@as(c_long, __helpers.signedRemainder(global.rfd, @as(c_int, 8) * @as(c_int, @bitCast(@as(c_uint, @truncate(@sizeOf(__fd_mask))))))))))))))) != @as(__fd_mask, 0));
        _ = &tty_has_events;
        var resize_has_events: c_int = @intFromBool(((&fds).*.fds_bits[@bitCast(@as(isize, @intCast(@divTrunc(global.resize_pipefd[@as(c_int, 0)], @as(c_int, 8) * @as(c_int, @bitCast(@as(c_uint, @truncate(@sizeOf(__fd_mask)))))))))] & @as(__fd_mask, @bitCast(@as(c_ulong, @truncate(@as(c_ulong, 1) << @intCast(@as(c_ulong, @bitCast(@as(c_long, __helpers.signedRemainder(global.resize_pipefd[@as(c_int, 0)], @as(c_int, 8) * @as(c_int, @bitCast(@as(c_uint, @truncate(@sizeOf(__fd_mask))))))))))))))) != @as(__fd_mask, 0));
        _ = &resize_has_events;
        if (tty_has_events != 0) {
            var read_rv: isize = read(global.rfd, @ptrCast(@alignCast(@as([*c]u8, @ptrCast(@alignCast(&buf))))), @sizeOf(@TypeOf(buf)));
            _ = &read_rv;
            if (read_rv < @as(isize, 0)) {
                global.last_errno = __errno_location().*;
                return -@as(c_int, 10);
            } else if (read_rv > @as(isize, 0)) {
                _ = bytebuf_nputs(&global.in, @ptrCast(@alignCast(&buf)), @bitCast(@as(c_long, read_rv)));
            }
        }
        if (resize_has_events != 0) {
            var ignore: c_int = 0;
            _ = &ignore;
            _ = read(global.resize_pipefd[@as(c_int, 0)], @ptrCast(@alignCast(&ignore)), @sizeOf(@TypeOf(ignore)));
            if ((blk: {
                const tmp = update_term_size();
                rv = tmp;
                break :blk tmp;
            }) != TB_OK) return rv;
            if ((blk: {
                const tmp = resize_cellbufs();
                rv = tmp;
                break :blk tmp;
            }) != TB_OK) return rv;
            event.*.type = TB_EVENT_RESIZE;
            event.*.w = global.width;
            event.*.h = global.height;
            return TB_OK;
        }
        _ = memset(@ptrCast(@alignCast(event)), 0, @sizeOf(@TypeOf(event.*)));
        if ((blk: {
            const tmp = extract_event(event);
            rv = tmp;
            break :blk tmp;
        }) == TB_OK) return rv;
        if (!(timeout == -@as(c_int, 1))) break;
    }
    return rv;
}
pub fn extract_event(arg_event: [*c]struct_tb_event) callconv(.c) c_int {
    var event = arg_event;
    _ = &event;
    var rv: c_int = undefined;
    _ = &rv;
    var in: [*c]struct_bytebuf = &global.in;
    _ = &in;
    if (in.*.len == @as(usize, 0)) return -@as(c_int, 1);
    if (@as(c_int, in.*.buf[@as(c_int, 0)]) == @as(c_int, '\x1b')) {
        if (!(((global.input_mode & TB_INPUT_ESC) != 0) and (in.*.len == @as(usize, 1)))) {
            if (((blk: {
                const tmp = extract_esc(event);
                rv = tmp;
                break :blk tmp;
            }) == TB_OK) or (rv == -@as(c_int, 2))) return rv;
        }
        if ((global.input_mode & TB_INPUT_ESC) != 0) {
            event.*.type = TB_EVENT_KEY;
            event.*.ch = 0;
            event.*.key = TB_KEY_ESC;
            event.*.mod = 0;
            _ = bytebuf_shift(in, 1);
            return TB_OK;
        }
        {
            const ref = &event.*.mod;
            ref.* = @bitCast(@as(i8, @truncate(@as(c_int, ref.*) | TB_MOD_ALT)));
        }
        _ = bytebuf_shift(in, 1);
        return extract_event(event);
    }
    var is_ctrl: c_int = @intFromBool((@as(c_int, @as(u16, in.*.buf[@as(c_int, 0)])) < TB_KEY_SPACE) or (@as(c_int, in.*.buf[@as(c_int, 0)]) == TB_KEY_BACKSPACE2));
    _ = &is_ctrl;
    if (is_ctrl != 0) {
        event.*.type = TB_EVENT_KEY;
        event.*.ch = 0;
        event.*.key = in.*.buf[@as(c_int, 0)];
        {
            const ref = &event.*.mod;
            ref.* = @bitCast(@as(i8, @truncate(@as(c_int, ref.*) | TB_MOD_CTRL)));
        }
        _ = bytebuf_shift(in, 1);
        return TB_OK;
    }
    if (in.*.len >= @as(usize, @bitCast(@as(c_long, tb_utf8_char_length(in.*.buf[@as(c_int, 0)]))))) {
        event.*.type = TB_EVENT_KEY;
        _ = tb_utf8_char_to_unicode(&event.*.ch, in.*.buf);
        event.*.key = 0;
        _ = bytebuf_shift(in, @bitCast(@as(c_long, tb_utf8_char_length(in.*.buf[@as(c_int, 0)]))));
        return TB_OK;
    }
    return -@as(c_int, 1);
}
pub fn extract_esc(arg_event: [*c]struct_tb_event) callconv(.c) c_int {
    var event = arg_event;
    _ = &event;
    var rv: c_int = undefined;
    _ = &rv;
    if (((blk: {
        const tmp = extract_esc_user(event, 0);
        rv = tmp;
        break :blk tmp;
    }) == TB_OK) or (rv == -@as(c_int, 2))) return rv;
    if (((blk: {
        const tmp = extract_esc_cap(event);
        rv = tmp;
        break :blk tmp;
    }) == TB_OK) or (rv == -@as(c_int, 2))) return rv;
    if (((blk: {
        const tmp = extract_esc_mouse(event);
        rv = tmp;
        break :blk tmp;
    }) == TB_OK) or (rv == -@as(c_int, 2))) return rv;
    if (((blk: {
        const tmp = extract_esc_user(event, 1);
        rv = tmp;
        break :blk tmp;
    }) == TB_OK) or (rv == -@as(c_int, 2))) return rv;
    return -@as(c_int, 1);
}
pub fn extract_esc_user(arg_event: [*c]struct_tb_event, arg_is_post: c_int) callconv(.c) c_int {
    var event = arg_event;
    _ = &event;
    var is_post = arg_is_post;
    _ = &is_post;
    var rv: c_int = undefined;
    _ = &rv;
    var consumed: usize = 0;
    _ = &consumed;
    var in: [*c]struct_bytebuf = &global.in;
    _ = &in;
    var @"fn": ?*const fn ([*c]struct_tb_event, [*c]usize) callconv(.c) c_int = undefined;
    _ = &@"fn";
    @"fn" = if (is_post != 0) global.fn_extract_esc_post else global.fn_extract_esc_pre;
    if (!(@"fn" != null)) return -@as(c_int, 1);
    rv = @"fn".?(event, &consumed);
    if (rv == TB_OK) {
        _ = bytebuf_shift(in, consumed);
    }
    if (((blk: {
        const tmp = rv;
        rv = tmp;
        break :blk tmp;
    }) == TB_OK) or (rv == -@as(c_int, 2))) return rv;
    return -@as(c_int, 1);
}
pub fn extract_esc_cap(arg_event: [*c]struct_tb_event) callconv(.c) c_int {
    var event = arg_event;
    _ = &event;
    var rv: c_int = undefined;
    _ = &rv;
    var in: [*c]struct_bytebuf = &global.in;
    _ = &in;
    var node: [*c]struct_cap_trie = undefined;
    _ = &node;
    var depth: usize = undefined;
    _ = &depth;
    if ((blk: {
        const tmp = cap_trie_find(in.*.buf, in.*.len, &node, &depth);
        rv = tmp;
        break :blk tmp;
    }) != TB_OK) return rv;
    if (node.*.is_leaf != 0) {
        event.*.type = TB_EVENT_KEY;
        event.*.ch = 0;
        event.*.key = node.*.key;
        event.*.mod = node.*.mod;
        _ = bytebuf_shift(in, depth);
        return TB_OK;
    } else if ((node.*.nchildren > @as(usize, 0)) and (in.*.len <= depth)) {
        return -@as(c_int, 2);
    }
    return -@as(c_int, 1);
}
pub fn extract_esc_mouse(arg_event: [*c]struct_tb_event) callconv(.c) c_int {
    var event = arg_event;
    _ = &event;
    var in: [*c]struct_bytebuf = &global.in;
    _ = &in;
    var buf_shift: usize = 0;
    _ = &buf_shift;
    if (in.*.len < @as(usize, 2)) {
        return -@as(c_int, 2);
    } else if (@as(c_int, in.*.buf[@as(c_int, 1)]) != @as(c_int, '[')) {
        return -@as(c_int, 1);
    } else if (in.*.len < @as(usize, 3)) {
        return -@as(c_int, 2);
    }
    var @"type": c_int = 0;
    _ = &@"type";
    const TYPE_VT200: c_int = 0;
    _ = &TYPE_VT200;
    const TYPE_1006: c_int = 1;
    _ = &TYPE_1006;
    const TYPE_1015: c_int = 2;
    _ = &TYPE_1015;
    const TYPE_MAX: c_int = 3;
    _ = &TYPE_MAX;
    const enum_unnamed_37 = c_uint;
    _ = &enum_unnamed_37;
    if (@as(c_int, in.*.buf[@as(c_int, 2)]) == @as(c_int, 'M')) {
        @"type" = TYPE_VT200;
    } else if (@as(c_int, in.*.buf[@as(c_int, 2)]) == @as(c_int, '<')) {
        @"type" = TYPE_1006;
    } else {
        @"type" = TYPE_1015;
    }
    while (true) {
        switch (@"type") {
            TYPE_VT200 => {
                {
                    if (in.*.len < @as(usize, 6)) return -@as(c_int, 2);
                    var b: c_int = @as(c_int, in.*.buf[@as(c_int, 3)]) - @as(c_int, 32);
                    _ = &b;
                    while (true) {
                        switch (b & @as(c_int, 3)) {
                            @as(c_int, 0) => {
                                event.*.key = @bitCast(@as(c_short, @truncate(if ((b & @as(c_int, 64)) != @as(c_int, 0)) @as(c_int, 65535) - @as(c_int, 27) else @as(c_int, 65535) - @as(c_int, 23))));
                                break;
                            },
                            @as(c_int, 1) => {
                                event.*.key = @bitCast(@as(c_short, @truncate(if ((b & @as(c_int, 64)) != @as(c_int, 0)) @as(c_int, 65535) - @as(c_int, 28) else @as(c_int, 65535) - @as(c_int, 25))));
                                break;
                            },
                            @as(c_int, 2) => {
                                event.*.key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 24))));
                                break;
                            },
                            @as(c_int, 3) => {
                                event.*.key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 26))));
                                break;
                            },
                            else => {
                                return -@as(c_int, 1);
                            },
                        }
                        break;
                    }
                    if ((b & @as(c_int, 32)) != @as(c_int, 0)) {
                        {
                            const ref = &event.*.mod;
                            ref.* = @bitCast(@as(i8, @truncate(@as(c_int, ref.*) | TB_MOD_MOTION)));
                        }
                    }
                    event.*.x = @as(c_int, @as(u8, in.*.buf[@as(c_int, 4)])) - @as(c_int, 33);
                    event.*.y = @as(c_int, @as(u8, in.*.buf[@as(c_int, 5)])) - @as(c_int, 33);
                    buf_shift = 6;
                    break;
                }
            },
            TYPE_1006, TYPE_1015 => {
                {
                    var num: [3]c_int = [3]c_int{
                        -@as(c_int, 1),
                        -@as(c_int, 1),
                        -@as(c_int, 1),
                    };
                    _ = &num;
                    var num_i: c_int = 0;
                    _ = &num_i;
                    var cur_num: c_int = -@as(c_int, 1);
                    _ = &cur_num;
                    var trail: u8 = ' ';
                    _ = &trail;
                    var i: usize = 2;
                    _ = &i;
                    if (@"type" == TYPE_1006) {
                        i +%= 1;
                    }
                    while ((i < in.*.len) and (num_i < @as(c_int, 3))) {
                        var c: u8 = in.*.buf[i];
                        _ = &c;
                        if ((@as(c_int, c) >= @as(c_int, '0')) and (@as(c_int, c) <= @as(c_int, '9'))) {
                            if (cur_num == -@as(c_int, 1)) {
                                cur_num = 0;
                            }
                            cur_num *= 10;
                            cur_num += @as(c_int, c) - @as(c_int, '0');
                        } else if ((cur_num != -@as(c_int, 1)) and (((num_i < @as(c_int, 2)) and (@as(c_int, c) == @as(c_int, ';'))) or ((num_i == @as(c_int, 2)) and ((@as(c_int, c) == @as(c_int, 'm')) or (@as(c_int, c) == @as(c_int, 'M')))))) {
                            num[@bitCast(@as(isize, @intCast(num_i)))] = cur_num;
                            num_i += 1;
                            cur_num = -@as(c_int, 1);
                            trail = c;
                        } else {
                            return -@as(c_int, 1);
                        }
                        i +%= 1;
                    }
                    if (num[@as(c_int, 2)] == -@as(c_int, 1)) return -@as(c_int, 2);
                    buf_shift = i;
                    if (@"type" == TYPE_1015) {
                        num[@as(c_int, 0)] -= 32;
                    }
                    while (true) {
                        switch (num[@as(c_int, 0)] & @as(c_int, 3)) {
                            @as(c_int, 0) => {
                                event.*.key = @bitCast(@as(c_short, @truncate(if ((num[@as(c_int, 0)] & @as(c_int, 64)) != @as(c_int, 0)) @as(c_int, 65535) - @as(c_int, 27) else @as(c_int, 65535) - @as(c_int, 23))));
                                break;
                            },
                            @as(c_int, 1) => {
                                event.*.key = @bitCast(@as(c_short, @truncate(if ((num[@as(c_int, 0)] & @as(c_int, 64)) != @as(c_int, 0)) @as(c_int, 65535) - @as(c_int, 28) else @as(c_int, 65535) - @as(c_int, 25))));
                                break;
                            },
                            @as(c_int, 2) => {
                                event.*.key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 24))));
                                break;
                            },
                            @as(c_int, 3) => {
                                event.*.key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 26))));
                                break;
                            },
                            else => {
                                return -@as(c_int, 1);
                            },
                        }
                        break;
                    }
                    if (@as(c_int, trail) == @as(c_int, 'm')) {
                        event.*.key = @bitCast(@as(c_short, @truncate(@as(c_int, 65535) - @as(c_int, 26))));
                    }
                    if ((num[@as(c_int, 0)] & @as(c_int, 32)) != @as(c_int, 0)) {
                        {
                            const ref = &event.*.mod;
                            ref.* = @bitCast(@as(i8, @truncate(@as(c_int, ref.*) | TB_MOD_MOTION)));
                        }
                    }
                    event.*.x = if ((num[@as(c_int, 1)] - @as(c_int, 1)) < @as(c_int, 0)) @as(c_int, 0) else num[@as(c_int, 1)] - @as(c_int, 1);
                    event.*.y = if ((num[@as(c_int, 2)] - @as(c_int, 1)) < @as(c_int, 0)) @as(c_int, 0) else num[@as(c_int, 2)] - @as(c_int, 1);
                    break;
                }
            },
            else => {},
        }
        break;
    }
    if (buf_shift > @as(usize, 0)) {
        _ = bytebuf_shift(in, buf_shift);
    }
    event.*.type = TB_EVENT_MOUSE;
    return TB_OK;
}
pub fn resize_cellbufs() callconv(.c) c_int {
    var rv: c_int = undefined;
    _ = &rv;
    if ((blk: {
        const tmp = cellbuf_resize(&global.back, global.width, global.height);
        rv = tmp;
        break :blk tmp;
    }) != TB_OK) return rv;
    if ((blk: {
        const tmp = cellbuf_resize(&global.front, global.width, global.height);
        rv = tmp;
        break :blk tmp;
    }) != TB_OK) return rv;
    if ((blk: {
        const tmp = cellbuf_clear(&global.front);
        rv = tmp;
        break :blk tmp;
    }) != TB_OK) return rv;
    if ((blk: {
        const tmp = send_clear();
        rv = tmp;
        break :blk tmp;
    }) != TB_OK) return rv;
    return TB_OK;
}
pub fn handle_resize(arg_sig: c_int) callconv(.c) void {
    var sig = arg_sig;
    _ = &sig;
    var errno_copy: c_int = __errno_location().*;
    _ = &errno_copy;
    _ = write(global.resize_pipefd[@as(c_int, 1)], @ptrCast(@alignCast(&sig)), @sizeOf(@TypeOf(sig)));
    __errno_location().* = errno_copy;
}
pub fn send_attr(arg_fg: uintattr_t, arg_bg: uintattr_t) callconv(.c) c_int {
    var fg = arg_fg;
    _ = &fg;
    var bg = arg_bg;
    _ = &bg;
    var rv: c_int = undefined;
    _ = &rv;
    if ((fg == global.last_fg) and (bg == global.last_bg)) {
        return TB_OK;
    }
    if ((blk: {
        const tmp = bytebuf_puts(&global.out, global.caps[TB_CAP_SGR0]);
        rv = tmp;
        break :blk tmp;
    }) != TB_OK) return rv;
    var cfg: u32 = undefined;
    _ = &cfg;
    var cbg: u32 = undefined;
    _ = &cbg;
    while (true) {
        switch (global.output_mode) {
            else => {
                cfg = (@as(uintattr_t, @bitCast(@as(c_int, if ((fg & @as(uintattr_t, TB_BRIGHT)) != 0) @as(c_int, 90) else @as(c_int, 30)))) +% (fg & @as(uintattr_t, 15))) -% @as(uintattr_t, 1);
                cbg = (@as(uintattr_t, @bitCast(@as(c_int, if ((bg & @as(uintattr_t, TB_BRIGHT)) != 0) @as(c_int, 100) else @as(c_int, 40)))) +% (bg & @as(uintattr_t, 15))) -% @as(uintattr_t, 1);
                break;
            },
            TB_OUTPUT_256 => {
                cfg = fg & @as(uintattr_t, 255);
                cbg = bg & @as(uintattr_t, 255);
                if ((fg & @as(uintattr_t, TB_HI_BLACK)) != 0) {
                    cfg = 0;
                }
                if ((bg & @as(uintattr_t, TB_HI_BLACK)) != 0) {
                    cbg = 0;
                }
                break;
            },
            TB_OUTPUT_216 => {
                cfg = fg & @as(uintattr_t, 255);
                cbg = bg & @as(uintattr_t, 255);
                if (cfg > @as(u32, 216)) {
                    cfg = 216;
                }
                if (cbg > @as(u32, 216)) {
                    cbg = 216;
                }
                cfg +%= 15;
                cbg +%= 15;
                break;
            },
            TB_OUTPUT_GRAYSCALE => {
                cfg = fg & @as(uintattr_t, 255);
                cbg = bg & @as(uintattr_t, 255);
                if (cfg > @as(u32, 24)) {
                    cfg = 24;
                }
                if (cbg > @as(u32, 24)) {
                    cbg = 24;
                }
                cfg +%= 231;
                cbg +%= 231;
                break;
            },
            TB_OUTPUT_TRUECOLOR => {
                cfg = fg & @as(uintattr_t, 16777215);
                cbg = bg & @as(uintattr_t, 16777215);
                if ((fg & @as(uintattr_t, TB_HI_BLACK)) != 0) {
                    cfg = 0;
                }
                if ((bg & @as(uintattr_t, TB_HI_BLACK)) != 0) {
                    cbg = 0;
                }
                break;
            },
        }
        break;
    }
    if ((fg & @as(uintattr_t, TB_BOLD)) != 0) if ((blk: {
        const tmp = bytebuf_puts(&global.out, global.caps[TB_CAP_BOLD]);
        rv = tmp;
        break :blk tmp;
    }) != TB_OK) return rv;
    if ((fg & @as(uintattr_t, TB_BLINK)) != 0) if ((blk: {
        const tmp = bytebuf_puts(&global.out, global.caps[TB_CAP_BLINK]);
        rv = tmp;
        break :blk tmp;
    }) != TB_OK) return rv;
    if ((fg & @as(uintattr_t, TB_UNDERLINE)) != 0) if ((blk: {
        const tmp = bytebuf_puts(&global.out, global.caps[TB_CAP_UNDERLINE]);
        rv = tmp;
        break :blk tmp;
    }) != TB_OK) return rv;
    if ((fg & @as(uintattr_t, TB_ITALIC)) != 0) if ((blk: {
        const tmp = bytebuf_puts(&global.out, global.caps[TB_CAP_ITALIC]);
        rv = tmp;
        break :blk tmp;
    }) != TB_OK) return rv;
    if ((fg & TB_DIM) != 0) if ((blk: {
        const tmp = bytebuf_puts(&global.out, global.caps[TB_CAP_DIM]);
        rv = tmp;
        break :blk tmp;
    }) != TB_OK) return rv;
    if (((fg & @as(uintattr_t, TB_REVERSE)) != 0) or ((bg & @as(uintattr_t, TB_REVERSE)) != 0)) if ((blk: {
        const tmp = bytebuf_puts(&global.out, global.caps[TB_CAP_REVERSE]);
        rv = tmp;
        break :blk tmp;
    }) != TB_OK) return rv;
    var fg_is_default: c_int = @intFromBool((fg & @as(uintattr_t, 255)) == @as(uintattr_t, 0));
    _ = &fg_is_default;
    var bg_is_default: c_int = @intFromBool((bg & @as(uintattr_t, 255)) == @as(uintattr_t, 0));
    _ = &bg_is_default;
    if (global.output_mode == TB_OUTPUT_256) {
        if ((fg & @as(uintattr_t, TB_HI_BLACK)) != 0) {
            fg_is_default = 0;
        }
        if ((bg & @as(uintattr_t, TB_HI_BLACK)) != 0) {
            bg_is_default = 0;
        }
    }
    if (global.output_mode == TB_OUTPUT_TRUECOLOR) {
        fg_is_default = @intFromBool(((fg & @as(uintattr_t, 16777215)) == @as(uintattr_t, 0)) and ((fg & @as(uintattr_t, TB_HI_BLACK)) == @as(uintattr_t, 0)));
        bg_is_default = @intFromBool(((bg & @as(uintattr_t, 16777215)) == @as(uintattr_t, 0)) and ((bg & @as(uintattr_t, TB_HI_BLACK)) == @as(uintattr_t, 0)));
    }
    if ((blk: {
        const tmp = send_sgr(cfg, cbg, fg_is_default, bg_is_default);
        rv = tmp;
        break :blk tmp;
    }) != TB_OK) return rv;
    global.last_fg = fg;
    global.last_bg = bg;
    return TB_OK;
}
pub fn send_sgr(arg_cfg: u32, arg_cbg: u32, arg_fg_is_default: c_int, arg_bg_is_default: c_int) callconv(.c) c_int {
    var cfg = arg_cfg;
    _ = &cfg;
    var cbg = arg_cbg;
    _ = &cbg;
    var fg_is_default = arg_fg_is_default;
    _ = &fg_is_default;
    var bg_is_default = arg_bg_is_default;
    _ = &bg_is_default;
    var rv: c_int = undefined;
    _ = &rv;
    var nbuf: [32]u8 = undefined;
    _ = &nbuf;
    if ((fg_is_default != 0) and (bg_is_default != 0)) {
        return TB_OK;
    }
    while (true) {
        switch (global.output_mode) {
            else => {
                if ((blk: {
                    const tmp = bytebuf_nputs(&global.out, "\x1b[", @sizeOf(@TypeOf("\x1b[".*)) -% @as(c_ulong, 1));
                    rv = tmp;
                    break :blk tmp;
                }) != TB_OK) return rv;
                if (!(fg_is_default != 0)) {
                    if ((blk: {
                        const tmp = bytebuf_nputs(&global.out, @ptrCast(@alignCast(&nbuf)), @bitCast(@as(c_long, convert_num(cfg, @ptrCast(@alignCast(&nbuf))))));
                        rv = tmp;
                        break :blk tmp;
                    }) != TB_OK) return rv;
                    if (!(bg_is_default != 0)) {
                        if ((blk: {
                            const tmp = bytebuf_nputs(&global.out, ";", @sizeOf(@TypeOf(";".*)) -% @as(c_ulong, 1));
                            rv = tmp;
                            break :blk tmp;
                        }) != TB_OK) return rv;
                    }
                }
                if (!(bg_is_default != 0)) {
                    if ((blk: {
                        const tmp = bytebuf_nputs(&global.out, @ptrCast(@alignCast(&nbuf)), @bitCast(@as(c_long, convert_num(cbg, @ptrCast(@alignCast(&nbuf))))));
                        rv = tmp;
                        break :blk tmp;
                    }) != TB_OK) return rv;
                }
                if ((blk: {
                    const tmp = bytebuf_nputs(&global.out, "m", @sizeOf(@TypeOf("m".*)) -% @as(c_ulong, 1));
                    rv = tmp;
                    break :blk tmp;
                }) != TB_OK) return rv;
                break;
            },
            TB_OUTPUT_256, TB_OUTPUT_216, TB_OUTPUT_GRAYSCALE => {
                if ((blk: {
                    const tmp = bytebuf_nputs(&global.out, "\x1b[", @sizeOf(@TypeOf("\x1b[".*)) -% @as(c_ulong, 1));
                    rv = tmp;
                    break :blk tmp;
                }) != TB_OK) return rv;
                if (!(fg_is_default != 0)) {
                    if ((blk: {
                        const tmp = bytebuf_nputs(&global.out, "38;5;", @sizeOf(@TypeOf("38;5;".*)) -% @as(c_ulong, 1));
                        rv = tmp;
                        break :blk tmp;
                    }) != TB_OK) return rv;
                    if ((blk: {
                        const tmp = bytebuf_nputs(&global.out, @ptrCast(@alignCast(&nbuf)), @bitCast(@as(c_long, convert_num(cfg, @ptrCast(@alignCast(&nbuf))))));
                        rv = tmp;
                        break :blk tmp;
                    }) != TB_OK) return rv;
                    if (!(bg_is_default != 0)) {
                        if ((blk: {
                            const tmp = bytebuf_nputs(&global.out, ";", @sizeOf(@TypeOf(";".*)) -% @as(c_ulong, 1));
                            rv = tmp;
                            break :blk tmp;
                        }) != TB_OK) return rv;
                    }
                }
                if (!(bg_is_default != 0)) {
                    if ((blk: {
                        const tmp = bytebuf_nputs(&global.out, "48;5;", @sizeOf(@TypeOf("48;5;".*)) -% @as(c_ulong, 1));
                        rv = tmp;
                        break :blk tmp;
                    }) != TB_OK) return rv;
                    if ((blk: {
                        const tmp = bytebuf_nputs(&global.out, @ptrCast(@alignCast(&nbuf)), @bitCast(@as(c_long, convert_num(cbg, @ptrCast(@alignCast(&nbuf))))));
                        rv = tmp;
                        break :blk tmp;
                    }) != TB_OK) return rv;
                }
                if ((blk: {
                    const tmp = bytebuf_nputs(&global.out, "m", @sizeOf(@TypeOf("m".*)) -% @as(c_ulong, 1));
                    rv = tmp;
                    break :blk tmp;
                }) != TB_OK) return rv;
                break;
            },
            TB_OUTPUT_TRUECOLOR => {
                if ((blk: {
                    const tmp = bytebuf_nputs(&global.out, "\x1b[", @sizeOf(@TypeOf("\x1b[".*)) -% @as(c_ulong, 1));
                    rv = tmp;
                    break :blk tmp;
                }) != TB_OK) return rv;
                if (!(fg_is_default != 0)) {
                    if ((blk: {
                        const tmp = bytebuf_nputs(&global.out, "38;2;", @sizeOf(@TypeOf("38;2;".*)) -% @as(c_ulong, 1));
                        rv = tmp;
                        break :blk tmp;
                    }) != TB_OK) return rv;
                    if ((blk: {
                        const tmp = bytebuf_nputs(&global.out, @ptrCast(@alignCast(&nbuf)), @bitCast(@as(c_long, convert_num((cfg >> @intCast(@as(u32, 16))) & @as(u32, 255), @ptrCast(@alignCast(&nbuf))))));
                        rv = tmp;
                        break :blk tmp;
                    }) != TB_OK) return rv;
                    if ((blk: {
                        const tmp = bytebuf_nputs(&global.out, ";", @sizeOf(@TypeOf(";".*)) -% @as(c_ulong, 1));
                        rv = tmp;
                        break :blk tmp;
                    }) != TB_OK) return rv;
                    if ((blk: {
                        const tmp = bytebuf_nputs(&global.out, @ptrCast(@alignCast(&nbuf)), @bitCast(@as(c_long, convert_num((cfg >> @intCast(@as(u32, 8))) & @as(u32, 255), @ptrCast(@alignCast(&nbuf))))));
                        rv = tmp;
                        break :blk tmp;
                    }) != TB_OK) return rv;
                    if ((blk: {
                        const tmp = bytebuf_nputs(&global.out, ";", @sizeOf(@TypeOf(";".*)) -% @as(c_ulong, 1));
                        rv = tmp;
                        break :blk tmp;
                    }) != TB_OK) return rv;
                    if ((blk: {
                        const tmp = bytebuf_nputs(&global.out, @ptrCast(@alignCast(&nbuf)), @bitCast(@as(c_long, convert_num(cfg & @as(u32, 255), @ptrCast(@alignCast(&nbuf))))));
                        rv = tmp;
                        break :blk tmp;
                    }) != TB_OK) return rv;
                    if (!(bg_is_default != 0)) {
                        if ((blk: {
                            const tmp = bytebuf_nputs(&global.out, ";", @sizeOf(@TypeOf(";".*)) -% @as(c_ulong, 1));
                            rv = tmp;
                            break :blk tmp;
                        }) != TB_OK) return rv;
                    }
                }
                if (!(bg_is_default != 0)) {
                    if ((blk: {
                        const tmp = bytebuf_nputs(&global.out, "48;2;", @sizeOf(@TypeOf("48;2;".*)) -% @as(c_ulong, 1));
                        rv = tmp;
                        break :blk tmp;
                    }) != TB_OK) return rv;
                    if ((blk: {
                        const tmp = bytebuf_nputs(&global.out, @ptrCast(@alignCast(&nbuf)), @bitCast(@as(c_long, convert_num((cbg >> @intCast(@as(u32, 16))) & @as(u32, 255), @ptrCast(@alignCast(&nbuf))))));
                        rv = tmp;
                        break :blk tmp;
                    }) != TB_OK) return rv;
                    if ((blk: {
                        const tmp = bytebuf_nputs(&global.out, ";", @sizeOf(@TypeOf(";".*)) -% @as(c_ulong, 1));
                        rv = tmp;
                        break :blk tmp;
                    }) != TB_OK) return rv;
                    if ((blk: {
                        const tmp = bytebuf_nputs(&global.out, @ptrCast(@alignCast(&nbuf)), @bitCast(@as(c_long, convert_num((cbg >> @intCast(@as(u32, 8))) & @as(u32, 255), @ptrCast(@alignCast(&nbuf))))));
                        rv = tmp;
                        break :blk tmp;
                    }) != TB_OK) return rv;
                    if ((blk: {
                        const tmp = bytebuf_nputs(&global.out, ";", @sizeOf(@TypeOf(";".*)) -% @as(c_ulong, 1));
                        rv = tmp;
                        break :blk tmp;
                    }) != TB_OK) return rv;
                    if ((blk: {
                        const tmp = bytebuf_nputs(&global.out, @ptrCast(@alignCast(&nbuf)), @bitCast(@as(c_long, convert_num(cbg & @as(u32, 255), @ptrCast(@alignCast(&nbuf))))));
                        rv = tmp;
                        break :blk tmp;
                    }) != TB_OK) return rv;
                }
                if ((blk: {
                    const tmp = bytebuf_nputs(&global.out, "m", @sizeOf(@TypeOf("m".*)) -% @as(c_ulong, 1));
                    rv = tmp;
                    break :blk tmp;
                }) != TB_OK) return rv;
                break;
            },
        }
        break;
    }
    return TB_OK;
}
pub fn send_cursor_if(arg_x: c_int, arg_y: c_int) callconv(.c) c_int {
    var x = arg_x;
    _ = &x;
    var y = arg_y;
    _ = &y;
    var rv: c_int = undefined;
    _ = &rv;
    var nbuf: [32]u8 = undefined;
    _ = &nbuf;
    if ((x < @as(c_int, 0)) or (y < @as(c_int, 0))) {
        return TB_OK;
    }
    if ((blk: {
        const tmp = bytebuf_nputs(&global.out, "\x1b[", @sizeOf(@TypeOf("\x1b[".*)) -% @as(c_ulong, 1));
        rv = tmp;
        break :blk tmp;
    }) != TB_OK) return rv;
    if ((blk: {
        const tmp = bytebuf_nputs(&global.out, @ptrCast(@alignCast(&nbuf)), @bitCast(@as(c_long, convert_num(@bitCast(@as(c_int, y + @as(c_int, 1))), @ptrCast(@alignCast(&nbuf))))));
        rv = tmp;
        break :blk tmp;
    }) != TB_OK) return rv;
    if ((blk: {
        const tmp = bytebuf_nputs(&global.out, ";", @sizeOf(@TypeOf(";".*)) -% @as(c_ulong, 1));
        rv = tmp;
        break :blk tmp;
    }) != TB_OK) return rv;
    if ((blk: {
        const tmp = bytebuf_nputs(&global.out, @ptrCast(@alignCast(&nbuf)), @bitCast(@as(c_long, convert_num(@bitCast(@as(c_int, x + @as(c_int, 1))), @ptrCast(@alignCast(&nbuf))))));
        rv = tmp;
        break :blk tmp;
    }) != TB_OK) return rv;
    if ((blk: {
        const tmp = bytebuf_nputs(&global.out, "H", @sizeOf(@TypeOf("H".*)) -% @as(c_ulong, 1));
        rv = tmp;
        break :blk tmp;
    }) != TB_OK) return rv;
    return TB_OK;
}
pub fn send_char(arg_x: c_int, arg_y: c_int, arg_ch: u32) callconv(.c) c_int {
    var x = arg_x;
    _ = &x;
    var y = arg_y;
    _ = &y;
    var ch = arg_ch;
    _ = &ch;
    return send_cluster(x, y, &ch, 1);
}
pub fn send_cluster(arg_x: c_int, arg_y: c_int, arg_ch: [*c]u32, arg_nch: usize) callconv(.c) c_int {
    var x = arg_x;
    _ = &x;
    var y = arg_y;
    _ = &y;
    var ch = arg_ch;
    _ = &ch;
    var nch = arg_nch;
    _ = &nch;
    var rv: c_int = undefined;
    _ = &rv;
    var chu8: [8]u8 = undefined;
    _ = &chu8;
    if ((global.last_x != (x - @as(c_int, 1))) or (global.last_y != y)) {
        if ((blk: {
            const tmp = send_cursor_if(x, y);
            rv = tmp;
            break :blk tmp;
        }) != TB_OK) return rv;
    }
    global.last_x = x;
    global.last_y = y;
    var i: c_int = undefined;
    _ = &i;
    {
        i = 0;
        while (i < @as(c_int, @bitCast(@as(c_uint, @truncate(nch))))) : (i += 1) {
            var ch32: u32 = (ch + @as(usize, @bitCast(@as(isize, @intCast(i))))).*;
            _ = &ch32;
            if (!(tb_iswprint(ch32) != 0)) {
                ch32 = 65533;
            }
            var chu8_len: c_int = tb_utf8_unicode_to_char(@ptrCast(@alignCast(&chu8)), ch32);
            _ = &chu8_len;
            if ((blk: {
                const tmp = bytebuf_nputs(&global.out, @ptrCast(@alignCast(&chu8)), @bitCast(@as(c_long, chu8_len)));
                rv = tmp;
                break :blk tmp;
            }) != TB_OK) return rv;
        }
    }
    return TB_OK;
}
pub fn convert_num(arg_num: u32, arg_buf: [*c]u8) callconv(.c) c_int {
    var num = arg_num;
    _ = &num;
    var buf = arg_buf;
    _ = &buf;
    var i: c_int = undefined;
    _ = &i;
    var l: c_int = 0;
    _ = &l;
    var ch: u8 = undefined;
    _ = &ch;
    while (true) {
        buf[
            @bitCast(@as(isize, @intCast(blk: {
                const ref = &l;
                const tmp = ref.*;
                ref.* += 1;
                break :blk tmp;
            })))
        ] = @truncate(@as(u32, '0') +% (num % @as(u32, 10)));
        num /= 10;
        if (!(num != 0)) break;
    }
    {
        i = 0;
        while (i < @divTrunc(l, @as(c_int, 2))) : (i += 1) {
            ch = buf[@bitCast(@as(isize, @intCast(i)))];
            buf[@bitCast(@as(isize, @intCast(i)))] = buf[@bitCast(@as(isize, @intCast((l - @as(c_int, 1)) - i)))];
            buf[@bitCast(@as(isize, @intCast((l - @as(c_int, 1)) - i)))] = ch;
        }
    }
    return l;
}
pub fn cell_cmp(arg_a: [*c]struct_tb_cell, arg_b: [*c]struct_tb_cell) callconv(.c) c_int {
    var a = arg_a;
    _ = &a;
    var b = arg_b;
    _ = &b;
    if (((a.*.ch != b.*.ch) or (a.*.fg != b.*.fg)) or (a.*.bg != b.*.bg)) {
        return 1;
    }
    return 0;
}
pub fn cell_copy(arg_dst: [*c]struct_tb_cell, arg_src: [*c]struct_tb_cell) callconv(.c) c_int {
    var dst = arg_dst;
    _ = &dst;
    var src = arg_src;
    _ = &src;
    return cell_set(dst, &src.*.ch, 1, src.*.fg, src.*.bg);
}
pub fn cell_set(arg_cell: [*c]struct_tb_cell, arg_ch: [*c]u32, arg_nch: usize, arg_fg: uintattr_t, arg_bg: uintattr_t) callconv(.c) c_int {
    var cell = arg_cell;
    _ = &cell;
    var ch = arg_ch;
    _ = &ch;
    var nch = arg_nch;
    _ = &nch;
    var fg = arg_fg;
    _ = &fg;
    var bg = arg_bg;
    _ = &bg;
    cell.*.ch = if (ch != null) ch.* else @as(u32, 0);
    cell.*.fg = fg;
    cell.*.bg = bg;
    _ = &nch;
    _ = &cell_reserve_ech;
    return TB_OK;
}
pub fn cell_reserve_ech(arg_cell: [*c]struct_tb_cell, arg_n: usize) callconv(.c) c_int {
    var cell = arg_cell;
    _ = &cell;
    var n = arg_n;
    _ = &n;
    _ = &cell;
    _ = &n;
    return -@as(c_int, 1);
}
pub fn cell_free(arg_cell: [*c]struct_tb_cell) callconv(.c) c_int {
    var cell = arg_cell;
    _ = &cell;
    _ = memset(@ptrCast(@alignCast(cell)), 0, @sizeOf(@TypeOf(cell.*)));
    return TB_OK;
}
pub fn cellbuf_init(arg_c: [*c]struct_cellbuf, arg_w: c_int, arg_h: c_int) callconv(.c) c_int {
    var c = arg_c;
    _ = &c;
    var w = arg_w;
    _ = &w;
    var h = arg_h;
    _ = &h;
    c.*.cells = @ptrCast(@alignCast(malloc((@sizeOf(struct_tb_cell) *% @as(c_ulong, @bitCast(@as(c_long, w)))) *% @as(c_ulong, @bitCast(@as(c_long, h))))));
    if (!(c.*.cells != null)) return -@as(c_int, 5);
    _ = memset(@ptrCast(@alignCast(c.*.cells)), 0, (@sizeOf(struct_tb_cell) *% @as(c_ulong, @bitCast(@as(c_long, w)))) *% @as(c_ulong, @bitCast(@as(c_long, h))));
    c.*.width = w;
    c.*.height = h;
    return TB_OK;
}
pub fn cellbuf_free(arg_c: [*c]struct_cellbuf) callconv(.c) c_int {
    var c = arg_c;
    _ = &c;
    if (c.*.cells != null) {
        var i: c_int = undefined;
        _ = &i;
        {
            i = 0;
            while (i < (c.*.width * c.*.height)) : (i += 1) {
                _ = cell_free(&c.*.cells[@bitCast(@as(isize, @intCast(i)))]);
            }
        }
        free(@ptrCast(@alignCast(c.*.cells)));
    }
    _ = memset(@ptrCast(@alignCast(c)), 0, @sizeOf(@TypeOf(c.*)));
    return TB_OK;
}
pub fn cellbuf_clear(arg_c: [*c]struct_cellbuf) callconv(.c) c_int {
    var c = arg_c;
    _ = &c;
    var rv: c_int = undefined;
    _ = &rv;
    var i: c_int = undefined;
    _ = &i;
    var space: u32 = @bitCast(@as(c_int, @as(c_int, ' ')));
    _ = &space;
    {
        i = 0;
        while (i < (c.*.width * c.*.height)) : (i += 1) {
            if ((blk: {
                const tmp = cell_set(&c.*.cells[@bitCast(@as(isize, @intCast(i)))], &space, 1, global.fg, global.bg);
                rv = tmp;
                break :blk tmp;
            }) != TB_OK) return rv;
        }
    }
    return TB_OK;
}
pub fn cellbuf_get(arg_c: [*c]struct_cellbuf, arg_x: c_int, arg_y: c_int, arg_out: [*c][*c]struct_tb_cell) callconv(.c) c_int {
    var c = arg_c;
    _ = &c;
    var x = arg_x;
    _ = &x;
    var y = arg_y;
    _ = &y;
    var out = arg_out;
    _ = &out;
    if (!(cellbuf_in_bounds(c, x, y) != 0)) {
        out.* = null;
        return -@as(c_int, 9);
    }
    out.* = &c.*.cells[@bitCast(@as(isize, @intCast((y * c.*.width) + x)))];
    return TB_OK;
}
pub fn cellbuf_in_bounds(arg_c: [*c]struct_cellbuf, arg_x: c_int, arg_y: c_int) callconv(.c) c_int {
    var c = arg_c;
    _ = &c;
    var x = arg_x;
    _ = &x;
    var y = arg_y;
    _ = &y;
    if ((((x < @as(c_int, 0)) or (x >= c.*.width)) or (y < @as(c_int, 0))) or (y >= c.*.height)) {
        return 0;
    }
    return 1;
}
pub fn cellbuf_resize(arg_c: [*c]struct_cellbuf, arg_w: c_int, arg_h: c_int) callconv(.c) c_int {
    var c = arg_c;
    _ = &c;
    var w = arg_w;
    _ = &w;
    var h = arg_h;
    _ = &h;
    var rv: c_int = undefined;
    _ = &rv;
    var ow: c_int = c.*.width;
    _ = &ow;
    var oh: c_int = c.*.height;
    _ = &oh;
    if ((ow == w) and (oh == h)) {
        return TB_OK;
    }
    w = if (w < @as(c_int, 1)) @as(c_int, 1) else w;
    h = if (h < @as(c_int, 1)) @as(c_int, 1) else h;
    var minw: c_int = if (w < ow) w else ow;
    _ = &minw;
    var minh: c_int = if (h < oh) h else oh;
    _ = &minh;
    var prev: [*c]struct_tb_cell = c.*.cells;
    _ = &prev;
    if ((blk: {
        const tmp = cellbuf_init(c, w, h);
        rv = tmp;
        break :blk tmp;
    }) != TB_OK) return rv;
    if ((blk: {
        const tmp = cellbuf_clear(c);
        rv = tmp;
        break :blk tmp;
    }) != TB_OK) return rv;
    var x: c_int = undefined;
    _ = &x;
    var y: c_int = undefined;
    _ = &y;
    {
        x = 0;
        while (x < minw) : (x += 1) {
            {
                y = 0;
                while (y < minh) : (y += 1) {
                    var src: [*c]struct_tb_cell = undefined;
                    _ = &src;
                    var dst: [*c]struct_tb_cell = undefined;
                    _ = &dst;
                    src = &prev[@bitCast(@as(isize, @intCast((y * ow) + x)))];
                    if ((blk: {
                        const tmp = cellbuf_get(c, x, y, &dst);
                        rv = tmp;
                        break :blk tmp;
                    }) != TB_OK) return rv;
                    if ((blk: {
                        const tmp = cell_copy(dst, src);
                        rv = tmp;
                        break :blk tmp;
                    }) != TB_OK) return rv;
                }
            }
        }
    }
    free(@ptrCast(@alignCast(prev)));
    return TB_OK;
}
pub fn bytebuf_puts(arg_b: [*c]struct_bytebuf, arg_str: [*c]const u8) callconv(.c) c_int {
    var b = arg_b;
    _ = &b;
    var str = arg_str;
    _ = &str;
    if (!(str != null) or (strlen(str) <= @as(usize, 0))) return TB_OK;
    return bytebuf_nputs(b, str, strlen(str));
}
pub fn bytebuf_nputs(arg_b: [*c]struct_bytebuf, arg_str: [*c]const u8, arg_nstr: usize) callconv(.c) c_int {
    var b = arg_b;
    _ = &b;
    var str = arg_str;
    _ = &str;
    var nstr = arg_nstr;
    _ = &nstr;
    var rv: c_int = undefined;
    _ = &rv;
    if ((blk: {
        const tmp = bytebuf_reserve(b, (b.*.len +% nstr) +% @as(usize, 1));
        rv = tmp;
        break :blk tmp;
    }) != TB_OK) return rv;
    _ = memcpy(@ptrCast(@alignCast(b.*.buf + b.*.len)), @ptrCast(@alignCast(str)), nstr);
    b.*.len +%= nstr;
    b.*.buf[b.*.len] = '\x00';
    return TB_OK;
}
pub fn bytebuf_shift(arg_b: [*c]struct_bytebuf, arg_n: usize) callconv(.c) c_int {
    var b = arg_b;
    _ = &b;
    var n = arg_n;
    _ = &n;
    if (n > b.*.len) {
        n = b.*.len;
    }
    var nmove: usize = b.*.len -% n;
    _ = &nmove;
    _ = memmove(@ptrCast(@alignCast(b.*.buf)), @ptrCast(@alignCast(b.*.buf + n)), nmove);
    b.*.len -%= n;
    return TB_OK;
}
pub fn bytebuf_flush(arg_b: [*c]struct_bytebuf, arg_fd: c_int) callconv(.c) c_int {
    var b = arg_b;
    _ = &b;
    var fd = arg_fd;
    _ = &fd;
    if (b.*.len <= @as(usize, 0)) return TB_OK;
    var write_rv: isize = write(fd, @ptrCast(@alignCast(b.*.buf)), b.*.len);
    _ = &write_rv;
    if ((write_rv < @as(isize, 0)) or (@as(usize, @bitCast(@as(c_long, write_rv))) != b.*.len)) {
        global.last_errno = __errno_location().*;
        return -@as(c_int, 1);
    }
    b.*.len = 0;
    return TB_OK;
}
pub fn bytebuf_reserve(arg_b: [*c]struct_bytebuf, arg_sz: usize) callconv(.c) c_int {
    var b = arg_b;
    _ = &b;
    var sz = arg_sz;
    _ = &sz;
    if (b.*.cap >= sz) return TB_OK;
    var newcap: usize = if (b.*.cap > @as(usize, 0)) b.*.cap else @as(usize, 1);
    _ = &newcap;
    while (newcap < sz) {
        newcap *%= 2;
    }
    var newbuf: [*c]u8 = undefined;
    _ = &newbuf;
    if (b.*.buf != null) {
        newbuf = @ptrCast(@alignCast(realloc(@ptrCast(@alignCast(b.*.buf)), newcap)));
    } else {
        newbuf = @ptrCast(@alignCast(malloc(newcap)));
    }
    if (!(newbuf != null)) return -@as(c_int, 5);
    b.*.buf = newbuf;
    b.*.cap = newcap;
    return TB_OK;
}
pub fn bytebuf_free(arg_b: [*c]struct_bytebuf) callconv(.c) c_int {
    var b = arg_b;
    _ = &b;
    if (b.*.buf != null) {
        free(@ptrCast(@alignCast(b.*.buf)));
    }
    _ = memset(@ptrCast(@alignCast(b)), 0, @sizeOf(@TypeOf(b.*)));
    return TB_OK;
}
pub fn tb_iswprint_ex(arg_ch: u32, arg_w: [*c]c_int) callconv(.c) c_int {
    var ch = arg_ch;
    _ = &ch;
    var w = arg_w;
    _ = &w;
    if (((ch >= @as(u32, 32)) and (ch <= @as(u32, 126))) or ((ch >= @as(u32, 160)) and (ch <= @as(u32, 255)))) {
        if (w != null) {
            w.* = 1;
        }
        return 1;
    } else if (ch <= @as(u32, 255)) {
        if (w != null) {
            w.* = if (ch == @as(u32, 0)) @as(c_int, 0) else -@as(c_int, 1);
        }
        return 0;
    }
    var lo: c_int = 0;
    _ = &lo;
    var hi: c_int = WCWIDTH_TABLE_LENGTH - @as(c_int, 1);
    _ = &hi;
    while (lo <= hi) {
        var i: c_int = @divTrunc(lo + hi, @as(c_int, 2));
        _ = &i;
        if (ch < wcwidth_table[@bitCast(@as(isize, @intCast(i)))].range_start) {
            hi = i - @as(c_int, 1);
        } else if (ch > wcwidth_table[@bitCast(@as(isize, @intCast(i)))].range_end) {
            lo = i + @as(c_int, 1);
        } else {
            if (w != null) {
                w.* = wcwidth_table[@bitCast(@as(isize, @intCast(i)))].width;
            }
            return if (wcwidth_table[@bitCast(@as(isize, @intCast(i)))].width >= @as(c_int, 0)) @as(c_int, 1) else @as(c_int, 0);
        }
    }
    if (w != null) {
        w.* = -@as(c_int, 1);
    }
    return 0;
}
pub fn tb_cluster_width(arg_ch: [*c]u32, arg_nch: usize) callconv(.c) c_int {
    var ch = arg_ch;
    _ = &ch;
    var nch = arg_nch;
    _ = &nch;
    var wmax: c_int = -@as(c_int, 1);
    _ = &wmax;
    var vs15: c_int = 0;
    _ = &vs15;
    var vs16: c_int = 0;
    _ = &vs16;
    var ri: c_int = 0;
    _ = &ri;
    var zwj: c_int = 0;
    _ = &zwj;
    var i: usize = 0;
    _ = &i;
    {
        i = 0;
        while (i < nch) : (i +%= 1) {
            var c: u32 = ch[i];
            _ = &c;
            while (true) {
                switch (c) {
                    @as(u32, 65038) => {
                        vs15 += 1;
                        break;
                    },
                    @as(u32, 65039) => {
                        vs16 += 1;
                        break;
                    },
                    @as(u32, 8205) => {
                        zwj += 1;
                        break;
                    },
                    else => {
                        if ((c >= @as(u32, 127462)) and (c <= @as(u32, 127487))) {
                            ri += 1;
                        }
                    },
                }
                break;
            }
            var w: c_int = tb_wcwidth(c);
            _ = &w;
            if (w > wmax) {
                wmax = w;
            }
        }
    }
    if (wmax >= @as(c_int, 1)) {
        if (vs15 != 0) return 1 else if (((vs16 != 0) or (zwj != 0)) or (ri >= @as(c_int, 2))) return 2;
    }
    return wmax;
}

pub const __VERSION__ = "Aro 0.0.0";
pub const __Aro__ = "";
pub const __STDC__ = @as(c_int, 1);
pub const __STDC_HOSTED__ = @as(c_int, 1);
pub const __STDC_UTF_16__ = @as(c_int, 1);
pub const __STDC_UTF_32__ = @as(c_int, 1);
pub const __STDC_EMBED_NOT_FOUND__ = @as(c_int, 0);
pub const __STDC_EMBED_FOUND__ = @as(c_int, 1);
pub const __STDC_EMBED_EMPTY__ = @as(c_int, 2);
pub const __STDC_VERSION__ = @as(c_long, 201710);
pub const __GNUC__ = @as(c_int, 7);
pub const __GNUC_MINOR__ = @as(c_int, 1);
pub const __GNUC_PATCHLEVEL__ = @as(c_int, 0);
pub const __ARO_EMULATE_NO__ = @as(c_int, 0);
pub const __ARO_EMULATE_CLANG__ = @as(c_int, 1);
pub const __ARO_EMULATE_GCC__ = @as(c_int, 2);
pub const __ARO_EMULATE_MSVC__ = @as(c_int, 3);
pub const __ARO_EMULATE__ = __ARO_EMULATE_GCC__;
pub inline fn __building_module(x: anytype) @TypeOf(@as(c_int, 0)) {
    _ = &x;
    return @as(c_int, 0);
}
pub const linux = @as(c_int, 1);
pub const __linux = @as(c_int, 1);
pub const __linux__ = @as(c_int, 1);
pub const unix = @as(c_int, 1);
pub const __unix = @as(c_int, 1);
pub const __unix__ = @as(c_int, 1);
pub const __code_model_small__ = @as(c_int, 1);
pub const __amd64__ = @as(c_int, 1);
pub const __amd64 = @as(c_int, 1);
pub const __x86_64__ = @as(c_int, 1);
pub const __x86_64 = @as(c_int, 1);
pub const __SEG_GS = @as(c_int, 1);
pub const __SEG_FS = @as(c_int, 1);
pub const __seg_gs = @compileError("unable to translate macro: undefined identifier `address_space`"); // <builtin>:33:9
pub const __seg_fs = @compileError("unable to translate macro: undefined identifier `address_space`"); // <builtin>:34:9
pub const __LAHF_SAHF__ = @as(c_int, 1);
pub const __AES__ = @as(c_int, 1);
pub const __VAES__ = @as(c_int, 1);
pub const __PCLMUL__ = @as(c_int, 1);
pub const __VPCLMULQDQ__ = @as(c_int, 1);
pub const __LZCNT__ = @as(c_int, 1);
pub const __RDRND__ = @as(c_int, 1);
pub const __FSGSBASE__ = @as(c_int, 1);
pub const __BMI__ = @as(c_int, 1);
pub const __BMI2__ = @as(c_int, 1);
pub const __POPCNT__ = @as(c_int, 1);
pub const __PRFCHW__ = @as(c_int, 1);
pub const __RDSEED__ = @as(c_int, 1);
pub const __ADX__ = @as(c_int, 1);
pub const __MWAITX__ = @as(c_int, 1);
pub const __MOVBE__ = @as(c_int, 1);
pub const __SSE4A__ = @as(c_int, 1);
pub const __FMA__ = @as(c_int, 1);
pub const __F16C__ = @as(c_int, 1);
pub const __GFNI__ = @as(c_int, 1);
pub const __EVEX512__ = @as(c_int, 1);
pub const __AVX512CD__ = @as(c_int, 1);
pub const __AVX512VPOPCNTDQ__ = @as(c_int, 1);
pub const __AVX512VNNI__ = @as(c_int, 1);
pub const __AVX512BF16__ = @as(c_int, 1);
pub const __AVX512DQ__ = @as(c_int, 1);
pub const __AVX512BITALG__ = @as(c_int, 1);
pub const __AVX512BW__ = @as(c_int, 1);
pub const __AVX512VL__ = @as(c_int, 1);
pub const __EVEX256__ = @as(c_int, 1);
pub const __AVX512VBMI__ = @as(c_int, 1);
pub const __AVX512VBMI2__ = @as(c_int, 1);
pub const __AVX512IFMA__ = @as(c_int, 1);
pub const __SHA__ = @as(c_int, 1);
pub const __FXSR__ = @as(c_int, 1);
pub const __XSAVE__ = @as(c_int, 1);
pub const __XSAVEOPT__ = @as(c_int, 1);
pub const __XSAVEC__ = @as(c_int, 1);
pub const __XSAVES__ = @as(c_int, 1);
pub const __PKU__ = @as(c_int, 1);
pub const __CLFLUSHOPT__ = @as(c_int, 1);
pub const __CLWB__ = @as(c_int, 1);
pub const __WBNOINVD__ = @as(c_int, 1);
pub const __SHSTK__ = @as(c_int, 1);
pub const __CLZERO__ = @as(c_int, 1);
pub const __RDPID__ = @as(c_int, 1);
pub const __RDPRU__ = @as(c_int, 1);
pub const __INVPCID__ = @as(c_int, 1);
pub const __CRC32__ = @as(c_int, 1);
pub const __AVX512F__ = @as(c_int, 1);
pub const __AVX2__ = @as(c_int, 1);
pub const __AVX__ = @as(c_int, 1);
pub const __SSE4_2__ = @as(c_int, 1);
pub const __SSE4_1__ = @as(c_int, 1);
pub const __SSSE3__ = @as(c_int, 1);
pub const __SSE3__ = @as(c_int, 1);
pub const __SSE2__ = @as(c_int, 1);
pub const __SSE__ = @as(c_int, 1);
pub const __SSE_MATH__ = @as(c_int, 1);
pub const __MMX__ = @as(c_int, 1);
pub const __GCC_HAVE_SYNC_COMPARE_AND_SWAP_8 = @as(c_int, 1);
pub const __SIZEOF_FLOAT128__ = @as(c_int, 16);
pub const _LP64 = @as(c_int, 1);
pub const __LP64__ = @as(c_int, 1);
pub const __FLOAT128__ = @as(c_int, 1);
pub const __ORDER_LITTLE_ENDIAN__ = @as(c_int, 1234);
pub const __ORDER_BIG_ENDIAN__ = @as(c_int, 4321);
pub const __ORDER_PDP_ENDIAN__ = @as(c_int, 3412);
pub const __BYTE_ORDER__ = __ORDER_LITTLE_ENDIAN__;
pub const __LITTLE_ENDIAN__ = @as(c_int, 1);
pub const __ELF__ = @as(c_int, 1);
pub const __ATOMIC_RELAXED = @as(c_int, 0);
pub const __ATOMIC_CONSUME = @as(c_int, 1);
pub const __ATOMIC_ACQUIRE = @as(c_int, 2);
pub const __ATOMIC_RELEASE = @as(c_int, 3);
pub const __ATOMIC_ACQ_REL = @as(c_int, 4);
pub const __ATOMIC_SEQ_CST = @as(c_int, 5);
pub const __ATOMIC_BOOL_LOCK_FREE = @as(c_int, 1);
pub const __ATOMIC_CHAR_LOCK_FREE = @as(c_int, 1);
pub const __ATOMIC_CHAR16_T_LOCK_FREE = @as(c_int, 1);
pub const __ATOMIC_CHAR32_T_LOCK_FREE = @as(c_int, 1);
pub const __ATOMIC_WCHAR_T_LOCK_FREE = @as(c_int, 1);
pub const __ATOMIC_WINT_T_LOCK_FREE = @as(c_int, 1);
pub const __ATOMIC_SHORT_LOCK_FREE = @as(c_int, 1);
pub const __ATOMIC_INT_LOCK_FREE = @as(c_int, 1);
pub const __ATOMIC_LONG_LOCK_FREE = @as(c_int, 1);
pub const __ATOMIC_LLONG_LOCK_FREE = @as(c_int, 1);
pub const __ATOMIC_POINTER_LOCK_FREE = @as(c_int, 1);
pub const __WINT_UNSIGNED__ = @as(c_int, 1);
pub const __CHAR_BIT__ = @as(c_int, 8);
pub const __BOOL_WIDTH__ = @as(c_int, 8);
pub const __SCHAR_MAX__ = @as(c_int, 127);
pub const __SCHAR_WIDTH__ = @as(c_int, 8);
pub const __SHRT_MAX__ = @as(c_int, 32767);
pub const __SHRT_WIDTH__ = @as(c_int, 16);
pub const __INT_MAX__ = __helpers.promoteIntLiteral(c_int, 2147483647, .decimal);
pub const __INT_WIDTH__ = @as(c_int, 32);
pub const __LONG_MAX__ = __helpers.promoteIntLiteral(c_long, 9223372036854775807, .decimal);
pub const __LONG_WIDTH__ = @as(c_int, 64);
pub const __LONG_LONG_MAX__ = @as(c_longlong, 9223372036854775807);
pub const __LONG_LONG_WIDTH__ = @as(c_int, 64);
pub const __WCHAR_MAX__ = __helpers.promoteIntLiteral(c_int, 2147483647, .decimal);
pub const __WCHAR_WIDTH__ = @as(c_int, 32);
pub const __WINT_MAX__ = __helpers.promoteIntLiteral(c_uint, 4294967295, .decimal);
pub const __WINT_WIDTH__ = @as(c_int, 32);
pub const __INTMAX_MAX__ = __helpers.promoteIntLiteral(c_long, 9223372036854775807, .decimal);
pub const __INTMAX_WIDTH__ = @as(c_int, 64);
pub const __SIZE_MAX__ = __helpers.promoteIntLiteral(c_ulong, 18446744073709551615, .decimal);
pub const __SIZE_WIDTH__ = @as(c_int, 64);
pub const __UINTMAX_MAX__ = __helpers.promoteIntLiteral(c_ulong, 18446744073709551615, .decimal);
pub const __UINTMAX_WIDTH__ = @as(c_int, 64);
pub const __PTRDIFF_MAX__ = __helpers.promoteIntLiteral(c_long, 9223372036854775807, .decimal);
pub const __PTRDIFF_WIDTH__ = @as(c_int, 64);
pub const __INTPTR_MAX__ = __helpers.promoteIntLiteral(c_long, 9223372036854775807, .decimal);
pub const __INTPTR_WIDTH__ = @as(c_int, 64);
pub const __UINTPTR_MAX__ = __helpers.promoteIntLiteral(c_ulong, 18446744073709551615, .decimal);
pub const __UINTPTR_WIDTH__ = @as(c_int, 64);
pub const __SIG_ATOMIC_MAX__ = __helpers.promoteIntLiteral(c_int, 2147483647, .decimal);
pub const __SIG_ATOMIC_WIDTH__ = @as(c_int, 32);
pub const __BITINT_MAXWIDTH__ = __helpers.promoteIntLiteral(c_int, 65535, .decimal);
pub const __SIZEOF_FLOAT__ = @as(c_int, 4);
pub const __SIZEOF_DOUBLE__ = @as(c_int, 8);
pub const __SIZEOF_LONG_DOUBLE__ = @as(c_int, 10);
pub const __SIZEOF_SHORT__ = @as(c_int, 2);
pub const __SIZEOF_INT__ = @as(c_int, 4);
pub const __SIZEOF_LONG__ = @as(c_int, 8);
pub const __SIZEOF_LONG_LONG__ = @as(c_int, 8);
pub const __SIZEOF_POINTER__ = @as(c_int, 8);
pub const __SIZEOF_PTRDIFF_T__ = @as(c_int, 8);
pub const __SIZEOF_SIZE_T__ = @as(c_int, 8);
pub const __SIZEOF_WCHAR_T__ = @as(c_int, 4);
pub const __SIZEOF_WINT_T__ = @as(c_int, 4);
pub const __SIZEOF_INT128__ = @as(c_int, 16);
pub const __INTPTR_TYPE__ = c_long;
pub const __UINTPTR_TYPE__ = c_ulong;
pub const __INTMAX_TYPE__ = c_long;
pub const __INTMAX_C_SUFFIX__ = @compileError("unable to translate macro: undefined identifier `L`"); // <builtin>:172:9
pub const __INTMAX_C = __helpers.L_SUFFIX;
pub const __UINTMAX_TYPE__ = c_ulong;
pub const __UINTMAX_C_SUFFIX__ = @compileError("unable to translate macro: undefined identifier `UL`"); // <builtin>:175:9
pub const __UINTMAX_C = __helpers.UL_SUFFIX;
pub const __PTRDIFF_TYPE__ = c_long;
pub const __SIZE_TYPE__ = c_ulong;
pub const __WCHAR_TYPE__ = c_int;
pub const __WINT_TYPE__ = c_uint;
pub const __CHAR16_TYPE__ = c_ushort;
pub const __CHAR32_TYPE__ = c_uint;
pub const __INT8_TYPE__ = i8;
pub const __INT8_FMTd__ = "hhd";
pub const __INT8_FMTi__ = "hhi";
pub const __INT8_C_SUFFIX__ = "";
pub inline fn __INT8_C(c: anytype) @TypeOf(c) {
    _ = &c;
    return c;
}
pub const __INT16_TYPE__ = c_short;
pub const __INT16_FMTd__ = "hd";
pub const __INT16_FMTi__ = "hi";
pub const __INT16_C_SUFFIX__ = "";
pub inline fn __INT16_C(c: anytype) @TypeOf(c) {
    _ = &c;
    return c;
}
pub const __INT32_TYPE__ = c_int;
pub const __INT32_FMTd__ = "d";
pub const __INT32_FMTi__ = "i";
pub const __INT32_C_SUFFIX__ = "";
pub inline fn __INT32_C(c: anytype) @TypeOf(c) {
    _ = &c;
    return c;
}
pub const __INT64_TYPE__ = c_long;
pub const __INT64_FMTd__ = "ld";
pub const __INT64_FMTi__ = "li";
pub const __INT64_C_SUFFIX__ = @compileError("unable to translate macro: undefined identifier `L`"); // <builtin>:201:9
pub const __UINT8_TYPE__ = u8;
pub const __UINT8_FMTo__ = "hho";
pub const __UINT8_FMTu__ = "hhu";
pub const __UINT8_FMTx__ = "hhx";
pub const __UINT8_FMTX__ = "hhX";
pub const __UINT8_C_SUFFIX__ = "";
pub inline fn __UINT8_C(c: anytype) @TypeOf(c) {
    _ = &c;
    return c;
}
pub const __UINT8_MAX__ = @as(c_int, 255);
pub const __INT8_MAX__ = @as(c_int, 127);
pub const __UINT16_TYPE__ = c_ushort;
pub const __UINT16_FMTo__ = "ho";
pub const __UINT16_FMTu__ = "hu";
pub const __UINT16_FMTx__ = "hx";
pub const __UINT16_FMTX__ = "hX";
pub const __UINT16_C_SUFFIX__ = "";
pub inline fn __UINT16_C(c: anytype) @TypeOf(c) {
    _ = &c;
    return c;
}
pub const __UINT16_MAX__ = __helpers.promoteIntLiteral(c_int, 65535, .decimal);
pub const __INT16_MAX__ = @as(c_int, 32767);
pub const __UINT32_TYPE__ = c_uint;
pub const __UINT32_FMTo__ = "o";
pub const __UINT32_FMTu__ = "u";
pub const __UINT32_FMTx__ = "x";
pub const __UINT32_FMTX__ = "X";
pub const __UINT32_C_SUFFIX__ = @compileError("unable to translate macro: undefined identifier `U`"); // <builtin>:226:9
pub const __UINT32_C = __helpers.U_SUFFIX;
pub const __UINT32_MAX__ = __helpers.promoteIntLiteral(c_uint, 4294967295, .decimal);
pub const __INT32_MAX__ = __helpers.promoteIntLiteral(c_int, 2147483647, .decimal);
pub const __UINT64_TYPE__ = c_ulong;
pub const __UINT64_FMTo__ = "lo";
pub const __UINT64_FMTu__ = "lu";
pub const __UINT64_FMTx__ = "lx";
pub const __UINT64_FMTX__ = "lX";
pub const __UINT64_C_SUFFIX__ = @compileError("unable to translate macro: undefined identifier `UL`"); // <builtin>:235:9
pub const __UINT64_MAX__ = __helpers.promoteIntLiteral(c_ulong, 18446744073709551615, .decimal);
pub const __INT64_MAX__ = __helpers.promoteIntLiteral(c_long, 9223372036854775807, .decimal);
pub const __INT_LEAST8_TYPE__ = i8;
pub const __INT_LEAST8_MAX__ = @as(c_int, 127);
pub const __INT_LEAST8_WIDTH__ = @as(c_int, 8);
pub const INT_LEAST8_FMTd__ = "hhd";
pub const INT_LEAST8_FMTi__ = "hhi";
pub const __UINT_LEAST8_TYPE__ = u8;
pub const __UINT_LEAST8_MAX__ = @as(c_int, 255);
pub const UINT_LEAST8_FMTo__ = "hho";
pub const UINT_LEAST8_FMTu__ = "hhu";
pub const UINT_LEAST8_FMTx__ = "hhx";
pub const UINT_LEAST8_FMTX__ = "hhX";
pub const __INT_FAST8_TYPE__ = i8;
pub const __INT_FAST8_MAX__ = @as(c_int, 127);
pub const __INT_FAST8_WIDTH__ = @as(c_int, 8);
pub const INT_FAST8_FMTd__ = "hhd";
pub const INT_FAST8_FMTi__ = "hhi";
pub const __UINT_FAST8_TYPE__ = u8;
pub const __UINT_FAST8_MAX__ = @as(c_int, 255);
pub const UINT_FAST8_FMTo__ = "hho";
pub const UINT_FAST8_FMTu__ = "hhu";
pub const UINT_FAST8_FMTx__ = "hhx";
pub const UINT_FAST8_FMTX__ = "hhX";
pub const __INT_LEAST16_TYPE__ = c_short;
pub const __INT_LEAST16_MAX__ = @as(c_int, 32767);
pub const __INT_LEAST16_WIDTH__ = @as(c_int, 16);
pub const INT_LEAST16_FMTd__ = "hd";
pub const INT_LEAST16_FMTi__ = "hi";
pub const __UINT_LEAST16_TYPE__ = c_ushort;
pub const __UINT_LEAST16_MAX__ = __helpers.promoteIntLiteral(c_int, 65535, .decimal);
pub const UINT_LEAST16_FMTo__ = "ho";
pub const UINT_LEAST16_FMTu__ = "hu";
pub const UINT_LEAST16_FMTx__ = "hx";
pub const UINT_LEAST16_FMTX__ = "hX";
pub const __INT_FAST16_TYPE__ = c_short;
pub const __INT_FAST16_MAX__ = @as(c_int, 32767);
pub const __INT_FAST16_WIDTH__ = @as(c_int, 16);
pub const INT_FAST16_FMTd__ = "hd";
pub const INT_FAST16_FMTi__ = "hi";
pub const __UINT_FAST16_TYPE__ = c_ushort;
pub const __UINT_FAST16_MAX__ = __helpers.promoteIntLiteral(c_int, 65535, .decimal);
pub const UINT_FAST16_FMTo__ = "ho";
pub const UINT_FAST16_FMTu__ = "hu";
pub const UINT_FAST16_FMTx__ = "hx";
pub const UINT_FAST16_FMTX__ = "hX";
pub const __INT_LEAST32_TYPE__ = c_int;
pub const __INT_LEAST32_MAX__ = __helpers.promoteIntLiteral(c_int, 2147483647, .decimal);
pub const __INT_LEAST32_WIDTH__ = @as(c_int, 32);
pub const INT_LEAST32_FMTd__ = "d";
pub const INT_LEAST32_FMTi__ = "i";
pub const __UINT_LEAST32_TYPE__ = c_uint;
pub const __UINT_LEAST32_MAX__ = __helpers.promoteIntLiteral(c_uint, 4294967295, .decimal);
pub const UINT_LEAST32_FMTo__ = "o";
pub const UINT_LEAST32_FMTu__ = "u";
pub const UINT_LEAST32_FMTx__ = "x";
pub const UINT_LEAST32_FMTX__ = "X";
pub const __INT_FAST32_TYPE__ = c_int;
pub const __INT_FAST32_MAX__ = __helpers.promoteIntLiteral(c_int, 2147483647, .decimal);
pub const __INT_FAST32_WIDTH__ = @as(c_int, 32);
pub const INT_FAST32_FMTd__ = "d";
pub const INT_FAST32_FMTi__ = "i";
pub const __UINT_FAST32_TYPE__ = c_uint;
pub const __UINT_FAST32_MAX__ = __helpers.promoteIntLiteral(c_uint, 4294967295, .decimal);
pub const UINT_FAST32_FMTo__ = "o";
pub const UINT_FAST32_FMTu__ = "u";
pub const UINT_FAST32_FMTx__ = "x";
pub const UINT_FAST32_FMTX__ = "X";
pub const __INT_LEAST64_TYPE__ = c_long;
pub const __INT_LEAST64_MAX__ = __helpers.promoteIntLiteral(c_long, 9223372036854775807, .decimal);
pub const __INT_LEAST64_WIDTH__ = @as(c_int, 64);
pub const INT_LEAST64_FMTd__ = "ld";
pub const INT_LEAST64_FMTi__ = "li";
pub const __UINT_LEAST64_TYPE__ = c_ulong;
pub const __UINT_LEAST64_MAX__ = __helpers.promoteIntLiteral(c_ulong, 18446744073709551615, .decimal);
pub const UINT_LEAST64_FMTo__ = "lo";
pub const UINT_LEAST64_FMTu__ = "lu";
pub const UINT_LEAST64_FMTx__ = "lx";
pub const UINT_LEAST64_FMTX__ = "lX";
pub const __INT_FAST64_TYPE__ = c_long;
pub const __INT_FAST64_MAX__ = __helpers.promoteIntLiteral(c_long, 9223372036854775807, .decimal);
pub const __INT_FAST64_WIDTH__ = @as(c_int, 64);
pub const INT_FAST64_FMTd__ = "ld";
pub const INT_FAST64_FMTi__ = "li";
pub const __UINT_FAST64_TYPE__ = c_ulong;
pub const __UINT_FAST64_MAX__ = __helpers.promoteIntLiteral(c_ulong, 18446744073709551615, .decimal);
pub const UINT_FAST64_FMTo__ = "lo";
pub const UINT_FAST64_FMTu__ = "lu";
pub const UINT_FAST64_FMTx__ = "lx";
pub const UINT_FAST64_FMTX__ = "lX";
pub const __FLT16_DENORM_MIN__ = @as(f16, 5.9604644775390625e-8);
pub const __FLT16_HAS_DENORM__ = "";
pub const __FLT16_DIG__ = @as(c_int, 3);
pub const __FLT16_DECIMAL_DIG__ = @as(c_int, 5);
pub const __FLT16_EPSILON__ = @as(f16, 9.765625e-4);
pub const __FLT16_HAS_INFINITY__ = "";
pub const __FLT16_HAS_QUIET_NAN__ = "";
pub const __FLT16_MANT_DIG__ = @as(c_int, 11);
pub const __FLT16_MAX_10_EXP__ = @as(c_int, 4);
pub const __FLT16_MAX_EXP__ = @as(c_int, 16);
pub const __FLT16_MAX__ = @as(f16, 6.5504e+4);
pub const __FLT16_MIN_10_EXP__ = -@as(c_int, 4);
pub const __FLT16_MIN_EXP__ = -@as(c_int, 13);
pub const __FLT16_MIN__ = @as(f16, 6.103515625e-5);
pub const __FLT_DENORM_MIN__ = @as(f32, 1.40129846e-45);
pub const __FLT_HAS_DENORM__ = "";
pub const __FLT_DIG__ = @as(c_int, 6);
pub const __FLT_DECIMAL_DIG__ = @as(c_int, 9);
pub const __FLT_EPSILON__ = @as(f32, 1.19209290e-7);
pub const __FLT_HAS_INFINITY__ = "";
pub const __FLT_HAS_QUIET_NAN__ = "";
pub const __FLT_MANT_DIG__ = @as(c_int, 24);
pub const __FLT_MAX_10_EXP__ = @as(c_int, 38);
pub const __FLT_MAX_EXP__ = @as(c_int, 128);
pub const __FLT_MAX__ = @as(f32, 3.40282347e+38);
pub const __FLT_MIN_10_EXP__ = -@as(c_int, 37);
pub const __FLT_MIN_EXP__ = -@as(c_int, 125);
pub const __FLT_MIN__ = @as(f32, 1.17549435e-38);
pub const __DBL_DENORM_MIN__ = @as(f64, 4.9406564584124654e-324);
pub const __DBL_HAS_DENORM__ = "";
pub const __DBL_DIG__ = @as(c_int, 15);
pub const __DBL_DECIMAL_DIG__ = @as(c_int, 17);
pub const __DBL_EPSILON__ = @as(f64, 2.2204460492503131e-16);
pub const __DBL_HAS_INFINITY__ = "";
pub const __DBL_HAS_QUIET_NAN__ = "";
pub const __DBL_MANT_DIG__ = @as(c_int, 53);
pub const __DBL_MAX_10_EXP__ = @as(c_int, 308);
pub const __DBL_MAX_EXP__ = @as(c_int, 1024);
pub const __DBL_MAX__ = @as(f64, 1.7976931348623157e+308);
pub const __DBL_MIN_10_EXP__ = -@as(c_int, 307);
pub const __DBL_MIN_EXP__ = -@as(c_int, 1021);
pub const __DBL_MIN__ = @as(f64, 2.2250738585072014e-308);
pub const __LDBL_DENORM_MIN__ = @as(c_longdouble, 3.64519953188247460253e-4951);
pub const __LDBL_HAS_DENORM__ = "";
pub const __LDBL_DIG__ = @as(c_int, 18);
pub const __LDBL_DECIMAL_DIG__ = @as(c_int, 21);
pub const __LDBL_EPSILON__ = @as(c_longdouble, 1.08420217248550443401e-19);
pub const __LDBL_HAS_INFINITY__ = "";
pub const __LDBL_HAS_QUIET_NAN__ = "";
pub const __LDBL_MANT_DIG__ = @as(c_int, 64);
pub const __LDBL_MAX_10_EXP__ = @as(c_int, 4932);
pub const __LDBL_MAX_EXP__ = @as(c_int, 16384);
pub const __LDBL_MAX__ = @as(c_longdouble, 1.18973149535723176502e+4932);
pub const __LDBL_MIN_10_EXP__ = -@as(c_int, 4931);
pub const __LDBL_MIN_EXP__ = -@as(c_int, 16381);
pub const __LDBL_MIN__ = @as(c_longdouble, 3.36210314311209350626e-4932);
pub const __FLT_EVAL_METHOD__ = @as(c_int, 0);
pub const __FLT_RADIX__ = @as(c_int, 2);
pub const __DECIMAL_DIG__ = __LDBL_DECIMAL_DIG__;
pub const TB_IMPL = @as(c_int, 1);
pub const _XOPEN_SOURCE = @as(c_int, 700);
pub const TB_OPT_ATTR_W = @as(c_int, 32);
pub const _FORTIFY_SOURCE = @as(c_int, 0);
pub const TERMBOX_H_INCL = "";
pub const _ERRNO_H = @as(c_int, 1);
pub const _FEATURES_H = @as(c_int, 1);
pub const __KERNEL_STRICT_NAMES = "";
pub inline fn __GNUC_PREREQ(maj: anytype, min: anytype) @TypeOf(((__GNUC__ << @as(c_int, 16)) + __GNUC_MINOR__) >= ((maj << @as(c_int, 16)) + min)) {
    _ = &maj;
    _ = &min;
    return ((__GNUC__ << @as(c_int, 16)) + __GNUC_MINOR__) >= ((maj << @as(c_int, 16)) + min);
}
pub inline fn __glibc_clang_prereq(maj: anytype, min: anytype) @TypeOf(@as(c_int, 0)) {
    _ = &maj;
    _ = &min;
    return @as(c_int, 0);
}
pub const __GLIBC_USE = @compileError("unable to translate macro: undefined identifier `__GLIBC_USE_`"); // /usr/include/features.h:197:9
pub const _DEFAULT_SOURCE = @as(c_int, 1);
pub const __GLIBC_USE_ISOC2Y = @as(c_int, 0);
pub const __GLIBC_USE_ISOC23 = @as(c_int, 0);
pub const __USE_ISOC11 = @as(c_int, 1);
pub const __USE_POSIX_IMPLICITLY = @as(c_int, 1);
pub const _POSIX_SOURCE = @as(c_int, 1);
pub const _POSIX_C_SOURCE = @as(c_long, 202405);
pub const __USE_POSIX = @as(c_int, 1);
pub const __USE_POSIX2 = @as(c_int, 1);
pub const __USE_POSIX199309 = @as(c_int, 1);
pub const __USE_POSIX199506 = @as(c_int, 1);
pub const __USE_XOPEN2K = @as(c_int, 1);
pub const __USE_XOPEN2K8 = @as(c_int, 1);
pub const _ATFILE_SOURCE = @as(c_int, 1);
pub const __USE_XOPEN2K24 = @as(c_int, 1);
pub const __USE_XOPEN = @as(c_int, 1);
pub const __USE_XOPEN_EXTENDED = @as(c_int, 1);
pub const __USE_UNIX98 = @as(c_int, 1);
pub const _LARGEFILE_SOURCE = @as(c_int, 1);
pub const __USE_XOPEN2K8XSI = @as(c_int, 1);
pub const __USE_XOPEN2KXSI = @as(c_int, 1);
pub const __USE_ISOC95 = @as(c_int, 1);
pub const __USE_ISOC99 = @as(c_int, 1);
pub const __USE_LARGEFILE = @as(c_int, 1);
pub const __WORDSIZE = @as(c_int, 64);
pub const __WORDSIZE_TIME64_COMPAT32 = @as(c_int, 1);
pub const __SYSCALL_WORDSIZE = @as(c_int, 64);
pub const __TIMESIZE = __WORDSIZE;
pub const __USE_TIME_BITS64 = @as(c_int, 1);
pub const __USE_MISC = @as(c_int, 1);
pub const __USE_ATFILE = @as(c_int, 1);
pub const __USE_FORTIFY_LEVEL = @as(c_int, 0);
pub const __GLIBC_USE_DEPRECATED_GETS = @as(c_int, 0);
pub const __GLIBC_USE_DEPRECATED_SCANF = @as(c_int, 0);
pub const __GLIBC_USE_C23_STRTOL = @as(c_int, 0);
pub const _STDC_PREDEF_H = @as(c_int, 1);
pub const __STDC_IEC_559__ = @as(c_int, 1);
pub const __STDC_IEC_60559_BFP__ = @as(c_long, 201404);
pub const __STDC_IEC_559_COMPLEX__ = @as(c_int, 1);
pub const __STDC_IEC_60559_COMPLEX__ = @as(c_long, 201404);
pub const __STDC_ISO_10646__ = @as(c_long, 201706);
pub const __GNU_LIBRARY__ = @as(c_int, 6);
pub const __GLIBC__ = @as(c_int, 2);
pub const __GLIBC_MINOR__ = @as(c_int, 43);
pub inline fn __GLIBC_PREREQ(maj: anytype, min: anytype) @TypeOf(((__GLIBC__ << @as(c_int, 16)) + __GLIBC_MINOR__) >= ((maj << @as(c_int, 16)) + min)) {
    _ = &maj;
    _ = &min;
    return ((__GLIBC__ << @as(c_int, 16)) + __GLIBC_MINOR__) >= ((maj << @as(c_int, 16)) + min);
}
pub const _SYS_CDEFS_H = @as(c_int, 1);
pub const __glibc_has_attribute = @compileError("unable to translate macro: undefined identifier `__has_attribute`"); // /usr/include/sys/cdefs.h:45:10
pub inline fn __glibc_has_builtin(name: anytype) @TypeOf(__builtin.has_builtin(name)) {
    _ = &name;
    return __builtin.has_builtin(name);
}
pub const __glibc_has_extension = @compileError("unable to translate macro: undefined identifier `__has_extension`"); // /usr/include/sys/cdefs.h:55:10
pub const __LEAF = @compileError("unable to translate macro: undefined identifier `__leaf__`"); // /usr/include/sys/cdefs.h:65:11
pub const __LEAF_ATTR = @compileError("unable to translate macro: undefined identifier `__leaf__`"); // /usr/include/sys/cdefs.h:66:11
pub const __THROW = @compileError("unable to translate macro: undefined identifier `__nothrow__`"); // /usr/include/sys/cdefs.h:79:11
pub const __THROWNL = @compileError("unable to translate macro: undefined identifier `__nothrow__`"); // /usr/include/sys/cdefs.h:80:11
pub const __NTH = @compileError("unable to translate macro: undefined identifier `__nothrow__`"); // /usr/include/sys/cdefs.h:81:11
pub const __NTHNL = @compileError("unable to translate macro: undefined identifier `__nothrow__`"); // /usr/include/sys/cdefs.h:82:11
pub const __COLD = @compileError("unable to translate macro: undefined identifier `__cold__`"); // /usr/include/sys/cdefs.h:102:11
pub inline fn __P(args: anytype) @TypeOf(args) {
    _ = &args;
    return args;
}
pub inline fn __PMT(args: anytype) @TypeOf(args) {
    _ = &args;
    return args;
}
pub const __CONCAT = @compileError("unable to translate C expr: unexpected token '##'"); // /usr/include/sys/cdefs.h:131:9
pub const __STRING = @compileError("unable to translate C expr: unexpected token ''"); // /usr/include/sys/cdefs.h:132:9
pub const __ptr_t = ?*anyopaque;
pub const __BEGIN_DECLS = "";
pub const __END_DECLS = "";
pub const __attribute_overloadable__ = "";
pub inline fn __bos(ptr: anytype) @TypeOf(__builtin.object_size(ptr, __USE_FORTIFY_LEVEL > @as(c_int, 1))) {
    _ = &ptr;
    return __builtin.object_size(ptr, __USE_FORTIFY_LEVEL > @as(c_int, 1));
}
pub inline fn __bos0(ptr: anytype) @TypeOf(__builtin.object_size(ptr, @as(c_int, 0))) {
    _ = &ptr;
    return __builtin.object_size(ptr, @as(c_int, 0));
}
pub inline fn __glibc_objsize0(__o: anytype) @TypeOf(__bos0(__o)) {
    _ = &__o;
    return __bos0(__o);
}
pub inline fn __glibc_objsize(__o: anytype) @TypeOf(__bos(__o)) {
    _ = &__o;
    return __bos(__o);
}
pub const __warnattr = @compileError("unable to translate macro: undefined identifier `__warning__`"); // /usr/include/sys/cdefs.h:366:10
pub const __errordecl = @compileError("unable to translate macro: undefined identifier `__error__`"); // /usr/include/sys/cdefs.h:367:10
pub const __flexarr = @compileError("unable to translate C expr: unexpected token '['"); // /usr/include/sys/cdefs.h:379:10
pub const __glibc_c99_flexarr_available = @as(c_int, 1);
pub const __REDIRECT = @compileError("unable to translate C expr: unexpected token '__asm__'"); // /usr/include/sys/cdefs.h:410:10
pub const __REDIRECT_NTH = @compileError("unable to translate C expr: unexpected token '__asm__'"); // /usr/include/sys/cdefs.h:417:11
pub const __REDIRECT_NTHNL = @compileError("unable to translate C expr: unexpected token '__asm__'"); // /usr/include/sys/cdefs.h:419:11
pub const __ASMNAME = @compileError("unable to translate macro: undefined identifier `__USER_LABEL_PREFIX__`"); // /usr/include/sys/cdefs.h:422:10
pub inline fn __ASMNAME2(prefix: anytype, cname: anytype) @TypeOf(__STRING(prefix) ++ cname) {
    _ = &prefix;
    _ = &cname;
    return __STRING(prefix) ++ cname;
}
pub const __REDIRECT_FORTIFY = __REDIRECT;
pub const __REDIRECT_FORTIFY_NTH = __REDIRECT_NTH;
pub const __attribute_malloc__ = @compileError("unable to translate macro: undefined identifier `__malloc__`"); // /usr/include/sys/cdefs.h:452:10
pub const __attribute_alloc_size__ = @compileError("unable to translate macro: undefined identifier `__alloc_size__`"); // /usr/include/sys/cdefs.h:460:10
pub const __attribute_alloc_align__ = @compileError("unable to translate macro: undefined identifier `__alloc_align__`"); // /usr/include/sys/cdefs.h:469:10
pub const __attribute_pure__ = @compileError("unable to translate macro: undefined identifier `__pure__`"); // /usr/include/sys/cdefs.h:479:10
pub const __attribute_const__ = @compileError("unable to translate C expr: unexpected token '__attribute__'"); // /usr/include/sys/cdefs.h:486:10
pub const __attribute_maybe_unused__ = @compileError("unable to translate macro: undefined identifier `__unused__`"); // /usr/include/sys/cdefs.h:492:10
pub const __attribute_used__ = @compileError("unable to translate macro: undefined identifier `__used__`"); // /usr/include/sys/cdefs.h:501:10
pub const __attribute_noinline__ = @compileError("unable to translate macro: undefined identifier `__noinline__`"); // /usr/include/sys/cdefs.h:502:10
pub const __attribute_deprecated__ = @compileError("unable to translate macro: undefined identifier `__deprecated__`"); // /usr/include/sys/cdefs.h:510:10
pub const __attribute_deprecated_msg__ = @compileError("unable to translate macro: undefined identifier `__deprecated__`"); // /usr/include/sys/cdefs.h:520:10
pub const __attribute_format_arg__ = @compileError("unable to translate macro: undefined identifier `__format_arg__`"); // /usr/include/sys/cdefs.h:533:10
pub const __attribute_format_strfmon__ = @compileError("unable to translate macro: undefined identifier `__format__`"); // /usr/include/sys/cdefs.h:543:10
pub const __attribute_nonnull__ = @compileError("unable to translate macro: undefined identifier `__nonnull__`"); // /usr/include/sys/cdefs.h:555:11
pub inline fn __nonnull(params: anytype) @TypeOf(__attribute_nonnull__(params)) {
    _ = &params;
    return __attribute_nonnull__(params);
}
pub const __returns_nonnull = @compileError("unable to translate macro: undefined identifier `__returns_nonnull__`"); // /usr/include/sys/cdefs.h:568:10
pub const __attribute_warn_unused_result__ = @compileError("unable to translate macro: undefined identifier `__warn_unused_result__`"); // /usr/include/sys/cdefs.h:577:10
pub const __wur = "";
pub const __always_inline = @compileError("unable to translate macro: undefined identifier `__always_inline__`"); // /usr/include/sys/cdefs.h:595:10
pub const __attribute_artificial__ = @compileError("unable to translate macro: undefined identifier `__artificial__`"); // /usr/include/sys/cdefs.h:604:10
pub const __extern_inline = @compileError("unable to translate C expr: unexpected token 'extern'"); // /usr/include/sys/cdefs.h:626:11
pub const __extern_always_inline = @compileError("unable to translate C expr: unexpected token 'extern'"); // /usr/include/sys/cdefs.h:627:11
pub const __fortify_function = __extern_always_inline ++ __attribute_artificial__;
pub const __va_arg_pack = @compileError("unable to translate macro: undefined identifier `__builtin_va_arg_pack`"); // /usr/include/sys/cdefs.h:638:10
pub const __va_arg_pack_len = @compileError("unable to translate macro: undefined identifier `__builtin_va_arg_pack_len`"); // /usr/include/sys/cdefs.h:639:10
pub const __restrict_arr = @compileError("unable to translate C expr: unexpected token '__restrict'"); // /usr/include/sys/cdefs.h:666:10
pub inline fn __glibc_unlikely(cond: anytype) @TypeOf(__builtin.expect(cond, @as(c_int, 0))) {
    _ = &cond;
    return __builtin.expect(cond, @as(c_int, 0));
}
pub inline fn __glibc_likely(cond: anytype) @TypeOf(__builtin.expect(cond, @as(c_int, 1))) {
    _ = &cond;
    return __builtin.expect(cond, @as(c_int, 1));
}
pub const __attribute_nonstring__ = "";
pub inline fn __attribute_copy__(arg: anytype) void {
    _ = &arg;
    return;
}
pub const __LDOUBLE_REDIRECTS_TO_FLOAT128_ABI = @as(c_int, 0);
pub inline fn __LDBL_REDIR1(name: anytype, proto: anytype, alias: anytype) @TypeOf(name ++ proto) {
    _ = &name;
    _ = &proto;
    _ = &alias;
    return name ++ proto;
}
pub inline fn __LDBL_REDIR(name: anytype, proto: anytype) @TypeOf(name ++ proto) {
    _ = &name;
    _ = &proto;
    return name ++ proto;
}
pub inline fn __LDBL_REDIR1_NTH(name: anytype, proto: anytype, alias: anytype) @TypeOf(name ++ proto ++ __THROW) {
    _ = &name;
    _ = &proto;
    _ = &alias;
    return name ++ proto ++ __THROW;
}
pub inline fn __LDBL_REDIR_NTH(name: anytype, proto: anytype) @TypeOf(name ++ proto ++ __THROW) {
    _ = &name;
    _ = &proto;
    return name ++ proto ++ __THROW;
}
pub inline fn __LDBL_REDIR2_DECL(name: anytype) void {
    _ = &name;
    return;
}
pub inline fn __LDBL_REDIR_DECL(name: anytype) void {
    _ = &name;
    return;
}
pub inline fn __REDIRECT_LDBL(name: anytype, proto: anytype, alias: anytype) @TypeOf(__REDIRECT(name, proto, alias)) {
    _ = &name;
    _ = &proto;
    _ = &alias;
    return __REDIRECT(name, proto, alias);
}
pub inline fn __REDIRECT_NTH_LDBL(name: anytype, proto: anytype, alias: anytype) @TypeOf(__REDIRECT_NTH(name, proto, alias)) {
    _ = &name;
    _ = &proto;
    _ = &alias;
    return __REDIRECT_NTH(name, proto, alias);
}
pub const __glibc_macro_warning1 = @compileError("unable to translate macro: undefined identifier `_Pragma`"); // /usr/include/sys/cdefs.h:807:10
pub const __glibc_macro_warning = @compileError("unable to translate macro: undefined identifier `GCC`"); // /usr/include/sys/cdefs.h:808:10
pub const __HAVE_GENERIC_SELECTION = @as(c_int, 1);
pub const __glibc_const_generic = @compileError("unable to translate C expr: expected type instead got 'const'"); // /usr/include/sys/cdefs.h:837:10
pub inline fn __fortified_attr_access(a: anytype, o: anytype, s: anytype) void {
    _ = &a;
    _ = &o;
    _ = &s;
    return;
}
pub inline fn __attr_access(x: anytype) void {
    _ = &x;
    return;
}
pub inline fn __attr_access_none(argno: anytype) void {
    _ = &argno;
    return;
}
pub inline fn __attr_dealloc(dealloc: anytype, argno: anytype) void {
    _ = &dealloc;
    _ = &argno;
    return;
}
pub const __attr_dealloc_free = "";
pub const __attribute_returns_twice__ = @compileError("unable to translate macro: undefined identifier `__returns_twice__`"); // /usr/include/sys/cdefs.h:884:10
pub const __attribute_struct_may_alias__ = @compileError("unable to translate macro: undefined identifier `__may_alias__`"); // /usr/include/sys/cdefs.h:893:10
pub const __stub___compat_bdflush = "";
pub const __stub_chflags = "";
pub const __stub_fchflags = "";
pub const __stub_gtty = "";
pub const __stub_revoke = "";
pub const __stub_setlogin = "";
pub const __stub_sigreturn = "";
pub const __stub_stty = "";
pub const _BITS_ERRNO_H = @as(c_int, 1);
pub const _ASM_GENERIC_ERRNO_H = "";
pub const _ASM_GENERIC_ERRNO_BASE_H = "";
pub const EPERM = @as(c_int, 1);
pub const ENOENT = @as(c_int, 2);
pub const ESRCH = @as(c_int, 3);
pub const EINTR = @as(c_int, 4);
pub const EIO = @as(c_int, 5);
pub const ENXIO = @as(c_int, 6);
pub const E2BIG = @as(c_int, 7);
pub const ENOEXEC = @as(c_int, 8);
pub const EBADF = @as(c_int, 9);
pub const ECHILD = @as(c_int, 10);
pub const EAGAIN = @as(c_int, 11);
pub const ENOMEM = @as(c_int, 12);
pub const EACCES = @as(c_int, 13);
pub const EFAULT = @as(c_int, 14);
pub const ENOTBLK = @as(c_int, 15);
pub const EBUSY = @as(c_int, 16);
pub const EEXIST = @as(c_int, 17);
pub const EXDEV = @as(c_int, 18);
pub const ENODEV = @as(c_int, 19);
pub const ENOTDIR = @as(c_int, 20);
pub const EISDIR = @as(c_int, 21);
pub const EINVAL = @as(c_int, 22);
pub const ENFILE = @as(c_int, 23);
pub const EMFILE = @as(c_int, 24);
pub const ENOTTY = @as(c_int, 25);
pub const ETXTBSY = @as(c_int, 26);
pub const EFBIG = @as(c_int, 27);
pub const ENOSPC = @as(c_int, 28);
pub const ESPIPE = @as(c_int, 29);
pub const EROFS = @as(c_int, 30);
pub const EMLINK = @as(c_int, 31);
pub const EPIPE = @as(c_int, 32);
pub const EDOM = @as(c_int, 33);
pub const ERANGE = @as(c_int, 34);
pub const EDEADLK = @as(c_int, 35);
pub const ENAMETOOLONG = @as(c_int, 36);
pub const ENOLCK = @as(c_int, 37);
pub const ENOSYS = @as(c_int, 38);
pub const ENOTEMPTY = @as(c_int, 39);
pub const ELOOP = @as(c_int, 40);
pub const EWOULDBLOCK = EAGAIN;
pub const ENOMSG = @as(c_int, 42);
pub const EIDRM = @as(c_int, 43);
pub const ECHRNG = @as(c_int, 44);
pub const EL2NSYNC = @as(c_int, 45);
pub const EL3HLT = @as(c_int, 46);
pub const EL3RST = @as(c_int, 47);
pub const ELNRNG = @as(c_int, 48);
pub const EUNATCH = @as(c_int, 49);
pub const ENOCSI = @as(c_int, 50);
pub const EL2HLT = @as(c_int, 51);
pub const EBADE = @as(c_int, 52);
pub const EBADR = @as(c_int, 53);
pub const EXFULL = @as(c_int, 54);
pub const ENOANO = @as(c_int, 55);
pub const EBADRQC = @as(c_int, 56);
pub const EBADSLT = @as(c_int, 57);
pub const EDEADLOCK = EDEADLK;
pub const EBFONT = @as(c_int, 59);
pub const ENOSTR = @as(c_int, 60);
pub const ENODATA = @as(c_int, 61);
pub const ETIME = @as(c_int, 62);
pub const ENOSR = @as(c_int, 63);
pub const ENONET = @as(c_int, 64);
pub const ENOPKG = @as(c_int, 65);
pub const EREMOTE = @as(c_int, 66);
pub const ENOLINK = @as(c_int, 67);
pub const EADV = @as(c_int, 68);
pub const ESRMNT = @as(c_int, 69);
pub const ECOMM = @as(c_int, 70);
pub const EPROTO = @as(c_int, 71);
pub const EMULTIHOP = @as(c_int, 72);
pub const EDOTDOT = @as(c_int, 73);
pub const EBADMSG = @as(c_int, 74);
pub const EFSBADCRC = EBADMSG;
pub const EOVERFLOW = @as(c_int, 75);
pub const ENOTUNIQ = @as(c_int, 76);
pub const EBADFD = @as(c_int, 77);
pub const EREMCHG = @as(c_int, 78);
pub const ELIBACC = @as(c_int, 79);
pub const ELIBBAD = @as(c_int, 80);
pub const ELIBSCN = @as(c_int, 81);
pub const ELIBMAX = @as(c_int, 82);
pub const ELIBEXEC = @as(c_int, 83);
pub const EILSEQ = @as(c_int, 84);
pub const ERESTART = @as(c_int, 85);
pub const ESTRPIPE = @as(c_int, 86);
pub const EUSERS = @as(c_int, 87);
pub const ENOTSOCK = @as(c_int, 88);
pub const EDESTADDRREQ = @as(c_int, 89);
pub const EMSGSIZE = @as(c_int, 90);
pub const EPROTOTYPE = @as(c_int, 91);
pub const ENOPROTOOPT = @as(c_int, 92);
pub const EPROTONOSUPPORT = @as(c_int, 93);
pub const ESOCKTNOSUPPORT = @as(c_int, 94);
pub const EOPNOTSUPP = @as(c_int, 95);
pub const EPFNOSUPPORT = @as(c_int, 96);
pub const EAFNOSUPPORT = @as(c_int, 97);
pub const EADDRINUSE = @as(c_int, 98);
pub const EADDRNOTAVAIL = @as(c_int, 99);
pub const ENETDOWN = @as(c_int, 100);
pub const ENETUNREACH = @as(c_int, 101);
pub const ENETRESET = @as(c_int, 102);
pub const ECONNABORTED = @as(c_int, 103);
pub const ECONNRESET = @as(c_int, 104);
pub const ENOBUFS = @as(c_int, 105);
pub const EISCONN = @as(c_int, 106);
pub const ENOTCONN = @as(c_int, 107);
pub const ESHUTDOWN = @as(c_int, 108);
pub const ETOOMANYREFS = @as(c_int, 109);
pub const ETIMEDOUT = @as(c_int, 110);
pub const ECONNREFUSED = @as(c_int, 111);
pub const EHOSTDOWN = @as(c_int, 112);
pub const EHOSTUNREACH = @as(c_int, 113);
pub const EALREADY = @as(c_int, 114);
pub const EINPROGRESS = @as(c_int, 115);
pub const ESTALE = @as(c_int, 116);
pub const EUCLEAN = @as(c_int, 117);
pub const EFSCORRUPTED = EUCLEAN;
pub const ENOTNAM = @as(c_int, 118);
pub const ENAVAIL = @as(c_int, 119);
pub const EISNAM = @as(c_int, 120);
pub const EREMOTEIO = @as(c_int, 121);
pub const EDQUOT = @as(c_int, 122);
pub const ENOMEDIUM = @as(c_int, 123);
pub const EMEDIUMTYPE = @as(c_int, 124);
pub const ECANCELED = @as(c_int, 125);
pub const ENOKEY = @as(c_int, 126);
pub const EKEYEXPIRED = @as(c_int, 127);
pub const EKEYREVOKED = @as(c_int, 128);
pub const EKEYREJECTED = @as(c_int, 129);
pub const EOWNERDEAD = @as(c_int, 130);
pub const ENOTRECOVERABLE = @as(c_int, 131);
pub const ERFKILL = @as(c_int, 132);
pub const EHWPOISON = @as(c_int, 133);
pub const ENOTSUP = EOPNOTSUPP;
pub const errno = __errno_location().*;
pub const _FCNTL_H = @as(c_int, 1);
pub const _BITS_TYPES_H = @as(c_int, 1);
pub const __S16_TYPE = c_short;
pub const __U16_TYPE = c_ushort;
pub const __S32_TYPE = c_int;
pub const __U32_TYPE = c_uint;
pub const __SLONGWORD_TYPE = c_long;
pub const __ULONGWORD_TYPE = c_ulong;
pub const __SQUAD_TYPE = c_long;
pub const __UQUAD_TYPE = c_ulong;
pub const __SWORD_TYPE = c_long;
pub const __UWORD_TYPE = c_ulong;
pub const __SLONG32_TYPE = c_int;
pub const __ULONG32_TYPE = c_uint;
pub const __S64_TYPE = c_long;
pub const __U64_TYPE = c_ulong;
pub const _BITS_TYPESIZES_H = @as(c_int, 1);
pub const __SYSCALL_SLONG_TYPE = __SLONGWORD_TYPE;
pub const __SYSCALL_ULONG_TYPE = __ULONGWORD_TYPE;
pub const __DEV_T_TYPE = __UQUAD_TYPE;
pub const __UID_T_TYPE = __U32_TYPE;
pub const __GID_T_TYPE = __U32_TYPE;
pub const __INO_T_TYPE = __SYSCALL_ULONG_TYPE;
pub const __INO64_T_TYPE = __UQUAD_TYPE;
pub const __MODE_T_TYPE = __U32_TYPE;
pub const __NLINK_T_TYPE = __SYSCALL_ULONG_TYPE;
pub const __FSWORD_T_TYPE = __SYSCALL_SLONG_TYPE;
pub const __OFF_T_TYPE = __SYSCALL_SLONG_TYPE;
pub const __OFF64_T_TYPE = __SQUAD_TYPE;
pub const __PID_T_TYPE = __S32_TYPE;
pub const __RLIM_T_TYPE = __SYSCALL_ULONG_TYPE;
pub const __RLIM64_T_TYPE = __UQUAD_TYPE;
pub const __BLKCNT_T_TYPE = __SYSCALL_SLONG_TYPE;
pub const __BLKCNT64_T_TYPE = __SQUAD_TYPE;
pub const __FSBLKCNT_T_TYPE = __SYSCALL_ULONG_TYPE;
pub const __FSBLKCNT64_T_TYPE = __UQUAD_TYPE;
pub const __FSFILCNT_T_TYPE = __SYSCALL_ULONG_TYPE;
pub const __FSFILCNT64_T_TYPE = __UQUAD_TYPE;
pub const __ID_T_TYPE = __U32_TYPE;
pub const __CLOCK_T_TYPE = __SYSCALL_SLONG_TYPE;
pub const __TIME_T_TYPE = __SYSCALL_SLONG_TYPE;
pub const __USECONDS_T_TYPE = __U32_TYPE;
pub const __SUSECONDS_T_TYPE = __SYSCALL_SLONG_TYPE;
pub const __SUSECONDS64_T_TYPE = __SQUAD_TYPE;
pub const __DADDR_T_TYPE = __S32_TYPE;
pub const __KEY_T_TYPE = __S32_TYPE;
pub const __CLOCKID_T_TYPE = __S32_TYPE;
pub const __TIMER_T_TYPE = ?*anyopaque;
pub const __BLKSIZE_T_TYPE = __SYSCALL_SLONG_TYPE;
pub const __FSID_T_TYPE = @compileError("unable to translate macro: undefined identifier `__val`"); // /usr/include/bits/typesizes.h:73:9
pub const __SSIZE_T_TYPE = __SWORD_TYPE;
pub const __CPU_MASK_TYPE = __SYSCALL_ULONG_TYPE;
pub const __OFF_T_MATCHES_OFF64_T = @as(c_int, 1);
pub const __INO_T_MATCHES_INO64_T = @as(c_int, 1);
pub const __RLIM_T_MATCHES_RLIM64_T = @as(c_int, 1);
pub const __STATFS_MATCHES_STATFS64 = @as(c_int, 1);
pub const __KERNEL_OLD_TIMEVAL_MATCHES_TIMEVAL64 = @as(c_int, 1);
pub const __FD_SETSIZE = @as(c_int, 1024);
pub const _BITS_TIME64_H = @as(c_int, 1);
pub const __TIME64_T_TYPE = __TIME_T_TYPE;
pub const __O_LARGEFILE = @as(c_int, 0);
pub const F_GETLK64 = @as(c_int, 5);
pub const F_SETLK64 = @as(c_int, 6);
pub const F_SETLKW64 = @as(c_int, 7);
pub const O_ACCMODE = @as(c_int, 0o003);
pub const O_RDONLY = @as(c_int, 0o0);
pub const O_WRONLY = @as(c_int, 0o1);
pub const O_RDWR = @as(c_int, 0o2);
pub const O_CREAT = @as(c_int, 0o100);
pub const O_EXCL = @as(c_int, 0o200);
pub const O_NOCTTY = @as(c_int, 0o400);
pub const O_TRUNC = @as(c_int, 0o1000);
pub const O_APPEND = @as(c_int, 0o2000);
pub const O_NONBLOCK = @as(c_int, 0o4000);
pub const O_NDELAY = O_NONBLOCK;
pub const O_SYNC = __helpers.promoteIntLiteral(c_int, 0o4010000, .octal);
pub const O_FSYNC = O_SYNC;
pub const O_ASYNC = @as(c_int, 0o20000);
pub const __O_DIRECTORY = __helpers.promoteIntLiteral(c_int, 0o200000, .octal);
pub const __O_NOFOLLOW = __helpers.promoteIntLiteral(c_int, 0o400000, .octal);
pub const __O_CLOEXEC = __helpers.promoteIntLiteral(c_int, 0o2000000, .octal);
pub const __O_DIRECT = @as(c_int, 0o40000);
pub const __O_NOATIME = __helpers.promoteIntLiteral(c_int, 0o1000000, .octal);
pub const __O_PATH = __helpers.promoteIntLiteral(c_int, 0o10000000, .octal);
pub const __O_DSYNC = @as(c_int, 0o10000);
pub const __O_TMPFILE = __helpers.promoteIntLiteral(c_int, 0o20000000, .octal) | __O_DIRECTORY;
pub const F_GETLK = F_GETLK64;
pub const F_SETLK = F_SETLK64;
pub const F_SETLKW = F_SETLKW64;
pub const O_DIRECTORY = __O_DIRECTORY;
pub const O_NOFOLLOW = __O_NOFOLLOW;
pub const O_CLOEXEC = __O_CLOEXEC;
pub const O_DSYNC = __O_DSYNC;
pub const O_RSYNC = O_SYNC;
pub const F_DUPFD = @as(c_int, 0);
pub const F_GETFD = @as(c_int, 1);
pub const F_SETFD = @as(c_int, 2);
pub const F_GETFL = @as(c_int, 3);
pub const F_SETFL = @as(c_int, 4);
pub const __F_SETOWN = @as(c_int, 8);
pub const __F_GETOWN = @as(c_int, 9);
pub const F_SETOWN = __F_SETOWN;
pub const F_GETOWN = __F_GETOWN;
pub const __F_SETSIG = @as(c_int, 10);
pub const __F_GETSIG = @as(c_int, 11);
pub const __F_SETOWN_EX = @as(c_int, 15);
pub const __F_GETOWN_EX = @as(c_int, 16);
pub const F_DUPFD_CLOEXEC = @as(c_int, 1030);
pub const FD_CLOEXEC = @as(c_int, 1);
pub const F_RDLCK = @as(c_int, 0);
pub const F_WRLCK = @as(c_int, 1);
pub const F_UNLCK = @as(c_int, 2);
pub const F_EXLCK = @as(c_int, 4);
pub const F_SHLCK = @as(c_int, 8);
pub const LOCK_SH = @as(c_int, 1);
pub const LOCK_EX = @as(c_int, 2);
pub const LOCK_NB = @as(c_int, 4);
pub const LOCK_UN = @as(c_int, 8);
pub const FAPPEND = O_APPEND;
pub const FFSYNC = O_FSYNC;
pub const FASYNC = O_ASYNC;
pub const FNONBLOCK = O_NONBLOCK;
pub const FNDELAY = O_NDELAY;
pub const __POSIX_FADV_DONTNEED = @as(c_int, 4);
pub const __POSIX_FADV_NOREUSE = @as(c_int, 5);
pub const POSIX_FADV_NORMAL = @as(c_int, 0);
pub const POSIX_FADV_RANDOM = @as(c_int, 1);
pub const POSIX_FADV_SEQUENTIAL = @as(c_int, 2);
pub const POSIX_FADV_WILLNEED = @as(c_int, 3);
pub const POSIX_FADV_DONTNEED = __POSIX_FADV_DONTNEED;
pub const POSIX_FADV_NOREUSE = __POSIX_FADV_NOREUSE;
pub inline fn __OPEN_NEEDS_MODE(oflag: anytype) @TypeOf(((oflag & O_CREAT) != @as(c_int, 0)) or ((oflag & __O_TMPFILE) == __O_TMPFILE)) {
    _ = &oflag;
    return ((oflag & O_CREAT) != @as(c_int, 0)) or ((oflag & __O_TMPFILE) == __O_TMPFILE);
}
pub const __mode_t_defined = "";
pub const __off_t_defined = "";
pub const __pid_t_defined = "";
pub const _STRUCT_TIMESPEC = @as(c_int, 1);
pub const _BITS_ENDIAN_H = @as(c_int, 1);
pub const __LITTLE_ENDIAN = @as(c_int, 1234);
pub const __BIG_ENDIAN = @as(c_int, 4321);
pub const __PDP_ENDIAN = @as(c_int, 3412);
pub const _BITS_ENDIANNESS_H = @as(c_int, 1);
pub const __BYTE_ORDER = __LITTLE_ENDIAN;
pub const __FLOAT_WORD_ORDER = __BYTE_ORDER;
pub inline fn __LONG_LONG_PAIR(HI: anytype, LO: anytype) @TypeOf(HI) {
    _ = &HI;
    _ = &LO;
    return blk: {
        _ = &LO;
        break :blk HI;
    };
}
pub const __time_t_defined = @as(c_int, 1);
pub const _BITS_STAT_H = @as(c_int, 1);
pub const _BITS_STRUCT_STAT_H = @as(c_int, 1);
pub const st_atime = @compileError("unable to translate macro: undefined identifier `st_atim`"); // /usr/include/bits/struct_stat.h:77:11
pub const st_mtime = @compileError("unable to translate macro: undefined identifier `st_mtim`"); // /usr/include/bits/struct_stat.h:78:11
pub const st_ctime = @compileError("unable to translate macro: undefined identifier `st_ctim`"); // /usr/include/bits/struct_stat.h:79:11
pub const _STATBUF_ST_BLKSIZE = "";
pub const _STATBUF_ST_RDEV = "";
pub const _STATBUF_ST_NSEC = "";
pub const __S_IFMT = __helpers.promoteIntLiteral(c_int, 0o170000, .octal);
pub const __S_IFDIR = @as(c_int, 0o040000);
pub const __S_IFCHR = @as(c_int, 0o020000);
pub const __S_IFBLK = @as(c_int, 0o060000);
pub const __S_IFREG = __helpers.promoteIntLiteral(c_int, 0o100000, .octal);
pub const __S_IFIFO = @as(c_int, 0o010000);
pub const __S_IFLNK = __helpers.promoteIntLiteral(c_int, 0o120000, .octal);
pub const __S_IFSOCK = __helpers.promoteIntLiteral(c_int, 0o140000, .octal);
pub inline fn __S_TYPEISMQ(buf: anytype) @TypeOf(buf.*.st_mode - buf.*.st_mode) {
    _ = &buf;
    return buf.*.st_mode - buf.*.st_mode;
}
pub inline fn __S_TYPEISSEM(buf: anytype) @TypeOf(buf.*.st_mode - buf.*.st_mode) {
    _ = &buf;
    return buf.*.st_mode - buf.*.st_mode;
}
pub inline fn __S_TYPEISSHM(buf: anytype) @TypeOf(buf.*.st_mode - buf.*.st_mode) {
    _ = &buf;
    return buf.*.st_mode - buf.*.st_mode;
}
pub const __S_ISUID = @as(c_int, 0o4000);
pub const __S_ISGID = @as(c_int, 0o2000);
pub const __S_ISVTX = @as(c_int, 0o1000);
pub const __S_IREAD = @as(c_int, 0o400);
pub const __S_IWRITE = @as(c_int, 0o200);
pub const __S_IEXEC = @as(c_int, 0o100);
pub const UTIME_NOW = (@as(c_long, 1) << @as(c_int, 30)) - @as(c_long, 1);
pub const UTIME_OMIT = (@as(c_long, 1) << @as(c_int, 30)) - @as(c_long, 2);
pub const S_IFMT = __S_IFMT;
pub const S_IFDIR = __S_IFDIR;
pub const S_IFCHR = __S_IFCHR;
pub const S_IFBLK = __S_IFBLK;
pub const S_IFREG = __S_IFREG;
pub const S_IFIFO = __S_IFIFO;
pub const S_IFLNK = __S_IFLNK;
pub const S_IFSOCK = __S_IFSOCK;
pub const S_ISUID = __S_ISUID;
pub const S_ISGID = __S_ISGID;
pub const S_ISVTX = __S_ISVTX;
pub const S_IRUSR = __S_IREAD;
pub const S_IWUSR = __S_IWRITE;
pub const S_IXUSR = __S_IEXEC;
pub const S_IRWXU = (__S_IREAD | __S_IWRITE) | __S_IEXEC;
pub const S_IRGRP = S_IRUSR >> @as(c_int, 3);
pub const S_IWGRP = S_IWUSR >> @as(c_int, 3);
pub const S_IXGRP = S_IXUSR >> @as(c_int, 3);
pub const S_IRWXG = S_IRWXU >> @as(c_int, 3);
pub const S_IROTH = S_IRGRP >> @as(c_int, 3);
pub const S_IWOTH = S_IWGRP >> @as(c_int, 3);
pub const S_IXOTH = S_IXGRP >> @as(c_int, 3);
pub const S_IRWXO = S_IRWXG >> @as(c_int, 3);
pub const R_OK = @as(c_int, 4);
pub const W_OK = @as(c_int, 2);
pub const X_OK = @as(c_int, 1);
pub const F_OK = @as(c_int, 0);
pub const SEEK_SET = @as(c_int, 0);
pub const SEEK_CUR = @as(c_int, 1);
pub const SEEK_END = @as(c_int, 2);
pub const AT_FDCWD = -@as(c_int, 100);
pub const AT_SYMLINK_NOFOLLOW = @as(c_int, 0x100);
pub const AT_REMOVEDIR = @as(c_int, 0x200);
pub const AT_SYMLINK_FOLLOW = @as(c_int, 0x400);
pub const AT_EACCESS = @as(c_int, 0x200);
pub const F_ULOCK = @as(c_int, 0);
pub const F_LOCK = @as(c_int, 1);
pub const F_TLOCK = @as(c_int, 2);
pub const F_TEST = @as(c_int, 3);
pub const _LIBC_LIMITS_H_ = @as(c_int, 1);
pub const MB_LEN_MAX = @as(c_int, 16);
pub const _GCC_LIMITS_H_ = "";
pub const __CLANG_LIMITS_H = "";
pub const SCHAR_MAX = __SCHAR_MAX__;
pub const SHRT_MAX = __SHRT_MAX__;
pub const INT_MAX = __INT_MAX__;
pub const LONG_MAX = __LONG_MAX__;
pub const SCHAR_MIN = -__SCHAR_MAX__ - @as(c_int, 1);
pub const SHRT_MIN = -__SHRT_MAX__ - @as(c_int, 1);
pub const INT_MIN = -__INT_MAX__ - @as(c_int, 1);
pub const LONG_MIN = -__LONG_MAX__ - @as(c_long, 1);
pub const UCHAR_MAX = (__SCHAR_MAX__ * @as(c_int, 2)) + @as(c_int, 1);
pub const USHRT_MAX = (__SHRT_MAX__ * @as(c_int, 2)) + @as(c_int, 1);
pub const UINT_MAX = (__INT_MAX__ * @as(c_uint, 2)) + @as(c_uint, 1);
pub const ULONG_MAX = (__LONG_MAX__ * @as(c_ulong, 2)) + @as(c_ulong, 1);
pub const CHAR_BIT = __CHAR_BIT__;
pub const CHAR_MIN = SCHAR_MIN;
pub const CHAR_MAX = __SCHAR_MAX__;
pub const LLONG_MIN = -__LONG_LONG_MAX__ - @as(c_longlong, 1);
pub const LLONG_MAX = __LONG_LONG_MAX__;
pub const ULLONG_MAX = (__LONG_LONG_MAX__ * @as(c_ulonglong, 2)) + @as(c_ulonglong, 1);
pub const _BITS_POSIX1_LIM_H = @as(c_int, 1);
pub const _POSIX_AIO_LISTIO_MAX = @as(c_int, 2);
pub const _POSIX_AIO_MAX = @as(c_int, 1);
pub const _POSIX_ARG_MAX = @as(c_int, 4096);
pub const _POSIX_CHILD_MAX = @as(c_int, 25);
pub const _POSIX_DELAYTIMER_MAX = @as(c_int, 32);
pub const _POSIX_HOST_NAME_MAX = @as(c_int, 255);
pub const _POSIX_LINK_MAX = @as(c_int, 8);
pub const _POSIX_LOGIN_NAME_MAX = @as(c_int, 9);
pub const _POSIX_MAX_CANON = @as(c_int, 255);
pub const _POSIX_MAX_INPUT = @as(c_int, 255);
pub const _POSIX_MQ_OPEN_MAX = @as(c_int, 8);
pub const _POSIX_MQ_PRIO_MAX = @as(c_int, 32);
pub const _POSIX_NAME_MAX = @as(c_int, 14);
pub const _POSIX_NGROUPS_MAX = @as(c_int, 8);
pub const _POSIX_OPEN_MAX = @as(c_int, 20);
pub const _POSIX_PATH_MAX = @as(c_int, 256);
pub const _POSIX_PIPE_BUF = @as(c_int, 512);
pub const _POSIX_RE_DUP_MAX = @as(c_int, 255);
pub const _POSIX_RTSIG_MAX = @as(c_int, 8);
pub const _POSIX_SEM_NSEMS_MAX = @as(c_int, 256);
pub const _POSIX_SEM_VALUE_MAX = @as(c_int, 32767);
pub const _POSIX_SIGQUEUE_MAX = @as(c_int, 32);
pub const _POSIX_SSIZE_MAX = @as(c_int, 32767);
pub const _POSIX_STREAM_MAX = @as(c_int, 8);
pub const _POSIX_SYMLINK_MAX = @as(c_int, 255);
pub const _POSIX_SYMLOOP_MAX = @as(c_int, 8);
pub const _POSIX_TIMER_MAX = @as(c_int, 32);
pub const _POSIX_TTY_NAME_MAX = @as(c_int, 9);
pub const _POSIX_TZNAME_MAX = @as(c_int, 6);
pub const _POSIX_CLOCKRES_MIN = __helpers.promoteIntLiteral(c_int, 20000000, .decimal);
pub const _LINUX_LIMITS_H = "";
pub const NGROUPS_MAX = __helpers.promoteIntLiteral(c_int, 65536, .decimal);
pub const MAX_CANON = @as(c_int, 255);
pub const MAX_INPUT = @as(c_int, 255);
pub const NAME_MAX = @as(c_int, 255);
pub const PATH_MAX = @as(c_int, 4096);
pub const PIPE_BUF = @as(c_int, 4096);
pub const XATTR_NAME_MAX = @as(c_int, 255);
pub const XATTR_SIZE_MAX = __helpers.promoteIntLiteral(c_int, 65536, .decimal);
pub const XATTR_LIST_MAX = __helpers.promoteIntLiteral(c_int, 65536, .decimal);
pub const RTSIG_MAX = @as(c_int, 32);
pub const _POSIX_THREAD_KEYS_MAX = @as(c_int, 128);
pub const PTHREAD_KEYS_MAX = @as(c_int, 1024);
pub const _POSIX_THREAD_DESTRUCTOR_ITERATIONS = @as(c_int, 4);
pub const PTHREAD_DESTRUCTOR_ITERATIONS = _POSIX_THREAD_DESTRUCTOR_ITERATIONS;
pub const _POSIX_THREAD_THREADS_MAX = @as(c_int, 64);
pub const AIO_PRIO_DELTA_MAX = @as(c_int, 20);
pub const PTHREAD_STACK_MIN = @as(c_int, 16384);
pub const DELAYTIMER_MAX = __helpers.promoteIntLiteral(c_int, 2147483647, .decimal);
pub const TTY_NAME_MAX = @as(c_int, 32);
pub const LOGIN_NAME_MAX = @as(c_int, 256);
pub const HOST_NAME_MAX = @as(c_int, 64);
pub const MQ_PRIO_MAX = __helpers.promoteIntLiteral(c_int, 32768, .decimal);
pub const SEM_VALUE_MAX = __helpers.promoteIntLiteral(c_int, 2147483647, .decimal);
pub const SSIZE_MAX = LONG_MAX;
pub const _BITS_POSIX2_LIM_H = @as(c_int, 1);
pub const _POSIX2_BC_BASE_MAX = @as(c_int, 99);
pub const _POSIX2_BC_DIM_MAX = @as(c_int, 2048);
pub const _POSIX2_BC_SCALE_MAX = @as(c_int, 99);
pub const _POSIX2_BC_STRING_MAX = @as(c_int, 1000);
pub const _POSIX2_COLL_WEIGHTS_MAX = @as(c_int, 2);
pub const _POSIX2_EXPR_NEST_MAX = @as(c_int, 32);
pub const _POSIX2_LINE_MAX = @as(c_int, 2048);
pub const _POSIX2_RE_DUP_MAX = @as(c_int, 255);
pub const _POSIX2_CHARCLASS_NAME_MAX = @as(c_int, 14);
pub const BC_BASE_MAX = _POSIX2_BC_BASE_MAX;
pub const BC_DIM_MAX = _POSIX2_BC_DIM_MAX;
pub const BC_SCALE_MAX = _POSIX2_BC_SCALE_MAX;
pub const BC_STRING_MAX = _POSIX2_BC_STRING_MAX;
pub const COLL_WEIGHTS_MAX = @as(c_int, 255);
pub const EXPR_NEST_MAX = _POSIX2_EXPR_NEST_MAX;
pub const LINE_MAX = _POSIX2_LINE_MAX;
pub const CHARCLASS_NAME_MAX = @as(c_int, 2048);
pub const RE_DUP_MAX = @as(c_int, 0x7fff);
pub const _XOPEN_LIM_H = @as(c_int, 1);
pub const _XOPEN_IOV_MAX = @compileError("unable to translate macro: undefined identifier `_POSIX_UIO_MAXIOV`"); // /usr/include/bits/xopen_lim.h:62:9
pub const _BITS_UIO_LIM_H = @as(c_int, 1);
pub const __IOV_MAX = @as(c_int, 1024);
pub const IOV_MAX = __IOV_MAX;
pub const NL_ARGMAX = _POSIX_ARG_MAX;
pub const NL_LANGMAX = _POSIX2_LINE_MAX;
pub const NL_MSGMAX = INT_MAX;
pub const NL_SETMAX = INT_MAX;
pub const NL_TEXTMAX = INT_MAX;
pub const NZERO = @as(c_int, 20);
pub const WORD_BIT = @as(c_int, 32);
pub const LONG_BIT = @as(c_int, 64);
pub const _SIGNAL_H = "";
pub const _BITS_SIGNUM_GENERIC_H = @as(c_int, 1);
pub const SIG_ERR = __helpers.cast(__sighandler_t, -@as(c_int, 1));
pub const SIG_DFL = __helpers.cast(__sighandler_t, @as(c_int, 0));
pub const SIG_IGN = __helpers.cast(__sighandler_t, @as(c_int, 1));
pub const SIG_HOLD = __helpers.cast(__sighandler_t, @as(c_int, 2));
pub const SIGINT = @as(c_int, 2);
pub const SIGILL = @as(c_int, 4);
pub const SIGABRT = @as(c_int, 6);
pub const SIGFPE = @as(c_int, 8);
pub const SIGSEGV = @as(c_int, 11);
pub const SIGTERM = @as(c_int, 15);
pub const SIGHUP = @as(c_int, 1);
pub const SIGQUIT = @as(c_int, 3);
pub const SIGTRAP = @as(c_int, 5);
pub const SIGKILL = @as(c_int, 9);
pub const SIGPIPE = @as(c_int, 13);
pub const SIGALRM = @as(c_int, 14);
pub const SIGIO = SIGPOLL;
pub const SIGIOT = SIGABRT;
pub const SIGCLD = SIGCHLD;
pub const _BITS_SIGNUM_ARCH_H = @as(c_int, 1);
pub const SIGSTKFLT = @as(c_int, 16);
pub const SIGPWR = @as(c_int, 30);
pub const SIGBUS = @as(c_int, 7);
pub const SIGSYS = @as(c_int, 31);
pub const SIGURG = @as(c_int, 23);
pub const SIGSTOP = @as(c_int, 19);
pub const SIGTSTP = @as(c_int, 20);
pub const SIGCONT = @as(c_int, 18);
pub const SIGCHLD = @as(c_int, 17);
pub const SIGTTIN = @as(c_int, 21);
pub const SIGTTOU = @as(c_int, 22);
pub const SIGPOLL = @as(c_int, 29);
pub const SIGXFSZ = @as(c_int, 25);
pub const SIGXCPU = @as(c_int, 24);
pub const SIGVTALRM = @as(c_int, 26);
pub const SIGPROF = @as(c_int, 27);
pub const SIGUSR1 = @as(c_int, 10);
pub const SIGUSR2 = @as(c_int, 12);
pub const SIGWINCH = @as(c_int, 28);
pub const __SIGRTMIN = @as(c_int, 32);
pub const __SIGRTMAX = @as(c_int, 64);
pub const _NSIG = __SIGRTMAX + @as(c_int, 1);
pub const __sig_atomic_t_defined = @as(c_int, 1);
pub const __sigset_t_defined = @as(c_int, 1);
pub const ____sigset_t_defined = "";
pub const _SIGSET_NWORDS = __helpers.div(@as(c_int, 1024), @as(c_int, 8) * __helpers.sizeof(c_ulong));
pub const __uid_t_defined = "";
pub const __siginfo_t_defined = @as(c_int, 1);
pub const ____sigval_t_defined = "";
pub const __SI_MAX_SIZE = @as(c_int, 128);
pub const __SI_PAD_SIZE = __helpers.div(__SI_MAX_SIZE, __helpers.sizeof(c_int)) - @as(c_int, 4);
pub const _BITS_SIGINFO_ARCH_H = @as(c_int, 1);
pub const __SI_ALIGNMENT = "";
pub const __SI_BAND_TYPE = c_long;
pub const __SI_CLOCK_T = __clock_t;
pub const __SI_ERRNO_THEN_CODE = @as(c_int, 1);
pub const __SI_HAVE_SIGSYS = @as(c_int, 1);
pub const __SI_SIGFAULT_ADDL = "";
pub const si_pid = @compileError("unable to translate macro: undefined identifier `_sifields`"); // /usr/include/bits/types/siginfo_t.h:128:9
pub const si_uid = @compileError("unable to translate macro: undefined identifier `_sifields`"); // /usr/include/bits/types/siginfo_t.h:129:9
pub const si_timerid = @compileError("unable to translate macro: undefined identifier `_sifields`"); // /usr/include/bits/types/siginfo_t.h:130:9
pub const si_overrun = @compileError("unable to translate macro: undefined identifier `_sifields`"); // /usr/include/bits/types/siginfo_t.h:131:9
pub const si_status = @compileError("unable to translate macro: undefined identifier `_sifields`"); // /usr/include/bits/types/siginfo_t.h:132:9
pub const si_utime = @compileError("unable to translate macro: undefined identifier `_sifields`"); // /usr/include/bits/types/siginfo_t.h:133:9
pub const si_stime = @compileError("unable to translate macro: undefined identifier `_sifields`"); // /usr/include/bits/types/siginfo_t.h:134:9
pub const si_value = @compileError("unable to translate macro: undefined identifier `_sifields`"); // /usr/include/bits/types/siginfo_t.h:135:9
pub const si_int = @compileError("unable to translate macro: undefined identifier `_sifields`"); // /usr/include/bits/types/siginfo_t.h:136:9
pub const si_ptr = @compileError("unable to translate macro: undefined identifier `_sifields`"); // /usr/include/bits/types/siginfo_t.h:137:9
pub const si_addr = @compileError("unable to translate macro: undefined identifier `_sifields`"); // /usr/include/bits/types/siginfo_t.h:138:9
pub const si_addr_lsb = @compileError("unable to translate macro: undefined identifier `_sifields`"); // /usr/include/bits/types/siginfo_t.h:139:9
pub const si_lower = @compileError("unable to translate macro: undefined identifier `_sifields`"); // /usr/include/bits/types/siginfo_t.h:140:9
pub const si_upper = @compileError("unable to translate macro: undefined identifier `_sifields`"); // /usr/include/bits/types/siginfo_t.h:141:9
pub const si_pkey = @compileError("unable to translate macro: undefined identifier `_sifields`"); // /usr/include/bits/types/siginfo_t.h:142:9
pub const si_band = @compileError("unable to translate macro: undefined identifier `_sifields`"); // /usr/include/bits/types/siginfo_t.h:143:9
pub const si_fd = @compileError("unable to translate macro: undefined identifier `_sifields`"); // /usr/include/bits/types/siginfo_t.h:144:9
pub const si_call_addr = @compileError("unable to translate macro: undefined identifier `_sifields`"); // /usr/include/bits/types/siginfo_t.h:146:10
pub const si_syscall = @compileError("unable to translate macro: undefined identifier `_sifields`"); // /usr/include/bits/types/siginfo_t.h:147:10
pub const si_arch = @compileError("unable to translate macro: undefined identifier `_sifields`"); // /usr/include/bits/types/siginfo_t.h:148:10
pub const _BITS_SIGINFO_CONSTS_H = @as(c_int, 1);
pub const __SI_ASYNCIO_AFTER_SIGIO = @as(c_int, 1);
pub const __sigval_t_defined = "";
pub const __sigevent_t_defined = @as(c_int, 1);
pub const __SIGEV_MAX_SIZE = @as(c_int, 64);
pub const __SIGEV_PAD_SIZE = __helpers.div(__SIGEV_MAX_SIZE, __helpers.sizeof(c_int)) - @as(c_int, 4);
pub const __have_pthread_attr_t = @as(c_int, 1);
pub const sigev_notify_function = @compileError("unable to translate macro: undefined identifier `_sigev_un`"); // /usr/include/bits/types/sigevent_t.h:45:9
pub const sigev_notify_attributes = @compileError("unable to translate macro: undefined identifier `_sigev_un`"); // /usr/include/bits/types/sigevent_t.h:46:9
pub const _BITS_SIGEVENT_CONSTS_H = @as(c_int, 1);
pub inline fn sigmask(sig: anytype) @TypeOf(__glibc_macro_warning("sigmask is deprecated")(__helpers.cast(c_int, @as(c_uint, 1) << (sig - @as(c_int, 1))))) {
    _ = &sig;
    return __glibc_macro_warning("sigmask is deprecated")(__helpers.cast(c_int, @as(c_uint, 1) << (sig - @as(c_int, 1))));
}
pub const NSIG = _NSIG;
pub const _BITS_SIGACTION_H = @as(c_int, 1);
pub const sa_handler = @compileError("unable to translate macro: undefined identifier `__sigaction_handler`"); // /usr/include/bits/sigaction.h:39:10
pub const sa_sigaction = @compileError("unable to translate macro: undefined identifier `__sigaction_handler`"); // /usr/include/bits/sigaction.h:40:10
pub const SA_NOCLDSTOP = @as(c_int, 1);
pub const SA_NOCLDWAIT = @as(c_int, 2);
pub const SA_SIGINFO = @as(c_int, 4);
pub const SA_ONSTACK = __helpers.promoteIntLiteral(c_int, 0x08000000, .hex);
pub const SA_RESTART = __helpers.promoteIntLiteral(c_int, 0x10000000, .hex);
pub const SA_NODEFER = __helpers.promoteIntLiteral(c_int, 0x40000000, .hex);
pub const SA_RESETHAND = __helpers.promoteIntLiteral(c_int, 0x80000000, .hex);
pub const SA_INTERRUPT = __helpers.promoteIntLiteral(c_int, 0x20000000, .hex);
pub const SA_NOMASK = SA_NODEFER;
pub const SA_ONESHOT = SA_RESETHAND;
pub const SA_STACK = SA_ONSTACK;
pub const SIG_BLOCK = @as(c_int, 0);
pub const SIG_UNBLOCK = @as(c_int, 1);
pub const SIG_SETMASK = @as(c_int, 2);
pub const _BITS_SIGCONTEXT_H = @as(c_int, 1);
pub const FP_XSTATE_MAGIC1 = __helpers.promoteIntLiteral(c_uint, 0x46505853, .hex);
pub const FP_XSTATE_MAGIC2 = __helpers.promoteIntLiteral(c_uint, 0x46505845, .hex);
pub const FP_XSTATE_MAGIC2_SIZE = __helpers.sizeof(FP_XSTATE_MAGIC2);
pub const __need_size_t = "";
pub const __STDC_VERSION_STDDEF_H__ = @as(c_long, 202311);
pub const NULL = __helpers.cast(?*anyopaque, @as(c_int, 0));
pub const offsetof = @compileError("unable to translate macro: undefined identifier `__builtin_offsetof`"); // ./zig-pkg/aro-0.0.0-JSD1Qi7QNgDnfcrdEJf82v3o6MhZySjYVrtdfEf3E4Se/include/stddef.h:18:9
pub const __stack_t_defined = @as(c_int, 1);
pub const _SYS_UCONTEXT_H = @as(c_int, 1);
pub const __NGREG = @as(c_int, 23);
pub const NGREG = __NGREG;
pub const _BITS_SIGSTACK_H = @as(c_int, 1);
pub const MINSIGSTKSZ = @as(c_int, 2048);
pub const SIGSTKSZ = @as(c_int, 8192);
pub const _BITS_SS_FLAGS_H = @as(c_int, 1);
pub const __sigstack_defined = @as(c_int, 1);
pub const _BITS_PTHREADTYPES_COMMON_H = @as(c_int, 1);
pub const _THREAD_SHARED_TYPES_H = @as(c_int, 1);
pub const _BITS_PTHREADTYPES_ARCH_H = @as(c_int, 1);
pub const __SIZEOF_PTHREAD_MUTEX_T = @as(c_int, 40);
pub const __SIZEOF_PTHREAD_ATTR_T = @as(c_int, 56);
pub const __SIZEOF_PTHREAD_RWLOCK_T = @as(c_int, 56);
pub const __SIZEOF_PTHREAD_BARRIER_T = @as(c_int, 32);
pub const __SIZEOF_PTHREAD_MUTEXATTR_T = @as(c_int, 4);
pub const __SIZEOF_PTHREAD_COND_T = @as(c_int, 48);
pub const __SIZEOF_PTHREAD_CONDATTR_T = @as(c_int, 4);
pub const __SIZEOF_PTHREAD_RWLOCKATTR_T = @as(c_int, 8);
pub const __SIZEOF_PTHREAD_BARRIERATTR_T = @as(c_int, 4);
pub const __LOCK_ALIGNMENT = "";
pub const __ONCE_ALIGNMENT = "";
pub const _BITS_ATOMIC_WIDE_COUNTER_H = "";
pub const _THREAD_MUTEX_INTERNAL_H = @as(c_int, 1);
pub const __PTHREAD_MUTEX_HAVE_PREV = @as(c_int, 1);
pub const __PTHREAD_MUTEX_INITIALIZER = @compileError("unable to translate C expr: unexpected token '{'"); // /usr/include/bits/struct_mutex.h:55:10
pub const _RWLOCK_INTERNAL_H = "";
pub inline fn __PTHREAD_RWLOCK_INITIALIZER(__flags: anytype) @TypeOf(__flags) {
    _ = &__flags;
    return blk: {
        _ = @as(c_int, 0);
        _ = @as(c_int, 0);
        _ = @as(c_int, 0);
        _ = @as(c_int, 0);
        _ = @as(c_int, 0);
        _ = @as(c_int, 0);
        _ = @as(c_int, 0);
        _ = @as(c_int, 0);
        _ = @as(c_int, 0);
        _ = @as(c_int, 0);
        break :blk __flags;
    };
}
pub const __ONCE_FLAG_INIT = @compileError("unable to translate C expr: unexpected token '{'"); // /usr/include/bits/thread-shared-types.h:114:9
pub const _BITS_SIGTHREAD_H = @as(c_int, 1);
pub const SIGRTMIN = __libc_current_sigrtmin();
pub const SIGRTMAX = __libc_current_sigrtmax();
pub const __STDC_VERSION_STDARG_H__ = @as(c_int, 0);
pub const va_start = @compileError("unable to translate macro: undefined identifier `__builtin_va_start`"); // ./zig-pkg/aro-0.0.0-JSD1Qi7QNgDnfcrdEJf82v3o6MhZySjYVrtdfEf3E4Se/include/stdarg.h:12:9
pub const va_end = @compileError("unable to translate macro: undefined identifier `__builtin_va_end`"); // ./zig-pkg/aro-0.0.0-JSD1Qi7QNgDnfcrdEJf82v3o6MhZySjYVrtdfEf3E4Se/include/stdarg.h:14:9
pub const va_arg = @compileError("unable to translate macro: undefined identifier `__builtin_va_arg`"); // ./zig-pkg/aro-0.0.0-JSD1Qi7QNgDnfcrdEJf82v3o6MhZySjYVrtdfEf3E4Se/include/stdarg.h:15:9
pub const __va_copy = @compileError("unable to translate macro: undefined identifier `__builtin_va_copy`"); // ./zig-pkg/aro-0.0.0-JSD1Qi7QNgDnfcrdEJf82v3o6MhZySjYVrtdfEf3E4Se/include/stdarg.h:18:9
pub const va_copy = @compileError("unable to translate macro: undefined identifier `__builtin_va_copy`"); // ./zig-pkg/aro-0.0.0-JSD1Qi7QNgDnfcrdEJf82v3o6MhZySjYVrtdfEf3E4Se/include/stdarg.h:22:9
pub const __GNUC_VA_LIST = @as(c_int, 1);
pub const _STDINT_H = @as(c_int, 1);
pub const _BITS_WCHAR_H = @as(c_int, 1);
pub const __WCHAR_MAX = __WCHAR_MAX__;
pub const __WCHAR_MIN = -__WCHAR_MAX - @as(c_int, 1);
pub const _BITS_STDINT_INTN_H = @as(c_int, 1);
pub const _BITS_STDINT_UINTN_H = @as(c_int, 1);
pub const _BITS_STDINT_LEAST_H = @as(c_int, 1);
pub const __intptr_t_defined = "";
pub const __INT64_C = __helpers.L_SUFFIX;
pub const __UINT64_C = __helpers.UL_SUFFIX;
pub const INT8_MIN = -@as(c_int, 128);
pub const INT16_MIN = -@as(c_int, 32767) - @as(c_int, 1);
pub const INT32_MIN = -__helpers.promoteIntLiteral(c_int, 2147483647, .decimal) - @as(c_int, 1);
pub const INT64_MIN = -__INT64_C(__helpers.promoteIntLiteral(c_int, 9223372036854775807, .decimal)) - @as(c_int, 1);
pub const INT8_MAX = @as(c_int, 127);
pub const INT16_MAX = @as(c_int, 32767);
pub const INT32_MAX = __helpers.promoteIntLiteral(c_int, 2147483647, .decimal);
pub const INT64_MAX = __INT64_C(__helpers.promoteIntLiteral(c_int, 9223372036854775807, .decimal));
pub const UINT8_MAX = @as(c_int, 255);
pub const UINT16_MAX = __helpers.promoteIntLiteral(c_int, 65535, .decimal);
pub const UINT32_MAX = __helpers.promoteIntLiteral(c_uint, 4294967295, .decimal);
pub const UINT64_MAX = __UINT64_C(__helpers.promoteIntLiteral(c_int, 18446744073709551615, .decimal));
pub const INT_LEAST8_MIN = -@as(c_int, 128);
pub const INT_LEAST16_MIN = -@as(c_int, 32767) - @as(c_int, 1);
pub const INT_LEAST32_MIN = -__helpers.promoteIntLiteral(c_int, 2147483647, .decimal) - @as(c_int, 1);
pub const INT_LEAST64_MIN = -__INT64_C(__helpers.promoteIntLiteral(c_int, 9223372036854775807, .decimal)) - @as(c_int, 1);
pub const INT_LEAST8_MAX = @as(c_int, 127);
pub const INT_LEAST16_MAX = @as(c_int, 32767);
pub const INT_LEAST32_MAX = __helpers.promoteIntLiteral(c_int, 2147483647, .decimal);
pub const INT_LEAST64_MAX = __INT64_C(__helpers.promoteIntLiteral(c_int, 9223372036854775807, .decimal));
pub const UINT_LEAST8_MAX = @as(c_int, 255);
pub const UINT_LEAST16_MAX = __helpers.promoteIntLiteral(c_int, 65535, .decimal);
pub const UINT_LEAST32_MAX = __helpers.promoteIntLiteral(c_uint, 4294967295, .decimal);
pub const UINT_LEAST64_MAX = __UINT64_C(__helpers.promoteIntLiteral(c_int, 18446744073709551615, .decimal));
pub const INT_FAST8_MIN = -@as(c_int, 128);
pub const INT_FAST16_MIN = -__helpers.promoteIntLiteral(c_long, 9223372036854775807, .decimal) - @as(c_int, 1);
pub const INT_FAST32_MIN = -__helpers.promoteIntLiteral(c_long, 9223372036854775807, .decimal) - @as(c_int, 1);
pub const INT_FAST64_MIN = -__INT64_C(__helpers.promoteIntLiteral(c_int, 9223372036854775807, .decimal)) - @as(c_int, 1);
pub const INT_FAST8_MAX = @as(c_int, 127);
pub const INT_FAST16_MAX = __helpers.promoteIntLiteral(c_long, 9223372036854775807, .decimal);
pub const INT_FAST32_MAX = __helpers.promoteIntLiteral(c_long, 9223372036854775807, .decimal);
pub const INT_FAST64_MAX = __INT64_C(__helpers.promoteIntLiteral(c_int, 9223372036854775807, .decimal));
pub const UINT_FAST8_MAX = @as(c_int, 255);
pub const UINT_FAST16_MAX = __helpers.promoteIntLiteral(c_ulong, 18446744073709551615, .decimal);
pub const UINT_FAST32_MAX = __helpers.promoteIntLiteral(c_ulong, 18446744073709551615, .decimal);
pub const UINT_FAST64_MAX = __UINT64_C(__helpers.promoteIntLiteral(c_int, 18446744073709551615, .decimal));
pub const INTPTR_MIN = -__helpers.promoteIntLiteral(c_long, 9223372036854775807, .decimal) - @as(c_int, 1);
pub const INTPTR_MAX = __helpers.promoteIntLiteral(c_long, 9223372036854775807, .decimal);
pub const UINTPTR_MAX = __helpers.promoteIntLiteral(c_ulong, 18446744073709551615, .decimal);
pub const INTMAX_MIN = -__INT64_C(__helpers.promoteIntLiteral(c_int, 9223372036854775807, .decimal)) - @as(c_int, 1);
pub const INTMAX_MAX = __INT64_C(__helpers.promoteIntLiteral(c_int, 9223372036854775807, .decimal));
pub const UINTMAX_MAX = __UINT64_C(__helpers.promoteIntLiteral(c_int, 18446744073709551615, .decimal));
pub const PTRDIFF_MIN = -__helpers.promoteIntLiteral(c_long, 9223372036854775807, .decimal) - @as(c_int, 1);
pub const PTRDIFF_MAX = __helpers.promoteIntLiteral(c_long, 9223372036854775807, .decimal);
pub const SIG_ATOMIC_MIN = -__helpers.promoteIntLiteral(c_int, 2147483647, .decimal) - @as(c_int, 1);
pub const SIG_ATOMIC_MAX = __helpers.promoteIntLiteral(c_int, 2147483647, .decimal);
pub const SIZE_MAX = __helpers.promoteIntLiteral(c_ulong, 18446744073709551615, .decimal);
pub const WCHAR_MIN = __WCHAR_MIN;
pub const WCHAR_MAX = __WCHAR_MAX;
pub const WINT_MIN = @as(c_uint, 0);
pub const WINT_MAX = __helpers.promoteIntLiteral(c_uint, 4294967295, .decimal);
pub inline fn INT8_C(c: anytype) @TypeOf(c) {
    _ = &c;
    return c;
}
pub inline fn INT16_C(c: anytype) @TypeOf(c) {
    _ = &c;
    return c;
}
pub inline fn INT32_C(c: anytype) @TypeOf(c) {
    _ = &c;
    return c;
}
pub const INT64_C = __helpers.L_SUFFIX;
pub inline fn UINT8_C(c: anytype) @TypeOf(c) {
    _ = &c;
    return c;
}
pub inline fn UINT16_C(c: anytype) @TypeOf(c) {
    _ = &c;
    return c;
}
pub const UINT32_C = __helpers.U_SUFFIX;
pub const UINT64_C = __helpers.UL_SUFFIX;
pub const INTMAX_C = __helpers.L_SUFFIX;
pub const UINTMAX_C = __helpers.UL_SUFFIX;
pub const _STDIO_H = @as(c_int, 1);
pub const __need_NULL = "";
pub const __need___va_list = "";
pub const _____fpos_t_defined = @as(c_int, 1);
pub const ____mbstate_t_defined = @as(c_int, 1);
pub const _____fpos64_t_defined = @as(c_int, 1);
pub const ____FILE_defined = @as(c_int, 1);
pub const __FILE_defined = @as(c_int, 1);
pub const __struct_FILE_defined = @as(c_int, 1);
pub const __getc_unlocked_body = @compileError("TODO postfix inc/dec expr"); // /usr/include/bits/types/struct_FILE.h:113:9
pub const __putc_unlocked_body = @compileError("TODO postfix inc/dec expr"); // /usr/include/bits/types/struct_FILE.h:117:9
pub const _IO_EOF_SEEN = @as(c_int, 0x0010);
pub inline fn __feof_unlocked_body(_fp: anytype) @TypeOf((_fp.*._flags & _IO_EOF_SEEN) != @as(c_int, 0)) {
    _ = &_fp;
    return (_fp.*._flags & _IO_EOF_SEEN) != @as(c_int, 0);
}
pub const _IO_ERR_SEEN = @as(c_int, 0x0020);
pub inline fn __ferror_unlocked_body(_fp: anytype) @TypeOf((_fp.*._flags & _IO_ERR_SEEN) != @as(c_int, 0)) {
    _ = &_fp;
    return (_fp.*._flags & _IO_ERR_SEEN) != @as(c_int, 0);
}
pub const _IO_USER_LOCK = __helpers.promoteIntLiteral(c_int, 0x8000, .hex);
pub const __cookie_io_functions_t_defined = @as(c_int, 1);
pub const _VA_LIST_DEFINED = "";
pub const __ssize_t_defined = "";
pub const _IOFBF = @as(c_int, 0);
pub const _IOLBF = @as(c_int, 1);
pub const _IONBF = @as(c_int, 2);
pub const BUFSIZ = @as(c_int, 8192);
pub const EOF = -@as(c_int, 1);
pub const P_tmpdir = "/tmp";
pub const L_tmpnam = @as(c_int, 20);
pub const TMP_MAX = __helpers.promoteIntLiteral(c_int, 238328, .decimal);
pub const _BITS_STDIO_LIM_H = @as(c_int, 1);
pub const FILENAME_MAX = @as(c_int, 4096);
pub const L_ctermid = @as(c_int, 9);
pub const FOPEN_MAX = @as(c_int, 16);
pub const __attr_dealloc_fclose = __attr_dealloc(fclose, @as(c_int, 1));
pub const _BITS_FLOATN_H = "";
pub const __HAVE_FLOAT128 = @as(c_int, 1);
pub const __HAVE_DISTINCT_FLOAT128 = @as(c_int, 1);
pub const __HAVE_FLOAT64X = @as(c_int, 1);
pub const __HAVE_FLOAT64X_LONG_DOUBLE = @as(c_int, 1);
pub const __f128 = @compileError("unable to translate macro: undefined identifier `f128`"); // /usr/include/bits/floatn.h:72:12
pub const __CFLOAT128 = @compileError("unable to translate: invalid numeric type"); // /usr/include/bits/floatn.h:86:12
pub const _BITS_FLOATN_COMMON_H = "";
pub const __HAVE_FLOAT16 = @as(c_int, 0);
pub const __HAVE_FLOAT32 = @as(c_int, 1);
pub const __HAVE_FLOAT64 = @as(c_int, 1);
pub const __HAVE_FLOAT32X = @as(c_int, 1);
pub const __HAVE_FLOAT128X = @as(c_int, 0);
pub const __HAVE_DISTINCT_FLOAT16 = __HAVE_FLOAT16;
pub const __HAVE_DISTINCT_FLOAT32 = @as(c_int, 0);
pub const __HAVE_DISTINCT_FLOAT64 = @as(c_int, 0);
pub const __HAVE_DISTINCT_FLOAT32X = @as(c_int, 0);
pub const __HAVE_DISTINCT_FLOAT64X = @as(c_int, 0);
pub const __HAVE_DISTINCT_FLOAT128X = __HAVE_FLOAT128X;
pub const __HAVE_FLOAT128_UNLIKE_LDBL = (__HAVE_DISTINCT_FLOAT128 != 0) and (__LDBL_MANT_DIG__ != @as(c_int, 113));
pub const __HAVE_FLOATN_NOT_TYPEDEF = @as(c_int, 1);
pub const __f32 = @compileError("unable to translate macro: undefined identifier `f32`"); // /usr/include/bits/floatn-common.h:93:12
pub const __f64 = @compileError("unable to translate macro: undefined identifier `f64`"); // /usr/include/bits/floatn-common.h:105:12
pub const __f32x = @compileError("unable to translate macro: undefined identifier `f32x`"); // /usr/include/bits/floatn-common.h:113:12
pub const __f64x = @compileError("unable to translate macro: undefined identifier `f64x`"); // /usr/include/bits/floatn-common.h:125:12
pub const __CFLOAT32 = @compileError("unable to translate: invalid numeric type"); // /usr/include/bits/floatn-common.h:151:12
pub const __CFLOAT64 = @compileError("unable to translate: invalid numeric type"); // /usr/include/bits/floatn-common.h:163:12
pub const __CFLOAT32X = @compileError("unable to translate: invalid numeric type"); // /usr/include/bits/floatn-common.h:171:12
pub const __CFLOAT64X = @compileError("unable to translate: invalid numeric type"); // /usr/include/bits/floatn-common.h:183:12
pub const __need_wchar_t = "";
pub const _STDLIB_H = @as(c_int, 1);
pub const WNOHANG = @as(c_int, 1);
pub const WUNTRACED = @as(c_int, 2);
pub const WSTOPPED = @as(c_int, 2);
pub const WEXITED = @as(c_int, 4);
pub const WCONTINUED = @as(c_int, 8);
pub const WNOWAIT = __helpers.promoteIntLiteral(c_int, 0x01000000, .hex);
pub const __WNOTHREAD = __helpers.promoteIntLiteral(c_int, 0x20000000, .hex);
pub const __WALL = __helpers.promoteIntLiteral(c_int, 0x40000000, .hex);
pub const __WCLONE = __helpers.promoteIntLiteral(c_int, 0x80000000, .hex);
pub inline fn __WEXITSTATUS(status: anytype) @TypeOf((status & __helpers.promoteIntLiteral(c_int, 0xff00, .hex)) >> @as(c_int, 8)) {
    _ = &status;
    return (status & __helpers.promoteIntLiteral(c_int, 0xff00, .hex)) >> @as(c_int, 8);
}
pub inline fn __WTERMSIG(status: anytype) @TypeOf(status & @as(c_int, 0x7f)) {
    _ = &status;
    return status & @as(c_int, 0x7f);
}
pub inline fn __WSTOPSIG(status: anytype) @TypeOf(__WEXITSTATUS(status)) {
    _ = &status;
    return __WEXITSTATUS(status);
}
pub inline fn __WIFEXITED(status: anytype) @TypeOf(__WTERMSIG(status) == @as(c_int, 0)) {
    _ = &status;
    return __WTERMSIG(status) == @as(c_int, 0);
}
pub inline fn __WIFSIGNALED(status: anytype) @TypeOf((__helpers.cast(i8, (status & @as(c_int, 0x7f)) + @as(c_int, 1)) >> @as(c_int, 1)) > @as(c_int, 0)) {
    _ = &status;
    return (__helpers.cast(i8, (status & @as(c_int, 0x7f)) + @as(c_int, 1)) >> @as(c_int, 1)) > @as(c_int, 0);
}
pub inline fn __WIFSTOPPED(status: anytype) @TypeOf((status & @as(c_int, 0xff)) == @as(c_int, 0x7f)) {
    _ = &status;
    return (status & @as(c_int, 0xff)) == @as(c_int, 0x7f);
}
pub inline fn __WIFCONTINUED(status: anytype) @TypeOf(status == __W_CONTINUED) {
    _ = &status;
    return status == __W_CONTINUED;
}
pub inline fn __WCOREDUMP(status: anytype) @TypeOf(status & __WCOREFLAG) {
    _ = &status;
    return status & __WCOREFLAG;
}
pub inline fn __W_EXITCODE(ret: anytype, sig: anytype) @TypeOf((ret << @as(c_int, 8)) | sig) {
    _ = &ret;
    _ = &sig;
    return (ret << @as(c_int, 8)) | sig;
}
pub inline fn __W_STOPCODE(sig: anytype) @TypeOf((sig << @as(c_int, 8)) | @as(c_int, 0x7f)) {
    _ = &sig;
    return (sig << @as(c_int, 8)) | @as(c_int, 0x7f);
}
pub const __W_CONTINUED = __helpers.promoteIntLiteral(c_int, 0xffff, .hex);
pub const __WCOREFLAG = @as(c_int, 0x80);
pub inline fn WEXITSTATUS(status: anytype) @TypeOf(__WEXITSTATUS(status)) {
    _ = &status;
    return __WEXITSTATUS(status);
}
pub inline fn WTERMSIG(status: anytype) @TypeOf(__WTERMSIG(status)) {
    _ = &status;
    return __WTERMSIG(status);
}
pub inline fn WSTOPSIG(status: anytype) @TypeOf(__WSTOPSIG(status)) {
    _ = &status;
    return __WSTOPSIG(status);
}
pub inline fn WIFEXITED(status: anytype) @TypeOf(__WIFEXITED(status)) {
    _ = &status;
    return __WIFEXITED(status);
}
pub inline fn WIFSIGNALED(status: anytype) @TypeOf(__WIFSIGNALED(status)) {
    _ = &status;
    return __WIFSIGNALED(status);
}
pub inline fn WIFSTOPPED(status: anytype) @TypeOf(__WIFSTOPPED(status)) {
    _ = &status;
    return __WIFSTOPPED(status);
}
pub inline fn WIFCONTINUED(status: anytype) @TypeOf(__WIFCONTINUED(status)) {
    _ = &status;
    return __WIFCONTINUED(status);
}
pub const __ldiv_t_defined = @as(c_int, 1);
pub const __lldiv_t_defined = @as(c_int, 1);
pub const RAND_MAX = __helpers.promoteIntLiteral(c_int, 2147483647, .decimal);
pub const EXIT_FAILURE = @as(c_int, 1);
pub const EXIT_SUCCESS = @as(c_int, 0);
pub const MB_CUR_MAX = __ctype_get_mb_cur_max();
pub const _SYS_TYPES_H = @as(c_int, 1);
pub const __u_char_defined = "";
pub const __ino_t_defined = "";
pub const __dev_t_defined = "";
pub const __gid_t_defined = "";
pub const __nlink_t_defined = "";
pub const __id_t_defined = "";
pub const __daddr_t_defined = "";
pub const __key_t_defined = "";
pub const __clock_t_defined = @as(c_int, 1);
pub const __clockid_t_defined = @as(c_int, 1);
pub const __timer_t_defined = @as(c_int, 1);
pub const __useconds_t_defined = "";
pub const __suseconds_t_defined = "";
pub const __BIT_TYPES_DEFINED__ = @as(c_int, 1);
pub const _ENDIAN_H = @as(c_int, 1);
pub const LITTLE_ENDIAN = __LITTLE_ENDIAN;
pub const BIG_ENDIAN = __BIG_ENDIAN;
pub const PDP_ENDIAN = __PDP_ENDIAN;
pub const BYTE_ORDER = __BYTE_ORDER;
pub const _BITS_BYTESWAP_H = @as(c_int, 1);
pub inline fn __bswap_constant_16(x: anytype) __uint16_t {
    _ = &x;
    return __helpers.cast(__uint16_t, ((x >> @as(c_int, 8)) & @as(c_int, 0xff)) | ((x & @as(c_int, 0xff)) << @as(c_int, 8)));
}
pub inline fn __bswap_constant_32(x: anytype) @TypeOf(((((x & __helpers.promoteIntLiteral(c_uint, 0xff000000, .hex)) >> @as(c_int, 24)) | ((x & __helpers.promoteIntLiteral(c_uint, 0x00ff0000, .hex)) >> @as(c_int, 8))) | ((x & @as(c_uint, 0x0000ff00)) << @as(c_int, 8))) | ((x & @as(c_uint, 0x000000ff)) << @as(c_int, 24))) {
    _ = &x;
    return ((((x & __helpers.promoteIntLiteral(c_uint, 0xff000000, .hex)) >> @as(c_int, 24)) | ((x & __helpers.promoteIntLiteral(c_uint, 0x00ff0000, .hex)) >> @as(c_int, 8))) | ((x & @as(c_uint, 0x0000ff00)) << @as(c_int, 8))) | ((x & @as(c_uint, 0x000000ff)) << @as(c_int, 24));
}
pub inline fn __bswap_constant_64(x: anytype) @TypeOf(((((((((x & @as(c_ulonglong, 0xff00000000000000)) >> @as(c_int, 56)) | ((x & @as(c_ulonglong, 0x00ff000000000000)) >> @as(c_int, 40))) | ((x & @as(c_ulonglong, 0x0000ff0000000000)) >> @as(c_int, 24))) | ((x & @as(c_ulonglong, 0x000000ff00000000)) >> @as(c_int, 8))) | ((x & @as(c_ulonglong, 0x00000000ff000000)) << @as(c_int, 8))) | ((x & @as(c_ulonglong, 0x0000000000ff0000)) << @as(c_int, 24))) | ((x & @as(c_ulonglong, 0x000000000000ff00)) << @as(c_int, 40))) | ((x & @as(c_ulonglong, 0x00000000000000ff)) << @as(c_int, 56))) {
    _ = &x;
    return ((((((((x & @as(c_ulonglong, 0xff00000000000000)) >> @as(c_int, 56)) | ((x & @as(c_ulonglong, 0x00ff000000000000)) >> @as(c_int, 40))) | ((x & @as(c_ulonglong, 0x0000ff0000000000)) >> @as(c_int, 24))) | ((x & @as(c_ulonglong, 0x000000ff00000000)) >> @as(c_int, 8))) | ((x & @as(c_ulonglong, 0x00000000ff000000)) << @as(c_int, 8))) | ((x & @as(c_ulonglong, 0x0000000000ff0000)) << @as(c_int, 24))) | ((x & @as(c_ulonglong, 0x000000000000ff00)) << @as(c_int, 40))) | ((x & @as(c_ulonglong, 0x00000000000000ff)) << @as(c_int, 56));
}
pub const _BITS_UINTN_IDENTITY_H = @as(c_int, 1);
pub inline fn htobe16(x: anytype) @TypeOf(__bswap_16(x)) {
    _ = &x;
    return __bswap_16(x);
}
pub inline fn htole16(x: anytype) @TypeOf(__uint16_identity(x)) {
    _ = &x;
    return __uint16_identity(x);
}
pub inline fn be16toh(x: anytype) @TypeOf(__bswap_16(x)) {
    _ = &x;
    return __bswap_16(x);
}
pub inline fn le16toh(x: anytype) @TypeOf(__uint16_identity(x)) {
    _ = &x;
    return __uint16_identity(x);
}
pub inline fn htobe32(x: anytype) @TypeOf(__bswap_32(x)) {
    _ = &x;
    return __bswap_32(x);
}
pub inline fn htole32(x: anytype) @TypeOf(__uint32_identity(x)) {
    _ = &x;
    return __uint32_identity(x);
}
pub inline fn be32toh(x: anytype) @TypeOf(__bswap_32(x)) {
    _ = &x;
    return __bswap_32(x);
}
pub inline fn le32toh(x: anytype) @TypeOf(__uint32_identity(x)) {
    _ = &x;
    return __uint32_identity(x);
}
pub inline fn htobe64(x: anytype) @TypeOf(__bswap_64(x)) {
    _ = &x;
    return __bswap_64(x);
}
pub inline fn htole64(x: anytype) @TypeOf(__uint64_identity(x)) {
    _ = &x;
    return __uint64_identity(x);
}
pub inline fn be64toh(x: anytype) @TypeOf(__bswap_64(x)) {
    _ = &x;
    return __bswap_64(x);
}
pub inline fn le64toh(x: anytype) @TypeOf(__uint64_identity(x)) {
    _ = &x;
    return __uint64_identity(x);
}
pub const _SYS_SELECT_H = @as(c_int, 1);
pub const __FD_ZERO = @compileError("unable to translate macro: undefined identifier `__i`"); // /usr/include/bits/select.h:25:9
pub const __FD_SET = @compileError("unable to translate C expr: expected ')' instead got '|='"); // /usr/include/bits/select.h:32:9
pub const __FD_CLR = @compileError("unable to translate C expr: expected ')' instead got '&='"); // /usr/include/bits/select.h:34:9
pub inline fn __FD_ISSET(d: anytype, s: anytype) @TypeOf((__FDS_BITS(s)[@as(usize, @intCast(__FD_ELT(d)))] & __FD_MASK(d)) != @as(c_int, 0)) {
    _ = &d;
    _ = &s;
    return (__FDS_BITS(s)[@as(usize, @intCast(__FD_ELT(d)))] & __FD_MASK(d)) != @as(c_int, 0);
}
pub const __timeval_defined = @as(c_int, 1);
pub const __NFDBITS = @as(c_int, 8) * __helpers.cast(c_int, __helpers.sizeof(__fd_mask));
pub inline fn __FD_ELT(d: anytype) @TypeOf(__helpers.div(d, __NFDBITS)) {
    _ = &d;
    return __helpers.div(d, __NFDBITS);
}
pub inline fn __FD_MASK(d: anytype) __fd_mask {
    _ = &d;
    return __helpers.cast(__fd_mask, @as(c_ulong, 1) << __helpers.rem(d, __NFDBITS));
}
pub inline fn __FDS_BITS(set: anytype) @TypeOf(set.*.fds_bits) {
    _ = &set;
    return set.*.fds_bits;
}
pub const FD_SETSIZE = __FD_SETSIZE;
pub const NFDBITS = __NFDBITS;
pub inline fn FD_SET(fd: anytype, fdsetp: anytype) @TypeOf(__FD_SET(fd, fdsetp)) {
    _ = &fd;
    _ = &fdsetp;
    return __FD_SET(fd, fdsetp);
}
pub inline fn FD_CLR(fd: anytype, fdsetp: anytype) @TypeOf(__FD_CLR(fd, fdsetp)) {
    _ = &fd;
    _ = &fdsetp;
    return __FD_CLR(fd, fdsetp);
}
pub inline fn FD_ISSET(fd: anytype, fdsetp: anytype) @TypeOf(__FD_ISSET(fd, fdsetp)) {
    _ = &fd;
    _ = &fdsetp;
    return __FD_ISSET(fd, fdsetp);
}
pub inline fn FD_ZERO(fdsetp: anytype) @TypeOf(__FD_ZERO(fdsetp)) {
    _ = &fdsetp;
    return __FD_ZERO(fdsetp);
}
pub const __blksize_t_defined = "";
pub const __blkcnt_t_defined = "";
pub const __fsblkcnt_t_defined = "";
pub const __fsfilcnt_t_defined = "";
pub const _ALLOCA_H = @as(c_int, 1);
pub const __COMPAR_FN_T = "";
pub const _STRING_H = @as(c_int, 1);
pub const __GLIBC_USE_LIB_EXT2 = @as(c_int, 0);
pub const __GLIBC_USE_IEC_60559_BFP_EXT = @as(c_int, 0);
pub const __GLIBC_USE_IEC_60559_BFP_EXT_C23 = @as(c_int, 0);
pub const __GLIBC_USE_IEC_60559_EXT = @as(c_int, 0);
pub const __GLIBC_USE_IEC_60559_FUNCS_EXT = @as(c_int, 0);
pub const __GLIBC_USE_IEC_60559_FUNCS_EXT_C23 = @as(c_int, 0);
pub const __GLIBC_USE_IEC_60559_TYPES_EXT = @as(c_int, 0);
pub const _BITS_TYPES_LOCALE_T_H = @as(c_int, 1);
pub const _BITS_TYPES___LOCALE_T_H = @as(c_int, 1);
pub const _STRINGS_H = @as(c_int, 1);
pub const _SYS_IOCTL_H = @as(c_int, 1);
pub const __ASM_GENERIC_IOCTLS_H = "";
pub const _LINUX_IOCTL_H = "";
pub const _ASM_GENERIC_IOCTL_H = "";
pub const _IOC_NRBITS = @as(c_int, 8);
pub const _IOC_TYPEBITS = @as(c_int, 8);
pub const _IOC_SIZEBITS = @as(c_int, 14);
pub const _IOC_DIRBITS = @as(c_int, 2);
pub const _IOC_NRMASK = (@as(c_int, 1) << _IOC_NRBITS) - @as(c_int, 1);
pub const _IOC_TYPEMASK = (@as(c_int, 1) << _IOC_TYPEBITS) - @as(c_int, 1);
pub const _IOC_SIZEMASK = (@as(c_int, 1) << _IOC_SIZEBITS) - @as(c_int, 1);
pub const _IOC_DIRMASK = (@as(c_int, 1) << _IOC_DIRBITS) - @as(c_int, 1);
pub const _IOC_NRSHIFT = @as(c_int, 0);
pub const _IOC_TYPESHIFT = _IOC_NRSHIFT + _IOC_NRBITS;
pub const _IOC_SIZESHIFT = _IOC_TYPESHIFT + _IOC_TYPEBITS;
pub const _IOC_DIRSHIFT = _IOC_SIZESHIFT + _IOC_SIZEBITS;
pub const _IOC_NONE = @as(c_uint, 0);
pub const _IOC_WRITE = @as(c_uint, 1);
pub const _IOC_READ = @as(c_uint, 2);
pub inline fn _IOC(dir: anytype, @"type": anytype, nr: anytype, size: anytype) @TypeOf((((dir << _IOC_DIRSHIFT) | (@"type" << _IOC_TYPESHIFT)) | (nr << _IOC_NRSHIFT)) | (size << _IOC_SIZESHIFT)) {
    _ = &dir;
    _ = &@"type";
    _ = &nr;
    _ = &size;
    return (((dir << _IOC_DIRSHIFT) | (@"type" << _IOC_TYPESHIFT)) | (nr << _IOC_NRSHIFT)) | (size << _IOC_SIZESHIFT);
}
pub inline fn _IOC_TYPECHECK(t: anytype) @TypeOf(__helpers.sizeof(t)) {
    _ = &t;
    return __helpers.sizeof(t);
}
pub inline fn _IO(@"type": anytype, nr: anytype) @TypeOf(_IOC(_IOC_NONE, @"type", nr, @as(c_int, 0))) {
    _ = &@"type";
    _ = &nr;
    return _IOC(_IOC_NONE, @"type", nr, @as(c_int, 0));
}
pub inline fn _IOR(@"type": anytype, nr: anytype, argtype: anytype) @TypeOf(_IOC(_IOC_READ, @"type", nr, _IOC_TYPECHECK(argtype))) {
    _ = &@"type";
    _ = &nr;
    _ = &argtype;
    return _IOC(_IOC_READ, @"type", nr, _IOC_TYPECHECK(argtype));
}
pub inline fn _IOW(@"type": anytype, nr: anytype, argtype: anytype) @TypeOf(_IOC(_IOC_WRITE, @"type", nr, _IOC_TYPECHECK(argtype))) {
    _ = &@"type";
    _ = &nr;
    _ = &argtype;
    return _IOC(_IOC_WRITE, @"type", nr, _IOC_TYPECHECK(argtype));
}
pub inline fn _IOWR(@"type": anytype, nr: anytype, argtype: anytype) @TypeOf(_IOC(_IOC_READ | _IOC_WRITE, @"type", nr, _IOC_TYPECHECK(argtype))) {
    _ = &@"type";
    _ = &nr;
    _ = &argtype;
    return _IOC(_IOC_READ | _IOC_WRITE, @"type", nr, _IOC_TYPECHECK(argtype));
}
pub inline fn _IOR_BAD(@"type": anytype, nr: anytype, argtype: anytype) @TypeOf(_IOC(_IOC_READ, @"type", nr, __helpers.sizeof(argtype))) {
    _ = &@"type";
    _ = &nr;
    _ = &argtype;
    return _IOC(_IOC_READ, @"type", nr, __helpers.sizeof(argtype));
}
pub inline fn _IOW_BAD(@"type": anytype, nr: anytype, argtype: anytype) @TypeOf(_IOC(_IOC_WRITE, @"type", nr, __helpers.sizeof(argtype))) {
    _ = &@"type";
    _ = &nr;
    _ = &argtype;
    return _IOC(_IOC_WRITE, @"type", nr, __helpers.sizeof(argtype));
}
pub inline fn _IOWR_BAD(@"type": anytype, nr: anytype, argtype: anytype) @TypeOf(_IOC(_IOC_READ | _IOC_WRITE, @"type", nr, __helpers.sizeof(argtype))) {
    _ = &@"type";
    _ = &nr;
    _ = &argtype;
    return _IOC(_IOC_READ | _IOC_WRITE, @"type", nr, __helpers.sizeof(argtype));
}
pub inline fn _IOC_DIR(nr: anytype) @TypeOf((nr >> _IOC_DIRSHIFT) & _IOC_DIRMASK) {
    _ = &nr;
    return (nr >> _IOC_DIRSHIFT) & _IOC_DIRMASK;
}
pub inline fn _IOC_TYPE(nr: anytype) @TypeOf((nr >> _IOC_TYPESHIFT) & _IOC_TYPEMASK) {
    _ = &nr;
    return (nr >> _IOC_TYPESHIFT) & _IOC_TYPEMASK;
}
pub inline fn _IOC_NR(nr: anytype) @TypeOf((nr >> _IOC_NRSHIFT) & _IOC_NRMASK) {
    _ = &nr;
    return (nr >> _IOC_NRSHIFT) & _IOC_NRMASK;
}
pub inline fn _IOC_SIZE(nr: anytype) @TypeOf((nr >> _IOC_SIZESHIFT) & _IOC_SIZEMASK) {
    _ = &nr;
    return (nr >> _IOC_SIZESHIFT) & _IOC_SIZEMASK;
}
pub const IOC_IN = _IOC_WRITE << _IOC_DIRSHIFT;
pub const IOC_OUT = _IOC_READ << _IOC_DIRSHIFT;
pub const IOC_INOUT = (_IOC_WRITE | _IOC_READ) << _IOC_DIRSHIFT;
pub const IOCSIZE_MASK = _IOC_SIZEMASK << _IOC_SIZESHIFT;
pub const IOCSIZE_SHIFT = _IOC_SIZESHIFT;
pub const TCGETS = @as(c_int, 0x5401);
pub const TCSETS = @as(c_int, 0x5402);
pub const TCSETSW = @as(c_int, 0x5403);
pub const TCSETSF = @as(c_int, 0x5404);
pub const TCGETA = @as(c_int, 0x5405);
pub const TCSETA = @as(c_int, 0x5406);
pub const TCSETAW = @as(c_int, 0x5407);
pub const TCSETAF = @as(c_int, 0x5408);
pub const TCSBRK = @as(c_int, 0x5409);
pub const TCXONC = @as(c_int, 0x540A);
pub const TCFLSH = @as(c_int, 0x540B);
pub const TIOCEXCL = @as(c_int, 0x540C);
pub const TIOCNXCL = @as(c_int, 0x540D);
pub const TIOCSCTTY = @as(c_int, 0x540E);
pub const TIOCGPGRP = @as(c_int, 0x540F);
pub const TIOCSPGRP = @as(c_int, 0x5410);
pub const TIOCOUTQ = @as(c_int, 0x5411);
pub const TIOCSTI = @as(c_int, 0x5412);
pub const TIOCGWINSZ = @as(c_int, 0x5413);
pub const TIOCSWINSZ = @as(c_int, 0x5414);
pub const TIOCMGET = @as(c_int, 0x5415);
pub const TIOCMBIS = @as(c_int, 0x5416);
pub const TIOCMBIC = @as(c_int, 0x5417);
pub const TIOCMSET = @as(c_int, 0x5418);
pub const TIOCGSOFTCAR = @as(c_int, 0x5419);
pub const TIOCSSOFTCAR = @as(c_int, 0x541A);
pub const FIONREAD = @as(c_int, 0x541B);
pub const TIOCINQ = FIONREAD;
pub const TIOCLINUX = @as(c_int, 0x541C);
pub const TIOCCONS = @as(c_int, 0x541D);
pub const TIOCGSERIAL = @as(c_int, 0x541E);
pub const TIOCSSERIAL = @as(c_int, 0x541F);
pub const TIOCPKT = @as(c_int, 0x5420);
pub const FIONBIO = @as(c_int, 0x5421);
pub const TIOCNOTTY = @as(c_int, 0x5422);
pub const TIOCSETD = @as(c_int, 0x5423);
pub const TIOCGETD = @as(c_int, 0x5424);
pub const TCSBRKP = @as(c_int, 0x5425);
pub const TIOCSBRK = @as(c_int, 0x5427);
pub const TIOCCBRK = @as(c_int, 0x5428);
pub const TIOCGSID = @as(c_int, 0x5429);
pub const TCGETS2 = @compileError("unable to translate macro: undefined identifier `termios2`"); // /usr/include/asm-generic/ioctls.h:61:9
pub const TCSETS2 = @compileError("unable to translate macro: undefined identifier `termios2`"); // /usr/include/asm-generic/ioctls.h:62:9
pub const TCSETSW2 = @compileError("unable to translate macro: undefined identifier `termios2`"); // /usr/include/asm-generic/ioctls.h:63:9
pub const TCSETSF2 = @compileError("unable to translate macro: undefined identifier `termios2`"); // /usr/include/asm-generic/ioctls.h:64:9
pub const TIOCGRS485 = @as(c_int, 0x542E);
pub const TIOCSRS485 = @as(c_int, 0x542F);
pub const TIOCGPTN = _IOR('T', @as(c_int, 0x30), c_uint);
pub const TIOCSPTLCK = _IOW('T', @as(c_int, 0x31), c_int);
pub const TIOCGDEV = _IOR('T', @as(c_int, 0x32), c_uint);
pub const TCGETX = @as(c_int, 0x5432);
pub const TCSETX = @as(c_int, 0x5433);
pub const TCSETXF = @as(c_int, 0x5434);
pub const TCSETXW = @as(c_int, 0x5435);
pub const TIOCSIG = _IOW('T', @as(c_int, 0x36), c_int);
pub const TIOCVHANGUP = @as(c_int, 0x5437);
pub const TIOCGPKT = _IOR('T', @as(c_int, 0x38), c_int);
pub const TIOCGPTLCK = _IOR('T', @as(c_int, 0x39), c_int);
pub const TIOCGEXCL = _IOR('T', @as(c_int, 0x40), c_int);
pub const TIOCGPTPEER = _IO('T', @as(c_int, 0x41));
pub const TIOCGISO7816 = @compileError("unable to translate macro: undefined identifier `serial_iso7816`"); // /usr/include/asm-generic/ioctls.h:82:9
pub const TIOCSISO7816 = @compileError("unable to translate macro: undefined identifier `serial_iso7816`"); // /usr/include/asm-generic/ioctls.h:83:9
pub const FIONCLEX = @as(c_int, 0x5450);
pub const FIOCLEX = @as(c_int, 0x5451);
pub const FIOASYNC = @as(c_int, 0x5452);
pub const TIOCSERCONFIG = @as(c_int, 0x5453);
pub const TIOCSERGWILD = @as(c_int, 0x5454);
pub const TIOCSERSWILD = @as(c_int, 0x5455);
pub const TIOCGLCKTRMIOS = @as(c_int, 0x5456);
pub const TIOCSLCKTRMIOS = @as(c_int, 0x5457);
pub const TIOCSERGSTRUCT = @as(c_int, 0x5458);
pub const TIOCSERGETLSR = @as(c_int, 0x5459);
pub const TIOCSERGETMULTI = @as(c_int, 0x545A);
pub const TIOCSERSETMULTI = @as(c_int, 0x545B);
pub const TIOCMIWAIT = @as(c_int, 0x545C);
pub const TIOCGICOUNT = @as(c_int, 0x545D);
pub const FIOQSIZE = @as(c_int, 0x5460);
pub const TIOCPKT_DATA = @as(c_int, 0);
pub const TIOCPKT_FLUSHREAD = @as(c_int, 1);
pub const TIOCPKT_FLUSHWRITE = @as(c_int, 2);
pub const TIOCPKT_STOP = @as(c_int, 4);
pub const TIOCPKT_START = @as(c_int, 8);
pub const TIOCPKT_NOSTOP = @as(c_int, 16);
pub const TIOCPKT_DOSTOP = @as(c_int, 32);
pub const TIOCPKT_IOCTL = @as(c_int, 64);
pub const TIOCSER_TEMT = @as(c_int, 0x01);
pub const _LINUX_SOCKIOS_H = "";
pub const __ASM_X86_BITSPERLONG_H = "";
pub const __BITS_PER_LONG = @as(c_int, 64);
pub const __ASM_GENERIC_BITS_PER_LONG = "";
pub const __BITS_PER_LONG_LONG = @as(c_int, 64);
pub const __ASM_GENERIC_SOCKIOS_H = "";
pub const FIOSETOWN = __helpers.promoteIntLiteral(c_int, 0x8901, .hex);
pub const SIOCSPGRP = __helpers.promoteIntLiteral(c_int, 0x8902, .hex);
pub const FIOGETOWN = __helpers.promoteIntLiteral(c_int, 0x8903, .hex);
pub const SIOCGPGRP = __helpers.promoteIntLiteral(c_int, 0x8904, .hex);
pub const SIOCATMARK = __helpers.promoteIntLiteral(c_int, 0x8905, .hex);
pub const SIOCGSTAMP_OLD = __helpers.promoteIntLiteral(c_int, 0x8906, .hex);
pub const SIOCGSTAMPNS_OLD = __helpers.promoteIntLiteral(c_int, 0x8907, .hex);
pub const SIOCINQ = FIONREAD;
pub const SIOCOUTQ = TIOCOUTQ;
pub const SOCK_IOC_TYPE = @as(c_int, 0x89);
pub const SIOCGSTAMP_NEW = _IOR(SOCK_IOC_TYPE, @as(c_int, 0x06), c_longlong[@as(usize, @intCast(@as(c_int, 2)))]);
pub const SIOCGSTAMPNS_NEW = _IOR(SOCK_IOC_TYPE, @as(c_int, 0x07), c_longlong[@as(usize, @intCast(@as(c_int, 2)))]);
pub const SIOCGSTAMP = SIOCGSTAMP_OLD;
pub const SIOCGSTAMPNS = SIOCGSTAMPNS_OLD;
pub const SIOCADDRT = __helpers.promoteIntLiteral(c_int, 0x890B, .hex);
pub const SIOCDELRT = __helpers.promoteIntLiteral(c_int, 0x890C, .hex);
pub const SIOCRTMSG = __helpers.promoteIntLiteral(c_int, 0x890D, .hex);
pub const SIOCGIFNAME = __helpers.promoteIntLiteral(c_int, 0x8910, .hex);
pub const SIOCSIFLINK = __helpers.promoteIntLiteral(c_int, 0x8911, .hex);
pub const SIOCGIFCONF = __helpers.promoteIntLiteral(c_int, 0x8912, .hex);
pub const SIOCGIFFLAGS = __helpers.promoteIntLiteral(c_int, 0x8913, .hex);
pub const SIOCSIFFLAGS = __helpers.promoteIntLiteral(c_int, 0x8914, .hex);
pub const SIOCGIFADDR = __helpers.promoteIntLiteral(c_int, 0x8915, .hex);
pub const SIOCSIFADDR = __helpers.promoteIntLiteral(c_int, 0x8916, .hex);
pub const SIOCGIFDSTADDR = __helpers.promoteIntLiteral(c_int, 0x8917, .hex);
pub const SIOCSIFDSTADDR = __helpers.promoteIntLiteral(c_int, 0x8918, .hex);
pub const SIOCGIFBRDADDR = __helpers.promoteIntLiteral(c_int, 0x8919, .hex);
pub const SIOCSIFBRDADDR = __helpers.promoteIntLiteral(c_int, 0x891a, .hex);
pub const SIOCGIFNETMASK = __helpers.promoteIntLiteral(c_int, 0x891b, .hex);
pub const SIOCSIFNETMASK = __helpers.promoteIntLiteral(c_int, 0x891c, .hex);
pub const SIOCGIFMETRIC = __helpers.promoteIntLiteral(c_int, 0x891d, .hex);
pub const SIOCSIFMETRIC = __helpers.promoteIntLiteral(c_int, 0x891e, .hex);
pub const SIOCGIFMEM = __helpers.promoteIntLiteral(c_int, 0x891f, .hex);
pub const SIOCSIFMEM = __helpers.promoteIntLiteral(c_int, 0x8920, .hex);
pub const SIOCGIFMTU = __helpers.promoteIntLiteral(c_int, 0x8921, .hex);
pub const SIOCSIFMTU = __helpers.promoteIntLiteral(c_int, 0x8922, .hex);
pub const SIOCSIFNAME = __helpers.promoteIntLiteral(c_int, 0x8923, .hex);
pub const SIOCSIFHWADDR = __helpers.promoteIntLiteral(c_int, 0x8924, .hex);
pub const SIOCGIFENCAP = __helpers.promoteIntLiteral(c_int, 0x8925, .hex);
pub const SIOCSIFENCAP = __helpers.promoteIntLiteral(c_int, 0x8926, .hex);
pub const SIOCGIFHWADDR = __helpers.promoteIntLiteral(c_int, 0x8927, .hex);
pub const SIOCGIFSLAVE = __helpers.promoteIntLiteral(c_int, 0x8929, .hex);
pub const SIOCSIFSLAVE = __helpers.promoteIntLiteral(c_int, 0x8930, .hex);
pub const SIOCADDMULTI = __helpers.promoteIntLiteral(c_int, 0x8931, .hex);
pub const SIOCDELMULTI = __helpers.promoteIntLiteral(c_int, 0x8932, .hex);
pub const SIOCGIFINDEX = __helpers.promoteIntLiteral(c_int, 0x8933, .hex);
pub const SIOGIFINDEX = SIOCGIFINDEX;
pub const SIOCSIFPFLAGS = __helpers.promoteIntLiteral(c_int, 0x8934, .hex);
pub const SIOCGIFPFLAGS = __helpers.promoteIntLiteral(c_int, 0x8935, .hex);
pub const SIOCDIFADDR = __helpers.promoteIntLiteral(c_int, 0x8936, .hex);
pub const SIOCSIFHWBROADCAST = __helpers.promoteIntLiteral(c_int, 0x8937, .hex);
pub const SIOCGIFCOUNT = __helpers.promoteIntLiteral(c_int, 0x8938, .hex);
pub const SIOCGIFBR = __helpers.promoteIntLiteral(c_int, 0x8940, .hex);
pub const SIOCSIFBR = __helpers.promoteIntLiteral(c_int, 0x8941, .hex);
pub const SIOCGIFTXQLEN = __helpers.promoteIntLiteral(c_int, 0x8942, .hex);
pub const SIOCSIFTXQLEN = __helpers.promoteIntLiteral(c_int, 0x8943, .hex);
pub const SIOCETHTOOL = __helpers.promoteIntLiteral(c_int, 0x8946, .hex);
pub const SIOCGMIIPHY = __helpers.promoteIntLiteral(c_int, 0x8947, .hex);
pub const SIOCGMIIREG = __helpers.promoteIntLiteral(c_int, 0x8948, .hex);
pub const SIOCSMIIREG = __helpers.promoteIntLiteral(c_int, 0x8949, .hex);
pub const SIOCWANDEV = __helpers.promoteIntLiteral(c_int, 0x894A, .hex);
pub const SIOCOUTQNSD = __helpers.promoteIntLiteral(c_int, 0x894B, .hex);
pub const SIOCGSKNS = __helpers.promoteIntLiteral(c_int, 0x894C, .hex);
pub const SIOCDARP = __helpers.promoteIntLiteral(c_int, 0x8953, .hex);
pub const SIOCGARP = __helpers.promoteIntLiteral(c_int, 0x8954, .hex);
pub const SIOCSARP = __helpers.promoteIntLiteral(c_int, 0x8955, .hex);
pub const SIOCDRARP = __helpers.promoteIntLiteral(c_int, 0x8960, .hex);
pub const SIOCGRARP = __helpers.promoteIntLiteral(c_int, 0x8961, .hex);
pub const SIOCSRARP = __helpers.promoteIntLiteral(c_int, 0x8962, .hex);
pub const SIOCGIFMAP = __helpers.promoteIntLiteral(c_int, 0x8970, .hex);
pub const SIOCSIFMAP = __helpers.promoteIntLiteral(c_int, 0x8971, .hex);
pub const SIOCADDDLCI = __helpers.promoteIntLiteral(c_int, 0x8980, .hex);
pub const SIOCDELDLCI = __helpers.promoteIntLiteral(c_int, 0x8981, .hex);
pub const SIOCGIFVLAN = __helpers.promoteIntLiteral(c_int, 0x8982, .hex);
pub const SIOCSIFVLAN = __helpers.promoteIntLiteral(c_int, 0x8983, .hex);
pub const SIOCBONDENSLAVE = __helpers.promoteIntLiteral(c_int, 0x8990, .hex);
pub const SIOCBONDRELEASE = __helpers.promoteIntLiteral(c_int, 0x8991, .hex);
pub const SIOCBONDSETHWADDR = __helpers.promoteIntLiteral(c_int, 0x8992, .hex);
pub const SIOCBONDSLAVEINFOQUERY = __helpers.promoteIntLiteral(c_int, 0x8993, .hex);
pub const SIOCBONDINFOQUERY = __helpers.promoteIntLiteral(c_int, 0x8994, .hex);
pub const SIOCBONDCHANGEACTIVE = __helpers.promoteIntLiteral(c_int, 0x8995, .hex);
pub const SIOCBRADDBR = __helpers.promoteIntLiteral(c_int, 0x89a0, .hex);
pub const SIOCBRDELBR = __helpers.promoteIntLiteral(c_int, 0x89a1, .hex);
pub const SIOCBRADDIF = __helpers.promoteIntLiteral(c_int, 0x89a2, .hex);
pub const SIOCBRDELIF = __helpers.promoteIntLiteral(c_int, 0x89a3, .hex);
pub const SIOCSHWTSTAMP = __helpers.promoteIntLiteral(c_int, 0x89b0, .hex);
pub const SIOCGHWTSTAMP = __helpers.promoteIntLiteral(c_int, 0x89b1, .hex);
pub const SIOCDEVPRIVATE = __helpers.promoteIntLiteral(c_int, 0x89F0, .hex);
pub const SIOCPROTOPRIVATE = __helpers.promoteIntLiteral(c_int, 0x89E0, .hex);
pub const TIOCM_LE = @as(c_int, 0x001);
pub const TIOCM_DTR = @as(c_int, 0x002);
pub const TIOCM_RTS = @as(c_int, 0x004);
pub const TIOCM_ST = @as(c_int, 0x008);
pub const TIOCM_SR = @as(c_int, 0x010);
pub const TIOCM_CTS = @as(c_int, 0x020);
pub const TIOCM_CAR = @as(c_int, 0x040);
pub const TIOCM_RNG = @as(c_int, 0x080);
pub const TIOCM_DSR = @as(c_int, 0x100);
pub const TIOCM_CD = TIOCM_CAR;
pub const TIOCM_RI = TIOCM_RNG;
pub const N_TTY = @as(c_int, 0);
pub const N_SLIP = @as(c_int, 1);
pub const N_MOUSE = @as(c_int, 2);
pub const N_PPP = @as(c_int, 3);
pub const N_STRIP = @as(c_int, 4);
pub const N_AX25 = @as(c_int, 5);
pub const N_X25 = @as(c_int, 6);
pub const N_6PACK = @as(c_int, 7);
pub const N_MASC = @as(c_int, 8);
pub const N_R3964 = @as(c_int, 9);
pub const N_PROFIBUS_FDL = @as(c_int, 10);
pub const N_IRDA = @as(c_int, 11);
pub const N_SMSBLOCK = @as(c_int, 12);
pub const N_HDLC = @as(c_int, 13);
pub const N_SYNC_PPP = @as(c_int, 14);
pub const N_HCI = @as(c_int, 15);
pub const _SYS_TTYDEFAULTS_H_ = "";
pub const TTYDEF_IFLAG = ((((BRKINT | ISTRIP) | ICRNL) | IMAXBEL) | IXON) | IXANY;
pub const TTYDEF_OFLAG = (OPOST | ONLCR) | XTABS;
pub const TTYDEF_LFLAG = (((((ECHO | ICANON) | ISIG) | IEXTEN) | ECHOE) | ECHOKE) | ECHOCTL;
pub const TTYDEF_CFLAG = ((CREAD | CS7) | PARENB) | HUPCL;
pub const TTYDEF_SPEED = B9600;
pub inline fn CTRL(x: anytype) @TypeOf(x & @as(c_int, 0o37)) {
    _ = &x;
    return x & @as(c_int, 0o37);
}
pub const CEOF = CTRL('d');
pub const CEOL = '\x00';
pub const CERASE = @as(c_int, 0o177);
pub const CINTR = CTRL('c');
pub const CSTATUS = '\x00';
pub const CKILL = CTRL('u');
pub const CMIN = @as(c_int, 1);
pub const CQUIT = @as(c_int, 0o34);
pub const CSUSP = CTRL('z');
pub const CTIME = @as(c_int, 0);
pub const CDSUSP = CTRL('y');
pub const CSTART = CTRL('q');
pub const CSTOP = CTRL('s');
pub const CLNEXT = CTRL('v');
pub const CDISCARD = CTRL('o');
pub const CWERASE = CTRL('w');
pub const CREPRINT = CTRL('r');
pub const CEOT = CEOF;
pub const CBRK = CEOL;
pub const CRPRNT = CREPRINT;
pub const CFLUSH = CDISCARD;
pub const _SYS_STAT_H = @as(c_int, 1);
pub inline fn __S_ISTYPE(mode: anytype, mask: anytype) @TypeOf((mode & __S_IFMT) == mask) {
    _ = &mode;
    _ = &mask;
    return (mode & __S_IFMT) == mask;
}
pub inline fn S_ISDIR(mode: anytype) @TypeOf(__S_ISTYPE(mode, __S_IFDIR)) {
    _ = &mode;
    return __S_ISTYPE(mode, __S_IFDIR);
}
pub inline fn S_ISCHR(mode: anytype) @TypeOf(__S_ISTYPE(mode, __S_IFCHR)) {
    _ = &mode;
    return __S_ISTYPE(mode, __S_IFCHR);
}
pub inline fn S_ISBLK(mode: anytype) @TypeOf(__S_ISTYPE(mode, __S_IFBLK)) {
    _ = &mode;
    return __S_ISTYPE(mode, __S_IFBLK);
}
pub inline fn S_ISREG(mode: anytype) @TypeOf(__S_ISTYPE(mode, __S_IFREG)) {
    _ = &mode;
    return __S_ISTYPE(mode, __S_IFREG);
}
pub inline fn S_ISFIFO(mode: anytype) @TypeOf(__S_ISTYPE(mode, __S_IFIFO)) {
    _ = &mode;
    return __S_ISTYPE(mode, __S_IFIFO);
}
pub inline fn S_ISLNK(mode: anytype) @TypeOf(__S_ISTYPE(mode, __S_IFLNK)) {
    _ = &mode;
    return __S_ISTYPE(mode, __S_IFLNK);
}
pub inline fn S_ISSOCK(mode: anytype) @TypeOf(__S_ISTYPE(mode, __S_IFSOCK)) {
    _ = &mode;
    return __S_ISTYPE(mode, __S_IFSOCK);
}
pub inline fn S_TYPEISMQ(buf: anytype) @TypeOf(__S_TYPEISMQ(buf)) {
    _ = &buf;
    return __S_TYPEISMQ(buf);
}
pub inline fn S_TYPEISSEM(buf: anytype) @TypeOf(__S_TYPEISSEM(buf)) {
    _ = &buf;
    return __S_TYPEISSEM(buf);
}
pub inline fn S_TYPEISSHM(buf: anytype) @TypeOf(__S_TYPEISSHM(buf)) {
    _ = &buf;
    return __S_TYPEISSHM(buf);
}
pub const S_IREAD = S_IRUSR;
pub const S_IWRITE = S_IWUSR;
pub const S_IEXEC = S_IXUSR;
pub const ACCESSPERMS = (S_IRWXU | S_IRWXG) | S_IRWXO;
pub const ALLPERMS = ((((S_ISUID | S_ISGID) | S_ISVTX) | S_IRWXU) | S_IRWXG) | S_IRWXO;
pub const DEFFILEMODE = ((((S_IRUSR | S_IWUSR) | S_IRGRP) | S_IWGRP) | S_IROTH) | S_IWOTH;
pub const S_BLKSIZE = @as(c_int, 512);
pub const _SYS_TIME_H = @as(c_int, 1);
pub inline fn timerisset(tvp: anytype) @TypeOf((tvp.*.tv_sec != 0) or (tvp.*.tv_usec != 0)) {
    _ = &tvp;
    return (tvp.*.tv_sec != 0) or (tvp.*.tv_usec != 0);
}
pub const timerclear = @compileError("unable to translate C expr: expected ')' instead got '='"); // /usr/include/sys/time.h:232:10
pub inline fn timercmp(a: anytype, b: anytype, CMP: anytype) @TypeOf(if (__helpers.cast(bool, a.*.tv_sec == b.*.tv_sec)) a.*.tv_usec ++ CMP(b).*.tv_usec else a.*.tv_sec ++ CMP(b).*.tv_sec) {
    _ = &a;
    _ = &b;
    _ = &CMP;
    return if (__helpers.cast(bool, a.*.tv_sec == b.*.tv_sec)) a.*.tv_usec ++ CMP(b).*.tv_usec else a.*.tv_sec ++ CMP(b).*.tv_sec;
}
pub const timeradd = @compileError("unable to translate C expr: unexpected token 'do'"); // /usr/include/sys/time.h:237:10
pub const timersub = @compileError("unable to translate C expr: unexpected token 'do'"); // /usr/include/sys/time.h:247:10
pub const _TERMIOS_H = @as(c_int, 1);
pub const NCCS = @as(c_int, 32);
pub const _HAVE_STRUCT_TERMIOS_C_ISPEED = @as(c_int, 1);
pub const _HAVE_STRUCT_TERMIOS_C_OSPEED = @as(c_int, 1);
pub const VINTR = @as(c_int, 0);
pub const VQUIT = @as(c_int, 1);
pub const VERASE = @as(c_int, 2);
pub const VKILL = @as(c_int, 3);
pub const VEOF = @as(c_int, 4);
pub const VTIME = @as(c_int, 5);
pub const VMIN = @as(c_int, 6);
pub const VSWTC = @as(c_int, 7);
pub const VSTART = @as(c_int, 8);
pub const VSTOP = @as(c_int, 9);
pub const VSUSP = @as(c_int, 10);
pub const VEOL = @as(c_int, 11);
pub const VREPRINT = @as(c_int, 12);
pub const VDISCARD = @as(c_int, 13);
pub const VWERASE = @as(c_int, 14);
pub const VLNEXT = @as(c_int, 15);
pub const VEOL2 = @as(c_int, 16);
pub const IGNBRK = @as(c_int, 0o000001);
pub const BRKINT = @as(c_int, 0o000002);
pub const IGNPAR = @as(c_int, 0o000004);
pub const PARMRK = @as(c_int, 0o000010);
pub const INPCK = @as(c_int, 0o000020);
pub const ISTRIP = @as(c_int, 0o000040);
pub const INLCR = @as(c_int, 0o000100);
pub const IGNCR = @as(c_int, 0o000200);
pub const ICRNL = @as(c_int, 0o000400);
pub const IUCLC = @as(c_int, 0o001000);
pub const IXON = @as(c_int, 0o002000);
pub const IXANY = @as(c_int, 0o004000);
pub const IXOFF = @as(c_int, 0o010000);
pub const IMAXBEL = @as(c_int, 0o020000);
pub const IUTF8 = @as(c_int, 0o040000);
pub const OPOST = @as(c_int, 0o000001);
pub const OLCUC = @as(c_int, 0o000002);
pub const ONLCR = @as(c_int, 0o000004);
pub const OCRNL = @as(c_int, 0o000010);
pub const ONOCR = @as(c_int, 0o000020);
pub const ONLRET = @as(c_int, 0o000040);
pub const OFILL = @as(c_int, 0o000100);
pub const OFDEL = @as(c_int, 0o000200);
pub const NLDLY = @as(c_int, 0o000400);
pub const NL0 = @as(c_int, 0o000000);
pub const NL1 = @as(c_int, 0o000400);
pub const CRDLY = @as(c_int, 0o003000);
pub const CR0 = @as(c_int, 0o000000);
pub const CR1 = @as(c_int, 0o001000);
pub const CR2 = @as(c_int, 0o002000);
pub const CR3 = @as(c_int, 0o003000);
pub const TABDLY = @as(c_int, 0o014000);
pub const TAB0 = @as(c_int, 0o000000);
pub const TAB1 = @as(c_int, 0o004000);
pub const TAB2 = @as(c_int, 0o010000);
pub const TAB3 = @as(c_int, 0o014000);
pub const BSDLY = @as(c_int, 0o020000);
pub const BS0 = @as(c_int, 0o000000);
pub const BS1 = @as(c_int, 0o020000);
pub const FFDLY = __helpers.promoteIntLiteral(c_int, 0o100000, .octal);
pub const FF0 = @as(c_int, 0o000000);
pub const FF1 = __helpers.promoteIntLiteral(c_int, 0o100000, .octal);
pub const VTDLY = @as(c_int, 0o040000);
pub const VT0 = @as(c_int, 0o000000);
pub const VT1 = @as(c_int, 0o040000);
pub const XTABS = @as(c_int, 0o014000);
pub const CSIZE = @as(c_int, 0o000060);
pub const CS5 = @as(c_int, 0o000000);
pub const CS6 = @as(c_int, 0o000020);
pub const CS7 = @as(c_int, 0o000040);
pub const CS8 = @as(c_int, 0o000060);
pub const CSTOPB = @as(c_int, 0o000100);
pub const CREAD = @as(c_int, 0o000200);
pub const PARENB = @as(c_int, 0o000400);
pub const PARODD = @as(c_int, 0o001000);
pub const HUPCL = @as(c_int, 0o002000);
pub const CLOCAL = @as(c_int, 0o004000);
pub const ADDRB = __helpers.promoteIntLiteral(c_int, 0o4000000000, .octal);
pub const CMSPAR = __helpers.promoteIntLiteral(c_int, 0o10000000000, .octal);
pub const CRTSCTS = __helpers.promoteIntLiteral(c_int, 0o20000000000, .octal);
pub const __B0 = @as(c_int, 0o000000);
pub const __B50 = @as(c_int, 0o000001);
pub const __B75 = @as(c_int, 0o000002);
pub const __B110 = @as(c_int, 0o000003);
pub const __B134 = @as(c_int, 0o000004);
pub const __B150 = @as(c_int, 0o000005);
pub const __B200 = @as(c_int, 0o000006);
pub const __B300 = @as(c_int, 0o000007);
pub const __B600 = @as(c_int, 0o000010);
pub const __B1200 = @as(c_int, 0o000011);
pub const __B1800 = @as(c_int, 0o000012);
pub const __B2400 = @as(c_int, 0o000013);
pub const __B4800 = @as(c_int, 0o000014);
pub const __B9600 = @as(c_int, 0o000015);
pub const __B19200 = @as(c_int, 0o000016);
pub const __B38400 = @as(c_int, 0o000017);
pub const CBAUD = @as(c_int, 0o00000010017);
pub const CBAUDEX = @as(c_int, 0o00000010000);
pub const CIBAUD = __helpers.promoteIntLiteral(c_int, 0o02003600000, .octal);
pub const IBSHIFT = @as(c_int, 16);
pub const __BOTHER = @as(c_int, 0o010000);
pub const __B57600 = @as(c_int, 0o010001);
pub const __B115200 = @as(c_int, 0o010002);
pub const __B230400 = @as(c_int, 0o010003);
pub const __B460800 = @as(c_int, 0o010004);
pub const __B500000 = @as(c_int, 0o010005);
pub const __B576000 = @as(c_int, 0o010006);
pub const __B921600 = @as(c_int, 0o010007);
pub const __B1000000 = @as(c_int, 0o010010);
pub const __B1152000 = @as(c_int, 0o010011);
pub const __B1500000 = @as(c_int, 0o010012);
pub const __B2000000 = @as(c_int, 0o010013);
pub const __B2500000 = @as(c_int, 0o010014);
pub const __B3000000 = @as(c_int, 0o010015);
pub const __B3500000 = @as(c_int, 0o010016);
pub const __B4000000 = @as(c_int, 0o010017);
pub const __EXTA = __B19200;
pub const __EXTB = __B38400;
pub const BOTHER = __BOTHER;
pub const ISIG = @as(c_int, 0o000001);
pub const ICANON = @as(c_int, 0o000002);
pub const XCASE = @as(c_int, 0o000004);
pub const ECHO = @as(c_int, 0o000010);
pub const ECHOE = @as(c_int, 0o000020);
pub const ECHOK = @as(c_int, 0o000040);
pub const ECHONL = @as(c_int, 0o000100);
pub const NOFLSH = @as(c_int, 0o000200);
pub const TOSTOP = @as(c_int, 0o000400);
pub const ECHOCTL = @as(c_int, 0o001000);
pub const ECHOPRT = @as(c_int, 0o002000);
pub const ECHOKE = @as(c_int, 0o004000);
pub const FLUSHO = @as(c_int, 0o010000);
pub const PENDIN = @as(c_int, 0o040000);
pub const IEXTEN = __helpers.promoteIntLiteral(c_int, 0o100000, .octal);
pub const EXTPROC = __helpers.promoteIntLiteral(c_int, 0o200000, .octal);
pub const TCOOFF = @as(c_int, 0);
pub const TCOON = @as(c_int, 1);
pub const TCIOFF = @as(c_int, 2);
pub const TCION = @as(c_int, 3);
pub const TCIFLUSH = @as(c_int, 0);
pub const TCOFLUSH = @as(c_int, 1);
pub const TCIOFLUSH = @as(c_int, 2);
pub const TCSANOW = @as(c_int, 0);
pub const TCSADRAIN = @as(c_int, 1);
pub const TCSAFLUSH = @as(c_int, 2);
pub const B0 = @as(c_uint, 0);
pub const B50 = @as(c_uint, 50);
pub const B75 = @as(c_uint, 75);
pub const B110 = @as(c_uint, 110);
pub const B134 = @as(c_uint, 134);
pub const B150 = @as(c_uint, 150);
pub const B200 = @as(c_uint, 200);
pub const B300 = @as(c_uint, 300);
pub const B600 = @as(c_uint, 600);
pub const B1200 = @as(c_uint, 1200);
pub const B1800 = @as(c_uint, 1800);
pub const B2400 = @as(c_uint, 2400);
pub const B4800 = @as(c_uint, 4800);
pub const B9600 = @as(c_uint, 9600);
pub const B19200 = @as(c_uint, 19200);
pub const B38400 = @as(c_uint, 38400);
pub const EXTA = B19200;
pub const EXTB = B38400;
pub const B7200 = @as(c_uint, 7200);
pub const B14400 = @as(c_uint, 14400);
pub const B28800 = @as(c_uint, 28800);
pub const B33600 = @as(c_uint, 33600);
pub const B57600 = @as(c_uint, 57600);
pub const B76800 = __helpers.promoteIntLiteral(c_uint, 76800, .decimal);
pub const B115200 = __helpers.promoteIntLiteral(c_uint, 115200, .decimal);
pub const B153600 = __helpers.promoteIntLiteral(c_uint, 153600, .decimal);
pub const B230400 = __helpers.promoteIntLiteral(c_uint, 230400, .decimal);
pub const B307200 = __helpers.promoteIntLiteral(c_uint, 307200, .decimal);
pub const B460800 = __helpers.promoteIntLiteral(c_uint, 460800, .decimal);
pub const B500000 = __helpers.promoteIntLiteral(c_uint, 500000, .decimal);
pub const B576000 = __helpers.promoteIntLiteral(c_uint, 576000, .decimal);
pub const B614400 = __helpers.promoteIntLiteral(c_uint, 614400, .decimal);
pub const B921600 = __helpers.promoteIntLiteral(c_uint, 921600, .decimal);
pub const B1000000 = __helpers.promoteIntLiteral(c_uint, 1000000, .decimal);
pub const B1152000 = __helpers.promoteIntLiteral(c_uint, 1152000, .decimal);
pub const B1500000 = __helpers.promoteIntLiteral(c_uint, 1500000, .decimal);
pub const B2000000 = __helpers.promoteIntLiteral(c_uint, 2000000, .decimal);
pub const B2500000 = __helpers.promoteIntLiteral(c_uint, 2500000, .decimal);
pub const B3000000 = __helpers.promoteIntLiteral(c_uint, 3000000, .decimal);
pub const B3500000 = __helpers.promoteIntLiteral(c_uint, 3500000, .decimal);
pub const B4000000 = __helpers.promoteIntLiteral(c_uint, 4000000, .decimal);
pub const B5000000 = __helpers.promoteIntLiteral(c_uint, 5000000, .decimal);
pub const B10000000 = __helpers.promoteIntLiteral(c_uint, 10000000, .decimal);
pub const __MAX_BAUD = __helpers.promoteIntLiteral(c_uint, 4294967295, .decimal);
pub inline fn CCEQ(val: anytype, c: anytype) @TypeOf((c == val) and (val != _POSIX_VDISABLE)) {
    _ = &val;
    _ = &c;
    return (c == val) and (val != _POSIX_VDISABLE);
}
pub const _UNISTD_H = @as(c_int, 1);
pub const _POSIX_VERSION = @as(c_long, 200809);
pub const __POSIX2_THIS_VERSION = @as(c_long, 200809);
pub const _POSIX2_VERSION = __POSIX2_THIS_VERSION;
pub const _POSIX2_C_VERSION = __POSIX2_THIS_VERSION;
pub const _POSIX2_C_BIND = __POSIX2_THIS_VERSION;
pub const _POSIX2_C_DEV = __POSIX2_THIS_VERSION;
pub const _POSIX2_SW_DEV = __POSIX2_THIS_VERSION;
pub const _POSIX2_LOCALEDEF = __POSIX2_THIS_VERSION;
pub const _XOPEN_VERSION = @as(c_int, 700);
pub const _XOPEN_XCU_VERSION = @as(c_int, 4);
pub const _XOPEN_XPG2 = @as(c_int, 1);
pub const _XOPEN_XPG3 = @as(c_int, 1);
pub const _XOPEN_XPG4 = @as(c_int, 1);
pub const _XOPEN_UNIX = @as(c_int, 1);
pub const _XOPEN_ENH_I18N = @as(c_int, 1);
pub const _XOPEN_LEGACY = @as(c_int, 1);
pub const _BITS_POSIX_OPT_H = @as(c_int, 1);
pub const _POSIX_JOB_CONTROL = @as(c_int, 1);
pub const _POSIX_SAVED_IDS = @as(c_int, 1);
pub const _POSIX_PRIORITY_SCHEDULING = @as(c_long, 200809);
pub const _POSIX_SYNCHRONIZED_IO = @as(c_long, 200809);
pub const _POSIX_FSYNC = @as(c_long, 200809);
pub const _POSIX_MAPPED_FILES = @as(c_long, 200809);
pub const _POSIX_MEMLOCK = @as(c_long, 200809);
pub const _POSIX_MEMLOCK_RANGE = @as(c_long, 200809);
pub const _POSIX_MEMORY_PROTECTION = @as(c_long, 200809);
pub const _POSIX_CHOWN_RESTRICTED = @as(c_int, 0);
pub const _POSIX_VDISABLE = @as(c_int, 0);
pub const _POSIX_NO_TRUNC = @as(c_int, 1);
pub const _XOPEN_REALTIME = @as(c_int, 1);
pub const _XOPEN_REALTIME_THREADS = @as(c_int, 1);
pub const _XOPEN_SHM = @as(c_int, 1);
pub const _POSIX_THREADS = @as(c_long, 200809);
pub const _POSIX_REENTRANT_FUNCTIONS = @as(c_int, 1);
pub const _POSIX_THREAD_SAFE_FUNCTIONS = @as(c_long, 200809);
pub const _POSIX_THREAD_PRIORITY_SCHEDULING = @as(c_long, 200809);
pub const _POSIX_THREAD_ATTR_STACKSIZE = @as(c_long, 200809);
pub const _POSIX_THREAD_ATTR_STACKADDR = @as(c_long, 200809);
pub const _POSIX_THREAD_PRIO_INHERIT = @as(c_long, 200809);
pub const _POSIX_THREAD_PRIO_PROTECT = @as(c_long, 200809);
pub const _POSIX_THREAD_ROBUST_PRIO_INHERIT = @as(c_long, 200809);
pub const _POSIX_THREAD_ROBUST_PRIO_PROTECT = -@as(c_int, 1);
pub const _POSIX_SEMAPHORES = @as(c_long, 200809);
pub const _POSIX_REALTIME_SIGNALS = @as(c_long, 200809);
pub const _POSIX_ASYNCHRONOUS_IO = @as(c_long, 200809);
pub const _POSIX_ASYNC_IO = @as(c_int, 1);
pub const _LFS_ASYNCHRONOUS_IO = @as(c_int, 1);
pub const _POSIX_PRIORITIZED_IO = @as(c_long, 200809);
pub const _LFS64_ASYNCHRONOUS_IO = @as(c_int, 1);
pub const _LFS_LARGEFILE = @as(c_int, 1);
pub const _LFS64_LARGEFILE = @as(c_int, 1);
pub const _LFS64_STDIO = @as(c_int, 1);
pub const _POSIX_SHARED_MEMORY_OBJECTS = @as(c_long, 200809);
pub const _POSIX_CPUTIME = @as(c_int, 0);
pub const _POSIX_THREAD_CPUTIME = @as(c_int, 0);
pub const _POSIX_REGEXP = @as(c_int, 1);
pub const _POSIX_READER_WRITER_LOCKS = @as(c_long, 200809);
pub const _POSIX_SHELL = @as(c_int, 1);
pub const _POSIX_TIMEOUTS = @as(c_long, 200809);
pub const _POSIX_SPIN_LOCKS = @as(c_long, 200809);
pub const _POSIX_SPAWN = @as(c_long, 200809);
pub const _POSIX_TIMERS = @as(c_long, 200809);
pub const _POSIX_BARRIERS = @as(c_long, 200809);
pub const _POSIX_MESSAGE_PASSING = @as(c_long, 200809);
pub const _POSIX_THREAD_PROCESS_SHARED = @as(c_long, 200809);
pub const _POSIX_MONOTONIC_CLOCK = @as(c_int, 0);
pub const _POSIX_CLOCK_SELECTION = @as(c_long, 200809);
pub const _POSIX_ADVISORY_INFO = @as(c_long, 200809);
pub const _POSIX_IPV6 = @as(c_long, 200809);
pub const _POSIX_RAW_SOCKETS = @as(c_long, 200809);
pub const _POSIX2_CHAR_TERM = @as(c_long, 200809);
pub const _POSIX_SPORADIC_SERVER = -@as(c_int, 1);
pub const _POSIX_THREAD_SPORADIC_SERVER = -@as(c_int, 1);
pub const _POSIX_TRACE = -@as(c_int, 1);
pub const _POSIX_TRACE_EVENT_FILTER = -@as(c_int, 1);
pub const _POSIX_TRACE_INHERIT = -@as(c_int, 1);
pub const _POSIX_TRACE_LOG = -@as(c_int, 1);
pub const _POSIX_TYPED_MEMORY_OBJECTS = -@as(c_int, 1);
pub const _POSIX_V7_LPBIG_OFFBIG = -@as(c_int, 1);
pub const _POSIX_V6_LPBIG_OFFBIG = -@as(c_int, 1);
pub const _XBS5_LPBIG_OFFBIG = -@as(c_int, 1);
pub const _POSIX_V7_LP64_OFF64 = @as(c_int, 1);
pub const _POSIX_V6_LP64_OFF64 = @as(c_int, 1);
pub const _XBS5_LP64_OFF64 = @as(c_int, 1);
pub const __ILP32_OFF32_CFLAGS = "-m32";
pub const __ILP32_OFF32_LDFLAGS = "-m32";
pub const __ILP32_OFFBIG_CFLAGS = "-m32 -D_LARGEFILE_SOURCE -D_FILE_OFFSET_BITS=64";
pub const __ILP32_OFFBIG_LDFLAGS = "-m32";
pub const __LP64_OFF64_CFLAGS = "-m64";
pub const __LP64_OFF64_LDFLAGS = "-m64";
pub const STDIN_FILENO = @as(c_int, 0);
pub const STDOUT_FILENO = @as(c_int, 1);
pub const STDERR_FILENO = @as(c_int, 2);
pub const __socklen_t_defined = "";
pub const L_SET = SEEK_SET;
pub const L_INCR = SEEK_CUR;
pub const L_XTND = SEEK_END;
pub const _SC_PAGE_SIZE = _SC_PAGESIZE;
pub const _CS_POSIX_V6_WIDTH_RESTRICTED_ENVS = _CS_V6_WIDTH_RESTRICTED_ENVS;
pub const _CS_POSIX_V5_WIDTH_RESTRICTED_ENVS = _CS_V5_WIDTH_RESTRICTED_ENVS;
pub const _CS_POSIX_V7_WIDTH_RESTRICTED_ENVS = _CS_V7_WIDTH_RESTRICTED_ENVS;
pub const _GETOPT_POSIX_H = @as(c_int, 1);
pub const _GETOPT_CORE_H = @as(c_int, 1);
pub const TB_PATH_MAX = PATH_MAX;
pub const TB_VERSION_STR = "2.7.0-dev";
pub const TB_KEY_CTRL_TILDE = @as(c_int, 0x00);
pub const TB_KEY_CTRL_2 = @as(c_int, 0x00);
pub const TB_KEY_CTRL_A = @as(c_int, 0x01);
pub const TB_KEY_CTRL_B = @as(c_int, 0x02);
pub const TB_KEY_CTRL_C = @as(c_int, 0x03);
pub const TB_KEY_CTRL_D = @as(c_int, 0x04);
pub const TB_KEY_CTRL_E = @as(c_int, 0x05);
pub const TB_KEY_CTRL_F = @as(c_int, 0x06);
pub const TB_KEY_CTRL_G = @as(c_int, 0x07);
pub const TB_KEY_BACKSPACE = @as(c_int, 0x08);
pub const TB_KEY_CTRL_H = @as(c_int, 0x08);
pub const TB_KEY_TAB = @as(c_int, 0x09);
pub const TB_KEY_CTRL_I = @as(c_int, 0x09);
pub const TB_KEY_CTRL_J = @as(c_int, 0x0a);
pub const TB_KEY_CTRL_K = @as(c_int, 0x0b);
pub const TB_KEY_CTRL_L = @as(c_int, 0x0c);
pub const TB_KEY_ENTER = @as(c_int, 0x0d);
pub const TB_KEY_CTRL_M = @as(c_int, 0x0d);
pub const TB_KEY_CTRL_N = @as(c_int, 0x0e);
pub const TB_KEY_CTRL_O = @as(c_int, 0x0f);
pub const TB_KEY_CTRL_P = @as(c_int, 0x10);
pub const TB_KEY_CTRL_Q = @as(c_int, 0x11);
pub const TB_KEY_CTRL_R = @as(c_int, 0x12);
pub const TB_KEY_CTRL_S = @as(c_int, 0x13);
pub const TB_KEY_CTRL_T = @as(c_int, 0x14);
pub const TB_KEY_CTRL_U = @as(c_int, 0x15);
pub const TB_KEY_CTRL_V = @as(c_int, 0x16);
pub const TB_KEY_CTRL_W = @as(c_int, 0x17);
pub const TB_KEY_CTRL_X = @as(c_int, 0x18);
pub const TB_KEY_CTRL_Y = @as(c_int, 0x19);
pub const TB_KEY_CTRL_Z = @as(c_int, 0x1a);
pub const TB_KEY_ESC = @as(c_int, 0x1b);
pub const TB_KEY_CTRL_LSQ_BRACKET = @as(c_int, 0x1b);
pub const TB_KEY_CTRL_3 = @as(c_int, 0x1b);
pub const TB_KEY_CTRL_4 = @as(c_int, 0x1c);
pub const TB_KEY_CTRL_BACKSLASH = @as(c_int, 0x1c);
pub const TB_KEY_CTRL_5 = @as(c_int, 0x1d);
pub const TB_KEY_CTRL_RSQ_BRACKET = @as(c_int, 0x1d);
pub const TB_KEY_CTRL_6 = @as(c_int, 0x1e);
pub const TB_KEY_CTRL_7 = @as(c_int, 0x1f);
pub const TB_KEY_CTRL_SLASH = @as(c_int, 0x1f);
pub const TB_KEY_CTRL_UNDERSCORE = @as(c_int, 0x1f);
pub const TB_KEY_SPACE = @as(c_int, 0x20);
pub const TB_KEY_BACKSPACE2 = @as(c_int, 0x7f);
pub const TB_KEY_CTRL_8 = @as(c_int, 0x7f);
pub inline fn tb_key_i(i: anytype) @TypeOf(__helpers.promoteIntLiteral(c_int, 0xffff, .hex) - i) {
    _ = &i;
    return __helpers.promoteIntLiteral(c_int, 0xffff, .hex) - i;
}
pub const TB_KEY_F1 = __helpers.promoteIntLiteral(c_int, 0xffff, .hex) - @as(c_int, 0);
pub const TB_KEY_F2 = __helpers.promoteIntLiteral(c_int, 0xffff, .hex) - @as(c_int, 1);
pub const TB_KEY_F3 = __helpers.promoteIntLiteral(c_int, 0xffff, .hex) - @as(c_int, 2);
pub const TB_KEY_F4 = __helpers.promoteIntLiteral(c_int, 0xffff, .hex) - @as(c_int, 3);
pub const TB_KEY_F5 = __helpers.promoteIntLiteral(c_int, 0xffff, .hex) - @as(c_int, 4);
pub const TB_KEY_F6 = __helpers.promoteIntLiteral(c_int, 0xffff, .hex) - @as(c_int, 5);
pub const TB_KEY_F7 = __helpers.promoteIntLiteral(c_int, 0xffff, .hex) - @as(c_int, 6);
pub const TB_KEY_F8 = __helpers.promoteIntLiteral(c_int, 0xffff, .hex) - @as(c_int, 7);
pub const TB_KEY_F9 = __helpers.promoteIntLiteral(c_int, 0xffff, .hex) - @as(c_int, 8);
pub const TB_KEY_F10 = __helpers.promoteIntLiteral(c_int, 0xffff, .hex) - @as(c_int, 9);
pub const TB_KEY_F11 = __helpers.promoteIntLiteral(c_int, 0xffff, .hex) - @as(c_int, 10);
pub const TB_KEY_F12 = __helpers.promoteIntLiteral(c_int, 0xffff, .hex) - @as(c_int, 11);
pub const TB_KEY_INSERT = __helpers.promoteIntLiteral(c_int, 0xffff, .hex) - @as(c_int, 12);
pub const TB_KEY_DELETE = __helpers.promoteIntLiteral(c_int, 0xffff, .hex) - @as(c_int, 13);
pub const TB_KEY_HOME = __helpers.promoteIntLiteral(c_int, 0xffff, .hex) - @as(c_int, 14);
pub const TB_KEY_END = __helpers.promoteIntLiteral(c_int, 0xffff, .hex) - @as(c_int, 15);
pub const TB_KEY_PGUP = __helpers.promoteIntLiteral(c_int, 0xffff, .hex) - @as(c_int, 16);
pub const TB_KEY_PGDN = __helpers.promoteIntLiteral(c_int, 0xffff, .hex) - @as(c_int, 17);
pub const TB_KEY_ARROW_UP = __helpers.promoteIntLiteral(c_int, 0xffff, .hex) - @as(c_int, 18);
pub const TB_KEY_ARROW_DOWN = __helpers.promoteIntLiteral(c_int, 0xffff, .hex) - @as(c_int, 19);
pub const TB_KEY_ARROW_LEFT = __helpers.promoteIntLiteral(c_int, 0xffff, .hex) - @as(c_int, 20);
pub const TB_KEY_ARROW_RIGHT = __helpers.promoteIntLiteral(c_int, 0xffff, .hex) - @as(c_int, 21);
pub const TB_KEY_BACK_TAB = __helpers.promoteIntLiteral(c_int, 0xffff, .hex) - @as(c_int, 22);
pub const TB_KEY_MOUSE_LEFT = __helpers.promoteIntLiteral(c_int, 0xffff, .hex) - @as(c_int, 23);
pub const TB_KEY_MOUSE_RIGHT = __helpers.promoteIntLiteral(c_int, 0xffff, .hex) - @as(c_int, 24);
pub const TB_KEY_MOUSE_MIDDLE = __helpers.promoteIntLiteral(c_int, 0xffff, .hex) - @as(c_int, 25);
pub const TB_KEY_MOUSE_RELEASE = __helpers.promoteIntLiteral(c_int, 0xffff, .hex) - @as(c_int, 26);
pub const TB_KEY_MOUSE_WHEEL_UP = __helpers.promoteIntLiteral(c_int, 0xffff, .hex) - @as(c_int, 27);
pub const TB_KEY_MOUSE_WHEEL_DOWN = __helpers.promoteIntLiteral(c_int, 0xffff, .hex) - @as(c_int, 28);
pub const TB_CAP_F1 = @as(c_int, 0);
pub const TB_CAP_F2 = @as(c_int, 1);
pub const TB_CAP_F3 = @as(c_int, 2);
pub const TB_CAP_F4 = @as(c_int, 3);
pub const TB_CAP_F5 = @as(c_int, 4);
pub const TB_CAP_F6 = @as(c_int, 5);
pub const TB_CAP_F7 = @as(c_int, 6);
pub const TB_CAP_F8 = @as(c_int, 7);
pub const TB_CAP_F9 = @as(c_int, 8);
pub const TB_CAP_F10 = @as(c_int, 9);
pub const TB_CAP_F11 = @as(c_int, 10);
pub const TB_CAP_F12 = @as(c_int, 11);
pub const TB_CAP_INSERT = @as(c_int, 12);
pub const TB_CAP_DELETE = @as(c_int, 13);
pub const TB_CAP_HOME = @as(c_int, 14);
pub const TB_CAP_END = @as(c_int, 15);
pub const TB_CAP_PGUP = @as(c_int, 16);
pub const TB_CAP_PGDN = @as(c_int, 17);
pub const TB_CAP_ARROW_UP = @as(c_int, 18);
pub const TB_CAP_ARROW_DOWN = @as(c_int, 19);
pub const TB_CAP_ARROW_LEFT = @as(c_int, 20);
pub const TB_CAP_ARROW_RIGHT = @as(c_int, 21);
pub const TB_CAP_BACK_TAB = @as(c_int, 22);
pub const TB_CAP__COUNT_KEYS = @as(c_int, 23);
pub const TB_CAP_ENTER_CA = @as(c_int, 23);
pub const TB_CAP_EXIT_CA = @as(c_int, 24);
pub const TB_CAP_SHOW_CURSOR = @as(c_int, 25);
pub const TB_CAP_HIDE_CURSOR = @as(c_int, 26);
pub const TB_CAP_CLEAR_SCREEN = @as(c_int, 27);
pub const TB_CAP_SGR0 = @as(c_int, 28);
pub const TB_CAP_UNDERLINE = @as(c_int, 29);
pub const TB_CAP_BOLD = @as(c_int, 30);
pub const TB_CAP_BLINK = @as(c_int, 31);
pub const TB_CAP_ITALIC = @as(c_int, 32);
pub const TB_CAP_REVERSE = @as(c_int, 33);
pub const TB_CAP_ENTER_KEYPAD = @as(c_int, 34);
pub const TB_CAP_EXIT_KEYPAD = @as(c_int, 35);
pub const TB_CAP_DIM = @as(c_int, 36);
pub const TB_CAP_INVISIBLE = @as(c_int, 37);
pub const TB_CAP__COUNT = @as(c_int, 38);
pub const TB_HARDCAP_ENTER_MOUSE = "\x1b[?1000h\x1b[?1002h\x1b[?1015h\x1b[?1006h";
pub const TB_HARDCAP_EXIT_MOUSE = "\x1b[?1006l\x1b[?1015l\x1b[?1002l\x1b[?1000l";
pub const TB_HARDCAP_STRIKEOUT = "\x1b[9m";
pub const TB_HARDCAP_UNDERLINE_2 = "\x1b[21m";
pub const TB_HARDCAP_OVERLINE = "\x1b[53m";
pub const TB_DEFAULT = @as(c_int, 0x0000);
pub const TB_BLACK = @as(c_int, 0x0001);
pub const TB_RED = @as(c_int, 0x0002);
pub const TB_GREEN = @as(c_int, 0x0003);
pub const TB_YELLOW = @as(c_int, 0x0004);
pub const TB_BLUE = @as(c_int, 0x0005);
pub const TB_MAGENTA = @as(c_int, 0x0006);
pub const TB_CYAN = @as(c_int, 0x0007);
pub const TB_WHITE = @as(c_int, 0x0008);
pub const TB_BOLD = __helpers.promoteIntLiteral(c_int, 0x01000000, .hex);
pub const TB_UNDERLINE = __helpers.promoteIntLiteral(c_int, 0x02000000, .hex);
pub const TB_REVERSE = __helpers.promoteIntLiteral(c_int, 0x04000000, .hex);
pub const TB_ITALIC = __helpers.promoteIntLiteral(c_int, 0x08000000, .hex);
pub const TB_BLINK = __helpers.promoteIntLiteral(c_int, 0x10000000, .hex);
pub const TB_HI_BLACK = __helpers.promoteIntLiteral(c_int, 0x20000000, .hex);
pub const TB_BRIGHT = __helpers.promoteIntLiteral(c_int, 0x40000000, .hex);
pub const TB_DIM = __helpers.promoteIntLiteral(c_int, 0x80000000, .hex);
pub const TB_TRUECOLOR_BOLD = TB_BOLD;
pub const TB_TRUECOLOR_UNDERLINE = TB_UNDERLINE;
pub const TB_TRUECOLOR_REVERSE = TB_REVERSE;
pub const TB_TRUECOLOR_ITALIC = TB_ITALIC;
pub const TB_TRUECOLOR_BLINK = TB_BLINK;
pub const TB_TRUECOLOR_BLACK = TB_HI_BLACK;
pub const TB_EVENT_KEY = @as(c_int, 1);
pub const TB_EVENT_RESIZE = @as(c_int, 2);
pub const TB_EVENT_MOUSE = @as(c_int, 3);
pub const TB_MOD_ALT = @as(c_int, 1);
pub const TB_MOD_CTRL = @as(c_int, 2);
pub const TB_MOD_SHIFT = @as(c_int, 4);
pub const TB_MOD_MOTION = @as(c_int, 8);
pub const TB_INPUT_CURRENT = @as(c_int, 0);
pub const TB_INPUT_ESC = @as(c_int, 1);
pub const TB_INPUT_ALT = @as(c_int, 2);
pub const TB_INPUT_MOUSE = @as(c_int, 4);
pub const TB_OUTPUT_CURRENT = @as(c_int, 0);
pub const TB_OUTPUT_NORMAL = @as(c_int, 1);
pub const TB_OUTPUT_256 = @as(c_int, 2);
pub const TB_OUTPUT_216 = @as(c_int, 3);
pub const TB_OUTPUT_GRAYSCALE = @as(c_int, 4);
pub const TB_OUTPUT_TRUECOLOR = @as(c_int, 5);
pub const TB_OK = @as(c_int, 0);
pub const TB_ERR = -@as(c_int, 1);
pub const TB_ERR_NEED_MORE = -@as(c_int, 2);
pub const TB_ERR_INIT_ALREADY = -@as(c_int, 3);
pub const TB_ERR_INIT_OPEN = -@as(c_int, 4);
pub const TB_ERR_MEM = -@as(c_int, 5);
pub const TB_ERR_NO_EVENT = -@as(c_int, 6);
pub const TB_ERR_NO_TERM = -@as(c_int, 7);
pub const TB_ERR_NOT_INIT = -@as(c_int, 8);
pub const TB_ERR_OUT_OF_BOUNDS = -@as(c_int, 9);
pub const TB_ERR_READ = -@as(c_int, 10);
pub const TB_ERR_RESIZE_IOCTL = -@as(c_int, 11);
pub const TB_ERR_RESIZE_PIPE = -@as(c_int, 12);
pub const TB_ERR_RESIZE_SIGACTION = -@as(c_int, 13);
pub const TB_ERR_POLL = -@as(c_int, 14);
pub const TB_ERR_TCGETATTR = -@as(c_int, 15);
pub const TB_ERR_TCSETATTR = -@as(c_int, 16);
pub const TB_ERR_UNSUPPORTED_TERM = -@as(c_int, 17);
pub const TB_ERR_RESIZE_WRITE = -@as(c_int, 18);
pub const TB_ERR_RESIZE_POLL = -@as(c_int, 19);
pub const TB_ERR_RESIZE_READ = -@as(c_int, 20);
pub const TB_ERR_RESIZE_SSCANF = -@as(c_int, 21);
pub const TB_ERR_CAP_COLLISION = -@as(c_int, 22);
pub const TB_ERR_SELECT = TB_ERR_POLL;
pub const TB_ERR_RESIZE_SELECT = TB_ERR_RESIZE_POLL;
pub const TB_FUNC_EXTRACT_PRE = @as(c_int, 0);
pub const TB_FUNC_EXTRACT_POST = @as(c_int, 1);
pub const TB_OPT_PRINTF_BUF = @as(c_int, 4096);
pub const TB_OPT_READ_BUF = @as(c_int, 64);
pub const tb_malloc = malloc;
pub const tb_realloc = realloc;
pub const tb_free = free;
pub const if_err_return = @compileError("unable to translate C expr: unexpected token 'if'"); // ./zig-pkg/N-V-__8AAAUXBQD6Fwpi9m0MBqWXFFaqW5l1lVrJC2Ynj7a-/termbox2.h:768:9
pub const if_err_break = @compileError("unable to translate C expr: unexpected token 'if'"); // ./zig-pkg/N-V-__8AAAUXBQD6Fwpi9m0MBqWXFFaqW5l1lVrJC2Ynj7a-/termbox2.h:770:9
pub const if_ok_return = @compileError("unable to translate C expr: unexpected token 'if'"); // ./zig-pkg/N-V-__8AAAUXBQD6Fwpi9m0MBqWXFFaqW5l1lVrJC2Ynj7a-/termbox2.h:772:9
pub const if_ok_or_need_more_return = @compileError("unable to translate C expr: unexpected token 'if'"); // ./zig-pkg/N-V-__8AAAUXBQD6Fwpi9m0MBqWXFFaqW5l1lVrJC2Ynj7a-/termbox2.h:774:9
pub inline fn send_literal(rv: anytype, a: anytype) @TypeOf(if_err_return(rv, bytebuf_nputs(&global.out, a, __helpers.sizeof(a) - @as(c_int, 1)))) {
    _ = &rv;
    _ = &a;
    return if_err_return(rv, bytebuf_nputs(&global.out, a, __helpers.sizeof(a) - @as(c_int, 1)));
}
pub inline fn send_num(rv: anytype, nbuf: anytype, n: anytype) @TypeOf(if_err_return(rv, bytebuf_nputs(&global.out, nbuf, convert_num(n, nbuf)))) {
    _ = &rv;
    _ = &nbuf;
    _ = &n;
    return if_err_return(rv, bytebuf_nputs(&global.out, nbuf, convert_num(n, nbuf)));
}
pub const snprintf_or_return = @compileError("unable to translate C expr: unexpected token 'do'"); // ./zig-pkg/N-V-__8AAAUXBQD6Fwpi9m0MBqWXFFaqW5l1lVrJC2Ynj7a-/termbox2.h:784:9
pub const if_not_init_return = @compileError("unable to translate C expr: unexpected token 'if'"); // ./zig-pkg/N-V-__8AAAUXBQD6Fwpi9m0MBqWXFFaqW5l1lVrJC2Ynj7a-/termbox2.h:790:9
pub const WCWIDTH_TABLE_LENGTH = @as(c_int, 2143);
pub const TB_RESIZE_FALLBACK_MS = @as(c_int, 1000);
pub const flock = struct_flock;
pub const timespec = struct_timespec;
pub const sigval = union_sigval;
pub const sigevent = struct_sigevent;
pub const _fpx_sw_bytes = struct__fpx_sw_bytes;
pub const _fpreg = struct__fpreg;
pub const _fpxreg = struct__fpxreg;
pub const _xmmreg = struct__xmmreg;
pub const _fpstate = struct__fpstate;
pub const sigcontext = struct_sigcontext;
pub const _xsave_hdr = struct__xsave_hdr;
pub const _ymmh_state = struct__ymmh_state;
pub const _xstate = struct__xstate;
pub const _libc_fpxreg = struct__libc_fpxreg;
pub const _libc_xmmreg = struct__libc_xmmreg;
pub const _libc_fpstate = struct__libc_fpstate;
pub const __pthread_internal_list = struct___pthread_internal_list;
pub const __pthread_internal_slist = struct___pthread_internal_slist;
pub const __pthread_mutex_s = struct___pthread_mutex_s;
pub const __pthread_rwlock_arch_t = struct___pthread_rwlock_arch_t;
pub const __pthread_cond_s = struct___pthread_cond_s;
pub const _G_fpos_t = struct__G_fpos_t;
pub const _G_fpos64_t = struct__G_fpos64_t;
pub const _IO_marker = struct__IO_marker;
pub const _IO_FILE = struct__IO_FILE;
pub const _IO_codecvt = struct__IO_codecvt;
pub const _IO_wide_data = struct__IO_wide_data;
pub const _IO_cookie_io_functions_t = struct__IO_cookie_io_functions_t;
pub const timeval = struct_timeval;
pub const random_data = struct_random_data;
pub const drand48_data = struct_drand48_data;
pub const __locale_struct = struct___locale_struct;
pub const winsize = struct_winsize;
pub const timezone = struct_timezone;
pub const __itimer_which = enum___itimer_which;
pub const itimerval = struct_itimerval;
pub const termios = struct_termios;
pub const tb_cell = struct_tb_cell;
pub const tb_event = struct_tb_event;
pub const bytebuf = struct_bytebuf;
pub const cellbuf = struct_cellbuf;
pub const cap_trie = struct_cap_trie;
pub const tb_global = struct_tb_global;
